@@ -1,26 +1,26 @@
-/* cfengine for GNU
+/*
+   Copyright (C) Cfengine AS
 
-        Copyright (C) 1995
-        Free Software Foundation, Inc.
-
-   This file is part of GNU cfengine - written and maintained
-   by Mark Burgess, Dept of Computing and Engineering, Oslo College,
-   Dept. of Theoretical physics, University of Oslo
+   This file is part of Cfengine 3 - written and maintained by Cfengine AS.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 2, or (at your option) any
-   later version.
+   Free Software Foundation; version 3.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 
+  To the extent this program is licensed as part of the Enterprise
+  versions of Cfengine, the applicable Commerical Open Source License
+  (COSL) may apply to this file if you as a licensee so wish it. See
+  included file COSL.txt.
+*/
 
 /*******************************************************************/
 /*                                                                 */
@@ -28,6 +28,9 @@
 /*                                                                 */
 /*******************************************************************/
 
+#ifndef CFENGINE_CF_DEFS_H
+#define CFENGINE_CF_DEFS_H
+
 /* Hard link this file between cf2/cf3 for consistent update */
 
 #include "conf.h"
@@ -35,10 +38,26 @@
 #ifdef NT
 #  define MAX_FILENAME 227
 #  define WINVER 0x501
+#  define FD_SETSIZE 512  // increase select(2) FD limit from 64
 #else
 #  define MAX_FILENAME 254
 #endif
 
+#ifdef MINGW
+# include <winsock2.h>
+# include <windows.h>
+# include <accctrl.h>
+# include <aclapi.h>
+# include <psapi.h>
+# include <wchar.h>
+# include <sddl.h>
+# include <tlhelp32.h>
+# include <iphlpapi.h>
+# include <ws2tcpip.h>
+# include <objbase.h>  // for disphelper
+#endif
+
+
 #include <stdio.h>
 #include <math.h>
 
@@ -104,6 +123,9 @@ struct utsname
 #define WTERMSIG(s) ((s) & 0)
 #endif
 
+#include "bool.h"
+#include "compiler.h"
+
 #include <openssl/err.h>
 #include <openssl/pem.h>
 #include <openssl/evp.h>
@@ -140,11 +162,14 @@ struct utsname
 #define LOG_LOCAL7      (23<<3)
 #define LOG_USER        (1<<3)
 #define LOG_DAEMON      (3<<3)
-#else
+
+#else  /* NOT MINGW */
+
 #include <syslog.h>
-#endif
 
 extern int errno;
+#endif
+
 
 /* Do this for ease of configuration from the Makefile */
 
@@ -177,6 +202,10 @@ extern int errno;
 #define bzero(s, n)     memset ((s), 0, (n))
 #endif
 
+#ifndef HAVE_STRNDUP
+char *strndup(const char *s, size_t n);
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -225,6 +254,8 @@ extern int errno;
 #  include <time.h>
 #endif
 
+#define _GNU_SOURCE
+
 #ifdef HAVE_TIME_H
 # include <time.h>
 #endif
@@ -242,24 +273,7 @@ extern int errno;
 # include <sys/sockio.h>
 #endif
 
-
-#ifdef MINGW
-# include <windows.h>
-# include <accctrl.h>
-# include <aclapi.h>
-# include <psapi.h>
-# include <wchar.h>
-# include <sddl.h>
-# include <tlhelp32.h>
-# include <iphlpapi.h>
-# include <ws2tcpip.h>
-# include <objbase.h>  // for disphelper
-# ifdef HAVE_WINSOCK2_H
-#  include <winsock2.h>
-# else
-#  include <winsock.h>
-# endif
-#else
+#ifndef MINGW
 # include <sys/socket.h>
 # include <sys/ioctl.h>
 # include <net/if.h>
@@ -285,14 +299,6 @@ extern int errno;
 #endif
 #endif
 
-#ifdef HAVE_PCRE_H
-# include <pcreposix.h>
-#elif HAVE_RXPOSIX_H
-# include <rxposix.h>
-#elif  HAVE_REGEX_H
-# include <regex.h>
-#endif
-
 #ifndef HAVE_SNPRINTF
 #include "../pub/snprintf.h"
 #endif
@@ -322,12 +328,22 @@ typedef int clockid_t;
 # include <selinux/selinux.h>
 #endif
 
+#ifndef MIN
+# define MIN(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
+#ifndef MAX
+# define MAX(a, b) ((a) > (b) ? (a) : (b))
+#endif
+
+#ifndef INADDR_NONE
+#define INADDR_NONE ((unsigned long int) 0xffffffff)
+#endif
+
 /*******************************************************************/
 /* Various defines                                                 */
 /*******************************************************************/
 
-#define true  1
-#define false 0
 #define CF_BUFSIZE 4096
 #define CF_BILLION 1000000000L
 #define CF_EXPANDSIZE (2*CF_BUFSIZE)
@@ -343,11 +359,13 @@ typedef int clockid_t;
 #define CF_MAXFARGS 8
 #define CF_MAX_IP_LEN 64       /* numerical ip length */
 #define CF_PROCCOLS 16
-#define CF_HASHTABLESIZE 4969 /* prime number */
+//#define CF_HASHTABLESIZE 7919 /* prime number */
+#define CF_HASHTABLESIZE 8192
 #define CF_MACROALPHABET 61    /* a-z, A-Z plus a bit */
+#define CF_ALPHABETSIZE 256
 #define CF_MAXSHELLARGS 64
 #define CF_MAX_SCLICODES 16
-#define CF_SAMEMODE 0
+#define CF_SAMEMODE 7777
 #define CF_SAME_OWNER ((uid_t)-1)
 #define CF_UNKNOWN_OWNER ((uid_t)-2)
 #define CF_SAME_GROUP ((gid_t)-1)
@@ -378,10 +396,12 @@ typedef int clockid_t;
 
 /* these should be >0 to prevent contention */
 
-#define CF_EXEC_IFELAPSED 1
+#define CF_EXEC_IFELAPSED 0
+#define CF_EDIT_IFELAPSED 5
 #define CF_EXEC_EXPIREAFTER 1
 
 #define MAXIP4CHARLEN 16
+#define PACK_UPIFELAPSED_SALT "packageuplist"
 
 /*******************************************************************/
 /*  DBM                                                            */
@@ -514,26 +534,26 @@ typedef u_long in_addr_t;  // as seen in in_addr struct in winsock.h
 /* database file names */
 
 #define CF_CLASSUSAGE     "cf_classes" "." DB_FEXT
+#define CF_VARIABLES      "cf_variables" "." DB_FEXT
 #define CF_PERFORMANCE    "performance" "." DB_FEXT
 #define CF_CHKDB          "checksum_digests" "." DB_FEXT
 #define CF_AVDB_FILE      "cf_observations" "." DB_FEXT
 #define CF_STATEDB_FILE   "cf_state" "." DB_FEXT
-#define CF_LASTDB_FILE    "cf_LastSeen" "." DB_FEXT
+#define CF_LASTDB_FILE    "cf_lastseen" "." DB_FEXT
 #define CF_AUDITDB_FILE   "cf_Audit" "." DB_FEXT
 #define CF_LOCKDB_FILE    "cf_lock" "." DB_FEXT
 
 /* end database file names */
 
 #define CF_VALUE_LOG      "cf_value.log"
-
+#define CF_FILECHANGE     "file_change.log"
+#define CF_PROMISE_LOG    "promise_summary.log"
 
 #define CF_STATELOG_FILE "state_log"
 #define CF_ENVNEW_FILE   "env_data.new"
 #define CF_ENV_FILE      "env_data"
 
 #define CF_TCPDUMP_COMM "/usr/sbin/tcpdump -t -n -v"
-#define CF_SCLI_COMM "/usr/local/bin/scli"
-
 
 #define CF_INPUTSVAR "CFINPUTS"          /* default name for file path var */
 #define CF_ALLCLASSESVAR "CFALLCLASSES"  /* default name for CFALLCLASSES env */
@@ -574,6 +594,14 @@ typedef u_long in_addr_t;  // as seen in in_addr struct in winsock.h
 # define ERESTARTSYS EINTR
 #endif
 
+#ifndef EOPNOTSUPP
+# define EOPNOTSUPP EINVAL
+#endif
+
+#ifndef ENOTSUPP
+# define ENOTSUPP EINVAL
+#endif
+
 #define CF_FAILEDSTR "BAD: Unspecified server refusal (see verbose server output)"
 #define CF_CHANGEDSTR1 "BAD: File changed "   /* Split this so it cannot be recognized */
 #define CF_CHANGEDSTR2 "while copying"
@@ -625,10 +653,13 @@ typedef u_long in_addr_t;  // as seen in in_addr struct in winsock.h
 #define ATTR     11
 #define CF_NETATTR   7 /* icmp udp dns tcpsyn tcpfin tcpack */
 #define PH_LIMIT 10
+#define CF_MONTH  (time_t)(3600*24*30)
 #define CF_WEEK   (7.0*24.0*3600.0)
 #define CF_HOUR   3600
+#define CF_DAY    3600*24
 #define CF_RELIABLE_CLASSES 7*24         /* CF_WEEK/CF_HOUR */
 #define CF_MEASURE_INTERVAL (5.0*60.0)
+#define CF_SHIFT_INTERVAL (6*3600.0)
 
 #define CF_OBSERVABLES 91
 
@@ -650,30 +681,6 @@ struct Averages
    struct QPoint Q[CF_OBSERVABLES];
    };
 
-struct OldAverages /* For conversion to new db */
-   {
-   double expect_number_of_users;
-   double expect_rootprocs;
-   double expect_otherprocs;
-   double expect_diskfree;
-   double expect_loadavg;
-   double expect_incoming[ATTR];
-   double expect_outgoing[ATTR];
-   double expect_pH[PH_LIMIT];
-   double var_number_of_users;
-   double var_rootprocs;
-   double var_otherprocs;
-   double var_diskfree;
-   double var_loadavg;
-   double var_incoming[ATTR];
-   double var_outgoing[ATTR];
-   double var_pH[PH_LIMIT];
-   double expect_netin[CF_NETATTR];
-   double expect_netout[CF_NETATTR];
-   double var_netin[CF_NETATTR];
-   double var_netout[CF_NETATTR];
-   };
-
 /******************************************************************/
 
 struct LockData
@@ -777,6 +784,7 @@ enum PROTOS
    cfd_svar,
    cfd_context,
    cfd_scontext,
+   cfd_squery,
    cfd_bad
    };
 
@@ -1108,33 +1116,15 @@ struct cfagent_connection
    int authenticated;
    int protoversion;
    int family;                              /* AF_INET or AF_INET6 */
+   char username[CF_SMALLBUF];
    char localip[CF_MAX_IP_LEN];
    char remoteip[CF_MAX_IP_LEN];
+   unsigned char digest[EVP_MAX_MD_SIZE+1];
    unsigned char *session_key;
    char encryption_type;
    short error;
    };
 
-
-/*******************************************************************/
-
-struct cfObject
-   {
-   char *scope;                         /* Name of object (scope) */
-   void *hashtable[CF_HASHTABLESIZE];   /* Variable heap  */
-   char type[CF_HASHTABLESIZE];         /* scalar or itlist? */
-   char *classlist;                     /* Private classes -- ? */
-   struct Item *actionsequence;
-   struct cfObject *next;
-   };
-
-/*
-
- $(globalvar)
- $(obj.name)
-
-*/
-
 /*******************************************************************/
 
 struct CompressedArray
@@ -1171,15 +1161,9 @@ struct Item
 
 /*******************************************************************/
 
-struct TwoDimList
+struct AlphaList  // Indexed itemlist
    {
-   short is2d;                  /* true if list > 1 */
-   short rounds;
-   short tied;                  /* do variables march together or in rounds ? */
-   char  sep;                   /* list separator */
-   struct Item *ilist;          /* Each node contains a list */
-   struct Item *current;        /* A static working pointer */
-   struct TwoDimList *next;
+   struct Item *list[256];
    };
 
 /*******************************************************************/
@@ -1389,8 +1373,11 @@ struct Checksum_Value
 #endif
 
 
-/********************************************************************/
-/* All prototypes                                                   */
-/********************************************************************/
+/* Nobody already knows why it was needed in first place. Please test whether
+   removing this variable is harmless on HP/UX nowadays. */
 
-#include "prototypes.h"
+#ifdef HPuUX
+int Error;
+#endif
+
+#endif
