@@ -1,22 +1,25 @@
-/* 
-   Copyright (C) 2008 - Cfengine AS
+/*
+   Copyright (C) Cfengine AS
 
    This file is part of Cfengine 3 - written and maintained by Cfengine AS.
- 
+
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 3, or (at your option) any
-   later version. 
+   Free Software Foundation; version 3.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
- 
+
   You should have received a copy of the GNU General Public License
-  
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 
+  To the extent this program is licensed as part of the Enterprise
+  versions of Cfengine, the applicable Commerical Open Source License
+  (COSL) may apply to this file if you as a licensee so wish it. See
+  included file COSL.txt.
 */
 
 /*******************************************************************/
@@ -25,6 +28,11 @@
 /*                                                                 */
 /*******************************************************************/
 
+#ifndef CFENGINE_CF3_SERVER_H
+#define CFENGINE_CF3_SERVER_H
+
+#ifndef HAVE_SERV_H
+#define HAVE_SERV_H 1
 #define queuesize 50
 #define connection 1
 #define RFC931_PORT 113
@@ -41,6 +49,7 @@ struct cfd_connection
    int  trust;
    int  sd_reply;
    unsigned char *session_key;
+   unsigned char digest[EVP_MAX_MD_SIZE+1];
    char hostname[CF_MAXVARSIZE];
    char username[CF_MAXVARSIZE];
    #ifdef MINGW
@@ -81,6 +90,6 @@ time_t CFDSTARTTIME;
 extern char *realpath(/* char *path; char resolved_path[MAXPATHLEN] */);
 #endif
 
-#ifndef INADDR_NONE
-#define INADDR_NONE ((unsigned long int) 0xffffffff)
+#endif
+
 #endif
