@@ -53,6 +53,10 @@ int LOOKUP = false;
 int VIEWS = true;
 int IGNORE_MISSING_INPUTS = false;
 int IGNORE_MISSING_BUNDLES = false;
+int FIPS_MODE = false;
+int ALWAYS_VALIDATE = false;
+
+unsigned int CFTEST_CLASS = 0;
 
 struct utsname VSYSNAME;
 
@@ -70,16 +74,19 @@ int AM_BACKGROUND_PROCESS = false;
 int CF_PERSISTENCE = 10;
 
 char *THIS_BUNDLE = NULL;
-char THIS_AGENT[CF_MAXVARSIZE];
+char THIS_AGENT[CF_MAXVARSIZE] = {0};
 enum cfagenttype THIS_AGENT_TYPE;
-char SYSLOGHOST[CF_MAXVARSIZE];
+char SYSLOGHOST[CF_MAXVARSIZE] = {0};
 unsigned short SYSLOGPORT = 514;
 int FACILITY = 0;
-time_t PROMISETIME;
+time_t PROMISETIME = 0;
+time_t CF_LOCKHORIZON = CF_MONTH;
 
 int LICENSES = 0;
-char EXPIRY[32];
+char EXPIRY[CF_SMALLBUF] = {0};
+char LICENSE_COMPANY[CF_SMALLBUF] = {0};
 int INSTALL_SKIP = false;
+int KEYTTL = 0;
 
 // These are used to measure graph complexity in know/agent
 
@@ -92,15 +99,13 @@ struct Rlist *MOUNTEDFSLIST = NULL;
 struct Rlist *SERVERLIST = NULL;
 struct PromiseIdent *PROMISE_ID_LIST = NULL;
 struct Item *PROCESSTABLE = NULL;
+struct Item *PROCESSREFRESH = NULL;
 struct Item *ROTATED = NULL;
 struct Item *FSTABLIST = NULL;
 struct Item *ABORTBUNDLEHEAP = NULL;
 struct Item *DONELIST = NULL;
 struct Rlist *CBUNDLESEQUENCE = NULL;
-
-#ifdef HAVE_LIBVIRT
-virConnectPtr CFVC[cfv_none];
-#endif
+struct Rlist *SERVER_KEYSEEN = NULL;
 
 int EDIT_MODEL = false;
 int CF_MOUNTALL = false;
@@ -109,7 +114,7 @@ int FSTAB_EDITS;
 int ABORTBUNDLE = false;
 int BOOTSTRAP = false;
 
-char HASHDB[CF_BUFSIZE];
+char HASHDB[CF_BUFSIZE] = {0};
 
 /*****************************************************************************/
 /* Measurements                                                              */
@@ -127,7 +132,7 @@ double Q_MIN;
 /* Internal data structures                                                  */
 /*****************************************************************************/
 
-struct PromiseParser P;
+struct PromiseParser P = {0};
 struct Bundle *BUNDLES = NULL;
 struct Body *BODIES = NULL;
 struct Scope *VSCOPE = NULL;
@@ -149,13 +154,22 @@ int LASTSEEN = false;
 
 struct Topic *TOPIC_MAP = NULL;
 
-char POLICY_SERVER[CF_BUFSIZE];
+char POLICY_SERVER[CF_BUFSIZE] = {0};
 
-char WEBDRIVER[CF_MAXVARSIZE];
-char BANNER[2*CF_BUFSIZE];
-char FOOTER[CF_BUFSIZE];
-char STYLESHEET[CF_BUFSIZE];
-char AGGREGATION[CF_BUFSIZE];
+char WEBDRIVER[CF_MAXVARSIZE] = {0};
+char DOCROOT[CF_MAXVARSIZE] = {0};
+char BANNER[2*CF_BUFSIZE] = {0};
+char FOOTER[CF_BUFSIZE] = {0};
+char STYLESHEET[CF_BUFSIZE] = {0};
+char AGGREGATION[CF_BUFSIZE] = {0};
+struct Topic *TOPICHASH[CF_HASHTABLESIZE];
+
+char SQL_DATABASE[CF_MAXVARSIZE] = {0};
+char SQL_OWNER[CF_MAXVARSIZE] = {0};
+char SQL_PASSWD[CF_MAXVARSIZE] = {0};
+char SQL_SERVER[CF_MAXVARSIZE] = {0};
+char SQL_CONNECT_NAME[CF_MAXVARSIZE] = {0};
+enum cfdbtype SQL_TYPE = cfd_notype;
 
 /*****************************************************************************/
 /* Windows version constants                                                 */
@@ -184,7 +198,8 @@ char *DAY_TEXT[] =
    "Thursday",
    "Friday",
    "Saturday",
-   "Sunday"
+   "Sunday",
+   NULL
    };
 
 char *MONTH_TEXT[] =
@@ -200,7 +215,8 @@ char *MONTH_TEXT[] =
    "September",
    "October",
    "November",
-   "December"
+   "December",
+   NULL
    };
 
 char *SHIFT_TEXT[] =
@@ -208,7 +224,8 @@ char *SHIFT_TEXT[] =
    "Night",
    "Morning",
    "Afternoon",
-   "Evening"
+   "Evening",
+   NULL
    };
 
 /*****************************************************************************/
@@ -246,6 +263,7 @@ char *CF_AGENTTYPES[] = /* see enum cfagenttype */
    CF_KNOWC,
    CF_REPORTC,
    CF_KEYGEN,
+   CF_HUBC,
    "<notype>",
    };
 
@@ -263,36 +281,37 @@ int D2 = false;
 int AUDIT = false;
 int LOGGING = false;
 
-char  VFQNAME[CF_MAXVARSIZE];
-char  VUQNAME[CF_MAXVARSIZE];
-char  VDOMAIN[CF_MAXVARSIZE];
+char  VFQNAME[CF_MAXVARSIZE] = {0};
+char  VUQNAME[CF_MAXVARSIZE] = {0};
+char  VDOMAIN[CF_MAXVARSIZE] = {0};
 
-char  VYEAR[5];
-char  VDAY[3];
-char  VMONTH[4];
-char  VHR[3];
-char  VMINUTE[3];
-char  VSEC[3];
-char  VSHIFT[12];
-char  VLIFECYCLE[12];
+char  VYEAR[5] = {0};
+char  VDAY[3] = {0};
+char  VMONTH[4] = {0};
+char  VHR[3] = {0};
+char  VMINUTE[3] = {0};
+char  VSEC[3] = {0};
+char  VSHIFT[12] = {0};
+char  VLIFECYCLE[12] = {0};
 
 char PADCHAR = ' ';
 char PURGE = 'n';
 
 int ERRORCOUNT = 0;
-char VPREFIX[CF_MAXVARSIZE];
-char VINPUTFILE[CF_BUFSIZE];
+char VPREFIX[CF_MAXVARSIZE] = {0};
+char VINPUTFILE[CF_BUFSIZE] = {0};
 
-char CONTEXTID[32];
-char CFPUBKEYFILE[CF_BUFSIZE];
-char CFPRIVKEYFILE[CF_BUFSIZE];
-char AVDB[CF_MAXVARSIZE];
-char CFWORKDIR[CF_BUFSIZE];
-char PIDFILE[CF_BUFSIZE];
+char CONTEXTID[32] = {0};
+char CFPUBKEYFILE[CF_BUFSIZE] = {0};
+char CFPRIVKEYFILE[CF_BUFSIZE] = {0};
+char AVDB[CF_MAXVARSIZE] = {0};
+char CFWORKDIR[CF_BUFSIZE] = {0};
+char PIDFILE[CF_BUFSIZE] = {0};
 
 char *DEFAULT_COPYTYPE = NULL;
 
 RSA *PRIVKEY = NULL, *PUBKEY = NULL;
+char PUBKEY_DIGEST[CF_MAXVARSIZE] = {0};
 
 pthread_attr_t PTHREADDEFAULTS;
 
@@ -305,6 +324,9 @@ pthread_mutex_t MUTEX_DBHANDLE = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
 pthread_mutex_t MUTEX_POLICY = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
 pthread_mutex_t MUTEX_GETADDR = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
 pthread_mutex_t MUTEX_DB_LASTSEEN = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
+pthread_mutex_t MUTEX_DB_REPORT = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
+pthread_mutex_t MUTEX_VSCOPE = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
+pthread_mutex_t MUTEX_SERVER_KEYSEEN = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
 #else
 # if defined HAVE_PTHREAD_H && (defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD)
 pthread_mutex_t MUTEX_SYSCALL = PTHREAD_MUTEX_INITIALIZER;
@@ -315,12 +337,14 @@ pthread_mutex_t MUTEX_DBHANDLE = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t MUTEX_POLICY = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t MUTEX_GETADDR = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t MUTEX_DB_LASTSEEN = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t MUTEX_DB_REPORT = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t MUTEX_VSCOPE = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t MUTEX_SERVER_KEYSEEN = PTHREAD_MUTEX_INITIALIZER;
 # endif
 #endif
 
 unsigned short PORTNUMBER = 0;
-char VIPADDRESS[18];
-int  CF_TIMEOUT = 10;
+char VIPADDRESS[18] = {0};
 int  CFSIGNATURE = 0;
 
 char *PROTOCOL[] =
@@ -343,15 +367,11 @@ char *PROTOCOL[] =
    "SVAR",
    "CONTEXT",
    "SCONTEXT",
+   "SQUERY",
    NULL
    };
 
 struct Item *IPADDRESSES = NULL;
-struct Item *VHEAP = NULL;
-struct Item *VNEGHEAP = NULL;
-struct Item *VDELCLASSES = NULL;
-struct Item *VADDCLASSES=NULL;           /* Action sequence defs  */
-struct Rlist *PRIVCLASSHEAP = NULL;
 
 int PR_KEPT = 0;
 int PR_REPAIRED = 0;
@@ -361,8 +381,22 @@ double VAL_KEPT = 0;
 double VAL_REPAIRED = 0;
 double VAL_NOTKEPT = 0;
 
-char FILE_SEPARATOR;
-char FILE_SEPARATOR_STR[2];
+char FILE_SEPARATOR = {0};
+char FILE_SEPARATOR_STR[2] = {0};
+
+
+/*******************************************************************/
+/* Context Management                                              */
+/*******************************************************************/
+
+struct AlphaList VHEAP;
+struct AlphaList VADDCLASSES;
+struct Item *VNEGHEAP = NULL;
+struct Item *VDELCLASSES = NULL;
+struct Rlist *PRIVCLASSHEAP = NULL;
+
+struct Rlist *GOALS = NULL;
+struct Rlist *GOALCATEGORIES = NULL;
 
 /*******************************************************************/
 /*                                                                 */
@@ -375,16 +409,16 @@ char FILE_SEPARATOR_STR[2];
 
 char *CF_DIGEST_TYPES[10][2] =
      {
-     "md5","m",
-     "sha224","c",
-     "sha256","C",
-     "sha384","h",
-     "sha512","H",
-     "sha1","S",
-     "sha","s",   /* Should come last, since substring */
-     "best","b",
-     "crypt","o",
-     NULL,NULL
+     {"md5","m"},
+     {"sha224","c"},
+     {"sha256","C"},
+     {"sha384","h"},
+     {"sha512","H"},
+     {"sha1","S"},
+     {"sha","s"},   /* Should come last, since substring */
+     {"best","b"},
+     {"crypt","o"},
+     {NULL,NULL}
      };
 
 int CF_DIGEST_SIZES[10] =
@@ -401,6 +435,9 @@ int CF_DIGEST_SIZES[10] =
      0
      };
 
+enum cfhashes CF_DEFAULT_DIGEST;
+int CF_DEFAULT_DIGEST_LEN;
+
 /***********************************************************/
 
 struct Audit *AUDITPTR;
@@ -408,13 +445,13 @@ struct Audit *VAUDIT = NULL;
 FILE *VLOGFP = NULL; 
 CF_DB  *AUDITDBP = NULL;
 
-char GRAPHDIR[CF_BUFSIZE];
-char CFLOCK[CF_BUFSIZE];
-char SAVELOCK[CF_BUFSIZE]; 
-char CFLOG[CF_BUFSIZE];
-char CFLAST[CF_BUFSIZE]; 
-char LOCKDB[CF_BUFSIZE];
-char LOGFILE[CF_MAXVARSIZE];
+char GRAPHDIR[CF_BUFSIZE] = {0};
+char CFLOCK[CF_BUFSIZE] = {0};
+char SAVELOCK[CF_BUFSIZE] = {0}; 
+char CFLOG[CF_BUFSIZE] = {0};
+char CFLAST[CF_BUFSIZE] = {0}; 
+char LOCKDB[CF_BUFSIZE] = {0};
+char LOGFILE[CF_MAXVARSIZE] = {0};
 
 char *SIGNALS[highest_signal];
 char *VSETUIDLOG = NULL;
@@ -422,12 +459,12 @@ char *VSETUIDLOG = NULL;
 time_t CFSTARTTIME;
 time_t CFINITSTARTTIME;
 dev_t ROOTDEVICE = 0;
-char  STR_CFENGINEPORT[16];
+char  STR_CFENGINEPORT[16] = {0};
 unsigned short SHORT_CFENGINEPORT;
-int RPCTIMEOUT = 60;          /* seconds */
+time_t CONNTIMEOUT = 10;	   /* seconds */
+time_t RECVTIMEOUT = 30;	   /* seconds */
+int RPCTIMEOUT = 60;			/* seconds */
 pid_t ALARM_PID = -1;
-int SENSIBLEFILECOUNT = 2;
-int SENSIBLEFSSIZE = 1000;
 int SKIPIDENTIFY = false;
 int ALL_SINGLECOPY = false;
 int FULLENCRYPT = false;
@@ -438,7 +475,7 @@ int VIFELAPSED = 1;
 int VEXPIREAFTER = 120;
 int UNDERSCORE_CLASSES=false;
 int CHECKSUMUPDATES = false;
-char BINDINTERFACE[CF_BUFSIZE];
+char BINDINTERFACE[CF_BUFSIZE] = {0};
 int MINUSF = false;
 int EXCLAIM = true;
 
@@ -497,98 +534,140 @@ char *TCPNAMES[CF_NETATTR] =
 
 char *OBS[CF_OBSERVABLES][2] =
     {
-    "users","Users logged in",
-    "rootprocs","Privileged system processes",
-    "otherprocs","Non-privileged process",
-    "diskfree","Free disk on / partition",
-    "loadavg","% kernel load utilization",
-    "netbiosns_in","netbios name lookups (in)",
-    "netbiosns_out","netbios name lookups (out)",
-    "netbiosdgm_in","netbios name datagrams (in)",
-    "netbiosdgm_out","netbios name datagrams (out)",
-    "netbiosssn_in","netbios name sessions (in)",
-    "netbiosssn_out","netbios name sessions (out)",
-    "irc_in","IRC connections (in)",
-    "irc_out","IRC connections (out)",
-    "cfengine_in","cfengine connections (in)",
-    "cfengine_out","cfengine connections (out)",
-    "nfsd_in","nfs connections (in)",
-    "nfsd_out","nfs connections (out)",
-    "smtp_in","smtp connections (in)",
-    "smtp_out","smtp connections (out)",
-    "www_in","www connections (in)",
-    "www_out","www connections (out)",
-    "ftp_in","ftp connections (in)",
-    "ftp_out","ftp connections (out)",
-    "ssh_in","ssh connections (in)",
-    "ssh_out","ssh connections (out)",
-    "wwws_in","wwws connections (in)",
-    "wwws_out","wwws connections (out)",
-    "icmp_in","ICMP packets (in)",
-    "icmp_out","ICMP packets (out)",
-    "udp_in","UDP dgrams (in)",
-    "udp_out","UDP dgrams (out)",
-    "dns_in","DNS requests (in)",
-    "dns_out","DNS requests (out)",
-    "tcpsyn_in","TCP sessions (in)",
-    "tcpsyn_out","TCP sessions (out)",
-    "tcpack_in","TCP acks (in)",
-    "tcpack_out","TCP acks (out)",
-    "tcpfin_in","TCP finish (in)",
-    "tcpfin_out","TCP finish (out)",
-    "tcpmisc_in","TCP misc (in)",
-    "tcpmisc_out","TCP misc (out)",
-    "webaccess","Webserver hits",
-    "weberrors","Webserver errors",
-    "syslog","New log entries (Syslog)",
-    "messages","New log entries (messages)",
-    "temp0","CPU Temperature 0",
-    "temp1","CPU Temperature 1",
-    "temp2","CPU Temperature 2",
-    "temp3","CPU Temperature 3",
-    "cpu","%CPU utilization (all)",
-    "cpu0","%CPU utilization 0",
-    "cpu1","%CPU utilization 1",
-    "cpu2","%CPU utilization 2",
-    "cpu3","%CPU utilization 3",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
-    "spare","unused",
+    {"users","Users with active processes"},
+    {"rootprocs","Sum privileged system processes"},
+    {"otherprocs","Sum non-privileged process"},
+    {"diskfree","Free disk on / partition"},
+    {"loadavg","% kernel load utilization"},
+    {"netbiosns_in","netbios name lookups (in)"},
+    {"netbiosns_out","netbios name lookups (out)"},
+    {"netbiosdgm_in","netbios name datagrams (in)"},
+    {"netbiosdgm_out","netbios name datagrams (out)"},
+    {"netbiosssn_in","netbios name sessions (in)"},
+    {"netbiosssn_out","netbios name sessions (out)"},
+    {"irc_in","IRC connections (in)"},
+    {"irc_out","IRC connections (out)"},
+    {"cfengine_in","cfengine connections (in)"},
+    {"cfengine_out","cfengine connections (out)"},
+    {"nfsd_in","nfs connections (in)"},
+    {"nfsd_out","nfs connections (out)"},
+    {"smtp_in","smtp connections (in)"},
+    {"smtp_out","smtp connections (out)"},
+    {"www_in","www connections (in)"},
+    {"www_out","www connections (out)"},
+    {"ftp_in","ftp connections (in)"},
+    {"ftp_out","ftp connections (out)"},
+    {"ssh_in","ssh connections (in)"},
+    {"ssh_out","ssh connections (out)"},
+    {"wwws_in","wwws connections (in)"},
+    {"wwws_out","wwws connections (out)"},
+    {"icmp_in","ICMP packets (in)"},
+    {"icmp_out","ICMP packets (out)"},
+    {"udp_in","UDP dgrams (in)"},
+    {"udp_out","UDP dgrams (out)"},
+    {"dns_in","DNS requests (in)"},
+    {"dns_out","DNS requests (out)"},
+    {"tcpsyn_in","TCP sessions (in)"},
+    {"tcpsyn_out","TCP sessions (out)"},
+    {"tcpack_in","TCP acks (in)"},
+    {"tcpack_out","TCP acks (out)"},
+    {"tcpfin_in","TCP finish (in)"},
+    {"tcpfin_out","TCP finish (out)"},
+    {"tcpmisc_in","TCP misc (in)"},
+    {"tcpmisc_out","TCP misc (out)"},
+    {"webaccess","Webserver hits"},
+    {"weberrors","Webserver errors"},
+    {"syslog","New log entries (Syslog)"},
+    {"messages","New log entries (messages)"},
+    {"temp0","CPU Temperature 0"},
+    {"temp1","CPU Temperature 1"},
+    {"temp2","CPU Temperature 2"},
+    {"temp3","CPU Temperature 3"},
+    {"cpu","%CPU utilization (all)"},
+    {"cpu0","%CPU utilization 0"},
+    {"cpu1","%CPU utilization 1"},
+    {"cpu2","%CPU utilization 2"},
+    {"cpu3","%CPU utilization 3"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
+    {"spare","unused"},
     };
 
-char *UNITS[CF_OBSERVABLES];
-time_t DATESTAMPS[CF_OBSERVABLES];
+char *UNITS[CF_OBSERVABLES] = {0};
+
+/**********************************************************************************/
+/* Report names                                                                   */
+/**********************************************************************************/
+
+// See enum in cf3.defs.h
+
+char *BASIC_REPORTS[cfrep_unknown+1][2] =
+    {
+    {"Bundle profile","Status of promise bundles and when they were last verified"},
+    {"Business value report","Accumulated value of promises kept"},
+    {"Class profile","User defined classes observed on the system"},
+    {"Compliance by promise","Compliance of each promise individually"},
+    {"Compliance summary","Total summary of host compliance"},
+    {"File change log","Log of all detected changes to files from changes promises"},
+    {"File change diffs","Delta/difference comparison showing file changes"},
+    {"Last saw hosts","Log of when neighbouring hosts were last observed online"},
+    {"Patches available","A list of patches currently claimed to be available by the local package manager"},
+    {"Patch status","A list of (un)applied patches according to the local package manager"},
+    {"Performance","Execution time used to verify selected promises"},
+    {"Promises repaired log","Log of actual repairs made to the system"},
+    {"Promises repaired summary","Cumulative (histogram) summary of promises repaired"},
+    {"Promises not kept log","Log of promises that could not or would not be kept"},
+    {"Promises not kept summary","Cumulative (histogram) summary of promises not kept"},
+    {"Setuid/gid root programs","Current list of observed setuid/setgid root programs"},
+    {"Software installed","List of software packages claimed to be installed according to the local package manager"},
+    {"Variables","Table of variable values last observed"},
+    {NULL,NULL}
+    };
+
+// See enum in cf3.defs.h
+
+char *CDP_REPORTS[cdp_unknown+1][2] =
+    {
+    {"ACLs","File access controls"},
+    {"Commands","Scheduled commands to execute"},
+    {"File Changes","File changes observed on the system"},
+    {"File Diffs","Delta/difference comparison showing file changes"},
+    {"Registry","Promised Windows registry setting status"},
+    {"Services","System services status"},
+    {NULL,NULL}
+    };
+        
