@@ -85,10 +85,12 @@ int    SMOOTHHISTOGRAM[CF_OBSERVABLES][7][CF_GRAINS];
 char   ERASE[CF_BUFSIZE];
 double AGE;
 
-static struct Averages ENTRY,MAX,MIN,DET;
+static struct Averages MAX,MIN;
 
 char OUTPUTDIR[CF_BUFSIZE],*sp;
 char REMOVEHOSTS[CF_BUFSIZE];
+char NOVA_EXPORT_TYPE[CF_MAXVARSIZE] = {0};
+char NOVA_IMPORT_FILE[CF_MAXVARSIZE] = {0};
 
 FILE *FPAV=NULL,*FPVAR=NULL, *FPNOW=NULL;
 FILE *FPE[CF_OBSERVABLES],*FPQ[CF_OBSERVABLES];
@@ -106,7 +108,7 @@ struct Rlist *CSVLIST = NULL;
             "data stored in cfengine's embedded databases in human\n"
             "readable form.";
 
- struct option OPTIONS[22] =
+ struct option OPTIONS[25] =
       {
       { "help",no_argument,0,'h' },
       { "debug",optional_argument,0,'d' },
@@ -125,14 +127,17 @@ struct Rlist *CSVLIST = NULL;
       { "timestamps",no_argument,0,'T'},
       { "resolution",no_argument,0,'R'},
       { "syntax",no_argument,0,'S'},
+      { "syntax-export",no_argument,0,'s'},
       { "no-error-bars",no_argument,0,'e'},
       { "no-scaling",no_argument,0,'n'},
       { "verbose",no_argument,0,'v'},
       { "remove-hosts",required_argument,0,'r'},
+      { "nova-export",required_argument,0,'x'},
+      { "nova-import",required_argument,0,'i'},
       { NULL,0,0,'\0' }
       };
 
- char *HINTS[22] =
+ char *HINTS[25] =
       {
       "Print the help message",
       "Set debugging level 0,1,2,3",
@@ -151,10 +156,13 @@ struct Rlist *CSVLIST = NULL;
       "Add a time stamp to directory name for graph file data",
       "Print graph data in high resolution",
       "Print a syntax summary for this cfengine version",
+      "Export a syntax tree in Javascript format",
       "Do not add error bars to the printed graphs",
       "Do not automatically scale the axes",
       "Generate verbose output",
       "Remove comma separated list of IP address entries from the hosts-seen database",
+      "Export Nova reports to file - delta or full report",
+      "Import Nova reports from file - specify the path (only on Nova policy hub)",
       NULL
       };
 
@@ -190,65 +198,69 @@ enum cf_format
    cfx_index,
    cfx_min,
    cfx_max,
-   cfx_end
+   cfx_end,
+   cfx_alias
    };
 
 char *CFRX[][2] =
    {
-    "<entry>\n","\n</entry>\n",
-    "<event>\n","\n</event>\n",
-    "<hostname>\n","\n</hostname>\n",
-    "<pm>\n","\n</pm>\n",
-    "<ip>\n","\n</ip>\n",
-    "<date>\n","\n</date>\n",
-    "<q>\n","\n</q>\n",
-    "<expect>\n","\n</expect>\n",
-    "<sigma>\n","\n</sigma>\n",
-    "<version>\n","\n</version>\n",
-    "<ref>\n","\n</ref>\n",
-    "<filename>\n","\n</filename>\n",
-    "<index>\n","\n</index>\n",
-    "<min>\n","\n</min>\n",
-    "<max>\n","\n</max>\n",
-    "<end>\n","\n</end>\n",
-    NULL,NULL
+    {"<entry>\n","\n</entry>\n"},
+    {"<event>\n","\n</event>\n"},
+    {"<hostname>\n","\n</hostname>\n"},
+    {"<pm>\n","\n</pm>\n"},
+    {"<ip>\n","\n</ip>\n"},
+    {"<date>\n","\n</date>\n"},
+    {"<q>\n","\n</q>\n"},
+    {"<expect>\n","\n</expect>\n"},
+    {"<sigma>\n","\n</sigma>\n"},
+    {"<version>\n","\n</version>\n"},
+    {"<ref>\n","\n</ref>\n"},
+    {"<filename>\n","\n</filename>\n"},
+    {"<index>\n","\n</index>\n"},
+    {"<min>\n","\n</min>\n"},
+    {"<max>\n","\n</max>\n"},
+    {"<end>\n","\n</end>\n"},
+    {"<alias>\n","\n</alias>\n"},
+    {NULL,NULL}
    };
 
 char *CFRH[][2] =
    {
-    "<tr>","</tr>\n\n",
-    "<td>","</td>\n",
-    "<td>","</td>\n",
-    "<td bgcolor=#add8e6>","</td>\n",
-    "<td bgcolor=#e0ffff>","</td>\n",
-    "<td bgcolor=#f0f8ff>","</td>\n",
-    "<td bgcolor=#fafafa>","</td>\n",
-    "<td bgcolor=#ededed>","</td>\n",
-    "<td bgcolor=#e0e0e0>","</td>\n",
-    "<td bgcolor=#add8e6>","</td>\n",
-    "<td bgcolor=#e0ffff>","</td>\n",
-    "<td bgcolor=#fafafa><small>","</small></td>\n",
-    "<td bgcolor=#fafafa><small>","</small></td>\n",
-    "<td>","</td>\n",
-    "<td>","</td>\n",
-    "<td>","</td>\n",
-    NULL,NULL
+    {"<tr>","</tr>\n\n"},
+    {"<td>","</td>\n"},
+    {"<td>","</td>\n"},
+    {"<td bgcolor=#add8e6>","</td>\n"},
+    {"<td bgcolor=#e0ffff>","</td>\n"},
+    {"<td bgcolor=#f0f8ff>","</td>\n"},
+    {"<td bgcolor=#fafafa>","</td>\n"},
+    {"<td bgcolor=#ededed>","</td>\n"},
+    {"<td bgcolor=#e0e0e0>","</td>\n"},
+    {"<td bgcolor=#add8e6>","</td>\n"},
+    {"<td bgcolor=#e0ffff>","</td>\n"},
+    {"<td bgcolor=#fafafa><small>","</small></td>\n"},
+    {"<td bgcolor=#fafafa><small>","</small></td>\n"},
+    {"<td>","</td>\n"},
+    {"<td>","</td>\n"},
+    {"<td>","</td>\n"},
+    {"<td>","</td>\n"},
+    {NULL,NULL}
    };
 
 /*****************************************************************************/
 
 int main(int argc,char *argv[])
 
-{
+{ 
 CheckOpts(argc,argv);
 GenericInitialize(argc,argv,"reporter");
 ThisAgentInit();
 KeepReportsControlPromises();
 KeepReportsPromises();
-Aggregate(STYLESHEET,BANNER,FOOTER,WEBDRIVER);
 return 0;
 }
 
+
+
 /*****************************************************************************/
 /* Level 1                                                                   */
 /*****************************************************************************/
@@ -256,12 +268,10 @@ return 0;
 void CheckOpts(int argc,char **argv)
 
 { extern char *optarg;
-  struct Item *actionList;
   int optindex = 0;
   int c;
-  char ld_library_path[CF_BUFSIZE];
 
-while ((c=getopt_long(argc,argv,"ghd:vVf:st:ar:PXHLMISKE:",OPTIONS,&optindex)) != EOF)
+while ((c=getopt_long(argc,argv,"ghd:vVf:st:ar:PXHLMISKE:x:i:",OPTIONS,&optindex)) != EOF)
    {
    switch ((char) c)
       {
@@ -289,6 +299,12 @@ while ((c=getopt_long(argc,argv,"ghd:vVf:st:ar:PXHLMISKE:",OPTIONS,&optindex)) !
 
       case 'S':
           SyntaxTree();
+          exit(0);
+          break;
+
+      case 's':
+          SyntaxExport();
+          exit(0);
           break;
 
       case'K':
@@ -358,6 +374,24 @@ while ((c=getopt_long(argc,argv,"ghd:vVf:st:ar:PXHLMISKE:",OPTIONS,&optindex)) !
           PURGE = 'y';
           break;
 
+      case 'x':
+          
+          if((String2Menu(optarg) != cfd_menu_delta) &&
+             (String2Menu(optarg) != cfd_menu_full))
+             {
+             Syntax("Wrong argument to export: should be delta or full",OPTIONS,HINTS,ID);
+             exit(1);
+             }
+          
+          snprintf(NOVA_EXPORT_TYPE, sizeof(NOVA_EXPORT_TYPE), "%s", optarg);
+
+          break;
+
+      case 'i':
+          
+          snprintf(NOVA_IMPORT_FILE, sizeof(NOVA_IMPORT_FILE), "%s", optarg);
+          break;
+
       default: Syntax("cf-report - cfengine's reporting agent",OPTIONS,HINTS,ID);
           exit(1);
 
@@ -375,8 +409,8 @@ if (argv[optind] != NULL)
 
 void ThisAgentInit()
 
-{ char vbuff[CF_BUFSIZE];
-  time_t now;
+{
+time_t now;
 
 if (strlen(OUTPUTDIR) == 0)
    {
@@ -410,6 +444,25 @@ MapName(VINPUTFILE);
 
 InitMeasurements();
 RemoveHostSeen(REMOVEHOSTS);
+
+#ifdef HAVE_NOVA
+if(!EMPTY(NOVA_EXPORT_TYPE))
+   {
+   Nova_ExportReports(NOVA_EXPORT_TYPE);
+   }
+
+if(!EMPTY(NOVA_IMPORT_FILE))
+   {
+   if(IsDefinedClass("am_policy_hub"))
+      {
+      Nova_ImportHostReports(NOVA_IMPORT_FILE);
+      }
+   else
+      {
+      CfOut(cf_error, "", "Importing reports is only possible on Nova policy hubs");
+      }
+   }
+#endif
 }
 
 /*****************************************************************************/
@@ -735,9 +788,7 @@ GrandSummary();
 void RemoveHostSeen(char *hosts)
 
 { CF_DB *dbp;
-  CF_DBC *dbcp;
-  char *key,name[CF_BUFSIZE];
-  void *value;
+  char name[CF_BUFSIZE];
   struct Item *ip,*list = SplitStringAsItemList(hosts,',');
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_LASTDB_FILE);
@@ -774,9 +825,9 @@ void ShowLastSeen()
   time_t tid = time(NULL);
   double now = (double)tid,average = 0, var = 0;
   double ticksperhr = (double)CF_TICKS_PER_HOUR;
-  char name[CF_BUFSIZE],hostname[CF_BUFSIZE];
-  struct QPoint entry;
-  int ret,ksize,vsize;
+  char name[CF_BUFSIZE],hostname[CF_BUFSIZE],address[CF_MAXVARSIZE];
+  struct CfKeyHostSeen entry;
+  int ksize,vsize;
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_LASTDB_FILE);
 MapName(name);
@@ -843,14 +894,16 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
    char tbuf[CF_BUFSIZE],addr[CF_BUFSIZE];
 
    memcpy(&then,value,sizeof(then));
-   strncpy(hostname,(char *)key,ksize);
-
+   
    if (value != NULL)
       {
       memcpy(&entry,value,sizeof(entry));
-      then = entry.q;
-      average = (double)entry.expect;
-      var = (double)entry.var;
+      strncpy(hostname,(char *)key,ksize);
+      strncpy(address,(char *)entry.address,ksize);   
+      then = entry.Q.q;
+      average = (double)entry.Q.expect;
+      var = (double)entry.Q.var;
+      strncpy(addr,entry.address,CF_MAXVARSIZE);
       }
    else
       {
@@ -868,21 +921,15 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
    snprintf(tbuf,CF_BUFSIZE-1,"%s",cf_ctime(&fthen));
    tbuf[strlen(tbuf)-9] = '\0';                     /* Chop off second and year */
 
-   if (strlen(hostname+1) > 15)
-      {
-      snprintf(addr,15,"...%s",hostname+strlen(hostname)-10); /* ipv6 */
-      }
-   else
-      {
-      snprintf(addr,15,"%s",hostname+1);
-      }
-
+   CfOut(cf_verbose,""," -> Reporting on %s",hostname);
+   
    if (XML)
       {
       fprintf(fout,"%s",CFRX[cfx_entry][cfb]);
       fprintf(fout,"%s%c%s",CFRX[cfx_pm][cfb],*hostname,CFRX[cfx_pm][cfe]);
-      fprintf(fout,"%s%s%s",CFRX[cfx_host][cfb],IPString2Hostname(hostname+1),CFRX[cfx_host][cfe]);
-      fprintf(fout,"%s%s%s",CFRX[cfx_ip][cfb],hostname+1,CFRX[cfx_ip][cfe]);
+      fprintf(fout,"%s%s%s",CFRX[cfx_host][cfb],hostname+1,CFRX[cfx_host][cfe]);
+      fprintf(fout,"%s%s%s",CFRX[cfx_alias][cfb],IPString2Hostname(address),CFRX[cfx_ip][cfe]);
+      fprintf(fout,"%s%s%s",CFRX[cfx_ip][cfb],address,CFRX[cfx_ip][cfe]);
       fprintf(fout,"%s%s%s",CFRX[cfx_date][cfb],tbuf,CFRX[cfx_date][cfe]);
       fprintf(fout,"%s%.2lf%s",CFRX[cfx_q][cfb],((double)(now-then))/ticksperhr,CFRX[cfx_q][cfe]);
       fprintf(fout,"%s%.2lf%s",CFRX[cfx_av][cfb],average/ticksperhr,CFRX[cfx_av][cfe]);
@@ -901,8 +948,9 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
          {
          fprintf(fout,"%s in (%c)%s",CFRH[cfx_pm][cfb],*hostname,CFRH[cfx_pm][cfe]);
          }
-      fprintf(fout,"%s%s%s",CFRH[cfx_host][cfb],IPString2Hostname(hostname+1),CFRH[cfx_host][cfe]);
-      fprintf(fout,"%s%s%s",CFRH[cfx_ip][cfb],hostname+1,CFRH[cfx_ip][cfe]);
+      fprintf(fout,"%s%s%s",CFRH[cfx_host][cfb],hostname+1,CFRH[cfx_host][cfe]);
+      fprintf(fout,"%s%s%s",CFRH[cfx_ip][cfb],address,CFRH[cfx_ip][cfe]);
+      fprintf(fout,"%s%s%s",CFRH[cfx_alias][cfb],IPString2Hostname(address),CFRH[cfx_ip][cfe]);
       fprintf(fout,"%s Last seen at %s%s",CFRH[cfx_date][cfb],tbuf,CFRH[cfx_date][cfe]);
       fprintf(fout,"%s %.2lf hrs ago %s",CFRH[cfx_q][cfb],((double)(now-then))/ticksperhr,CFRH[cfx_q][cfe]);
       fprintf(fout,"%s Av %.2lf hrs %s",CFRH[cfx_av][cfb],average/ticksperhr,CFRH[cfx_av][cfe]);
@@ -911,9 +959,10 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
       }
    else if (CSV)
       {
-      fprintf(fout,"%c,%25.25s,%15.15s,%s,%.2lf,%.2lf,%.2lf hrs\n",
+      fprintf(fout,"%c,%25.25s,%25.25s,%15.15s,%s,%.2lf,%.2lf,%.2lf hrs\n",
              *hostname,
-             IPString2Hostname(hostname+1),
+             hostname+1,
+             IPString2Hostname(address),
              addr,
              tbuf,
              ((double)(now-then))/ticksperhr,
@@ -922,9 +971,10 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
       }
    else
       {
-      fprintf(fout,"IP %c %25.25s %15.15s  @ [%s] not seen for (%.2lf) hrs, Av %.2lf +/- %.2lf hrs\n",
+      fprintf(fout,"IP %c %25.25s %25.25s %15.15s  @ [%s] not seen for (%.2lf) hrs, Av %.2lf +/- %.2lf hrs\n",
              *hostname,
-             IPString2Hostname(hostname+1),
+             hostname+1,
+             IPString2Hostname(address),
              addr,
              tbuf,
              ((double)(now-then))/ticksperhr,
@@ -962,7 +1012,7 @@ void ShowPerformance()
   double ticksperminute = 60.0;
   char name[CF_BUFSIZE],eventname[CF_BUFSIZE];
   struct Event entry;
-  int ret,ksize,vsize;
+  int ksize,vsize;
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_PERFORMANCE);
 
@@ -1027,7 +1077,7 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
    {
    double measure;
    time_t then;
-   char tbuf[CF_BUFSIZE],addr[CF_BUFSIZE];
+   char tbuf[CF_BUFSIZE];
 
    memcpy(&then,value,sizeof(then));
    strncpy(eventname,(char *)key,ksize);
@@ -1037,8 +1087,8 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
       memcpy(&entry,value,sizeof(entry));
 
       then    = entry.t;
-      measure = entry.Q.q/ticksperminute;;
-      average = entry.Q.expect/ticksperminute;;
+      measure = entry.Q.q/ticksperminute;
+      average = entry.Q.expect/ticksperminute;
       var     = entry.Q.var;
 
       snprintf(tbuf,CF_BUFSIZE-1,"%s",cf_ctime(&then));
@@ -1127,11 +1177,10 @@ void ShowClasses()
   FILE *fout,*fnotes;
   struct Item *already = NULL,*ip;
   double now = (double)time(NULL),average = 0, var = 0;
-  double ticksperminute = 60.0;
   char name[CF_BUFSIZE],eventname[CF_BUFSIZE];
   struct Event entry;
   struct CEnt array[1024];
-  int ret,i,ksize,vsize;
+  int i,ksize,vsize;
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_CLASSUSAGE);
 MapName(name);
@@ -1206,7 +1255,7 @@ while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
    {
    double measure;
    time_t then;
-   char tbuf[CF_BUFSIZE],addr[CF_BUFSIZE];
+   char tbuf[CF_BUFSIZE];
 
    memcpy(&then,value,sizeof(then));
    strncpy(eventname,(char *)key,CF_BUFSIZE-1);
@@ -1293,7 +1342,7 @@ for (i = 0; i < 1024; i++)
       }
    else
       {
-      fprintf(fout,"Probability %7.4lf +/- %7.4lf for %s (last oberved @ %s)\n",array[i].q,sqrt(array[i].d),array[i].name,array[i].date);
+      fprintf(fout,"Probability %7.4lf +/- %7.4lf for %s (last observed @ %s)\n",array[i].q,sqrt(array[i].d),array[i].name,array[i].date);
       }
    }
 
@@ -1304,39 +1353,44 @@ if (HTML && !EMBEDDED)
    fprintf(fout,"<table class=\"border\" cellpadding=\"5\">\n");
    }
 
-for (ip = VHEAP; ip != NULL; ip=ip->next)
+for (i = 0; i < CF_ALPHABETSIZE; i++)
    {
-   if (IsItemIn(already,ip->name))
+   for (ip = VHEAP.list[i]; ip != NULL; ip=ip->next)
       {
-      continue;
-      }
-
-   if (strncmp(ip->name,"Min",3) == 0 || strncmp(ip->name,"Hr",2) == 0 || strncmp(ip->name,"Q",1) == 0)
-      {
-      continue;
-      }
-   
-   if (XML)
-      {
-      fprintf(fout,"%s",CFRX[cfx_entry][cfb]);
-      fprintf(fout,"%s%s%s",CFRX[cfx_event][cfb],ip->name,CFRX[cfx_event][cfe]);
-      fprintf(fout,"%s%s%s",CFRX[cfx_date][cfb],"always",CFRX[cfx_date][cfe]);
-      fprintf(fout,"%s%.4lf%s",CFRX[cfx_av][cfb],1.0,CFRX[cfx_av][cfe]);
-      fprintf(fout,"%s%.4lf%s",CFRX[cfx_dev][cfb],0.0,CFRX[cfx_dev][cfe]);
-      fprintf(fout,"%s",CFRX[cfx_entry][cfe]);
-      }
-   else if (HTML)
-      {
-      fprintf(fout,"%s",CFRH[cfx_entry][cfb]);
-      fprintf(fout,"%s%s%s",CFRH[cfx_event][cfb],ip->name,CFRH[cfx_event][cfe]);
-      fprintf(fout,"%s occured %s%s",CFRH[cfx_date][cfb],"always",CFRH[cfx_date][cfe]);
-      fprintf(fout,"%s Probability %.4lf %s",CFRH[cfx_av][cfb],1.0,CFRH[cfx_av][cfe]);
-      fprintf(fout,"%s &plusmn; %.4lf %s",CFRH[cfx_dev][cfb],0.0,CFRH[cfx_dev][cfe]);
-      fprintf(fout,"%s",CFRH[cfx_entry][cfe]);
-      }
-   else if (CSV)
-      {
-      fprintf(fout,"%7.4lf,%7.4lf,%s,%s\n",1.0,0.0,ip->name,"always");
+      if (IsItemIn(already,ip->name))
+         {
+         continue;
+         }
+      
+      if (strncmp(ip->name,"Min",3) == 0 || strncmp(ip->name,"Hr",2) == 0 || strncmp(ip->name,"Q",1) == 0
+          || strncmp(ip->name,"Yr",1) == 0 || strncmp(ip->name,"Day",1) == 0 || strncmp(ip->name,"Morning",1) == 0
+          || strncmp(ip->name,"Afternoon",1) == 0 || strncmp(ip->name,"Evening",1) == 0 || strncmp(ip->name,"Night",1) == 0)
+         {
+         continue;
+         }
+      
+      if (XML)
+         {
+         fprintf(fout,"%s",CFRX[cfx_entry][cfb]);
+         fprintf(fout,"%s%s%s",CFRX[cfx_event][cfb],ip->name,CFRX[cfx_event][cfe]);
+         fprintf(fout,"%s%s%s",CFRX[cfx_date][cfb],"often",CFRX[cfx_date][cfe]);
+         fprintf(fout,"%s%.4lf%s",CFRX[cfx_av][cfb],1.0,CFRX[cfx_av][cfe]);
+         fprintf(fout,"%s%.4lf%s",CFRX[cfx_dev][cfb],0.0,CFRX[cfx_dev][cfe]);
+         fprintf(fout,"%s",CFRX[cfx_entry][cfe]);
+         }
+      else if (HTML)
+         {
+         fprintf(fout,"%s",CFRH[cfx_entry][cfb]);
+         fprintf(fout,"%s%s%s",CFRH[cfx_event][cfb],ip->name,CFRH[cfx_event][cfe]);
+         fprintf(fout,"%s occurred %s%s",CFRH[cfx_date][cfb],"often",CFRH[cfx_date][cfe]);
+         fprintf(fout,"%s Probability %.4lf %s",CFRH[cfx_av][cfb],1.0,CFRH[cfx_av][cfe]);
+         fprintf(fout,"%s &plusmn; %.4lf %s",CFRH[cfx_dev][cfb],0.0,CFRH[cfx_dev][cfe]);
+         fprintf(fout,"%s",CFRH[cfx_entry][cfe]);
+         }
+      else if (CSV)
+         {
+         fprintf(fout,"%7.4lf,%7.4lf,%s,%s\n",1.0,0.0,ip->name,"often");
+         }
       }
    }
 
@@ -1367,10 +1421,9 @@ void ShowChecksums()
   CF_DBC *dbcp;
   char *key;
   void *value;
-  int ret,ksize,vsize;
-  FILE *pp,*fout;
+  int ksize,vsize;
+  FILE *fout;
   char checksumdb[CF_BUFSIZE],name[CF_BUFSIZE];
-  struct stat statbuf;
 
 snprintf(checksumdb,CF_BUFSIZE,"%s/%s",CFWORKDIR,CF_CHKDB);
 
@@ -1418,7 +1471,8 @@ if (XML)
 
 if (!NewDBCursor(dbp,&dbcp))
    {
-   CfOut(cf_inform,""," !! Unable to scan last-seen db");
+   CfOut(cf_inform,""," !! Unable to scan checksum db");
+   CloseDB(dbp);
    return;
    }
 
@@ -1493,7 +1547,7 @@ void ShowLocks (int active)
   char *key;
   void *value;
   FILE *fout;
-  int ret,ksize,vsize;
+  int ksize,vsize;
   char lockdb[CF_BUFSIZE],name[CF_BUFSIZE];
   struct LockData entry;
 
@@ -1534,7 +1588,8 @@ else
 if ((fout = fopen(name,"w")) == NULL)
    {
    CfOut(cf_error,"fopen"," !! Unable to write to %s/%s\n",OUTPUTDIR,name);
-   exit(1);
+   CloseDB(dbp);
+   return;
    }
 
 if (HTML && !EMBEDDED)
@@ -1556,6 +1611,8 @@ if (XML)
 if (!NewDBCursor(dbp,&dbcp))
    {
    CfOut(cf_inform,""," !! Unable to scan last-seen db");
+   CloseDB(dbp);
+   fclose(fout);
    return;
    }
 
@@ -1654,7 +1711,7 @@ void ShowCurrentAudit()
   void *value;
   CF_DB *dbp;
   CF_DBC *dbcp;
-  int ret,ksize,vsize;
+  int ksize,vsize;
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_AUDITDB_FILE);
 MapName(name);
@@ -1896,7 +1953,7 @@ void ReadAverages()
   char timekey[CF_MAXVARSIZE];
   time_t now;
   CF_DB *dbp;
-  int i,err;
+  int i;
 
 CfOut(cf_verbose,"","\nLooking for database %s\n",VINPUTFILE);
 CfOut(cf_verbose,"","\nFinding MAXimum values...\n\n");
@@ -1953,7 +2010,7 @@ CloseDB(dbp);
 
 void EraseAverages()
 
-{ int i,err;
+{ int i;
  char timekey[CF_MAXVARSIZE],name[CF_MAXVARSIZE];
   struct Item *list = NULL;
   struct Averages entry;
@@ -2005,7 +2062,7 @@ CloseDB(dbp);
 
 void SummarizeAverages()
 
-{ int i,err;
+{ int i;
   FILE *fout;
   char name[CF_BUFSIZE];
   CF_DB *dbp;
@@ -2076,7 +2133,7 @@ for (i = 0; i < CF_OBSERVABLES; i++)
       }
    else
       {
-      CfOut(cf_verbose,"","%2d. MAX <%-10s-in>   = %10lf - %10lf u %10lf\n",i,name,MIN.Q[i].expect,MAX.Q[i].expect,sqrt(MAX.Q[i].var));
+      fprintf(fout,"%2d. MAX <%-10s-in>   = %10lf - %10lf u %10lf\n",i,name,MIN.Q[i].expect,MAX.Q[i].expect,sqrt(MAX.Q[i].var));
       }
    }
 
@@ -2111,11 +2168,10 @@ fclose(fout);
 
 void WriteGraphFiles()
 
-{ int its,i,j,k, count = 0,err;
+{ int its,i,j,count = 0;
   double kept = 0, not_kept = 0, repaired = 0;
-  struct stat statbuf;
   struct Averages entry,det;
-  char timekey[CF_MAXVARSIZE],name[CF_MAXVARSIZE];
+  char timekey[CF_MAXVARSIZE];
   time_t now;
   CF_DB *dbp;
 
@@ -2251,7 +2307,7 @@ CloseFiles();
 
 void MagnifyNow()
 
-{ int its,i,j,k, count = 0,err;
+{ int its,i,j,count = 0;
   struct Averages entry,det;
   time_t now,here_and_now;
   char timekey[CF_MAXVARSIZE];
@@ -2436,7 +2492,7 @@ void DiskArrivals(void)
 { DIR *dirh;
   FILE *fp;
   struct dirent *dirp;
-  int count = 0, index = 0, i,err;
+  int count = 0, index = 0, i;
   char filename[CF_BUFSIZE],database[CF_BUFSIZE],timekey[CF_MAXVARSIZE];
   double val, maxval = 1.0, *array, grain = 0.0;
   time_t now;
@@ -2553,17 +2609,17 @@ closedir(dirh);
 
 void PeerIntermittency()
 
-{ CF_DB *dbp,*dbpent;
+{ CF_DB *dbp;
   CF_DBC *dbcp;
   char *key;
   void *value;
-  int i,ret,ksize,vsize;
+  int ksize,vsize;
   FILE *fp1,*fp2;
   char name[CF_BUFSIZE],hostname[CF_BUFSIZE],timekey[CF_MAXVARSIZE];
   char out1[CF_BUFSIZE],out2[CF_BUFSIZE];
   struct QPoint entry;
   struct Item *ip, *hostlist = NULL;
-  double entropy,average,var,sum,sum_av;
+  double average,var;
   time_t now = time(NULL), then, lastseen = CF_WEEK;
 
 snprintf(name,CF_BUFSIZE-1,"%s/%s",CFWORKDIR,CF_LASTDB_FILE);
@@ -2588,7 +2644,7 @@ CfOut(cf_verbose,"","Examining known peers...\n");
 
 while(NextDB(dbp,dbcp,&key,&ksize,&value,&vsize))
    {
-   strcpy(hostname,IPString2Hostname((char *)key+1));
+   strcpy(hostname,(char *)key+1);
 
    if (!IsItemIn(hostlist,hostname))
       {
@@ -2784,6 +2840,8 @@ for (i = 0; i < CF_OBSERVABLES; i++)
 }
 
 
+
+
 /* EOF */
 
 
