@@ -35,6 +35,14 @@
 #include "cf3.defs.h"
 #include "cf3.extern.h"
 
+static void CacheServerConnection(struct cfagent_connection *conn,char *server);
+static int TryConnect(struct cfagent_connection *conn, struct timeval *tvp, struct sockaddr *cinp, int cinpSz);
+static void MarkServerOffline(char *server);
+static struct cfagent_connection *ServerConnectionReady(char *server);
+static int ServerOffline(char *server);
+static void FlushFileStream(int sd,int toget);
+static int CacheStat(char *file,struct stat *statbuf,char *stattype,struct Attributes attr,struct Promise *pp);
+
 /*********************************************************************/
 
 void DetermineCfenginePort()
@@ -87,7 +95,7 @@ for (rp = attr.copy.servers; rp != NULL; rp = rp->next)
       }
    else
       {   
-      if (conn = ServerConnectionReady(rp->item))
+      if ((conn = ServerConnectionReady(rp->item)))
          {
          return conn;
          }
@@ -118,12 +126,11 @@ return NULL;
 struct cfagent_connection *ServerConnection(char *server,struct Attributes attr,struct Promise *pp)
 
 { struct cfagent_connection *conn;
-  
+ 
 #ifndef MINGW
 static sigset_t   signal_mask;
 
 signal(SIGPIPE,SIG_IGN);
-
 sigemptyset(&signal_mask);
 sigaddset(&signal_mask,SIGPIPE);
 pthread_sigmask (SIG_BLOCK,&signal_mask, NULL);
@@ -144,13 +151,22 @@ if (strcmp(server,"localhost") == 0)
 conn->authenticated = false;
 conn->encryption_type = CfEnterpriseOptions();
 
+
+/* username of the client - say root from Windows */
+
+#ifdef MINGW
+snprintf(conn->username,CF_SMALLBUF,"root");
+#else
+GetCurrentUserName(conn->username,CF_SMALLBUF);
+#endif  /* NOT MINGW */
+
 if (conn->sd == CF_NOT_CONNECTED)
    {   
-   Debug("Opening server connnection to %s\n",server);
+   Debug("Opening server connection to %s\n",server);
    
    if (!ServerConnect(conn,server,attr,pp))
       {
-      CfOut(cf_inform,"socket"," !! No server is responding on this port");
+      CfOut(cf_inform,""," !! No server is responding on this port");
 
       if (conn->sd != CF_NOT_CONNECTED)
          {
@@ -203,8 +219,11 @@ Debug("Closing current server connection\n");
 
 if (conn)
    {
-   cf_closesocket(conn->sd);
-   conn->sd = CF_NOT_CONNECTED;
+   if (conn->sd > 0)
+      {
+      cf_closesocket(conn->sd);
+      conn->sd = CF_NOT_CONNECTED;
+      }
    DeleteAgentConn(conn);
    }
 }
@@ -562,13 +581,13 @@ pp->cache = NULL;
 
 int CompareHashNet(char *file1,char *file2,struct Attributes attr,struct Promise *pp)
 
-{ static unsigned char d[CF_MD5_LEN];
+{ static unsigned char d[EVP_MAX_MD_SIZE+1];
   char *sp,sendbuffer[CF_BUFSIZE],recvbuffer[CF_BUFSIZE],in[CF_BUFSIZE],out[CF_BUFSIZE];
   int i,tosend,cipherlen;
   struct cfagent_connection *conn = pp->conn;
 
-HashFile(file2,d,cf_md5);
-Debug("Send digest of %s to server, %s\n",file2,HashPrint(cf_md5,d));
+HashFile(file2,d,CF_DEFAULT_DIGEST);
+Debug("Send digest of %s to server, %s\n",file2,HashPrint(CF_DEFAULT_DIGEST,d));
 
 memset(recvbuffer,0,CF_BUFSIZE);
 
@@ -578,12 +597,12 @@ if (attr.copy.encrypt)
 
    sp = in + strlen(in) + CF_SMALL_OFFSET;
 
-   for (i = 0; i < CF_MD5_LEN; i++)
+   for (i = 0; i < CF_DEFAULT_DIGEST_LEN; i++)
       {
       *sp++ = d[i];
       }
    
-   cipherlen = EncryptString(conn->encryption_type,in,out,conn->session_key,strlen(in)+CF_SMALL_OFFSET+CF_MD5_LEN);
+   cipherlen = EncryptString(conn->encryption_type,in,out,conn->session_key,strlen(in)+CF_SMALL_OFFSET+CF_DEFAULT_DIGEST_LEN);
    snprintf(sendbuffer,CF_BUFSIZE,"SMD5 %d",cipherlen);
    memcpy(sendbuffer+CF_PROTO_OFFSET,out,cipherlen);
    tosend = cipherlen + CF_PROTO_OFFSET;
@@ -593,12 +612,12 @@ else
    snprintf(sendbuffer,CF_BUFSIZE,"MD5 %s",file1);
    sp = sendbuffer + strlen(sendbuffer) + CF_SMALL_OFFSET;
 
-   for (i = 0; i < CF_MD5_LEN; i++)
+   for (i = 0; i < CF_DEFAULT_DIGEST_LEN; i++)
       {
       *sp++ = d[i];
       }
    
-   tosend = strlen(sendbuffer)+CF_SMALL_OFFSET+CF_MD5_LEN;
+   tosend = strlen(sendbuffer)+CF_SMALL_OFFSET+CF_DEFAULT_DIGEST_LEN;
    } 
  
 if (SendTransaction(conn->sd,sendbuffer,tosend,CF_DONE) == -1)
@@ -616,12 +635,12 @@ if (ReceiveTransaction(conn->sd,recvbuffer,NULL) == -1)
 
 if (strcmp(CFD_TRUE,recvbuffer) == 0)
    {
-   Debug("MD5 mismatch: (reply - %s)\n",recvbuffer);
+   Debug("Hash mismatch: (reply - %s)\n",recvbuffer);
    return true; /* mismatch */
    }
 else
    {
-   Debug("MD5 matched ok: (reply - %s)\n",recvbuffer);
+   Debug("Hash matched ok: (reply - %s)\n",recvbuffer);
    return false;
    }
  
@@ -632,10 +651,10 @@ else
 
 int CopyRegularFileNet(char *source,char *new,off_t size,struct Attributes attr,struct Promise *pp)
 
-{ int dd, buf_size,n_read = 0,toget,towrite,plainlen,more = true, finlen;
-  int last_write_made_hole = 0, done = false,tosend,cipherlen=0,value;
-  char *buf,in[CF_BUFSIZE],out[CF_BUFSIZE],workbuf[CF_BUFSIZE],cfchangedstr[265];
-  unsigned char iv[32] = {1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8};
+{ int dd, buf_size,n_read = 0,toget,towrite;
+  int last_write_made_hole = 0, done = false,tosend,value;
+  char *buf,workbuf[CF_BUFSIZE],cfchangedstr[265];
+
   long n_read_total = 0;  
   EVP_CIPHER_CTX ctx;
   struct cfagent_connection *conn = pp->conn;
@@ -786,8 +805,8 @@ return true;
 
 int EncryptCopyRegularFileNet(char *source,char *new,off_t size,struct Attributes attr,struct Promise *pp)
 
-{ int dd, blocksize = 2048,n_read = 0,toget,towrite,plainlen,more = true, finlen,cnt = 0;
-  int last_write_made_hole = 0, done = false,tosend,cipherlen=0,value;
+{ int dd, blocksize = 2048,n_read = 0,towrite,plainlen,more = true, finlen,cnt = 0;
+  int last_write_made_hole = 0, tosend,cipherlen=0;
   char *buf,in[CF_BUFSIZE],out[CF_BUFSIZE],workbuf[CF_BUFSIZE],cfchangedstr[265];
   unsigned char iv[32] = {1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8};
   long n_read_total = 0;  
@@ -844,6 +863,7 @@ while (more)
    {
    if ((cipherlen = ReceiveTransaction(conn->sd,buf,&more)) == -1)
       {
+      free(buf);
       return false;
       }
 
@@ -867,7 +887,7 @@ while (more)
       return false;      
       }
 
-   EVP_DecryptInit(&ctx,CfengineCipher(CfEnterpriseOptions()),conn->session_key,iv);
+   EVP_DecryptInit_ex(&ctx,CfengineCipher(CfEnterpriseOptions()),NULL,conn->session_key,iv);
 
    if (!EVP_DecryptUpdate(&ctx,workbuf,&plainlen,buf,cipherlen))
       {
@@ -877,7 +897,7 @@ while (more)
       return false;
       }
 
-   if (!EVP_DecryptFinal(&ctx,workbuf+plainlen,&finlen))
+   if (!EVP_DecryptFinal_ex(&ctx,workbuf+plainlen,&finlen))
       {
       Debug("Final decrypt failed\n");
       close(dd);
@@ -930,9 +950,11 @@ return true;
 
 int ServerConnect(struct cfagent_connection *conn,char *host,struct Attributes attr, struct Promise *pp) 
 
-{ int err;
+{
   short shortport;
-  char strport[CF_MAXVARSIZE];
+  char strport[CF_MAXVARSIZE] = {0};
+  struct sockaddr_in cin = {0};
+  struct timeval tv = {0};
 
 if (attr.copy.portnumber == (short)CF_NOINT)
    {
@@ -947,15 +969,28 @@ else
    
 CfOut(cf_verbose,"","Set cfengine port number to %s = %u\n",strport,(int)ntohs(shortport));
 
+if (attr.copy.timeout == (short)CF_NOINT || attr.copy.timeout <= 0)
+   {
+   tv.tv_sec = CONNTIMEOUT;
+   }
+else
+   {
+   tv.tv_sec = attr.copy.timeout;
+   }
+
+CfOut(cf_verbose,"","Set connection timeout to %d\n",tv.tv_sec);
+
+tv.tv_usec = 0;
+
 #if defined(HAVE_GETADDRINFO)
  
 if (!attr.copy.force_ipv4)
    {
-   struct addrinfo query, *response, *ap;
-   struct addrinfo query2, *response2, *ap2;
+   struct addrinfo query = {0}, *response, *ap;
+   struct addrinfo query2 = {0}, *response2, *ap2;
    int err,connected = false;
 
-   memset(&query,0,sizeof(struct addrinfo));   
+   memset(&query,0,sizeof(query));
    query.ai_family = AF_UNSPEC;
    query.ai_socktype = SOCK_STREAM;
 
@@ -974,17 +1009,18 @@ if (!attr.copy.force_ipv4)
          CfOut(cf_inform,"socket"," !! Couldn't open a socket");
          continue;
          }
-      
+
       if (BINDINTERFACE[0] != '\0')
          {
-         memset(&query2,0,sizeof(struct addrinfo));   
-         
-         query.ai_family = AF_UNSPEC;
-         query.ai_socktype = SOCK_STREAM;
+	 memset(&query2,0,sizeof(query2));
+         query2.ai_family = AF_UNSPEC;
+         query2.ai_socktype = SOCK_STREAM;
          
          if ((err = getaddrinfo(BINDINTERFACE,NULL,&query2,&response2)) != 0)
             {
             cfPS(cf_error,CF_FAIL,"",pp,attr," !! Unable to lookup hostname or cfengine service: %s",gai_strerror(err));
+            cf_closesocket(conn->sd);
+            conn->sd = CF_NOT_CONNECTED;
             return false;
             }
          
@@ -1003,20 +1039,13 @@ if (!attr.copy.force_ipv4)
             freeaddrinfo(response2);
             }
          }
-      
-      signal(SIGALRM,(void *)TimeOut);
-      alarm(CF_TIMEOUT);
-      
-      if (connect(conn->sd,ap->ai_addr,ap->ai_addrlen) >= 0)
+
+      if (TryConnect(conn,&tv,ap->ai_addr,ap->ai_addrlen))
          {
          connected = true;
-         alarm(0);
-         signal(SIGALRM,SIG_DFL);
          break;
          }
-
-      alarm(0);
-      signal(SIGALRM,SIG_DFL);
+      
       }
    
    if (connected)
@@ -1037,9 +1066,11 @@ if (!attr.copy.force_ipv4)
    
    if (!connected && pp)
       {
-      cfPS(cf_verbose,CF_FAIL,"connect",pp,attr,"Unable to connect to server %s",host);
+      cfPS(cf_verbose,CF_FAIL,"connect",pp,attr," !! Unable to connect to server %s",host);
       return false;
       }
+
+   return true;
    }
  
  else
@@ -1048,7 +1079,6 @@ if (!attr.copy.force_ipv4)
 
    {
    struct hostent *hp;
-   struct sockaddr_in cin;
    memset(&cin,0,sizeof(cin));
    
    if ((hp = gethostbyname(host)) == NULL)
@@ -1077,31 +1107,17 @@ if (!attr.copy.force_ipv4)
    
    conn->family = AF_INET;
    snprintf(conn->remoteip,CF_MAX_IP_LEN-1,"%s",inet_ntoa(cin.sin_addr));
-    
-   signal(SIGALRM,(void *)TimeOut);
-   alarm(CF_TIMEOUT);
-    
-   if (err=connect(conn->sd,(void *)&cin,sizeof(cin)) == -1)
-      {
-      cfPS(cf_verbose,CF_INTERPT,"connect",pp,attr,"Unable to connect to server %s (old ipv4)",host);
-      return false;
-      }
-   
-   alarm(0);
-   signal(SIGALRM,SIG_DFL);
+
+
+   return TryConnect(conn,&tv,(struct sockaddr *)&cin,sizeof(cin));
    }
-
-LastSaw(host,cf_connect);
-return true; 
 }
 
-
 /*********************************************************************/
 
-int ServerOffline(char *server)
+static int ServerOffline(char *server)
     
 { struct Rlist *rp;
-  struct cfagent_connection *conn;
   struct ServerItem *svp;
   char ipname[CF_MAXVARSIZE];
 
@@ -1129,10 +1145,9 @@ return false;
 
 /*********************************************************************/
 
-struct cfagent_connection *ServerConnectionReady(char *server)
+static struct cfagent_connection *ServerConnectionReady(char *server)
 
 { struct Rlist *rp;
-  struct cfagent_connection *conn;
   struct ServerItem *svp;
   char ipname[CF_MAXVARSIZE];
 
@@ -1151,7 +1166,7 @@ for (rp = SERVERLIST; rp != NULL; rp=rp->next)
    
    if (svp->busy)
       {
-      CfOut(cf_verbose,"","Existing connection seems to be busy...\n",ipname);
+      CfOut(cf_verbose,"","Existing connection to %s seems to be active...\n",ipname);
       return NULL;
       }
    
@@ -1190,7 +1205,7 @@ CfOut(cf_verbose,"","Existing connection just became free...\n");
 
 /*********************************************************************/
 
-void MarkServerOffline(char *server)
+static void MarkServerOffline(char *server)
 
 /* Unable to contact the server so don't waste time trying for
    other connections, mark it offline */
@@ -1243,18 +1258,20 @@ if ((svp->server = strdup(ipname)) == NULL)
 free(rp->item);
 rp->item = svp;
 
-if (svp->conn = NewAgentConn())
+if ((svp->conn = NewAgentConn()))
    {
    /* If we couldn't connect, mark this server unavailable for everyone */
    svp->conn->sd = CF_COULD_NOT_CONNECT;
    }
 
+svp->busy = false;
+
 ThreadUnlock(cft_getaddr);
 }
 
 /*********************************************************************/
 
-void CacheServerConnection(struct cfagent_connection *conn,char *server)
+static void CacheServerConnection(struct cfagent_connection *conn,char *server)
 
 /* First time we open a connection, so store it */
     
@@ -1282,11 +1299,11 @@ ThreadUnlock(cft_getaddr);
 
 /*********************************************************************/
 
-int CacheStat(char *file,struct stat *statbuf,char *stattype,struct Attributes attr,struct Promise *pp)
+static int CacheStat(char *file,struct stat *statbuf,char *stattype,struct Attributes attr,struct Promise *pp)
 
 { struct cfstat *sp;
 
-Debug("CacheStat(%s,%d)\n",file,pp);
+Debug("CacheStat(%s)\n",file);
 
 for (sp = pp->cache; sp != NULL; sp=sp->next)
    {
@@ -1329,7 +1346,7 @@ return false;
 
 /*********************************************************************/
 
-void FlushFileStream(int sd,int toget)
+static void FlushFileStream(int sd,int toget)
 
 { int i;
   char buffer[2]; 
@@ -1343,4 +1360,112 @@ for (i = 0; i < toget; i++)
 }
 
 
+/*********************************************************************/
+
+static int TryConnect(struct cfagent_connection *conn, struct timeval *tvp, struct sockaddr *cinp, int cinpSz)
+
+/** 
+ * Tries a nonblocking connect and then restores blocking if
+ * successful. Returns true on success, false otherwise.
+ * NB! Do not use recv() timeout - see note below.
+ **/
+    
+#ifdef MINGW
+{
+return NovaWin_TryConnect(conn,tvp,cinp,cinpSz);
+}
+
+#else  /* NOT MINGW */
+
+{ int res;
+  long arg;
+  struct sockaddr_in emptyCin = {0};
+#ifdef LINUX
+  struct timeval tvRecv = {0};
+#endif
+
+  if (!cinp)
+     {
+     cinp = (struct sockaddr *)&emptyCin;
+     cinpSz = sizeof(emptyCin);
+     }
+  
+
+   /* set non-blocking socket */
+   arg = fcntl(conn->sd, F_GETFL, NULL);
+
+   if(fcntl(conn->sd, F_SETFL, arg | O_NONBLOCK) == -1)
+     {
+     CfOut(cf_error,"","!! Could not set socket to non-blocking mode");
+     }
+
+   res = connect(conn->sd,cinp,cinpSz);
+
+   if (res < 0)
+      {
+      if (errno == EINPROGRESS)
+         {
+         fd_set myset;
+         int valopt;
+         socklen_t lon = sizeof(int);
+         
+         FD_ZERO(&myset);
+         FD_SET(conn->sd,&myset);
+
+         /* now wait for connect, but no more than tvp.sec */
+         res = select(conn->sd + 1, NULL, &myset, NULL, tvp);
+         if(getsockopt(conn->sd, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon) != 0)
+	   {
+	     CfOut(cf_error,"getsockopt","!! Could not check connection status");
+	     return false;
+	   }
+
+         if (valopt || res <= 0)
+            {
+            CfOut(cf_inform,"connect"," !! Error connecting to server (timeout)");
+            return false;
+            }
+         }
+      else
+         {
+         CfOut(cf_inform,"connect"," !! Error connecting to server");
+         return false;
+         }
+      }
+
+
+   /* connection suceeded; return to blocking mode */
+
+   if(fcntl(conn->sd, F_SETFL, arg) == -1)
+     {
+     CfOut(cf_error,"","!! Could not set socket to blocking mode");
+     }
+
+
+   /*
+    * NB: recv() timeout is not portable.  struct timeval is very
+    *     unstable - interpreted differently on different
+    *     platforms. E.g. setting tv_sec to 50 (and tv_usec to 0)
+    *     results in a timeout of 0.5 seconds on Windows, but 50
+    *     seconds on Linux. Thus it must be tested thoroughly on
+    *     the affected platforms. */
+
+   
+#ifdef LINUX
+    
+   tvRecv.tv_sec = RECVTIMEOUT;
+   tvRecv.tv_usec = 0;
+
+   if (setsockopt(conn->sd, SOL_SOCKET, SO_RCVTIMEO, (char *)&tvRecv, sizeof(tvRecv)))
+      {
+      CfOut(cf_error,"setsockopt","!! Couldn't set socket timeout");
+      }
+
+#endif
+  
+  return true;
+}
+
+#endif  /* NOT MINGW */
+
 
