@@ -47,6 +47,10 @@ should be free, please let us know and we will consider this carefully.
 # include <cf.nova.h>
 #endif
 
+#ifdef HAVE_ZONE_H
+# include <zone.h>
+#endif
+
 void *Nova_LDAPValue(char *uri,char *basedn,char *filter,char *name,char *scope,char *sec);
 void *Nova_LDAPList(char *uri,char *dn,char *filter,char *name,char *scope,char *sec);
 void *Nova_LDAPArray(char *array,char *uri,char *dn,char *filter,char *scope,char *sec);
@@ -60,7 +64,7 @@ char CURRENT_SHIFT[CF_MAXVARSIZE];
 int IsEnterprise()
 
 {
-#if defined HAVE_LIBCFNOVA || defined HAVE_LIBCFCONSTELLATION || defined HAVE_LIBCFGALAXY
+#if defined HAVE_NOVA || defined HAVE_CONSTELLATION || defined HAVE_LIBCFGALAXY
 return true;
 #else
 return false;
@@ -69,10 +73,24 @@ return false;
 
 /*****************************************************************************/
 
+void EnterpriseModuleTrick()
+
+{
+#if defined HAVE_NOVA
+Nova_EnterpriseModuleTrick();
+#endif
+
+#if defined HAVE_CONSTELLATION
+Constellation_EnterpriseModuleTrick();
+#endif
+}
+
+/*****************************************************************************/
+
 void EnterpriseContext()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_EnterpriseContext();
 #endif
 }
@@ -82,7 +100,9 @@ void EnterpriseContext()
 void EnterpriseVersion()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_CONSTELLATION
+Constellation_Version();
+#elif defined HAVE_NOVA
 Nova_Version();
 #endif 
 }
@@ -91,7 +111,7 @@ Nova_Version();
 
 int CfSessionKeySize(char c)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_CfSessionKeySize(c);
 #else
 return CF_BLOWFISHSIZE; 
@@ -102,7 +122,7 @@ return CF_BLOWFISHSIZE;
 
 char CfEnterpriseOptions()
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_CfEnterpriseOptions();
 #else
 return 'c';
@@ -111,9 +131,10 @@ return 'c';
 
 /*****************************************************************************/
 
+const EVP_CIPHER *Nova_CfengineCipher(char type);
 const EVP_CIPHER *CfengineCipher(char type)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_CfengineCipher(type);
 #else
 return EVP_bf_cbc();
@@ -122,11 +143,11 @@ return EVP_bf_cbc();
 
 /*****************************************************************************/
 
-int EnterpriseExpiry(char *day,char *month,char *year)
+int EnterpriseExpiry(char *day,char *month,char *year,char *company)
 
 {
-#ifdef HAVE_LIBCFNOVA
-return Nova_EnterpriseExpiry(day,month,year);
+#ifdef HAVE_NOVA
+ return Nova_EnterpriseExpiry(day,month,year,company);
 #else
 return false;
 #endif
@@ -137,8 +158,8 @@ return false;
 void CheckLicenses()
 
 {
-#ifdef HAVE_LIBCFNOVA
-return Nova_CheckLicensePromise();
+#ifdef HAVE_NOVA
+Nova_CheckLicensePromise();
 #else
 return;
 #endif
@@ -149,7 +170,7 @@ return;
 void CheckAutoBootstrap()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_CheckAutoBootstrap();
 #else
 #endif
@@ -160,50 +181,12 @@ void CheckAutoBootstrap()
 char *GetConsolePrefix()
     
 {
-#ifdef HAVE_LIBCFNOVA
- return "nova>";
-#else
- return "cf3";
-#endif
-}
-
-/*****************************************************************************/
-
-pid_t StartTwin(int argc,char **argv)
-
-/* Self-monitor in case of crash or binary change */
-    
-{
-#ifdef HAVE_LIBCFNOVA
-return Nova_StartTwin(argc,argv);
-#else
-return 0;
-#endif
-}
-
-/*****************************************************************************/
-
-void SignalTwin()
-
-/* Self-monitor in case of crash or binary change */
-    
-{
-#ifdef HAVE_LIBCFNOVA
- Nova_SignalTwin();
-#else
-#endif
-}
-
-/*****************************************************************************/
-
-void ReviveOther(int argc,char **argv)
-
-/* Self-monitor in case of crash or binary change */
-    
-{
-#ifdef HAVE_LIBCFNOVA
- Nova_ReviveOther(argc,argv);
+#ifdef HAVE_CONSTELLATION
+return "constellation";
+#elif defined HAVE_NOVA
+return "nova";
 #else
+return "cf3";
 #endif
 }
 
@@ -212,14 +195,14 @@ void ReviveOther(int argc,char **argv)
 char *MailSubject()
 
 { static char buffer[CF_BUFSIZE];
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 if (LICENSES)
    {
    strcpy(buffer,"Nova");
    }
 else
    {
-   strcpy(buffer,"EXPIRED");
+   strcpy(buffer,"NO LICENSE");
    }
 #else
 strcpy(buffer,"community");
@@ -232,7 +215,7 @@ return buffer;
 void SetPolicyServer(char *name)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SetPolicyServer(name);
 #else
 CfOut(cf_verbose,"","Setting policy server requires version Nova or above");
@@ -243,7 +226,7 @@ CfOut(cf_verbose,"","Setting policy server requires version Nova or above");
 
 void LogFileChange(char *file,int change,struct Attributes a,struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_LogFileChange(file,change,a,pp);
 #else
 CfOut(cf_verbose,"","Logging file differences requires version Nova or above");
@@ -254,54 +237,30 @@ CfOut(cf_verbose,"","Logging file differences requires version Nova or above");
 
 void RemoteSyslog(struct Attributes a,struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_RemoteSyslog(a,pp);
 #else
 CfOut(cf_verbose,"","Remote logging requires version Nova or above");
 #endif 
 }
 
+/*****************************************************************************/
+
+#if !defined(HAVE_NOVA)
+void WebCache(char *s,char *t)
+{
+}
+#endif
+
 /*****************************************************************************/
 /* Knowledge                                                                 */
 /*****************************************************************************/
 
-void BundleNode(FILE *fp,char *bundle)
-
-{
-#ifdef HAVE_LIBCFNOVA
-Nova_BundleNode(fp,bundle);
-#else
-#endif
-}
-
-/*****************************************************************************/
-
-void BodyNode(FILE *fp,char *bundle,int calltype)
-
-{
-#ifdef HAVE_LIBCFNOVA
- Nova_BodyNode(fp,bundle,calltype);
-#else
-#endif
-}
-
-/*****************************************************************************/
-
-void TypeNode(FILE *fp,char *type)
-
-{
-#ifdef HAVE_LIBCFNOVA
- Nova_TypeNode(fp,type);
-#else
-#endif
-}
-
-/*****************************************************************************/
-
+char *Nova_PromiseID(struct Promise *pp);
 char *PromiseID(struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_PromiseID(pp);
 #else
 return "";
@@ -310,11 +269,11 @@ return "";
 
 /*****************************************************************************/
 
-void NotePromiseCompliance(struct Promise *pp,double val,enum cf_status status)
+void NotePromiseCompliance(struct Promise *pp,double val,enum cf_status status,char *reason)
 
 {
-#ifdef HAVE_LIBCFNOVA
-Nova_NotePromiseCompliance(pp,val,status);
+#ifdef HAVE_NOVA
+ Nova_NotePromiseCompliance(pp,val,status,reason);
 #else
 #endif
 }
@@ -324,7 +283,7 @@ Nova_NotePromiseCompliance(pp,val,status);
 void PreSanitizePromise(struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_PreSanitizePromise(pp);
 #else
 #endif
@@ -335,7 +294,7 @@ Nova_PreSanitizePromise(pp);
 void TrackValue(char *date,double kept,double repaired, double notkept)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_TrackValue(date,kept,repaired,notkept);
 #else
 #endif 
@@ -346,7 +305,7 @@ Nova_TrackValue(date,kept,repaired,notkept);
 time_t GetPromiseCompliance(struct Promise *pp,double *value,double *average,double *var,time_t *lastseen)
     
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_GetPromiseCompliance(pp,value,average,var,lastseen);
 #else
 return time(NULL);
@@ -355,23 +314,11 @@ return time(NULL);
 
 /*****************************************************************************/
 
-void PromiseNode(FILE *fp,struct Promise *pp,int type)
+void MapPromiseToTopic(FILE *fp,struct Promise *pp,const char *version)
 
 {
-#ifdef HAVE_LIBCFNOVA
- Nova_PromiseNode(fp,pp,type);
-#else
-#endif
-}
-
-/*****************************************************************************/
-
-void MapPromiseToTopic(FILE *fp,struct Promise *pp,char *version)
-
-{
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_MapPromiseToTopic(fp,pp,version); 
-#else
 #endif
 }
 
@@ -380,20 +327,41 @@ void MapPromiseToTopic(FILE *fp,struct Promise *pp,char *version)
 void ShowTopicRepresentation(FILE *fp)
 
 {
-#ifdef HAVE_LIBCFNOVA
- Nova_ShowTopicRepresentation(fp);
+#ifdef HAVE_NOVA
+Nova_ShowTopicRepresentation(fp);
 #else
  CfOut(cf_verbose,"","# Knowledge map reporting feature is only available in version Nova and above\n");
 #endif
 
 }
 
+/*****************************************************************************/
+
+void NewPromiser(struct Promise *pp)
+{
+#ifdef HAVE_NOVA
+Nova_NewPromiser(pp);
+#else
+#endif 
+}
+
+/*****************************************************************************/
+
+void AnalyzePromiseConflicts()
+{
+#ifdef HAVE_NOVA
+Nova_AnalyzePromiseConflicts();
+#else
+#endif
+}
+
+
 /*****************************************************************************/
 
 void RegisterBundleDependence(char *name,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_RegisterBundleDepedence(name,pp);
 #else
 #endif
@@ -403,7 +371,7 @@ void RegisterBundleDependence(char *name,struct Promise *pp)
 
 void SyntaxCompletion(char *s)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SyntaxCompletion(s);
 #else
  printf("Syntax completion is available in cfengine Nova,Constellation or Galaxy\n\n");
@@ -412,9 +380,20 @@ void SyntaxCompletion(char *s)
 
 /*****************************************************************************/
 
+void SyntaxExport()
+{
+#ifdef HAVE_NOVA
+Nova_SyntaxTree2JavaScript();
+#else
+printf("Syntax export is available in cfengine Nova,Constellation or Galaxy\n\n");
+#endif
+}
+
+/*****************************************************************************/
+
 void VerifyOutputsPromise(struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_VerifyOutputsPromise(pp);
 #else
  printf(" !! Outputs promises are not available in the community edition of Cfengine\n");
@@ -425,16 +404,25 @@ void VerifyOutputsPromise(struct Promise *pp)
 
 void SetPromiseOutputs(struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SetPromiseOutputs(pp);
 #endif
 }
 
 /*****************************************************************************/
 
+void LastSawBundle(char *name)
+{
+#ifdef HAVE_NOVA
+Nova_LastSawBundle(name);
+#endif
+}
+
+/*****************************************************************************/
+
 void SetBundleOutputs(char *name)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SetBundleOutputs(name);
 #endif
 }
@@ -443,7 +431,7 @@ Nova_SetBundleOutputs(name);
 
 void ResetBundleOutputs(char *name)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_ResetBundleOutputs(name);
 #endif
 }
@@ -452,7 +440,7 @@ Nova_ResetBundleOutputs(name);
 
 void SpecialQuote(char *topic,char *type)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SpecialQuote(topic,type);
 #endif
 }
@@ -463,13 +451,14 @@ Nova_SpecialQuote(topic,type);
 
 void HistoryUpdate(struct Averages newvals)
 
-{ struct Promise *pp = NewPromise("history_db","the long term memory");
-  struct Attributes dummyattr;
+{
+#ifdef HAVE_NOVA  
+  struct Promise *pp = NewPromise("history_db","the long term memory");
+  struct Attributes dummyattr = {{0}};
   struct CfLock thislock;
   time_t now = time(NULL);
   char timekey[CF_MAXVARSIZE];
 
-#ifdef HAVE_LIBCFNOVA  
 /* We do this only once per hour - this should not be changed */
 
 Banner("Update long-term history");
@@ -483,7 +472,7 @@ if (strlen(CURRENT_SHIFT) == 0)
 memset(&dummyattr,0,sizeof(dummyattr));
 dummyattr.transaction.ifelapsed = 59;
 
-thislock = AcquireLock(pp->promiser,VUQNAME,now,dummyattr,pp);
+thislock = AcquireLock(pp->promiser,VUQNAME,now,dummyattr,pp,false);
 
 if (thislock.lock == NULL)
    {
@@ -537,7 +526,7 @@ Nova_DumpSlowlyVaryingObservations();
 
 void CfGetClassName(int i,char *name)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  char desc[CF_BUFSIZE];
  Nova_GetClassName(i,name,desc);
 #else
@@ -549,7 +538,7 @@ strcpy(name,OBS[i][0]);
 
 void LookUpClassName(int i,char *name)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  char desc[CF_BUFSIZE];
 
 Nova_LookupClassName(i,name,desc);
@@ -563,7 +552,7 @@ strcpy(name,OBS[i][0]);
 void LoadSlowlyVaryingObservations()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_LoadSlowlyVaryingObservations();
 #else
 CfOut(cf_verbose,"","# Extended system discovery is only available in version Nova and above\n");
@@ -577,7 +566,7 @@ CfOut(cf_verbose,"","# Extended system discovery is only available in version No
 void RegisterLiteralServerData(char *handle,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_RegisterLiteralServerData(handle,pp);
 #else
 CfOut(cf_verbose,"","# Access to server literals is only available in version Nova and above\n");
@@ -589,7 +578,7 @@ CfOut(cf_verbose,"","# Access to server literals is only available in version No
 int ReturnLiteralData(char *handle,char *ret)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_ReturnLiteralData(handle,ret);
 #else
 CfOut(cf_verbose,"","# Access to server literals is only available in version Nova and above\n");
@@ -599,10 +588,11 @@ return 0;
 
 /*****************************************************************************/
 
+char *Nova_GetRemoteScalar(char *proto,char *handle,char *server,int encrypted,char *rcv);
 char *GetRemoteScalar(char *proto,char *handle,char *server,int encrypted,char *rcv)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_GetRemoteScalar(proto,handle,server,encrypted,rcv);
 #else
 CfOut(cf_verbose,"","# Access to server literals is only available in version Nova and above\n");
@@ -615,7 +605,7 @@ return "";
 void CacheUnreliableValue(char *caller,char *handle,char *buffer)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_CacheUnreliableValue(caller,handle,buffer);
 #else
 CfOut(cf_verbose,"","# Value fault-tolerance in version Nova and above\n");
@@ -628,7 +618,7 @@ return;
 int RetrieveUnreliableValue(char *caller,char *handle,char *buffer)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_RetrieveUnreliableValue(caller,handle,buffer);
 #else
 CfOut(cf_verbose,"","# Value fault-tolerance in version Nova and above\n");
@@ -640,13 +630,54 @@ return false;
 
 void TranslatePath(char *new,char *old)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_TranslatePath(new,old);
 #else
 strncpy(new,old,CF_BUFSIZE-1);
 #endif 
 }
 
+/*****************************************************************************/
+
+RSA *Nova_SelectKeyRing(char *name);
+RSA *SelectKeyRing(char *name)
+{
+#ifdef HAVE_NOVA
+if (KEYTTL > 0)
+   {
+   return Nova_SelectKeyRing(name);
+   }
+else
+   {
+   return NULL;
+   }
+#else
+return NULL;
+#endif 
+}
+
+/*****************************************************************************/
+
+void IdempAddToKeyRing(char *name,char *ip,RSA *key)
+{
+#ifdef HAVE_NOVA
+Nova_IdempAddToKeyRing(name,ip,key);
+#else
+return;
+#endif 
+}
+
+/*****************************************************************************/
+
+void PurgeKeyRing()
+{
+#ifdef HAVE_NOVA
+Nova_PurgeKeyRing();
+#else
+return;
+#endif 
+}
+
 /*****************************************************************************/
 /* Reporting                                                                 */
 /*****************************************************************************/
@@ -654,7 +685,7 @@ strncpy(new,old,CF_BUFSIZE-1);
 void SummarizeCompliance(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeCompliance(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Compliance reporting feature is only available in version Nova and above\n");
@@ -666,7 +697,7 @@ void SummarizeCompliance(int xml,int html,int csv,int embed,char *stylesheet,cha
 void SummarizeValue(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SummarizeValue(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
 CfOut(cf_verbose,"","# Value reporting feature is only available in version Nova and above - use the state/cf_value.log\n");
@@ -678,7 +709,7 @@ CfOut(cf_verbose,"","# Value reporting feature is only available in version Nova
 void SummarizePromiseRepaired(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SummarizePromiseRepaired(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
 CfOut(cf_verbose,"","# Compliance reporting feature is only available in version Nova and above\n");
@@ -690,7 +721,7 @@ CfOut(cf_verbose,"","# Compliance reporting feature is only available in version
 void SummarizePromiseNotKept(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_SummarizePromiseNotKept(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
 CfOut(cf_verbose,"","# Compliance reporting feature is only available in version Nova and above\n");
@@ -702,7 +733,7 @@ CfOut(cf_verbose,"","# Compliance reporting feature is only available in version
 void GrandSummary()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_GrandSummary();
 #else
  CfOut(cf_verbose,"","# Reporting feature is only available in version Nova and above\n");
@@ -714,7 +745,7 @@ void GrandSummary()
 void CSV2XML(struct Rlist *list)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_CSV2XML(list);
 #else
  CfOut(cf_verbose,"","# Format conversion feature is only available in version Nova and above\n");
@@ -723,30 +754,10 @@ void CSV2XML(struct Rlist *list)
 
 /*****************************************************************************/
 
-void Aggregate(char *stylesheet,char *banner,char *footer,char *webdriver)
-
-{
-#ifdef HAVE_LIBCFNOVA
-Nova_Aggregate(stylesheet,banner,footer,webdriver);
-#endif 
-}
-
-/*****************************************************************************/
-
-void NoteVarUsage()
-
-{
-#ifdef HAVE_LIBCFNOVA
-Nova_NoteVarUsage();
-#endif 
-}
-
-/*****************************************************************************/
-
 void SummarizeVariables(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeVariables(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Variable reporting feature is only available in version Nova and above\n");
@@ -758,7 +769,7 @@ void SummarizeVariables(int xml,int html,int csv,int embed,char *stylesheet,char
 void SummarizePerPromiseCompliance(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizePerPromiseCompliance(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Compliance reporting feature is only available in version Nova and above\n");
@@ -770,7 +781,7 @@ void SummarizePerPromiseCompliance(int xml,int html,int csv,int embed,char *styl
 void SummarizeFileChanges(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeFileChanges(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# File change reporting feature is only available in version Nova and above\n");
@@ -782,7 +793,7 @@ void SummarizeFileChanges(int xml,int html,int csv,int embed,char *stylesheet,ch
 void SummarizeSetuid(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeSetuid(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Setuid reporting feature is only available in version Nova and above\n");
@@ -792,19 +803,39 @@ void SummarizeSetuid(int xml,int html,int csv,int embed,char *stylesheet,char *h
 /*****************************************************************************/
 
 void ReportSoftware(struct CfPackageManager *list)
-{
-#ifdef HAVE_LIBCFNOVA
- Nova_ReportSoftware(list);
-#else
- CfOut(cf_verbose,"","# Software reporting feature is only available in version Nova and above\n");
-#endif
+
+{ FILE *fout;
+  struct CfPackageManager *mp = NULL;
+  struct CfPackageItem *pi;
+  char name[CF_BUFSIZE];
+
+snprintf(name,CF_BUFSIZE,"%s/state/%s",CFWORKDIR,NOVA_SOFTWARE_INSTALLED);
+MapName(name);
+
+if ((fout = fopen(name,"w")) == NULL)
+   {
+   CfOut(cf_error,"fopen","Cannot open the destination file %s",name);
+   return;
+   }
+
+for (mp = list; mp != NULL; mp = mp->next)
+   {
+   for (pi = mp->pack_list; pi != NULL; pi=pi->next)
+      {
+      fprintf(fout,"%s,",CanonifyChar(pi->name,','));
+      fprintf(fout,"%s,",CanonifyChar(pi->version,','));
+      fprintf(fout,"%s,%s\n",pi->arch,ReadLastNode(GetArg0(mp->manager)));
+      }
+   }
+
+fclose(fout);
 }
 
 /*****************************************************************************/
 
 void ReportPatches(struct CfPackageManager *list)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_ReportPatches(list);
 #else
  CfOut(cf_verbose,"","# Patch reporting feature is only available in version Nova and above\n");
@@ -816,7 +847,7 @@ void ReportPatches(struct CfPackageManager *list)
 void SummarizeSoftware(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeSoftware(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Software summary reporting feature is only available in version Nova and above\n");
@@ -828,7 +859,7 @@ void SummarizeSoftware(int xml,int html,int csv,int embed,char *stylesheet,char
 void SummarizeUpdates(int xml,int html,int csv,int embed,char *stylesheet,char *head,char *foot,char *web)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SummarizeUpdates(xml,html,csv,embed,stylesheet,head,foot,web);
 #else
  CfOut(cf_verbose,"","# Software summary reporting feature is only available in version Nova and above\n");
@@ -839,7 +870,7 @@ void SummarizeUpdates(int xml,int html,int csv,int embed,char *stylesheet,char *
 
 void VerifyServices(struct Attributes a,struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_VerifyServices(a,pp);
 #else
  CfOut(cf_verbose,"","# Services promises are only available in Cfengine Nova and above");
@@ -852,7 +883,7 @@ void VerifyServices(struct Attributes a,struct Promise *pp)
 
 void InitMeasurements()
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  NovaInitMeasurements();
 #endif
 }
@@ -862,7 +893,7 @@ void InitMeasurements()
 void VerifyMeasurement(double *this,struct Attributes a,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_VerifyMeasurement(this,a,pp);
 #else
  CfOut(cf_verbose,"","# Custom monitoring feature is only available in version Nova and above\n");
@@ -874,7 +905,7 @@ void VerifyMeasurement(double *this,struct Attributes a,struct Promise *pp)
 void LongHaul()
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_LongHaul(VDAY,VMONTH,VLIFECYCLE,VSHIFT);
 #else
 #endif
@@ -885,7 +916,7 @@ void LongHaul()
 void SetMeasurementPromises(struct Item **classlist)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
  Nova_SetMeasurementPromises(classlist);
 #else
 #endif
@@ -898,7 +929,7 @@ void SetMeasurementPromises(struct Item **classlist)
 void VerifyACL(char *file,struct Attributes a, struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 Nova_VerifyACL(file,a,pp);
 #else
 CfOut(cf_verbose, "", "Verifying ACL promises is only available with Cfengine Nova or above");
@@ -910,7 +941,7 @@ CfOut(cf_verbose, "", "Verifying ACL promises is only available with Cfengine No
 int CheckACLSyntax(char *file,struct CfACL acl,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_CheckACLSyntax(file,acl,pp);
 #else
 return true;
@@ -924,7 +955,7 @@ return true;
 void VerifyRegistryPromise(struct Attributes a,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 # ifdef NT
 Nova_VerifyRegistryPromise(a,pp);
 # endif
@@ -937,7 +968,7 @@ Nova_VerifyRegistryPromise(a,pp);
 int GetRegistryValue(char *key,char *value,char *buffer)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 # ifdef NT
 return Nova_CopyRegistryValue(key,value,buffer);
 # endif
@@ -951,59 +982,47 @@ return 0;
 /* LDAP                                                                      */
 /*****************************************************************************/
 
+#if !defined(HAVE_NOVA)
+
 void *CfLDAPValue(char *uri,char *dn,char *filter,char *name,char *scope,char *sec)
 {
-#if defined HAVE_LIBCFNOVA && defined HAVE_LIBLDAP
- return Nova_LDAPValue(uri,dn,filter,name,scope,sec);
-#else
- CfOut(cf_error,"","LDAP support available in Nova and above");
+CfOut(cf_error, "", "LDAP support is available in Nova and above");
 return NULL;
-#endif
 }
 
 /*****************************************************************************/
 
 void *CfLDAPList(char *uri,char *dn,char *filter,char *name,char *scope,char *sec)
 {
-#if defined HAVE_LIBCFNOVA && defined HAVE_LIBLDAP
- return Nova_LDAPList(uri,dn,filter,name,scope,sec);
-#else
- CfOut(cf_error,"","LDAP support available in Nova and above");
+CfOut(cf_error,"","LDAP support available in Nova and above");
 return NULL;
-#endif
 }
 
 /*****************************************************************************/
 
 void *CfLDAPArray(char *array,char *uri,char *dn,char *filter,char *scope,char *sec)
 {
-#if defined HAVE_LIBCFNOVA && defined HAVE_LIBLDAP
- return Nova_LDAPArray(array,uri,dn,filter,scope,sec);
-#else
- CfOut(cf_error,"","LDAP support available in Nova and above");
+CfOut(cf_error,"","LDAP support available in Nova and above");
 return NULL;
-#endif
 }
 
 /*****************************************************************************/
 
 void *CfRegLDAP(char *uri,char *dn,char *filter,char *name,char *scope,char *regex,char *sec)
 {
-#if defined HAVE_LIBCFNOVA && defined HAVE_LIBLDAP
-return Nova_RegLDAP(uri,dn,filter,name,scope,regex,sec);
-#else
 CfOut(cf_error,"","LDAP support available in Nova and above");
 return NULL;
-#endif
 }
 
+#endif
+
 /*****************************************************************************/
 /* SQL                                                                       */
 /*****************************************************************************/
 
 int CheckDatabaseSanity(struct Attributes a, struct Promise *pp)
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_CheckDatabaseSanity(a,pp);
 #else
 return false;
@@ -1015,7 +1034,7 @@ return false;
 int VerifyDatabasePromise(CfdbConn *cfdb,char *database,struct Attributes a,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
   char query[CF_BUFSIZE],name[CF_MAXVARSIZE];
   int found = false;
  
@@ -1103,7 +1122,7 @@ return false;
 int CfVerifyTablePromise(CfdbConn *cfdb,char *name,struct Rlist *columns,struct Attributes a,struct Promise *pp)
 
 {
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 return Nova_VerifyTablePromise(cfdb,name,columns,a,pp);
 #else
 CfOut(cf_verbose,"","Verifying SQL table promises is only available with Cfengine Nova or above");
@@ -1118,12 +1137,12 @@ return false;
 void NoteEfficiency(double e)
 
 {
-#ifdef HAVE_LIBCFNOVA
- struct Attributes a;
- struct Promise p;
+#ifdef HAVE_NOVA
+ struct Attributes a = {{0}};
+ struct Promise p = {0};
  
 NovaNamedEvent("Configuration model efficiency",e,a,&p);
-CfOut(cf_verbose,"","Configuration model efficiency for %s = %.2lf%%",VUQNAME,e);
+CfOut(cf_verbose,"","Configuration model efficiency for %s = %.4lf%%",VUQNAME,e);
 #endif 
 }
 
@@ -1131,297 +1150,102 @@ CfOut(cf_verbose,"","Configuration model efficiency for %s = %.2lf%%",VUQNAME,e)
 
 char *GetProcessOptions()
 {
-#ifdef HAVE_LIBCFNOVA
- return Nova_GetProcessOptions();
-#else
-CfOut(cf_verbose,"","Verifying SQL table promises is only available with Cfengine Nova or above");
-return VPSOPTS[VSYSTEMHARDCLASS];
+#ifdef HAVE_GETZONEID
+ zoneid_t zid;
+ char zone[ZONENAME_MAX];
+ static char psopts[CF_BUFSIZE];
+ 
+zid = getzoneid();
+getzonenamebyid(zid,zone,ZONENAME_MAX);
+
+if (cf_strcmp(zone,"global") == 0)
+   {
+   snprintf(psopts,CF_BUFSIZE,"%s,zone",VPSOPTS[VSYSTEMHARDCLASS]);
+   return psopts;
+   }
 #endif
+
+#ifdef LINUX
+if (strncmp(VSYSNAME.release,"2.4",3) == 0)
+   {
+   // No threads on 2.4 kernels
+   return "-eo user,pid,ppid,pgid,pcpu,pmem,vsz,pri,rss,stime,time,args";
+   }
+
+#endif
+
+return VPSOPTS[VSYSTEMHARDCLASS];
 }
 
-/*****************************************************************************/
-/* Linker troubles require this code to be here in the main body             */
 /*****************************************************************************/
 
-#ifdef HAVE_LIBCFNOVA
+int ForeignZone(char *s)
+{
+// We want to keep the banner
 
-int Nova_VerifyTablePromise(CfdbConn *cfdb,char *table_path,struct Rlist *columns,struct Attributes a,struct Promise *pp)
-
-{ char name[CF_MAXVARSIZE],type[CF_MAXVARSIZE],query[CF_MAXVARSIZE],table[CF_MAXVARSIZE],db[CF_MAXVARSIZE];
-  int i,count,size,no_of_cols,*size_table,*done,identified,retval = true;
-  char **name_table,**type_table;
-  struct Rlist *rp, *cols;
-
-CfOut(cf_verbose,""," -> Verifying promised table structure for \"%s\"",table_path);
-
-if (!Nova_ValidateSQLTableName(table_path,db,table))
+if (strstr(s,"%CPU"))
    {
-   CfOut(cf_error,""," !! The structure of the promiser did not match that for an SQL table, i.e. \"database.table\"\n",table_path);
    return false;
    }
-else
+
+#ifdef HAVE_GETZONEID
+ zoneid_t zid;
+ char *sp,zone[ZONENAME_MAX];
+ static psopts[CF_BUFSIZE];
+ 
+zid = getzoneid();
+getzonenamebyid(zid,zone,ZONENAME_MAX);
+
+if (cf_strcmp(zone,"global") == 0)
    {
-   CfOut(cf_verbose,""," -> Assuming database \"%s\" with table \"%s\"",db,table);
-   }
-
-/* Verify the existence of the tables within the database */
-
-if (!Nova_TableExists(cfdb,table))
-   {
-   CfOut(cf_error,""," !! The database did not contain the promised table \"%s\"\n",table_path);
-
-   if (a.database.operation && strcmp(a.database.operation,"create") == 0)
+   if (cf_strcmp(s+strlen(s)-6,"global") == 0)
       {
-      if (!DONTDO && a.transaction.action != cfa_warn)
+      *(s+strlen(s)-6) = '\0';
+
+      for (sp = s+strlen(s)-1; isspace(*sp); sp--)
          {
-         cfPS(cf_error,CF_CHG,"",pp,a," -> Database.table %s doesn't seem to exist, creating\n",table_path);
-         return Nova_CreateTableColumns(cfdb,table,columns,a,pp);
+         *sp = '\0';
          }
-      else
-         {
-         CfOut(cf_error,""," -> Database.table %s doesn't seem to exist, but only a warning was promised\n",table_path);
-         }
-      }
-   
-   return false;
-   }
 
-/* Get a list of the columns in the table */
-
-Nova_QueryTableColumns(query,db,table);
-CfNewQueryDB(cfdb,query);
-
-if (cfdb->maxcolumns != 3)
-   {
-   cfPS(cf_error,CF_FAIL,"",pp,a,"Could not make sense of the columns");
-   CfDeleteQuery(cfdb);
-   return false;
-   }
-
-/* Assume that the Rlist has been validated and consists of a,b,c */
-
-count = 0;
-no_of_cols = RlistLen(columns);
-
-if (!Nova_NewSQLColumns(table,columns,&name_table,&type_table,&size_table,&done))
-   {
-   cfPS(cf_error,CF_FAIL,"",pp,a,"Could not make sense of the columns");
-   return false;
-   }
-
-/* Obtain columns from the named table - if any */
-
-while(CfFetchRow(cfdb))
-   {
-   name[0] = '\0';
-   type[0] = '\0';
-   size = CF_NOINT;
-   
-   strncpy(name,CfFetchColumn(cfdb,0),CF_MAXVARSIZE-1);
-   strncpy(type,ToLowerStr(CfFetchColumn(cfdb,1)),CF_MAXVARSIZE-1);
-   size = Str2Int(CfFetchColumn(cfdb,2));
-
-   CfOut(cf_verbose,"","    ... discovered column (%s,%s,%d)",name,type,size);
-   
-   if (size == CF_NOINT)
-      {
-      cfPS(cf_error,CF_NOP,"",pp,a," !! Integer size of datatype could not be determined - invalid promise.");
-      Nova_DeleteSQLColumns(name_table,type_table,size_table,done,no_of_cols);
-      free(done);
-      CfDeleteQuery(cfdb);
       return false;
       }
-
-   identified = false;
-   
-   for (i = 0; i < no_of_cols; i++)
+   else
       {
-      if (done[i])
-         {
-         continue;
-         }
-
-      if (strcmp(name,name_table[i]) == 0)
-         {
-         NovaCheckSQLDataType(type,type_table[i],pp);
-
-         if (size != size_table[i])
-            {
-            cfPS(cf_error,CF_FAIL,"",pp,a," !! Promised column \"%s\" in database.table \"%s\" has a non-matching array size (%d != %d)",name,table_path,size,size_table[i]);
-            }
-         else
-            {
-            CfOut(cf_verbose,""," -> Promised column \"%s\" in database.table \"%s\" is as promised",name,table_path);
-            }
-         
-         count++;
-         done[i] = true;
-         identified = true;
-         break;
-         }
-      }
-
-   if (!identified)
-      {
-      cfPS(cf_error,CF_FAIL,"",pp,a,"Column \"%s\" found in database.table \"%s\" is not part of its promise.",name,table_path);
-
-      if (a.database.operation && strcmp(a.database.operation,"drop") == 0)
-         {
-         cfPS(cf_error,CF_FAIL,"",pp,a,"Cfengine will not promise to repair this, as the operation is potentially too destructive.");
-         // Future allow deletion?
-         }
-      
-      retval = false;
+      return true;
       }
    }
-
-CfDeleteQuery(cfdb);
-
-/* Now look for deviations - only if we have promised to create missing */
-
-if (a.database.operation && strcmp(a.database.operation,"drop") == 0)
-   {
-   return retval;
-   }
-
-if (count != no_of_cols)
-   {
-   for (i = 0; i < no_of_cols; i++)
-      {
-      if (!done[i])
-         {
-         CfOut(cf_error,""," !! Promised column \"%s\" missing from database table %s",name_table[i],pp->promiser);
-         
-         if (!DONTDO && a.transaction.action != cfa_warn)
-            {
-            if (size_table[i] > 0)
-               {
-               snprintf(query,CF_MAXVARSIZE-1,"ALTER TABLE %s ADD %s %s(%d)",table,name_table[i],type_table[i],size_table[i]);
-               }
-            else
-               {
-               snprintf(query,CF_MAXVARSIZE-1,"ALTER TABLE %s ADD %s %s",table,name_table[i],type_table[i]);
-               }
-            
-            CfVoidQueryDB(cfdb,query);
-            cfPS(cf_error,CF_CHG,"",pp,a," !! Adding promised column \"%s\" to database table %s",name_table[i],table);
-            retval = true;
-            }
-         else
-            {
-            cfPS(cf_error,CF_WARN,"",pp,a," !! Promised column \"%s\" missing from database table %s but only a warning was promised",name_table[i],table);
-            retval = false;
-            }
-         }
-      }
-   }
-
-Nova_DeleteSQLColumns(name_table,type_table,size_table,done,no_of_cols);
-
-return retval;
+#endif
+return false;
 }
 
 /*****************************************************************************/
 
-int Nova_TableExists(CfdbConn *cfdb,char *name)
+int GetInstalledPkgsRpath(struct CfPackageItem **pkgList, struct Attributes a, struct Promise *pp)
+{
+#ifdef HAVE_NOVA
 
-{ struct Rlist *rp,*list = NULL;
-  int match = false;
+return Nova_GetInstalledPkgsRpath(pkgList, a, pp);
 
-list = Nova_GetSQLTables(cfdb);
- 
-for (rp = list; rp != NULL; rp=rp->next)
-   {
-   if (strcmp(name,rp->item) == 0)
-      {
-      match = true;
-      }
-   }
+#else
 
-DeleteRlist(list);
-
-return match;
-}
-
-/*****************************************************************************/
-
-int Nova_CreateTableColumns(CfdbConn *cfdb,char *table,struct Rlist *columns,struct Attributes a,struct Promise *pp)
-
-{ char entry[CF_MAXVARSIZE],query[CF_BUFSIZE];
-  int i,count,size,*size_table,*done,identified,retval = true;
-  char **name_table,**type_table;
-  struct Rlist *rp, *cols;
-  int no_of_cols = RlistLen(columns);
-
-CfOut(cf_error,""," -> Trying to create table %s\n",table);
-  
-if (!Nova_NewSQLColumns(table,columns,&name_table,&type_table,&size_table,&done))
-   {
-   return false;
-   }
-
-if (no_of_cols > 0)
-   {
-   snprintf(query,CF_BUFSIZE-1,"create table %s(",table);
-   
-   for (i = 0; i < no_of_cols; i++)
-      {
-      CfOut(cf_verbose,""," -> Forming column template %s %s %d\n",name_table[i],type_table[i],size_table[i]);;
-      
-      if (size_table[i] > 0)
-         {
-         snprintf(entry,CF_MAXVARSIZE-1,"%s %s(%d)",name_table[i],type_table[i],size_table[i]);
-         }
-      else
-         {
-         snprintf(entry,CF_MAXVARSIZE-1,"%s %s",name_table[i],type_table[i]);
-         }
-
-      strcat(query,entry);
-
-      if (i < no_of_cols -1)
-         {
-         strcat(query,",");
-         }
-      }
-
-   strcat(query,")");
-   }
-
-CfVoidQueryDB(cfdb,query);
-Nova_DeleteSQLColumns(name_table,type_table,size_table,done,no_of_cols);
-return true;
-}
-
-
-/*****************************************************************************/
-/* Level                                                                     */
-/*****************************************************************************/
-
-struct Rlist *Nova_GetSQLTables(CfdbConn *cfdb)
-
-{ struct Rlist *list = NULL;
-  char query[CF_MAXVARSIZE];
-
-Nova_ListTables(cfdb->type,query);
-
-CfNewQueryDB(cfdb,query);
-
-if (cfdb->maxcolumns != 1)
-   {
-   CfOut(cf_error,"","Could not make sense of the columns");
-   CfDeleteQuery(cfdb);
-   return NULL;
-   }
-
-while(CfFetchRow(cfdb))
-   {
-   PrependRScalar(&list,CfFetchColumn(cfdb,0),CF_SCALAR);
-   }
-
-CfDeleteQuery(cfdb);
-
-return list;
-}
+CfOut(cf_error, "", "!! rPath internal package listing only available in Nova or above");
+return false;
 
 #endif
+}
+
+/*****************************************************************************/
+
+int ExecPackageCommandRpath(char *command,int verify,int setCmdClasses,struct Attributes a,struct Promise *pp)
+{
+#ifdef HAVE_NOVA
+
+return Nova_ExecPackageCommandRpath(command,verify,setCmdClasses,a,pp);
+
+#else
+
+CfOut(cf_error, "", "!! rPath internal package commands only available in Nova or above");
+return false;
+
+#endif
+}
