@@ -64,14 +64,12 @@ extern struct BodySyntax CFEX_CONTROLBODY[];
 
 void StartServer(int argc,char **argv);
 int ScheduleRun(void);
-static char *timestamp(time_t stamp, char *buf, size_t len);
 void *LocalExec(void *scheduled_run);
 int FileChecksum(char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],char type);
 int CompareResult(char *filename,char *prev_file);
 void MailResult(char *file,char *to);
 int Dialogue(int sd,char *s);
 void Apoptosis(void);
-void DebugExecMemory(char *ref);
 
 /*******************************************************************/
 /* Command line options                                            */
@@ -157,7 +155,6 @@ void CheckOpts(int argc,char **argv)
 
 { extern char *optarg;
   char arg[CF_BUFSIZE];
-  struct Item *actionList;
   int optindex = 0;
   int c;
   char ld_library_path[CF_BUFSIZE];
@@ -268,8 +265,7 @@ if (argv[optind] != NULL)
 
 void ThisAgentInit()
 
-{ char vbuff[CF_BUFSIZE];
-
+{
 umask(077);
 LOGGING = true;
 MAILTO[0] = '\0';
@@ -287,6 +283,7 @@ if (SCHEDULE == NULL)
    AppendItem(&SCHEDULE,"Min25",NULL);   
    AppendItem(&SCHEDULE,"Min30",NULL);
    AppendItem(&SCHEDULE,"Min35",NULL);
+   AppendItem(&SCHEDULE,"Min40",NULL);
    AppendItem(&SCHEDULE,"Min45",NULL);
    AppendItem(&SCHEDULE,"Min50",NULL);
    AppendItem(&SCHEDULE,"Min55",NULL);
@@ -355,7 +352,7 @@ for (cp = ControlBodyConstraints(cf_executor); cp != NULL; cp=cp->next)
       {
       int hash,time = Str2Int(retval);
       snprintf(splay,CF_BUFSIZE,"%s+%s+%d",VFQNAME,VIPADDRESS,getuid());
-      hash = Hash(splay);
+      hash = GetHash(splay);
       SPLAYTIME = (int)(time*60*hash/CF_HASHTABLESIZE);
       }
 
@@ -381,7 +378,7 @@ for (cp = ControlBodyConstraints(cf_executor); cp != NULL; cp=cp->next)
 
 void StartServer(int argc,char **argv)
 
-{ int pid,time_to_run = false;
+{ int time_to_run = false;
   time_t now = time(NULL);
   struct Promise *pp = NewPromise("exec_cfengine","the executor agent"); 
   struct Attributes dummyattr;
@@ -396,7 +393,7 @@ dummyattr.transaction.expireafter = CF_EXEC_EXPIREAFTER;
 
 if (!ONCE)
    {
-   thislock = AcquireLock(pp->promiser,VUQNAME,CFSTARTTIME,dummyattr,pp);
+   thislock = AcquireLock(pp->promiser,VUQNAME,CFSTARTTIME,dummyattr,pp,false);
 
    if (thislock.lock == NULL)
       {
@@ -409,7 +406,7 @@ Apoptosis();
 
 #ifdef MINGW
 
-if(!NO_FORK)
+if (!NO_FORK)
   {
   CfOut(cf_verbose, "", "Windows does not support starting processes in the background - starting in foreground");
   }
@@ -429,11 +426,6 @@ if (!NO_FORK)
    
 #endif  /* NOT MINGW */
 
-if (!ONCE)
-   {
-   MYTWIN = StartTwin(argc,argv);
-   }
-
 WritePID("cf-execd.pid");
 signal(SIGINT,HandleSignals);
 signal(SIGTERM,HandleSignals);
@@ -451,13 +443,14 @@ if (ONCE)
    LocalExec((void *)0);
    }
 else
-   { char **nargv;
-     int i;
+   {
 #ifdef HAVE_PTHREAD_H
-     pthread_t tid;
+   pthread_t tid;
 #endif
 
-#if defined NT && !(defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD) 
+#if defined NT && !(defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD)
+   int i;
+   char **nargv;
    /*
     * Append --once option to our arguments for spawned monitor process.
     */
@@ -507,27 +500,19 @@ else
          pthread_attr_setstacksize(&PTHREADDEFAULTS,(size_t)2048*1024);
 #endif
 
-         DebugExecMemory("Before thread");
-
          if (pthread_create(&tid,&PTHREADDEFAULTS,LocalExec,(void *)1) != 0)
             {
             CfOut(cf_inform,"pthread_create","Can't create thread!");
             LocalExec((void *)1);
             }
 
-         DebugExecMemory("after-thread-before-free");
-
          ThreadLock(cft_system);
          pthread_attr_destroy(&PTHREADDEFAULTS);
          ThreadUnlock(cft_system);
-         
-         DebugExecMemory("After free");
 #else
          LocalExec((void *)1);  
 #endif
          }
-
-      ReviveOther(argc,argv);
       }
    }
 
@@ -543,10 +528,9 @@ if (!ONCE)
 
 void Apoptosis()
 
-{ struct Promise pp;
+{ struct Promise pp = {0};
   struct Rlist *signals = NULL, *owners = NULL;
-  char mypid[32],pidrange[32];
-  char *psopts = GetProcessOptions();
+  char mypid[32];
   static char promiserBuf[CF_SMALLBUF];
 
 if (ONCE || VSYSTEMHARDCLASS == cfnt)
@@ -593,12 +577,12 @@ AppendConstraint(&(pp.conlist),"process_select",strdup("true"),CF_SCALAR,"any",f
 AppendConstraint(&(pp.conlist),"process_owner",owners,CF_LIST,"any",false);
 AppendConstraint(&(pp.conlist),"ifelapsed",strdup("0"),CF_SCALAR,"any",false);
 AppendConstraint(&(pp.conlist),"process_count",strdup("true"),CF_SCALAR,"any",false);
-AppendConstraint(&(pp.conlist),"match_range",strdup("0,4"),CF_SCALAR,"any",false);
+AppendConstraint(&(pp.conlist),"match_range",strdup("0,2"),CF_SCALAR,"any",false);
 AppendConstraint(&(pp.conlist),"process_result",strdup("process_owner.process_count"),CF_SCALAR,"any",false);
 
 CfOut(cf_verbose,""," -> Looking for cf-execd processes owned by %s",mypid);
 
-if (LoadProcessTable(&PROCESSTABLE,psopts))
+if (LoadProcessTable(&PROCESSTABLE))
    {
    VerifyProcessesPromise(&pp);   
    }
@@ -620,11 +604,6 @@ int ScheduleRun()
 { time_t now;
   char timekey[64];
   struct Item *ip;
-  
-DebugExecMemory("Run 0.1");
-
-SignalTwin();
-DebugExecMemory("Run 0.2");
 
 CfOut(cf_verbose,"","Sleeping...\n");
 sleep(CFPULSETIME);                /* 1 Minute resolution is enough */ 
@@ -632,28 +611,29 @@ now = time(NULL);
 
 // recheck license (in case of license updates or expiry)
 
-DebugExecMemory("0.3");
-
-/*
-if (EnterpriseExpiry(LIC_DAY,LIC_MONTH,LIC_YEAR)) 
+if (EnterpriseExpiry(LIC_DAY,LIC_MONTH,LIC_YEAR,LIC_COMPANY)) 
   {
   CfOut(cf_error,"","Cfengine - autonomous configuration engine. This enterprise license is invalid.\n");
   exit(1);
   }
-*/
-
-DebugExecMemory("Run 1");
 
 ThreadLock(cft_system);
-DeleteItemList(VHEAP);
-VHEAP = NULL;
-DeleteItemList(VADDCLASSES);
-VADDCLASSES = NULL;
+
+DeleteAlphaList(&VHEAP);
+InitAlphaList(&VHEAP);
+DeleteAlphaList(&VADDCLASSES);
+InitAlphaList(&VADDCLASSES);
+
 DeleteItemList(IPADDRESSES);
 IPADDRESSES = NULL;
+
 DeleteScope("this");
 DeleteScope("mon");
 DeleteScope("sys");
+NewScope("this");
+NewScope("mon");
+NewScope("sys");
+
 CfGetInterfaceInfo(cf_executor);
 Get3Environment();
 OSClasses();
@@ -662,8 +642,6 @@ snprintf(timekey,63,"%s",cf_ctime(&now));
 AddTimeClass(timekey); 
 ThreadUnlock(cft_system);
 
-DebugExecMemory("Run 1.1");
-
 for (ip = SCHEDULE; ip != NULL; ip = ip->next)
    {
    CfOut(cf_verbose,"","Checking schedule %s...\n",ip->name);
@@ -671,38 +649,19 @@ for (ip = SCHEDULE; ip != NULL; ip = ip->next)
    if (IsDefinedClass(ip->name))
       {
       CfOut(cf_verbose,"","Waking up the agent at %s ~ %s \n",timekey,ip->name);
-      DebugExecMemory("Run schedules");
       return true;
       }
    }
 
-DebugExecMemory("Run - unscheduled");
 return false;
 }
 
 /*************************************************************************/
 
-static char *timestamp(time_t stamp, char *buf, size_t len)
-
-{ struct tm *ltime;
- 
-ltime = localtime(&stamp);
-snprintf(buf, len, "%04d-%02d-%02d--%02d-%02d-%02d",
-         ltime->tm_year+1900,
-         ltime->tm_mon+1,
-         ltime->tm_mday,
-         ltime->tm_hour,
-         ltime->tm_min,
-         ltime->tm_sec);
-return buf;
-}
-
-/**************************************************************/
-
 void *LocalExec(void *scheduled_run)
 
 { FILE *pp; 
-  char line[CF_BUFSIZE],filename[CF_BUFSIZE],*sp;
+  char line[CF_BUFSIZE],lineEscaped[sizeof(line)*2],filename[CF_BUFSIZE],*sp;
   char cmd[CF_BUFSIZE],esc_command[CF_BUFSIZE];
   int print,count = 0;
   void *threadName;
@@ -725,8 +684,6 @@ CfOut(cf_verbose,"","-----------------------------------------------------------
 CfOut(cf_verbose,"","  LocalExec(%sscheduled) at %s\n", scheduled_run ? "" : "not ", cf_ctime(&starttime));
 CfOut(cf_verbose,"","------------------------------------------------------------------\n"); 
 
-DebugExecMemory("LocalExec 1");
-
 /* Need to make sure we have LD_LIBRARY_PATH here or children will die  */
 
 if (strlen(EXECCOMMAND) > 0)
@@ -781,7 +738,7 @@ else
          twin_exists = true;
          }
       
-      if (IsExecutable(cmd))
+      if (twin_exists && IsExecutable(cmd))
 	 {
          snprintf(cmd,CF_BUFSIZE-1,"\"%s/bin/cf-twin\" -f failsafe.cf && \"%s/bin/cf-agent%s\" -Dfrom_cfexecd%s",
                   CFWORKDIR,
@@ -800,8 +757,6 @@ else
       }   
    }
 
-DebugExecMemory("LocalExec 2");
-
 strncpy(esc_command,MapName(cmd),CF_BUFSIZE-1);
    
 snprintf(line,CF_BUFSIZE-1,"_%d_%s",starttime,CanonifyName(cf_ctime(&starttime)));
@@ -848,23 +803,28 @@ while (!feof(pp) && CfReadLine(line,CF_BUFSIZE,pp))
    
    if (print)
       {
-      fprintf(fp,"%s\n",line);
+      // we must escape print format chars (%) from output
+
+      ReplaceStr(line,lineEscaped,sizeof(lineEscaped),"%","%%");
+
+      fprintf(fp,"%s\n",lineEscaped);
       count++;
       
       /* If we can't send mail, log to syslog */
       
       if (strlen(MAILTO) == 0)
          {
-         strncat(line,"\n",CF_BUFSIZE-1-strlen(line));
-         if ((strchr(line,'\n')) == NULL)
+	 strncat(lineEscaped,"\n",sizeof(lineEscaped)-1-strlen(lineEscaped));
+         if ((strchr(lineEscaped,'\n')) == NULL)
             {
-            line[CF_BUFSIZE-2] = '\n';
+	    lineEscaped[sizeof(lineEscaped)-2] = '\n';
             }
          
-         CfOut(cf_inform,"",line);
+         CfOut(cf_inform,"",lineEscaped);
          }
       
       line[0] = '\0';
+      lineEscaped[0] = '\0';
       }
    }
  
@@ -872,8 +832,6 @@ cf_pclose(pp);
 Debug("Closing fp\n");
 fclose(fp);
 
-DebugExecMemory("LocalExec 3");
-
 if (ONCE)
    {
    Cf3CloseLog();
@@ -892,7 +850,6 @@ else
    unlink(filename);
    }
 
-DebugExecMemory("LocalExec 4");
 return NULL; 
 }
 
@@ -904,7 +861,8 @@ int FileChecksum(char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],char typ
 
 { FILE *file;
   EVP_MD_CTX context;
-  int len, md_len;
+  int len;
+  unsigned int md_len;
   unsigned char buffer[1024];
   const EVP_MD *md = NULL;
 
@@ -932,7 +890,7 @@ else
    
    EVP_DigestInit(&context,md);
 
-   while (len = fread(buffer,1,1024,file))
+   while ((len = fread(buffer,1,1024,file)))
       {
       EVP_DigestUpdate(&context,buffer,len);
       }
@@ -950,8 +908,8 @@ return 0;
 int CompareResult(char *filename,char *prev_file)
 
 { int i;
-  char digest1[EVP_MAX_MD_SIZE+1];
-  char digest2[EVP_MAX_MD_SIZE+1];
+  unsigned char digest1[EVP_MAX_MD_SIZE+1];
+  unsigned char digest2[EVP_MAX_MD_SIZE+1];
   int  md_len1, md_len2;
   FILE *fp;
   int rtn = 0;
@@ -996,23 +954,11 @@ if (!ThreadLock(cft_count))
 
 unlink(prev_file);
 
-#ifdef MINGW
-
-if (!CopyFile(filename, prev_file, TRUE))
-  {
-  CfOut(cf_inform,"CopyFile","Could copy %s to %s",filename,prev_file);
-  rtn = 1;
-  }
-
-#else  /* NOT MINGW */
-
-if (symlink(filename, prev_file) == -1)
+ if(!LinkOrCopy(filename,prev_file,true))
    {
-   CfOut(cf_inform,"symlink","Could not link %s and %s",filename,prev_file);
-   rtn = 1;
+     CfOut(cf_inform,"","Could not symlink or copy %s to %s",filename,prev_file);
+     rtn = 1;
    }
-   
-#endif  /* NOT MINGW */
 
 ThreadUnlock(cft_count);
 return(rtn);
@@ -1023,7 +969,7 @@ return(rtn);
 void MailResult(char *file,char *to)
 
 { int sd, sent, count = 0, anomaly = false;
- char domain[256], prev_file[CF_BUFSIZE],vbuff[CF_BUFSIZE];
+  char prev_file[CF_BUFSIZE],vbuff[CF_BUFSIZE];
   struct hostent *hp;
   struct sockaddr_in raddr;
   struct servent *server;
@@ -1031,12 +977,6 @@ void MailResult(char *file,char *to)
   time_t now = time(NULL);
   FILE *fp;
 
-if ((strlen(VMAILSERVER) == 0) || (strlen(to) == 0))
-   {
-   /* Syslog should have done this */
-   return;
-   }
-
 CfOut(cf_verbose,"","Mail result...\n");
 
 if (cfstat(file,&statbuf) == -1)
@@ -1060,6 +1000,13 @@ if (CompareResult(file,prev_file) == 0)
    return;
    }
 
+if ((strlen(VMAILSERVER) == 0) || (strlen(to) == 0))
+   {
+   /* Syslog should have done this */
+   CfOut(cf_verbose, "", "Empty mail server or address - skipping");
+   return;
+   }
+
 if (MAXLINES == 0)
    {
    Debug("Not mailing: EmailMaxLines was zero\n");
@@ -1313,53 +1260,6 @@ while (recv(sd,&ch,1,0))
 return ((f == '2') || (f == '3')); /* return code 200 or 300 from smtp*/
 }
 
-/******************************************************************/
-
-void DebugExecMemory(char *ref)
-
-{ char buffer[CF_EXPANDSIZE],line[CF_BUFSIZE],*sp;
- static int size = 0, mem;
-
-if (DEBUG)
-   {
-   sleep(1);
-
-   GetExecOutput("/bin/ps waux | grep cf-execd",buffer,true);
-   
-   strcpy(line,"(empty)");
-   
-   for (sp = buffer; sp < buffer + strlen(buffer); sp += strlen(line)+1)
-      {
-      sscanf(sp,"%[^\n]\n",line);
-      
-      if (strstr(line,"grep") || strstr(line,"sh") || strstr(line,"gdb"))
-         {
-         continue;
-         }
-      
-      mem = 0;
-      sscanf(line,"%*s%*s%*s%*s%*s%d",&mem);
-      
-      if (mem > size)
-         {
-         printf("MEMORY DEBUG> resident increased from %d to %d (delta %d) at checkpoint(%s)\n",size,mem,mem-size,ref);
-         size = mem;
-         
-         struct Scope *ptr;
-         
-         for (ptr = VSCOPE; ptr != NULL; ptr=ptr->next)
-            {
-            printf(" -- active SCOPE %s\n",ptr->scope);
-            }
-         }
-      else
-         {
-         //printf("MEMORY DEBUG> stat at checkpoint(%s): %d\n",ref,mem);
-         }
-      }
-   }
-}
-
 /* EOF */
 
 
