@@ -92,7 +92,7 @@ since these cannot be mapped into "this" without some magic.
 
 void ExpandPromise(enum cfagenttype agent,char *scopeid,struct Promise *pp,void *fnptr)
 
-{ struct Rlist *rp, *listvars = NULL, *scalarvars = NULL;
+{ struct Rlist *listvars = NULL, *scalarvars = NULL;
   struct Constraint *cp;
   struct Promise *pcopy;
 
@@ -102,6 +102,8 @@ Debug("****************************************************\n\n");
 
 DeleteScope("match"); /* in case we expand something expired accidentially */
 
+THIS_BUNDLE = scopeid;
+
 pcopy = DeRefCopyPromise(scopeid,pp);
 
 ScanRval(scopeid,&scalarvars,&listvars,pcopy->promiser,CF_SCALAR,pp);
@@ -130,7 +132,6 @@ DeleteRlist(listvars);
 struct Rval ExpandDanglers(char *scopeid,struct Rval rval,struct Promise *pp)
 
 { struct Rval final;
-  struct Rlist *rp;
 
  /* If there is still work left to do, expand and replace alloc */
  
@@ -144,12 +145,14 @@ switch (rval.rtype)
           }
        else
           {
-          final = rval;
+          final.item = CopyRvalItem(rval.item,rval.rtype);
+          final.rtype = rval.rtype;
           }
        break;
 
    default:
-       final = rval;
+       final.item = CopyRvalItem(rval.item,rval.rtype);
+       final.rtype = rval.rtype;
        break;
    }
 
@@ -171,7 +174,6 @@ if (rval == NULL)
 switch(type)
    {
    case CF_SCALAR:
-
        ScanScalar(scopeid,scalarvars,listvars,(char *)rval,0,pp);
        break;
        
@@ -202,7 +204,7 @@ switch(type)
 
 void ScanScalar(char *scopeid,struct Rlist **scal,struct Rlist **its,char *string,int level,struct Promise *pp)
 
-{ struct Rlist *rp;
+{
   char *sp,rtype;
   void *rval;
   char v[CF_BUFSIZE],var[CF_EXPANDSIZE],exp[CF_EXPANDSIZE],temp[CF_BUFSIZE];
@@ -226,7 +228,7 @@ for (sp = string; (*sp != '\0') ; sp++)
          {
          char absscope[CF_MAXVARSIZE];
 
-         if (strstr(v,"."))
+         if (IsQualifiedVariable(v))
             {
             strncpy(temp,var,CF_BUFSIZE-1);  
             absscope[0] = '\0';
@@ -238,14 +240,13 @@ for (sp = string; (*sp != '\0') ; sp++)
             }
 
          ExpandPrivateScalar(absscope,v,var); 
-
+         
          RegisterBundleDependence(absscope,pp);
 
          if (GetVariable(absscope,var,&rval,&rtype) != cf_notype)
             {
             if (rtype == CF_LIST)
                {
-               Debug("Found list %s\n",var);
                ExpandScalar(var,exp); 
 
                /* embedded iterators should be incremented fastest,
@@ -294,6 +295,20 @@ return ExpandPrivateScalar(CONTEXTID,string,buffer);
 
 /*********************************************************************/
 
+int ExpandThis(enum cfreport level,char *string,char buffer[CF_EXPANDSIZE])
+
+{
+if (level == cf_error || ((strstr(string,"$(this.") == 0) && (strstr(string,"${this.") == 0)))
+   {
+   strncpy(buffer,string,CF_EXPANDSIZE-1);
+   return true;
+   }
+
+return ExpandPrivateScalar(CONTEXTID,string,buffer); 
+}
+
+/*********************************************************************/
+
 struct Rlist *ExpandList(char *scopeid,struct Rlist *list,int expandnaked)
 
 { struct Rlist *rp, *start = NULL;
@@ -326,6 +341,7 @@ for (rp = (struct Rlist *)list; rp != NULL; rp=rp->next)
       }
 
    AppendRlist(&start,returnval.item,returnval.rtype);
+   DeleteRvalItem(returnval.item,returnval.rtype);
    }
 
 return start;
@@ -336,9 +352,8 @@ return start;
 struct Rval ExpandPrivateRval(char *scopeid,void *rval,char type)
 
 { char buffer[CF_EXPANDSIZE];
- struct Rlist *rp, *start = NULL;
  struct FnCall *fp,*fpe;
- struct Rval returnval,extra;
+ struct Rval returnval;
      
 Debug("ExpandPrivateRval(scope=%s,type=%c)\n",scopeid,type);
 
@@ -380,9 +395,8 @@ return returnval;
 struct Rval ExpandBundleReference(char *scopeid,void *rval,char type)
 
 { char buffer[CF_EXPANDSIZE];
- struct Rlist *rp, *start = NULL;
  struct FnCall *fp,*fpe;
- struct Rval returnval,extra;
+ struct Rval returnval;
      
 Debug("ExpandBundleReference(scope=%s,type=%c)\n",scopeid,type);
 
@@ -416,14 +430,28 @@ return returnval;
 
 /*********************************************************************/
 
+static int ExpandOverflow(char *str1,char *str2)
+
+{ int len = strlen(str2);
+
+if ((strlen(str1)+len) > (CF_EXPANDSIZE - CF_BUFFERMARGIN))
+   {
+   CfOut(cf_error,"","Expansion overflow constructing string. Increase CF_EXPANDSIZE macro. Tried to add %s to %s\n",str2,str1);
+   return true;
+   }
+
+return false;
+}
+
+/*********************************************************************/
+
 int ExpandPrivateScalar(char *scopeid,char *string,char buffer[CF_EXPANDSIZE]) 
 
 { char *sp,rtype;
   void *rval;
   int varstring = false;
   char currentitem[CF_EXPANDSIZE],temp[CF_BUFSIZE],name[CF_MAXVARSIZE];
-  int len,increment, returnval = true;
-  time_t tloc;
+  int increment, returnval = true;
   
 memset(buffer,0,CF_EXPANDSIZE);
 
@@ -576,20 +604,44 @@ void ExpandPromiseAndDo(enum cfagenttype agent,char *scopeid,struct Promise *pp,
 
 { struct Rlist *lol = NULL; 
   struct Promise *pexp;
-  struct Scope *ptr;
-  int i = 1;
-  char *handle = GetConstraint("handle",pp,CF_SCALAR);
+  const int cf_null_cutoff = 5;
+  char *handle = GetConstraint("handle",pp,CF_SCALAR),v[CF_MAXVARSIZE];
+  int cutoff = 0;
 
 lol = NewIterationContext(scopeid,listvars);
 
+if (lol && EndOfIteration(lol))
+   {
+   DeleteIterationContext(lol);
+   return;
+   }
+
+while (NullIterators(lol))
+   {
+   IncrementIterationContext(lol,1);
+
+   // In case a list is completely blank
+   if (cutoff++ > cf_null_cutoff)
+      {
+      break;
+      }
+   }
+
+if (lol && EndOfIteration(lol))
+   {
+   DeleteIterationContext(lol);
+   return;
+   }
+
 do
    {
+   char number[CF_SMALLBUF];
    /* Set scope "this" first to ensure list expansion ! */
    SetScope("this");  
-   DeRefListsInHashtable("this",listvars,lol);   
+   DeRefListsInHashtable("this",listvars,lol);
+
+   /* Allow $(this.handle) etc variables */
 
-   /* Allow $(this.handle) */
-   
    if (handle)
       {
       NewScalar("this","handle",handle,cf_str);
@@ -599,15 +651,29 @@ do
       NewScalar("this","handle",PromiseID(pp),cf_str);
       }
 
-   pexp = ExpandDeRefPromise("this",pp);
+   if (pp->audit && pp->audit->filename)
+      {
+      NewScalar("this","promise_filename",pp->audit->filename,cf_str);
+      snprintf(number,CF_SMALLBUF,"%d",pp->lineno);
+      NewScalar("this","promise_linenumber",number,cf_str);
+      }
+
+   snprintf(v,CF_MAXVARSIZE,"%d",(int)getuid());
+   NewScalar("this","promiser_uid",v,cf_int);
+   snprintf(v,CF_MAXVARSIZE,"%d",(int)getgid());
+   NewScalar("this","promiser_gid",v,cf_int);
    
+   /* End special variables */
+
+   pexp = ExpandDeRefPromise("this",pp);
+
    switch (agent)
       {
       case cf_common:
           ShowPromise(pexp,6);
           ReCheckAllConstraints(pexp);
           break;
-          
+
       default:
 
           if (fnptr != NULL)
@@ -616,19 +682,14 @@ do
              }
           break;
       }
-      
+
    if (strcmp(pp->agentsubtype,"vars") == 0)
       {
       ConvergeVarHashPromise(pp->bundle,pexp,true);
       }
-
+   
    DeletePromise(pexp);
 
-   if (lol && EndOfIteration(lol))
-      {
-      return;
-      }
-
    /* End thread monitor */
    }
 while (IncrementIterationContext(lol,1));
@@ -646,7 +707,7 @@ struct Rval EvaluateFinalRval(char *scopeid,void *rval,char rtype,int forcelist,
   struct FnCall *fp;
 
 Debug("EvaluateFinalRval -- type %c\n",rtype);
-  
+
 if ((rtype == CF_SCALAR) && IsNakedVar(rval,'@')) /* Treat lists specially here */
    {
    GetNaked(naked,rval);
@@ -873,7 +934,7 @@ if (count != 0)
    return false;
    }
 
-Debug1("IsNakedVar(%s,%c)!!\n",str,vtype);
+Debug("IsNakedVar(%s,%c)!!\n",str,vtype);
 return true;
 }
 
@@ -902,12 +963,12 @@ strncpy(s2,s1+2,strlen(s1)-3);
 void ConvergeVarHashPromise(char *scope,struct Promise *pp,int allow_redefine)
 
 { struct Constraint *cp,*cp_save = NULL;
-  struct Attributes a;
-  char *lval,rtype;
+  struct Attributes a = {{0}};
+  char rtype,type = 'x',*sp = NULL;
   void *rval = NULL,*retval;
-  int i = 0,ok_redefine = false,drop_undefined = false;;
+  int i = 0,ok_redefine = false,drop_undefined = false;
   struct Rval returnval; /* Must expand naked functions here for consistency */
-  struct Rlist *rp,*last = NULL;
+  struct Rlist *rp;
   
 if (pp->done)
    {
@@ -919,6 +980,11 @@ if (IsExcluded(pp->classes))
    return;
    }
 
+if (VarClassExcluded(pp,&sp))
+   {
+   return;
+   }
+
 for (cp = pp->conlist; cp != NULL; cp=cp->next)
    {
    if (strcmp(cp->lval,"comment") == 0)
@@ -983,29 +1049,49 @@ if (i > 2)
    return;
    }
 
-// More consideration needs to be given to using these
+//More consideration needs to be given to using these
 //a.transaction = GetTransactionConstraints(pp);
-//a.classes = GetClassDefinitionConstraints(pp);
+a.classes = GetClassDefinitionConstraints(pp);
 
 if (rval != NULL)
    {
    struct FnCall *fp = (struct FnCall *)rval;
-   
+
    if (cp->type == CF_FNCALL)
       {
       returnval = EvaluateFunctionCall(fp,pp);
-      cp->rval = rval = returnval.item;
-      cp->type = returnval.rtype;
-      DeleteFnCall(fp);
+      rval = returnval.item;
+      type = returnval.rtype;
       
       if (FNCALL_STATUS.status == FNCALL_FAILURE)
          {
          /* We do not assign variables to failed fn calls */
+         DeleteRvalItem(rval,type);
          return;
          }
       }
+   else
+      {
+      char conv[CF_MAXVARSIZE];
 
-   if (Epimenides(pp->promiser,rval,cp->type,0))
+      if (strcmp(cp->lval,"int") == 0)
+         {
+         snprintf(conv,CF_MAXVARSIZE,"%ld",Str2Int(cp->rval));
+         rval = CopyRvalItem(conv,cp->type);
+         }
+      else if (strcmp(cp->lval,"real") == 0)
+         {
+         snprintf(conv,CF_MAXVARSIZE,"%lf",Str2Double(cp->rval));
+         rval = CopyRvalItem(conv,cp->type);
+         }
+      else
+         {
+         rval = CopyRvalItem(cp->rval,cp->type);
+         }
+      type = cp->type;
+      }
+
+   if (Epimenides(pp->promiser,rval,type,0))
       {
       CfOut(cf_error,"","Variable \"%s\" contains itself indirectly - an unkeepable promise",pp->promiser);
       exit(1);
@@ -1013,10 +1099,13 @@ if (rval != NULL)
    else
       {
       /* See if the variable needs recursively expanding again */
-      returnval = EvaluateFinalRval(scope,rval,cp->type,true,pp);
-      DeleteRvalItem(cp->rval,cp->type);
-      cp->rval = rval = returnval.item;
-      cp->type = returnval.rtype;
+      
+      returnval = EvaluateFinalRval(scope,rval,type,true,pp);
+      DeleteRvalItem(rval,type);
+
+      // freed before function exit
+      rval = returnval.item;
+      type = returnval.rtype;
       }
 
    if (GetVariable(scope,pp->promiser,(void *)&retval,&rtype) != cf_notype)
@@ -1025,16 +1114,31 @@ if (rval != NULL)
          {
          DeleteVariable(scope,pp->promiser);
          }
-      else if ((THIS_AGENT_TYPE == cf_common) && (CompareRval(retval,rtype,rval,cp->type) == false))
+      else if ((THIS_AGENT_TYPE == cf_common) && (CompareRval(retval,rtype,rval,type) == false))
          {
-         CfOut(cf_error,""," !! Redefinition of a constant variable \"%s\"",pp->promiser);
-         PromiseRef(cf_error,pp);
+         switch (type)
+            {
+            case CF_SCALAR:
+                CfOut(cf_error,""," !! Redefinition of a constant scalar \"%s\" (was %s now %s)",pp->promiser,retval,rval);
+                PromiseRef(cf_error,pp);
+                break;
+            case CF_LIST:
+                CfOut(cf_error,""," !! Redefinition of a constant list \"%s\"",pp->promiser,retval,rval);
+                printf("%s>  -- Was ",VPREFIX);
+                ShowRlist(stdout,retval);      
+                printf("%s> now ",VPREFIX);
+                ShowRlist(stdout,rval);      
+                printf("%s>\n",VPREFIX);
+                PromiseRef(cf_error,pp);
+		break;
+            }
          }
       }
-
+   
    if (IsCf3VarString(pp->promiser))
       {
       // Unexpanded variables, we don't do anything with
+      DeleteRvalItem(rval,type);
       return;
       }
    
@@ -1042,43 +1146,41 @@ if (rval != NULL)
       {
       CfOut(cf_error,""," !! Variable identifier contains illegal characters");
       PromiseRef(cf_error,pp);
+      DeleteRvalItem(rval,type);
       return;
       }
 
-   if (drop_undefined && cp->type == CF_LIST)
+   if (drop_undefined && type == CF_LIST)
       {
       for (rp = rval; rp != NULL; rp=rp->next)
          {
          if (IsNakedVar(rp->item,'@'))
             {
-            if (rp == rval)
-               {
-               DeleteRvalItem(rp->item,rp->type);
-               rval = rp->next;
-               }
-            else if (last)
-               {
-               last->next = rp->next;
-               DeleteRvalItem(rp->item,rp->type);
-               }
+            free(rp->item);
+            rp->item = strdup(CF_NULL_VALUE);
             }
-
-         last = rp;
          }
       }
 
-   if (!AddVariableHash(scope,pp->promiser,rval,cp->type,Typename2Datatype(cp->lval),cp->audit->filename,cp->lineno))
+   if (!AddVariableHash(scope,pp->promiser,rval,type,Typename2Datatype(cp->lval),cp->audit->filename,cp->lineno))
       {
       CfOut(cf_verbose,"","Unable to converge %s.%s value (possibly empty or infinite regression)\n",scope,pp->promiser);
       PromiseRef(cf_verbose,pp);
+      cfPS(cf_noreport,CF_FAIL,"",pp,a," !! Couldn't add variable %s",pp->promiser);
+      }
+   else
+      {
+      cfPS(cf_noreport,CF_CHG,"",pp,a," -> Added variable %s",pp->promiser);
       }
    }
 else
    {
-   CfOut(cf_error,"","Variable %s has no promised value\n",pp->promiser);
-   CfOut(cf_error,"","Rule from %s at/before line %d\n",cp->audit->filename,cp->lineno);
+   CfOut(cf_error,""," !! Variable %s has no promised value\n",pp->promiser);
+   CfOut(cf_error,""," !! Rule from %s at/before line %d\n",cp->audit->filename,cp->lineno);
+   cfPS(cf_noreport,CF_FAIL,"",pp,a," !! Couldn't add variable %s",pp->promiser);
    }     
 
+ DeleteRvalItem(rval,type);
 }
 
 /*********************************************************************/
@@ -1088,6 +1190,7 @@ void ConvergePromiseValues(struct Promise *pp)
 { struct Constraint *cp;
   struct Rlist *rp;
   char expandbuf[CF_EXPANDSIZE];
+  struct FnCall *fp;
 
 switch (pp->petype)
    {
@@ -1153,6 +1256,21 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
                 }          
              }
           break;
+
+      case CF_FNCALL:
+
+       fp = (struct FnCall *)cp->rval;
+       for (rp = fp->args; rp != NULL; rp=rp->next)
+          {
+          if (rp->type == CF_SCALAR && IsCf3VarString(rp->item))
+             {
+             ExpandPrivateScalar(CONTEXTID,(char *)rp->item,expandbuf);
+             free(rp->item);
+             rp->item = strdup(expandbuf);
+             }
+          }
+
+          break;
       }
    }
 }
