@@ -69,14 +69,37 @@ int ScheduleEditLineOperations(char *filename,struct Bundle *bp,struct Attribute
 { enum editlinetypesequence type;
   struct SubType *sp;
   struct Promise *pp;
-  int pass;
+  char lockname[CF_BUFSIZE];
   char *bp_stack = THIS_BUNDLE;
-      
+  struct CfLock thislock;
+  int pass;
+
+snprintf(lockname,CF_BUFSIZE-1,"masterfilelock-%s",filename);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,parentp,true);
+
+if (thislock.lock == NULL)
+   {
+   return false;
+   }
+  
 NewScope("edit");
 NewScalar("edit","filename",filename,cf_str);
-         
+
 /* Reset the done state for every call here, since bundle is reusable */
 
+for (type = 0; EDITLINETYPESEQUENCE[type] != NULL; type++)
+   {
+   if ((sp = GetSubTypeForBundle(EDITLINETYPESEQUENCE[type],bp)) == NULL)
+      {
+      continue;      
+      }
+   
+   for (pp = sp->promiselist; pp != NULL; pp=pp->next)
+      {
+      pp->donep = false;
+      }
+   }
+
 for (pass = 1; pass < CF_DONEPASSES; pass++)
    {
    for (type = 0; EDITLINETYPESEQUENCE[type] != NULL; type++)
@@ -97,37 +120,24 @@ for (pass = 1; pass < CF_DONEPASSES; pass++)
          pp->edcontext = parentp->edcontext;
          pp->this_server = filename;
          pp->donep = &(pp->done);
-         
+
          ExpandPromise(cf_agent,bp->name,pp,KeepEditLinePromise);
          
          if (Abort())
             {
             THIS_BUNDLE = bp_stack;
             DeleteScope("edit");
+            YieldCurrentLock(thislock);
             return false;
             }         
          }
       }
    }
 
-/* Reset the promises after 3 passes since edit bundles are reusable */
-
-for (type = 0; EDITLINETYPESEQUENCE[type] != NULL; type++)
-   {
-   if ((sp = GetSubTypeForBundle(EDITLINETYPESEQUENCE[type],bp)) == NULL)
-      {
-      continue;      
-      }
-   
-   for (pp = sp->promiselist; pp != NULL; pp=pp->next)
-      {
-      pp->donep = false;
-      }
-   }
-
 DeleteScope("edit");
-
+SetScope(parentp->bundle);
 THIS_BUNDLE = bp_stack;
+YieldCurrentLock(thislock);
 return true;
 }
 
@@ -138,6 +148,7 @@ return true;
 void EditClassBanner(enum editlinetypesequence type)
 
 { struct Item *ip;
+  int i;
  
 if (type != elp_delete)   /* Just parsed all local classes */
    {
@@ -146,9 +157,12 @@ if (type != elp_delete)   /* Just parsed all local classes */
 
 CfOut(cf_verbose,"","     ??  Private class context\n");
 
-for (ip = VADDCLASSES; ip != NULL; ip=ip->next)
+for (i = 0; i < CF_ALPHABETSIZE; i++)
    {
-   CfOut(cf_verbose,"","     ??       %s\n",ip->name);
+   for (ip = VADDCLASSES.list[i]; ip != NULL; ip=ip->next)
+      {
+      CfOut(cf_verbose,"","     ??       %s\n",ip->name);
+      }
    }
 
 CfOut(cf_verbose,"","\n");
@@ -171,7 +185,7 @@ if (!IsDefinedClass(pp->classes))
 
 if (pp->done)
    {
-   return;
+//   return;
    }
 
 if (VarClassExcluded(pp,&sp))
@@ -228,24 +242,17 @@ if (strcmp("reports",pp->agentsubtype) == 0)
 
 void VerifyLineDeletions(struct Promise *pp)
 
-{ struct Item **start = &(pp->edcontext->file_start), *match, *prev;
-  struct Attributes a;
+{ struct Item **start = &(pp->edcontext->file_start);
+  struct Attributes a = {{0}};
   struct Item *begin_ptr,*end_ptr;
-  char lockname[CF_BUFSIZE];
   struct CfLock thislock;
+  char lockname[CF_BUFSIZE];
 
 /* *(pp->donep) = true;	*/
 	 
 a = GetDeletionAttributes(pp);
-
-snprintf(lockname,CF_BUFSIZE-1,"filedeletion-%s-%s",pp->promiser,pp->this_server);
-thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp);
-
-if (thislock.lock == NULL)
-   {
-   return;
-   }
-
+a.transaction.ifelapsed = CF_EDIT_IFELAPSED;
+    
 /* Are we working in a restricted region? */
 
 if (!a.haveregion)
@@ -255,8 +262,22 @@ if (!a.haveregion)
    }
 else if (!SelectRegion(*start,&begin_ptr,&end_ptr,a,pp))
    {
-   cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised line deletion (%s) could not select an edit region in %s",pp->promiser,pp->this_server);
-   YieldCurrentLock(thislock);
+   if (a.region.include_end || a.region.include_start)
+      {
+      cfPS(cf_verbose,CF_INTERPT,"",pp,a," !! The promised line deletion (%s) could not select an edit region in %s (this is a good thing, as policy suggests deleting the markers)",pp->promiser,pp->this_server);
+      }
+   else
+      {
+      cfPS(cf_inform,CF_INTERPT,"",pp,a," !! The promised line deletion (%s) could not select an edit region in %s (but the delimiters were expected in the file)",pp->promiser,pp->this_server);
+      }
+   return;
+   }
+
+snprintf(lockname,CF_BUFSIZE-1,"deleteline-%s-%s",pp->promiser,pp->this_server);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp,true);
+
+if (thislock.lock == NULL)
+   {
    return;
    }
 
@@ -272,29 +293,21 @@ YieldCurrentLock(thislock);
 
 void VerifyColumnEdits(struct Promise *pp)
 
-{ struct Item **start = &(pp->edcontext->file_start), *match, *prev;
-  struct Attributes a;
+{ struct Item **start = &(pp->edcontext->file_start);
+  struct Attributes a = {{0}};
   struct Item *begin_ptr,*end_ptr;
-  char lockname[CF_BUFSIZE];
   struct CfLock thislock;
-
+  char lockname[CF_BUFSIZE];
+  
 /* *(pp->donep) = true; */
 
 a = GetColumnAttributes(pp);
-
-snprintf(lockname,CF_BUFSIZE-1,"filecolumnedits-%s-%s",pp->promiser,pp->this_server);
-thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp);
-
-if (thislock.lock == NULL)
-   {
-   return;
-   }
+a.transaction.ifelapsed = CF_EDIT_IFELAPSED;
 
 if (a.column.column_separator == NULL)
    {
    cfPS(cf_error,CF_WARN,"",pp,a,"No field_separator in promise to edit by column for %s",pp->promiser);
    PromiseRef(cf_error,pp);
-   YieldCurrentLock(thislock);
    return;
    }
 
@@ -302,7 +315,6 @@ if (a.column.select_column <= 0)
    {
    cfPS(cf_error,CF_WARN,"",pp,a,"No select_field in promise to edit %s",pp->promiser);
    PromiseRef(cf_error,pp);
-   YieldCurrentLock(thislock);
    return;   
    }
 
@@ -310,7 +322,6 @@ if (!a.column.column_value)
    {
    cfPS(cf_error,CF_WARN,"",pp,a,"No field_value is promised to column_edit %s",pp->promiser);
    PromiseRef(cf_error,pp);
-   YieldCurrentLock(thislock);
    return;   
    }
 
@@ -324,12 +335,19 @@ if (!a.haveregion)
 else if (!SelectRegion(*start,&begin_ptr,&end_ptr,a,pp))
    {
    cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised column edit (%s) could not select an edit region in %s",pp->promiser,pp->this_server);
-   YieldCurrentLock(thislock);
    return;
    }
 
 /* locate and split line */
 
+snprintf(lockname,CF_BUFSIZE-1,"column-%s-%s",pp->promiser,pp->this_server);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp,true);
+
+if (thislock.lock == NULL)
+   {
+   return;
+   }
+
 if (EditColumns(begin_ptr,end_ptr,a,pp))
    {
    (pp->edcontext->num_edits)++;
@@ -342,12 +360,12 @@ YieldCurrentLock(thislock);
 
 void VerifyPatterns(struct Promise *pp)
 
-{ struct Item **start = &(pp->edcontext->file_start), *match, *prev;
-  struct Attributes a;
+{ struct Item **start = &(pp->edcontext->file_start);
+  struct Attributes a = {{0}};
   struct Item *begin_ptr,*end_ptr;
-  char lockname[CF_BUFSIZE];
   struct CfLock thislock;
-
+  char lockname[CF_BUFSIZE];
+  
 /* *(pp->donep) = true; */
 
 CfOut(cf_verbose,""," -> Looking at pattern %s\n",pp->promiser);
@@ -355,19 +373,11 @@ CfOut(cf_verbose,""," -> Looking at pattern %s\n",pp->promiser);
 /* Are we working in a restricted region? */
 
 a = GetReplaceAttributes(pp);
-
-snprintf(lockname,CF_BUFSIZE-1,"filepatterns-%s-%s",pp->promiser,pp->this_server);
-thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp);
-
-if (thislock.lock == NULL)
-   {
-   return;
-   }
+a.transaction.ifelapsed = CF_EDIT_IFELAPSED;
 
 if (!a.replace.replace_value)
    {
    cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised pattern replace (%s) had no replacement string",pp->promiser);
-   YieldCurrentLock(thislock);
    return;
    }
 
@@ -379,7 +389,14 @@ if (!a.haveregion)
 else if (!SelectRegion(*start,&begin_ptr,&end_ptr,a,pp))
    {
    cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised pattern replace (%s) could not select an edit region in %s",pp->promiser,pp->this_server);
-   YieldCurrentLock(thislock);
+   return;
+   }
+
+snprintf(lockname,CF_BUFSIZE-1,"replace-%s-%s",pp->promiser,pp->this_server);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp,true);
+
+if (thislock.lock == NULL)
+   {
    return;
    }
 
@@ -390,6 +407,8 @@ if (ReplacePatterns(begin_ptr,end_ptr,a,pp))
    (pp->edcontext->num_edits)++;
    }
 
+DeleteScope("match"); // because this might pollute the parent promise in next iteration
+
 YieldCurrentLock(thislock);
 }
 
@@ -399,13 +418,14 @@ void VerifyLineInsertions(struct Promise *pp)
 
 { struct Item **start = &(pp->edcontext->file_start), *match, *prev;
   struct Item *begin_ptr,*end_ptr;
-  struct Attributes a;
-  char lockname[CF_BUFSIZE];
+  struct Attributes a = {{0}};
   struct CfLock thislock;
-
+  char lockname[CF_BUFSIZE];
+  
 /* *(pp->donep) = true; */
-
+  
 a = GetInsertionAttributes(pp);
+a.transaction.ifelapsed = CF_EDIT_IFELAPSED;
 
 if (!SanityCheckInsertions(a))
    {
@@ -413,14 +433,6 @@ if (!SanityCheckInsertions(a))
    return;
    }
 
-snprintf(lockname,CF_BUFSIZE-1,"filepatterns-%s-%s",pp->promiser,pp->this_server);
-thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp);
-
-if (thislock.lock == NULL)
-   {
-   return;
-   }
-
 /* Are we working in a restricted region? */
 
 if (!a.haveregion)
@@ -431,7 +443,14 @@ if (!a.haveregion)
 else if (!SelectRegion(*start,&begin_ptr,&end_ptr,a,pp))
    {
    cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised line insertion (%s) could not select an edit region in %s",pp->promiser,pp->this_server);
-   YieldCurrentLock(thislock);
+   return;
+   }
+
+snprintf(lockname,CF_BUFSIZE-1,"insertline-%s-%s",pp->promiser,pp->this_server);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp,true);
+
+if (thislock.lock == NULL)
+   {
    return;
    }
 
@@ -446,7 +465,7 @@ if (a.location.line_matching == NULL)
    }
 else
    {
-   if (!SelectItemMatching(a.location.line_matching,begin_ptr,end_ptr,&match,&prev,a.location.first_last))
+   if (!SelectItemMatching(*start,a.location.line_matching,begin_ptr,end_ptr,&match,&prev,a.location.first_last))
       {
       cfPS(cf_error,CF_INTERPT,"",pp,a," !! The promised line insertion (%s) could not select a locator matching regex \"%s\" in %s",pp->promiser,a.location.line_matching,pp->this_server);
       YieldCurrentLock(thislock);
@@ -540,7 +559,7 @@ int InsertMissingLinesToRegion(struct Item **start,struct Item *begin_ptr,struct
 
 /* find prev for region */
 
- if (IsItemInRegion(pp->promiser,begin_ptr,end_ptr,a,pp))
+if (IsItemInRegion(pp->promiser,begin_ptr,end_ptr,a,pp))
    {
    cfPS(cf_verbose,CF_NOP,"",pp,a," -> Promised line \"%s\" exists within selected region of %s (promise kept)",pp->promiser,pp->this_server);
    return false;
@@ -593,7 +612,7 @@ int InsertMissingLinesAtLocation(struct Item **start,struct Item *begin_ptr,stru
   char buf[CF_BUFSIZE],exp[CF_EXPANDSIZE];
   struct Item *loc = NULL;
   int retval = false;
-  
+
 if (a.sourcetype && strcmp(a.sourcetype,"file") == 0)
    {
    if ((fin = fopen(pp->promiser,"r")) == NULL)
@@ -608,7 +627,7 @@ if (a.sourcetype && strcmp(a.sourcetype,"file") == 0)
       {
       buf[0] = '\0';
       fgets(buf,CF_BUFSIZE,fin);
-      Chop(buf);
+      StripTrailingNewline(buf);
 
       if (feof(fin) && strlen(buf) == 0)
          {
@@ -628,14 +647,15 @@ if (a.sourcetype && strcmp(a.sourcetype,"file") == 0)
          {
          continue;
          }
-      
+
       if (IsItemInRegion(exp,begin_ptr,end_ptr,a,pp))
          {
          cfPS(cf_verbose,CF_NOP,"",pp,a," -> Promised file line \"%s\" exists within file %s (promise kept)",exp,pp->this_server);
          continue;
          }
 
-      retval |= InsertMissingLineAtLocation(exp,start,loc,prev,a,pp);
+      retval |= InsertCompoundLineAtLocation(exp,start,loc,prev,a,pp);
+      //retval |= InsertMissingLineAtLocation(exp,start,loc,prev,a,pp);
 
       if (prev && prev != CF_UNDEFINED_ITEM)
          {
@@ -660,7 +680,7 @@ else
       {
       char *sp;
       loc = location;
-      
+
       for (sp = pp->promiser; sp <= pp->promiser+strlen(pp->promiser); sp++)
          {
          memset(buf,0,CF_BUFSIZE);
@@ -671,7 +691,7 @@ else
             {
             continue;
             }
-         
+
          if (IsItemInRegion(buf,begin_ptr,end_ptr,a,pp))
             {
             cfPS(cf_verbose,CF_NOP,"",pp,a," -> Promised file line \"%s\" exists within file %s (promise kept)",buf,pp->this_server);
@@ -680,7 +700,7 @@ else
 
          if (!multiline)
             {
-            retval |= InsertMissingLineAtLocation(buf,start,loc,prev,a,pp);
+            retval |= InsertCompoundLineAtLocation(buf,start,loc,prev,a,pp);
          
             if (prev && prev != CF_UNDEFINED_ITEM)
                {
@@ -697,17 +717,34 @@ else
             need_insert = true;            
             }
          }
-
+      
       if (need_insert)
-         {
-         return InsertMissingLineAtLocation(pp->promiser,start,location,prev,a,pp);
+         {         
+         for (sp = pp->promiser; sp <= pp->promiser+strlen(pp->promiser); sp++)
+            {
+            memset(buf,0,CF_BUFSIZE);
+            sscanf(sp,"%[^\n]",buf);
+            sp += strlen(buf);
+            
+            retval |= InsertCompoundLineAtLocation(buf,start,loc,prev,a,pp);
+            
+            if (prev && prev != CF_UNDEFINED_ITEM)
+               {
+               prev = prev->next;
+               }
+            
+            if (loc)
+               {
+               loc = loc->next;
+               }
+            }
          }
-
+      
       return retval;
       }
    else
       {
-      return InsertMissingLineAtLocation(pp->promiser,start,location,prev,a,pp);
+      return InsertCompoundLineAtLocation(pp->promiser,start,location,prev,a,pp);
       }
    }
 }
@@ -716,21 +753,31 @@ else
     
 int DeletePromisedLinesMatching(struct Item **start,struct Item *begin,struct Item *end,struct Attributes a,struct Promise *pp)
 
-{ struct Item *ip,*np,*lp;
- int in_region = false, retval = false, match, noedits = true;
-
+{ struct Item *ip,*np = NULL,*lp;
+  int in_region = false, retval = false, match, noedits = true;
+  int i,lines = 0;
+  char *sp,buf[CF_BUFSIZE];
+  
 if (start == NULL)
    {
    return false;
    }
-  
+
 for (ip = *start; ip != NULL; ip = np)
    {
+   lines = 0;
+   
    if (ip == begin)
       {
       in_region = true;
       }
 
+   if (a.region.include_end == false && ip == end)
+      {
+      in_region = false;
+      break;
+      }
+   
    if (!in_region)
       {
       np = ip->next;
@@ -739,21 +786,31 @@ for (ip = *start; ip != NULL; ip = np)
 
    if (a.not_matching)
       {
-      match = !FullTextMatch(pp->promiser,ip->name);
+      match = !MatchRegion(pp->promiser,ip,begin,end);
       }
    else
       {
-      match = FullTextMatch(pp->promiser,ip->name);
+      match = MatchRegion(pp->promiser,ip,begin,end);
       }
-
-   if (!SelectLine(ip->name,a,pp))
+   
+   if (!SelectLine(ip->name,a,pp)) // Start search from location
       {
       np = ip->next;
       continue;
       }
-         
+
    if (in_region && match)
       {
+      for (sp = pp->promiser; sp <= pp->promiser+strlen(pp->promiser); sp++)
+         {
+         memset(buf,0,CF_BUFSIZE);
+         sscanf(sp,"%[^\n]",buf);
+         sp += strlen(buf);
+         lines++;
+         }
+      
+      CfOut(cf_verbose,""," -> Delete chunk of %d lines\n",lines,ip->name);
+      
       if (a.transaction.action == cfa_warn)
          {
          cfPS(cf_error,CF_WARN,"",pp,a," -> Need to delete line \"%s\" from %s - but only a warning was promised",ip->name,pp->this_server);
@@ -762,40 +819,50 @@ for (ip = *start; ip != NULL; ip = np)
          }
       else
          {
-         cfPS(cf_verbose,CF_CHG,"",pp,a," -> Deleting the promised line \"%s\" from %s",ip->name,pp->this_server);
-         retval = true;
-         noedits = false;
-         
-         if (ip->name != NULL)
-            {
-            free(ip->name);
-            }
-         
-         np = ip->next;
-         free((char *)ip);
-         
-         lp = ip;
-         
-         if (ip == *start)
-            {
-            *start = np;
-            }
-         else
-            {
-            for (lp = *start; lp->next != ip; lp=lp->next)
-               {
-               }
+         for (i = 0; i < lines; i++)
+            {                     
+            cfPS(cf_verbose,CF_CHG,"",pp,a," -> Deleting the promised line \"%s\" from %s",ip->name,pp->this_server);
+            retval = true;
+            noedits = false;
             
-            lp->next = np;
+            if (ip->name != NULL)
+               {
+               free(ip->name);
+               }
+
+            np = ip->next;
+            free((char *)ip);
+            
+            lp = ip;
+            
+            if (ip == *start)
+               {
+               *start = np;
+               }
+            else
+               {
+               for (lp = *start; lp->next != ip; lp=lp->next)
+                  {
+                  }
+               
+               lp->next = np;
+               }
+
+            if (ip == end)
+               {
+               in_region = false;
+               break;
+               }
+
+            (pp->edcontext->num_edits)++;
+
+            ip = np;
             }
 
-         if (ip == end)
+         if (!in_region)
             {
-            in_region = false;
             break;
-            }
-         
-         (pp->edcontext->num_edits)++;
+            }         
          }
       }
    else
@@ -822,11 +889,11 @@ return retval;
 
 int ReplacePatterns(struct Item *file_start,struct Item *file_end,struct Attributes a,struct Promise *pp)
 
-{ char *sp, *start = NULL,*end,replace[CF_EXPANDSIZE],line_buff[CF_EXPANDSIZE];
+{ char replace[CF_EXPANDSIZE],line_buff[CF_EXPANDSIZE];
+  char before[CF_BUFSIZE],after[CF_BUFSIZE];
   int match_len,start_off,end_off,once_only = false,retval = false;
-  struct CfRegEx rex;
   struct Item *ip;
-  int notfound = true;
+  int notfound = true, cutoff = 1, replaced=false;
 
 if (a.replace.occurrences && (strcmp(a.replace.occurrences,"first") == 0))
    {
@@ -841,87 +908,87 @@ for (ip = file_start; ip != file_end; ip=ip->next)
       continue;
       }
 
-   if (!BlockTextMatch(pp->promiser,ip->name,&start_off,&end_off))
-      {
-      continue;
-      }
-   else
+   cutoff = 1;
+   strncpy(line_buff,ip->name,CF_BUFSIZE);
+   replaced = false;
+   match_len = 0;
+
+   while (BlockTextMatch(pp->promiser,line_buff,&start_off,&end_off))
       {
+      if (match_len == strlen(line_buff))
+         {
+         CfOut(cf_verbose,""," -> Improper convergent expression matches defacto convergence, so accepting");
+         break;
+         }      
+
+      if (cutoff++ > CF_MAX_REPLACE)
+         {
+         CfOut(cf_verbose,""," !! Too many replacements on this line");
+         break;
+         }
+      
       match_len = end_off - start_off;
       ExpandScalar(a.replace.replace_value,replace);
-      }
 
-   CfOut(cf_verbose,""," -> Verifying replacement of \"%s\" with \"%s\"\n",pp->promiser,replace);  
+      CfOut(cf_verbose,""," -> Verifying replacement of \"%s\" with \"%s\" (%d)\n",pp->promiser,replace,cutoff);  
 
-   memset(line_buff,0,CF_BUFSIZE);
-   sp = ip->name;   
+      before[0] = after[0] = '\0';         
 
-   while (*sp != '\0')
-      {
-      strncat(line_buff,sp,start_off);
-      sp += start_off;
-      strncat(line_buff,replace,CF_BUFSIZE);
-      sp += match_len;
-
-      CfOut(cf_verbose,""," -> << \"%s\"\n",ip->name);
-      CfOut(cf_verbose,""," -> >> \"%s\"\n",line_buff);
-
-      notfound = false;
+      // Model the partial substitution in line_buff to check convergence
       
+      strncat(before,line_buff,start_off);
+      strncat(after,line_buff+end_off,sizeof(after) - 1);
+      snprintf(line_buff,CF_EXPANDSIZE-1,"%s%s",before,replace);
+      notfound = false;
+      replaced = true;
+      
+      // Model the full substitution in line_buff
+      
+      snprintf(line_buff,CF_EXPANDSIZE-1,"%s%s%s",before,replace,after);
+            
       if (once_only)
          {
          CfOut(cf_verbose,""," -> Replace first occurrence only (warning, this is not a convergent policy)");
-         strncat(line_buff,ip->name+end_off,CF_BUFSIZE);
          break;
          }
-      else
-         {
-         if (!BlockTextMatch(pp->promiser,sp,&start_off,&end_off))
-            {
-            strncat(line_buff,sp,CF_BUFSIZE);
-            break;
-            }
-         else
-            {
-            match_len = end_off - start_off;
-            ExpandScalar(a.replace.replace_value,replace);
-            }
-         }
       }
 
-   if (BlockTextMatch(pp->promiser,line_buff,&start_off,&end_off))
+   if (NotAnchored(pp->promiser) && BlockTextMatch(pp->promiser,line_buff,&start_off,&end_off))
       {
-      if (start_off == 0 && end_off == strlen(line_buff))
-         {
-         ExpandScalar(a.replace.replace_value,replace);
-         
-         if (strcmp(replace,line_buff) != 0)
-            {      
-            cfPS(cf_error,CF_INTERPT,"",pp,a," -> Promised replacement \"%s\" on line \"%s\" for pattern \"%s\" is not convergent while editing %s",line_buff,ip->name,pp->promiser,pp->this_server);
-            CfOut(cf_error,"","Because the regular expression \"%s\" still matches the replacement string \"%s\"",pp->promiser,line_buff);
-            PromiseRef(cf_error,pp);
-            continue;
-            }
-         }
+      cfPS(cf_error,CF_INTERPT,"",pp,a," -> Promised replacement \"%s\" on line \"%s\" for pattern \"%s\" is not convergent while editing %s",line_buff,ip->name,pp->promiser,pp->this_server);
+      CfOut(cf_error,"","Because the regular expression \"%s\" still matches the replacement string \"%s\"",pp->promiser,line_buff);
+      PromiseRef(cf_error,pp);
+      break;
       }
-
+   
    if (a.transaction.action == cfa_warn)
       {
       cfPS(cf_verbose,CF_WARN,"",pp,a," -> Need to replace line \"%s\" in %s - but only a warning was promised",pp->promiser,pp->this_server);
       continue;
       }
-   else
+   else if (replaced)
       {
       free(ip->name);
       ip->name = strdup(line_buff);
       cfPS(cf_verbose,CF_CHG,"",pp,a," -> Replaced pattern \"%s\" in %s",pp->promiser,pp->this_server);
       (pp->edcontext->num_edits)++;
       retval = true;
-      }
 
-   if (once_only)
-      {
-      break;
+      CfOut(cf_verbose,""," -> << (%d)\"%s\"\n",cutoff,ip->name);
+      CfOut(cf_verbose,""," -> >> (%d)\"%s\"\n",cutoff,line_buff);
+
+      if (once_only)
+         {
+         CfOut(cf_verbose,""," -> Replace first occurrence only (warning, this is not a convergent policy)");
+         break;
+         }
+
+      if (BlockTextMatch(pp->promiser,ip->name,&start_off,&end_off))
+         {
+         cfPS(cf_inform,CF_INTERPT,"",pp,a," -> Promised replacement \"%s\" for pattern \"%s\" is not properly convergent while editing %s",ip->name,pp->promiser,pp->this_server);
+         CfOut(cf_inform,"","Because the regular expression \"%s\" still matches the end-state replacement string \"%s\"",pp->promiser,line_buff);
+         PromiseRef(cf_inform,pp);
+         }
       }
    }
 
@@ -930,8 +997,6 @@ if (notfound)
    cfPS(cf_verbose,CF_NOP,"",pp,a," -> No pattern \"%s\" in %s",pp->promiser,pp->this_server);
    }
 
-DeleteScope("match");
-NewScope("match");
 return retval;
 }
 
@@ -941,16 +1006,13 @@ int EditColumns(struct Item *file_start,struct Item *file_end,struct Attributes
 
 { char separator[CF_MAXVARSIZE]; 
   int s,e,retval = false;
-  struct CfRegEx rex;
   struct Item *ip;
   struct Rlist *columns = NULL;
- 
-rex = CompileRegExp(pp->promiser);
 
-if (rex.failed)
-   {
+if (!ValidateRegEx(pp->promiser))
+{
    return false;
-   }
+}
 
 for (ip = file_start; ip != file_end; ip=ip->next)
    {
@@ -970,7 +1032,7 @@ for (ip = file_start; ip != file_end; ip=ip->next)
 
    if (!BlockTextMatch(a.column.column_separator,ip->name,&s,&e))
       {
-      cfPS(cf_error,CF_INTERPT,"",pp,a,"Field edit - no fields found by promised pattern %s in %s",a.column.column_separator,pp->this_server);
+      cfPS(cf_verbose,CF_INTERPT,"",pp,a," ! Field edit - no fields found by promised pattern %s in %s",a.column.column_separator,pp->this_server);
       return false;
       }
 
@@ -979,7 +1041,6 @@ for (ip = file_start; ip != file_end; ip=ip->next)
       CfOut(cf_error,""," !! Line split criterion matches a huge part of the line -- seems to be in error");
       return false;
       }
-
    
    strncpy(separator,ip->name+s,e-s);
    separator[e-s]='\0';
@@ -1006,7 +1067,11 @@ int SanityCheckInsertions(struct Attributes a)
 { long not = 0;
   long with = 0;
   long ok = true;
-  
+  struct Rlist *rp;
+  enum insert_match opt;
+  int exact = false, ignore_something = false;
+  int multiline = a.sourcetype && strcmp(a.sourcetype,"preserve_block") == 0;
+
 if (a.line_select.startwith_from_list)
    {
    with++;
@@ -1039,13 +1104,39 @@ if (a.line_select.not_contains_from_list)
 
 if (not > 1)
    {
-   CfOut(cf_error,"","Line insertion selection promise is meaningless - the alternatives are mutually exclusive (only one is allowed)");
+   CfOut(cf_error,""," !! Line insertion selection promise is meaningless - the alternatives are mutually exclusive (only one is allowed)");
    ok = false;
    }
 
 if (with && not)
    {
-   CfOut(cf_error,"","Line insertion selection promise is meaningless - cannot mix positive and negative constraints");
+   CfOut(cf_error,""," !! Line insertion selection promise is meaningless - cannot mix positive and negative constraints");
+   ok = false;
+   }
+
+for (rp = a.insert_match; rp != NULL; rp=rp->next)
+   {
+   opt = String2InsertMatch(rp->item);
+
+   switch (opt)
+      {
+      case cf_exact_match:
+          exact = true;
+          break;
+      default:
+          ignore_something = true;
+          if (multiline)
+             {
+             CfOut(cf_error,""," !! Line insertion should not use whitespace policy with preserve_block");
+             ok = false;
+             }
+          break;
+      }
+   }
+
+if (exact && ignore_something)
+   {
+   CfOut(cf_error,""," !! Line insertion selection promise is meaningless - cannot mix exact_match with other ignore whitespace options");
    ok = false;
    }
 
@@ -1056,6 +1147,40 @@ return ok;
 /* Level                                                                   */
 /***************************************************************************/
 
+int InsertCompoundLineAtLocation(char *newline,struct Item **start,struct Item *location,struct Item *prev,struct Attributes a,struct Promise *pp)
+
+{
+  int result = false;
+  char buf[CF_EXPANDSIZE];
+
+if (strchr(newline,'\n') != NULL) /* Multi-line string */
+   {
+   char *sp;
+   
+   for (sp = newline; sp <= newline+strlen(newline); sp++)
+      {
+      memset(buf,0,CF_BUFSIZE);
+      sscanf(sp,"%2048[^\n]",buf);
+      sp += strlen(buf);
+
+      if (!SelectLine(buf,a,pp))
+         {
+         continue;
+         }
+      
+      result |= InsertMissingLineAtLocation(buf,start,location,prev,a,pp);
+      }
+   }
+else
+   {
+   result |= InsertMissingLineAtLocation(newline,start,location,prev,a,pp);
+   }
+
+return result;
+}
+
+/***************************************************************************/
+
 int InsertMissingLineAtLocation(char *newline,struct Item **start,struct Item *location,struct Item *prev,struct Attributes a,struct Promise *pp)
 
 /* Check line neighbourhood in whole file to avoid edge effects */
@@ -1156,7 +1281,7 @@ else
 
 int EditLineByColumn(struct Rlist **columns,struct Attributes a,struct Promise *pp)
 
-{ struct Rlist *rp,*this_column;
+{ struct Rlist *rp,*this_column = NULL;
   char sep[CF_MAXVARSIZE];
   int i,count = 0,retval = false;
 
@@ -1205,9 +1330,16 @@ if (a.column.value_separator != '\0')
    {
    /* internal separator, single char so split again */
 
-   this_column = SplitStringAsRList(rp->item,a.column.value_separator);
-   retval = EditColumn(&this_column,a,pp);
-
+   if (strcmp(rp->item,a.column.column_value) == 0)
+      {
+      retval = false;
+      }
+   else
+      {
+      this_column = SplitStringAsRList(rp->item,a.column.value_separator);
+      retval = EditColumn(&this_column,a,pp);
+      }
+   
    if (retval)
       {
       if (a.transaction.action == cfa_warn)
@@ -1229,7 +1361,7 @@ if (a.column.value_separator != '\0')
       {
       cfPS(cf_verbose,CF_NOP,"",pp,a," -> No need to edit field in %s",pp->this_server);
       }
-   
+
    DeleteRlist(this_column);
    return retval;
    }
@@ -1284,7 +1416,7 @@ int SelectLine(char *line,struct Attributes a,struct Promise *pp)
   int s,e;
   char *selector;
 
-if (c = a.line_select.startwith_from_list)
+if ((c = a.line_select.startwith_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1299,7 +1431,7 @@ if (c = a.line_select.startwith_from_list)
    return false;
    }
 
-if (c = a.line_select.not_startwith_from_list)
+if ((c = a.line_select.not_startwith_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1314,7 +1446,7 @@ if (c = a.line_select.not_startwith_from_list)
    return true;
    }
 
-if (c = a.line_select.match_from_list)
+if ((c = a.line_select.match_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1329,7 +1461,7 @@ if (c = a.line_select.match_from_list)
    return false;
    }
 
-if (c = a.line_select.not_match_from_list)
+if ((c = a.line_select.not_match_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1344,7 +1476,7 @@ if (c = a.line_select.not_match_from_list)
    return true;
    }
 
-if (c = a.line_select.contains_from_list)
+if ((c = a.line_select.contains_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1359,7 +1491,7 @@ if (c = a.line_select.contains_from_list)
    return false;
    }
 
-if (c = a.line_select.not_contains_from_list)
+if ((c = a.line_select.not_contains_from_list))
    {
    for (rp = c; rp != NULL; rp=rp->next)
       {
@@ -1384,11 +1516,11 @@ return true;
 int EditColumn(struct Rlist **columns,struct Attributes a,struct Promise *pp)
 
 { struct Rlist *rp, *found;
- int count = 0,retval = false;
+ int retval = false;
 
 if (a.column.column_operation && strcmp(a.column.column_operation,"delete") == 0)
    {
-   if (found = KeyInRlist(*columns,a.column.column_value))
+   if ((found = KeyInRlist(*columns,a.column.column_value)))
       {
       CfOut(cf_inform,""," -> Deleting column field sub-value %s in %s",a.column.column_value,pp->this_server);
       DeleteRlistEntry(columns,found);
@@ -1410,7 +1542,7 @@ if (a.column.column_operation && strcmp(a.column.column_operation,"set") == 0)
          return false;
          }
       }
-   
+
    CfOut(cf_inform,""," -> Setting field sub-value %s in %s",a.column.column_value,pp->this_server);
    DeleteRlist(*columns);
    *columns = NULL;
@@ -1458,3 +1590,19 @@ else
 return false;
 }
 
+/********************************************************************/
+
+int NotAnchored(char *s)
+{
+if (*s != '^')
+   {
+   return true;
+   }
+
+if (*(s+strlen(s)-1) != '$')
+   {
+   return true;
+   }
+
+return false;
+}
