@@ -35,11 +35,11 @@
 
 /*******************************************************************/
 
-int Hash(char *name)
+int RefHash(char *name) // This function wants HASHTABLESIZE to be prime
 
 { int i, slot = 0;
 
- for (i = 0; name[i] != '\0'; i++)
+for (i = 0; name[i] != '\0'; i++)
    {
    slot = (CF_MACROALPHABET * slot + name[i]) % CF_HASHTABLESIZE;
    }
@@ -51,24 +51,49 @@ return slot;
 
 int ElfHash(char *key)
 
-{ unsigned int h = 0;
-  unsigned int g;
+{ unsigned char *p = key;
+  int len = strlen(key);
+  unsigned h = 0, g;
+  unsigned int hashtablesize = CF_HASHTABLESIZE;
+  int i;
+ 
+for ( i = 0; i < len; i++ )
+   {
+   h = ( h << 4 ) + p[i];
+   g = h & 0xf0000000L;
+   
+   if ( g != 0 )
+      {
+      h ^= g >> 24;
+      }
+   
+   h &= ~g;
+   }
 
-while (*key)
-  {
-  h = (h << 4) + *key++;
+return (h & (hashtablesize-1));
+}
 
-  g = h & 0xF0000000;         /* Assumes int is 32 bit */
+/*******************************************************************/
 
-  if (g) 
-     {
-     h ^= g >> 24;
-     }
+int OatHash(char *key)
 
-  h &= ~g;
-  }
+{ unsigned int hashtablesize = CF_HASHTABLESIZE;
+  unsigned char *p = key;
+  unsigned h = 0;
+  int i, len = strlen(key);
+  
+for ( i = 0; i < len; i++ )
+   {
+   h += p[i];
+   h += ( h << 10 );
+   h ^= ( h >> 6 );
+   }
 
-return (h % CF_HASHTABLESIZE);
+h += ( h << 3 );
+h ^= ( h >> 11 );
+h += ( h << 15 );
+
+return (h & (hashtablesize-1));
 }
 
 /*****************************************************************************/
@@ -79,11 +104,10 @@ int FileHashChanged(char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],int w
    to the database. Returns true if hashes do not match and also potentially
    updates database to the new value */
 
-{ struct stat stat1, stat2;
-  int i,needupdate = false, size = 21;
-  unsigned char dbdigest[EVP_MAX_MD_SIZE+1],dbattr[EVP_MAX_MD_SIZE+1];
+{
+  int i,size = 21;
+  unsigned char dbdigest[EVP_MAX_MD_SIZE+1];
   CF_DB *dbp;
-  FILE *fp;
 
 Debug("HashChanged: key %s (type=%d) with data %s\n",filename,type,HashPrint(type,digest));
 
@@ -122,18 +146,22 @@ if (ReadHash(dbp,type,filename,dbdigest))
          
          if (attr.change.update)
             {
-            CfOut(cf_verbose,""," -> Updating cryptohash for %s to %s\n",filename,HashPrint(type,digest));
+	    cfPS(warnlevel,CF_CHG,"",pp,attr," -> Updating hash for %s to %s",filename,HashPrint(type,digest));
             
             DeleteHash(dbp,type,filename);
             WriteHash(dbp,type,filename,digest);
             }
+	 else
+	   {
+	   cfPS(warnlevel,CF_FAIL,"",pp,attr,"!! Hash for file \"%s\" changed",filename);
+	   }
          
 	 CloseDB(dbp);
-         return true;                        /* Checksum updated but was changed */
+         return true;
          }
       }
    
-   Debug("Found checksum for %s in database and it matched\n",filename);
+   cfPS(cf_verbose,CF_NOP,"",pp,attr," -> File hash for %s is correct",filename);
    CloseDB(dbp);
    return false;
    }
@@ -166,8 +194,8 @@ if (sstat->st_size != dstat->st_size)
   
 if (attr.copy.servers == NULL || strcmp(attr.copy.servers->item,"localhost") == 0)
    {
-   HashFile(file1,digest1,cf_md5);
-   HashFile(file2,digest2,cf_md5);
+   HashFile(file1,digest1,CF_DEFAULT_DIGEST);
+   HashFile(file2,digest2,CF_DEFAULT_DIGEST);
 
    for (i = 0; i < EVP_MAX_MD_SIZE; i++)
       {
@@ -228,7 +256,7 @@ if (attr.copy.servers == NULL || strcmp(attr.copy.servers->item,"localhost") ==
    }
 else
    {
-   Debug("Using network md5 checksum instead\n");
+   Debug("Using network checksum instead\n");
    return CompareHashNet(file1,file2,attr,pp); /* client.c */
    }
 }
@@ -255,7 +283,7 @@ else
    
    EVP_DigestInit(&context,md);
 
-   while (len = fread(buffer,1,1024,file))
+   while ((len = fread(buffer,1,1024,file)))
       {
       EVP_DigestUpdate(&context,buffer,len);
       }
@@ -299,7 +327,6 @@ void HashString(char *buffer,int len,unsigned char digest[EVP_MAX_MD_SIZE+1],enu
 
 { EVP_MD_CTX context;
   const EVP_MD *md = NULL;
-  char *file_buffer;
   int md_len;
 
 Debug2("HashString(%c)\n",type);
@@ -307,7 +334,8 @@ Debug2("HashString(%c)\n",type);
 switch (type)
    {
    case cf_crypt:
-       CfOut(cf_error,"","The crypt support is not presently implemented, please use md5 instead");
+       CfOut(cf_error,"","The crypt support is not presently implemented, please use another algorithm instead");
+       memset(digest,0,EVP_MAX_MD_SIZE+1);
        break;
        
    default:
@@ -319,7 +347,7 @@ switch (type)
           }
        
        EVP_DigestInit(&context,md); 
-       EVP_DigestUpdate(&context,(unsigned char*)buffer,len);
+       EVP_DigestUpdate(&context,(unsigned char*)buffer,(size_t)len);
        EVP_DigestFinal(&context,digest,&md_len);
        break;
    }
@@ -327,12 +355,77 @@ switch (type)
 
 /*******************************************************************/
 
+void HashPubKey(RSA *key,unsigned char digest[EVP_MAX_MD_SIZE+1],enum cfhashes type)
+
+{ EVP_MD_CTX context;
+  const EVP_MD *md = NULL;
+  int md_len,i,buf_len, actlen;
+  unsigned char *buffer;
+
+Debug("HashPubKey(%d)\n",type);
+
+//RSA_print_fp(stdout,key,0);
+
+if (key->n)
+   {
+   buf_len = (size_t)BN_num_bytes(key->n);
+   }
+else
+   {
+   buf_len = 0;
+   }
+
+if (key->e)
+   {
+   if (buf_len < (i = (size_t)BN_num_bytes(key->e)))
+      {
+      buf_len = i;
+      }
+   }
+
+if ((buffer = malloc(buf_len+10)) == NULL)
+   {
+   FatalError("Memory alloc in HashPubKey");
+   }
+
+switch (type)
+   {
+   case cf_crypt:
+       CfOut(cf_error,"","The crypt support is not presently implemented, please use sha256 instead");
+       break;
+       
+   default:
+       md = EVP_get_digestbyname(FileHashName(type));
+
+       if (md == NULL)
+          {
+          CfOut(cf_inform,""," !! Digest type %s not supported by OpenSSL library",CF_DIGEST_TYPES[type][0]);
+          }
+       
+       EVP_DigestInit(&context,md); 
+
+       actlen = BN_bn2bin(key->n,buffer);
+       EVP_DigestUpdate(&context,buffer,actlen);
+       actlen = BN_bn2bin(key->e,buffer);
+       EVP_DigestUpdate(&context,buffer,actlen);
+       EVP_DigestFinal(&context,digest,&md_len);
+       break;
+   }
+
+free(buffer);
+}
+
+/*******************************************************************/
+
 int HashesMatch(unsigned char digest1[EVP_MAX_MD_SIZE+1],unsigned char digest2[EVP_MAX_MD_SIZE+1],enum cfhashes type)
 
 { int i,size = EVP_MAX_MD_SIZE;
- 
+
 size = FileHashSize(type);
 
+Debug("1. CHECKING DIGEST type %d - size %d (%s)\n",type,size,HashPrint(type,digest1));
+Debug("2. CHECKING DIGEST type %d - size %d (%s)\n",type,size,HashPrint(type,digest2));
+
 for (i = 0; i < size; i++)
    {
    if (digest1[i] != digest2[i])
@@ -347,9 +440,26 @@ return true;
 /*********************************************************************/
 
 char *HashPrint(enum cfhashes type,unsigned char digest[EVP_MAX_MD_SIZE+1])
-
-{ unsigned int i;
+/** 
+ * WARNING: Not thread-safe (returns pointer to global memory).
+ * Use HashPrintSafe for a thread-safe equivalent
+ */
+{
   static char buffer[EVP_MAX_MD_SIZE*4];
+  
+  HashPrintSafe(type,digest,buffer);
+
+  return buffer;
+}    
+
+/*********************************************************************/
+
+char *HashPrintSafe(enum cfhashes type,unsigned char digest[EVP_MAX_MD_SIZE+1], char buffer[EVP_MAX_MD_SIZE*4])
+/**
+ * Thread safe. Note the buffer size.
+ */
+{
+ unsigned int i;
 
 switch (type)
    {
@@ -366,8 +476,8 @@ for (i = 0; i < CF_DIGEST_SIZES[type]; i++)
    sprintf((char *)(buffer+4+2*i),"%02x", digest[i]);
    }
 
-return buffer; 
-}    
+return buffer;   
+}
 
 /***************************************************************/
 
@@ -378,7 +488,7 @@ void PurgeHashes(char *path,struct Attributes attr,struct Promise *pp)
 { CF_DB *dbp;
   CF_DBC *dbcp;
   struct stat statbuf;
-  int ret,ksize,vsize;
+  int ksize,vsize;
   char *key;
   void *value;
 
@@ -393,7 +503,7 @@ if (path)
       {
       DeleteDB(dbp,path);
       }
-   
+   CloseDB(dbp);
    return;
    }
 
@@ -402,6 +512,7 @@ if (path)
 if (!NewDBCursor(dbp,&dbcp))
    {
    CfOut(cf_inform,""," !! Unable to scan hash database");
+   CloseDB(dbp);
    return;
    }
 
@@ -559,7 +670,6 @@ return CF_DIGEST_TYPES[id][0];
 
 int FileHashSize(enum cfhashes id)
 
-{ int i,size = 0;
- 
+{
 return CF_DIGEST_SIZES[id];
 }
