@@ -94,7 +94,7 @@ int VerifyFileLeaf(char *path,struct stat *sb,struct Attributes attr,struct Prom
 
 {
 /* Here we can assume that we are in the parent directory of the leaf */
-
+ 
 if (!SelectLeaf(path,sb,attr,pp))
    {
    Debug("Skipping non-selected file %s\n",path);
@@ -185,11 +185,17 @@ int CfCreateFile(char *file,struct Promise *pp,struct Attributes attr)
 
 // attr.move_obstructions for MakeParentDirectory
 
+if (!IsAbsoluteFileName(file))
+   {
+   cfPS(cf_inform,CF_FAIL,"creat",pp,attr," !! Cannot create a relative filename %s - has no invariant meaning\n",file);
+   return false;
+   }
+ 
 if (strcmp(".",ReadLastNode(file)) == 0)
    {
    Debug("File object \"%s \"seems to be a directory\n",file);
 
-   if (!DONTDO)
+   if (!DONTDO && attr.transaction.action != cfa_warn)
       {
       if (!MakeParentDirectory(file,attr.move_obstructions))
          {
@@ -199,10 +205,15 @@ if (strcmp(".",ReadLastNode(file)) == 0)
 
       cfPS(cf_inform,CF_CHG,"",pp,attr," -> Created directory %s\n",file);
       }
+   else
+      {
+      CfOut(cf_error,""," !! Warning promised, need to create directory %s",file);
+      return false;
+      }
    }
 else
    {
-   if (!DONTDO)
+   if (!DONTDO && attr.transaction.action != cfa_warn)
       {
       mode_t saveumask = umask(0);
       mode_t filemode = 0600;  /* Decide the mode for filecreation */
@@ -233,6 +244,11 @@ else
          umask(saveumask);
          }
       }
+   else
+      {
+      CfOut(cf_error,""," !! Warning promised, need to create file %s\n",file);
+      return false;
+      }
    }
 
 return true;
@@ -244,7 +260,7 @@ int ScheduleCopyOperation(char *destination,struct Attributes attr,struct Promis
 
 { struct cfagent_connection *conn;
 
- CfOut(cf_verbose,""," -> Copy file %s from %s check\n",destination,attr.copy.source);
+CfOut(cf_verbose,""," -> Copy file %s from %s check\n",destination,attr.copy.source);
 
 if (attr.copy.servers == NULL || strcmp(attr.copy.servers->item,"localhost") == 0)
    {
@@ -273,14 +289,15 @@ return true;
 
 /*****************************************************************************/
 
-int ScheduleLinkChildrenOperation(char *destination,struct Attributes attr,struct Promise *pp)
+int ScheduleLinkChildrenOperation(char *destination,char *source,int recurse,struct Attributes attr,struct Promise *pp)
 
 { DIR *dirh;
   struct dirent *dirp;
   char promiserpath[CF_BUFSIZE],sourcepath[CF_BUFSIZE];
   struct stat lsb;
+  int ret;
 
-if (lstat(destination,&lsb) != -1)
+if ((ret = lstat(destination,&lsb)) != -1)
    {
    if (attr.move_obstructions && S_ISLNK(lsb.st_mode))
       {
@@ -295,13 +312,13 @@ if (lstat(destination,&lsb) != -1)
 
 snprintf(promiserpath,CF_BUFSIZE,"%s/.",destination);
 
-if (!CfCreateFile(promiserpath,pp,attr))
+if ((ret == -1 || !S_ISDIR(lsb.st_mode)) && !CfCreateFile(promiserpath,pp,attr))
    {
    CfOut(cf_error,"","Cannot promise to link multiple files to children of %s as it is not a directory!",destination);
    return false;
    }
 
-if ((dirh = opendir(attr.link.source)) == NULL)
+if ((dirh = opendir(source)) == NULL)
    {
    cfPS(cf_error,CF_FAIL,"opendir",pp,attr,"Can't open source of children to link %s\n",attr.link.source);
    return false;
@@ -309,7 +326,7 @@ if ((dirh = opendir(attr.link.source)) == NULL)
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    {
-   if (!ConsiderFile(dirp->d_name,attr.link.source,attr,pp))
+   if (!ConsiderFile(dirp->d_name,source,attr,pp))
       {
       continue;
       }
@@ -326,7 +343,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       return false;
       }
 
-   strncpy(sourcepath,attr.link.source,CF_BUFSIZE-1);
+   strncpy(sourcepath,source,CF_BUFSIZE-1);
    AddSlash(sourcepath);
 
    if (!JoinPath(sourcepath,dirp->d_name))
@@ -336,7 +353,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       return false;
       }
 
-   if ((lstat(promiserpath,&lsb) != -1) && !S_ISLNK(lsb.st_mode))
+   if ((lstat(promiserpath,&lsb) != -1) && !S_ISLNK(lsb.st_mode) && !S_ISDIR(lsb.st_mode))
       {
       if (attr.link.when_linking_children == cfa_override)
          {
@@ -349,7 +366,14 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
          }
       }
 
-   ScheduleLinkOperation(promiserpath,sourcepath,attr,pp);
+   if ((attr.recursion.depth > recurse) && (lstat(sourcepath,&lsb) != -1) && S_ISDIR(lsb.st_mode))
+      {
+      ScheduleLinkChildrenOperation(promiserpath,sourcepath,recurse+1,attr,pp);
+      }
+   else
+      {
+      ScheduleLinkOperation(promiserpath,sourcepath,attr,pp);
+      }
    }
 
 closedir(dirh);
@@ -400,28 +424,38 @@ int ScheduleEditOperation(char *filename,struct Attributes a,struct Promise *pp)
 { struct Bundle *bp;
   void *vp;
   struct FnCall *fp;
-  char *edit_bundle_name = NULL;
+  char *edit_bundle_name = NULL,lockname[CF_BUFSIZE];
   struct Rlist *params;
   int retval = false;
+  struct CfLock thislock;
+
+snprintf(lockname,CF_BUFSIZE-1,"fileedit-%s",pp->promiser);
+thislock = AcquireLock(lockname,VUQNAME,CFSTARTTIME,a,pp,false);
+
+if (thislock.lock == NULL)
+   {
+   return false;
+   }
 
 pp->edcontext = NewEditContext(filename,a,pp);
 
 if (pp->edcontext == NULL)
    {
-   CfOut(cf_error,"","File %s was marked for editing but could not be opened\n",filename);
+   cfPS(cf_error,CF_FAIL,"",pp,a,"File %s was marked for editing but could not be opened\n",filename);
    FinishEditContext(pp->edcontext,a,pp);
+   YieldCurrentLock(thislock);
    return false;
    }
 
 if (a.haveeditline)
    {
-   if (vp = GetConstraint("edit_line",pp,CF_FNCALL))
+   if ((vp = GetConstraint("edit_line",pp,CF_FNCALL)))
       {
       fp = (struct FnCall *)vp;
       edit_bundle_name = fp->name;
-      params = fp->args;;
+      params = fp->args;
       }
-   else if (vp = GetConstraint("edit_line",pp,CF_SCALAR))
+   else if ((vp = GetConstraint("edit_line",pp,CF_SCALAR)))
       {
       edit_bundle_name = (char *)vp;
       params = NULL;
@@ -429,6 +463,7 @@ if (a.haveeditline)
    else
       {
       FinishEditContext(pp->edcontext,a,pp);
+      YieldCurrentLock(thislock);
       return false;
       }
 
@@ -436,10 +471,14 @@ if (a.haveeditline)
 
    // add current filename to context - already there?
 
-   if (bp = GetBundle(edit_bundle_name,"edit_line"))
+   if ((bp = GetBundle(edit_bundle_name,"edit_line")))
       {
       BannerSubBundle(bp,params);
+
+      DeleteScope(bp->name);
       NewScope(bp->name);
+      HashVariables(bp->name);
+
       AugmentScope(bp->name,bp->args,params);
       PushPrivateClassContext();
       retval = ScheduleEditLineOperations(filename,bp,a,pp);
@@ -449,6 +488,7 @@ if (a.haveeditline)
    }
 
 FinishEditContext(pp->edcontext,a,pp);
+YieldCurrentLock(thislock);
 return retval;
 }
 
@@ -1108,7 +1148,7 @@ if (cmpsb.st_dev != sb->st_dev)
 
 if (cmpsb.st_ino != sb->st_ino)
    {
-   CfOut(cf_error,"","ALERT: inode for %s changed %d -> %d",file,cmpsb.st_ino,sb->st_ino);
+   CfOut(cf_error,"","ALERT: inode for %s changed %lu -> %lu",file,cmpsb.st_ino,sb->st_ino);
    }
 
 if (cmpsb.st_mtime != sb->st_mtime)
@@ -1148,7 +1188,10 @@ CloseDB(dbp);
 int TransformFile(char *file,struct Attributes attr,struct Promise *pp)
 
 { char comm[CF_EXPANDSIZE],line[CF_BUFSIZE];
-  FILE *pop;
+  FILE *pop = NULL;
+  int print = false;
+  struct CfLock thislock;
+  int transRetcode = 0;
 
 if (attr.transformer == NULL || file == NULL)
    {
@@ -1158,21 +1201,65 @@ if (attr.transformer == NULL || file == NULL)
 ExpandScalar(attr.transformer,comm);
 CfOut(cf_inform,"","Transforming: %s ",comm);
 
-if ((pop = cf_popen(comm,"r")) == NULL)
+if (!IsExecutable(GetArg0(comm)))
    {
    cfPS(cf_inform,CF_FAIL,"",pp,attr,"Transformer %s %s failed",attr.transformer,file);
    return false;
    }
 
-while (!feof(pop))
+if (strncmp(comm,"/bin/echo",strlen("/bin/echo")) == 0)
    {
-   CfReadLine(line,CF_BUFSIZE,pop);
-   CfOut(cf_inform,"",line);
+   print = true;
    }
 
-cf_pclose(pop);
+if (!DONTDO)
+   {
+   thislock = AcquireLock(comm,VUQNAME,CFSTARTTIME,attr,pp,false);
+   
+   if (thislock.lock == NULL)
+      {
+      return false;
+      }
+   
+   if ((pop = cf_popen(comm,"r")) == NULL)
+      {
+      cfPS(cf_inform,CF_FAIL,"",pp,attr,"Transformer %s %s failed",attr.transformer,file);
+      YieldCurrentLock(thislock);
+      return false;
+      }
+   
+   while (!feof(pop))
+      {
+      CfReadLine(line,CF_BUFSIZE,pop);
 
-cfPS(cf_inform,CF_CHG,"",pp,attr,"Transformer %s => %s seemed ok",file,comm);
+      if (print)
+         {
+         CfOut(cf_reporting,"",line);
+         }
+      else
+         {
+         CfOut(cf_inform,"",line);
+         }
+      }
+   
+   transRetcode = cf_pclose(pop);
+   
+   if(VerifyCommandRetcode(transRetcode,true,attr,pp))
+     {
+     CfOut(cf_inform,"","-> Transformer %s => %s seemed to work ok",file,comm);
+     }
+   else
+     {
+     CfOut(cf_error,"","-> Transformer %s => %s returned error",file,comm);
+     }
+   
+   }
+else
+   {
+   CfOut(cf_error,""," -> Need to transform file \"%s\" with \"%s\"",file,comm);
+   }
+
+YieldCurrentLock(thislock);
 return true;
 }
 
@@ -1226,12 +1313,15 @@ if (lstat(pathbuf,&statbuf) != -1)
    {
    if (S_ISLNK(statbuf.st_mode))
       {
-      CfOut(cf_verbose,"","%s: INFO: %s is a symbolic link, not a true directory!\n",VPREFIX,pathbuf);
+      CfOut(cf_verbose,"","INFO: %s is a symbolic link, not a true directory!\n",VPREFIX,pathbuf);
       }
 
    if (force)   /* force in-the-way directories aside */
       {
-      if (!S_ISDIR(statbuf.st_mode))  /* if the dir exists - no problem */
+      struct stat dir;
+      stat(pathbuf,&dir);
+   
+      if (!S_ISDIR(dir.st_mode) || S_ISLNK(statbuf.st_mode))  /* if the dir exists - no problem */
          {
          struct stat sbuf;
 
@@ -1388,14 +1478,25 @@ else
 void LogHashChange(char *file)
 
 { FILE *fp;
-  char fname[CF_BUFSIZE],timebuf[CF_MAXVARSIZE];
+  char fname[CF_BUFSIZE];
   time_t now = time(NULL);
   struct stat sb;
   mode_t perm = 0600;
+  static char prevFile[CF_MAXVARSIZE] = {0};
+
+
+  // we might get called twice..
+  if(strcmp(file,prevFile) == 0)
+    {
+    return;
+    }
+  
+  snprintf(prevFile,sizeof(prevFile),file);
+
 
 /* This is inefficient but we don't want to lose any data */
 
-snprintf(fname,CF_BUFSIZE,"%s/state/file_hash_event_history",CFWORKDIR);
+snprintf(fname,CF_BUFSIZE,"%s/state/%s",CFWORKDIR,CF_FILECHANGE);
 MapName(fname);
 
 #ifndef MINGW
@@ -1414,9 +1515,7 @@ if ((fp = fopen(fname,"a")) == NULL)
    return;
    }
 
-snprintf(timebuf,CF_MAXVARSIZE-1,"%s",cf_ctime(&now));
-Chop(timebuf);
-fprintf(fp,"%s,%s\n",timebuf,file);
+fprintf(fp,"%ld,%s\n",(long)now,file);
 fclose(fp);
 
 cf_chmod(fname,perm);
@@ -1431,8 +1530,8 @@ void RotateFiles(char *name,int number)
 { int i, fd;
   struct stat statbuf;
   char from[CF_BUFSIZE],to[CF_BUFSIZE];
-  struct Attributes attr;
-  struct Promise dummyp;
+  struct Attributes attr = {{0}};
+  struct Promise dummyp = {0};
 
 if (IsItemIn(ROTATED,name))
    {
@@ -1523,7 +1622,7 @@ else
 
 void DeleteDirectoryTree(char *path,struct Promise *pp)
 
-{ struct Promise promise;
+{ struct Promise promise = {0};
   char s[CF_MAXVARSIZE];
   time_t now = time(NULL);
 
@@ -1671,8 +1770,8 @@ int Unix_VerifyOwner(char *file,struct Promise *pp,struct Attributes attr,struct
 
 { struct passwd *pw;
   struct group *gp;
-  struct UidList *ulp, *unknownulp;
-  struct GidList *glp, *unknownglp;
+  struct UidList *ulp;
+  struct GidList *glp;
   short uidmatch = false, gidmatch = false;
   uid_t uid = CF_SAME_OWNER;
   gid_t gid = CF_SAME_GROUP;
@@ -1681,18 +1780,6 @@ Debug("Unix_VerifyOwner: %d\n",sb->st_uid);
 
 for (ulp = attr.perms.owners; ulp != NULL; ulp=ulp->next)
    {
-   if (ulp->uid == CF_UNKNOWN_OWNER)
-      {
-      unknownulp = MakeUidList(ulp->uidname); /* Will only match one */
-
-      if (unknownulp != NULL && sb->st_uid == unknownulp->uid)
-         {
-         uid = unknownulp->uid;
-         uidmatch = true;
-         break;
-         }
-      }
-
    if (ulp->uid == CF_SAME_OWNER || sb->st_uid == ulp->uid)   /* "same" matches anything */
       {
       uid = ulp->uid;
@@ -1701,20 +1788,18 @@ for (ulp = attr.perms.owners; ulp != NULL; ulp=ulp->next)
       }
    }
 
+if (attr.perms.groups->next == NULL && attr.perms.groups->gid == CF_UNKNOWN_GROUP) // Only one non.existent item
+   {   
+   cfPS(cf_inform,CF_FAIL,"",pp,attr," !! Unable to make file belong to an unknown group");
+   }
+
+if (attr.perms.owners->next == NULL && attr.perms.owners->uid == CF_UNKNOWN_OWNER) // Only one non.existent item
+   {   
+   cfPS(cf_inform,CF_FAIL,"",pp,attr," !! Unable to make file belong to an unknown user");
+   }
+
 for (glp = attr.perms.groups; glp != NULL; glp=glp->next)
    {
-   if (glp->gid == CF_UNKNOWN_GROUP) /* means not found while parsing */
-      {
-      unknownglp = MakeGidList(glp->gidname); /* Will only match one */
-
-      if (unknownglp != NULL && sb->st_gid == unknownglp->gid)
-         {
-         gid = unknownglp->gid;
-         gidmatch = true;
-         break;
-         }
-      }
-
    if (glp->gid == CF_SAME_GROUP || sb->st_gid == glp->gid)  /* "same" matches anything */
       {
       gid = glp->gid;
@@ -1759,7 +1844,7 @@ else
 
           if (uid == CF_SAME_OWNER && gid == CF_SAME_GROUP)
              {
-             CfOut(cf_verbose,"","%s:   touching %s\n",VPREFIX,file);
+             CfOut(cf_verbose,""," -> Touching %s\n",file);
              }
           else
              {
@@ -1887,7 +1972,7 @@ for (sp = uidnames; *sp != '\0'; sp+=strlen(uidbuff))
             {
             if ((pw = getpwnam(ip->name)) == NULL)
                {
-               CfOut(cf_inform,"","Unknown user \'%s\'\n",ip->name);
+               CfOut(cf_inform,""," !! Unknown user \'%s\'\n",ip->name);
                uid = CF_UNKNOWN_OWNER; /* signal user not found */
                usercopy = ip->name;
                }
@@ -1973,7 +2058,7 @@ for (sp = gidnames; *sp != '\0'; sp+=strlen(gidbuff))
             }
          else if ((gr = getgrnam(gidbuff)) == NULL)
             {
-            CfOut(cf_inform,"","Unknown group %s\n",gidbuff);
+            CfOut(cf_inform,""," !! Unknown group %s\n",gidbuff);
             gid = CF_UNKNOWN_GROUP;
             groupcopy = gidbuff;
             }
@@ -2008,37 +2093,41 @@ void Unix_VerifyFileAttributes(char *file,struct stat *dstat,struct Attributes a
 maskvalue = umask(0);                 /* This makes the DEFAULT modes absolute */
 
 newperm = (dstat->st_mode & 07777);
-newperm |= attr.perms.plus;
-newperm &= ~(attr.perms.minus);
 
-Debug("Unix_VerifyFileAttributes(%s -> %o)\n",file,newperm);
-
- /* directories must have x set if r set, regardless  */
-
-if (S_ISDIR(dstat->st_mode))
+if ((attr.perms.plus != CF_SAMEMODE) && (attr.perms.minus != CF_SAMEMODE))
    {
-   if (attr.perms.rxdirs)
+   newperm |= attr.perms.plus;
+   newperm &= ~(attr.perms.minus);
+
+   Debug("Unix_VerifyFileAttributes(%s -> %o)\n",file,newperm);
+   
+   /* directories must have x set if r set, regardless  */
+   
+   if (S_ISDIR(dstat->st_mode))
       {
-      Debug("Directory...fixing x bits\n");
-
-      if (newperm & S_IRUSR)
+      if (attr.perms.rxdirs)
          {
-         newperm  |= S_IXUSR;
+         Debug("Directory...fixing x bits\n");
+         
+         if (newperm & S_IRUSR)
+            {
+            newperm  |= S_IXUSR;
+            }
+         
+         if (newperm & S_IRGRP)
+            {
+            newperm |= S_IXGRP;
+            }
+         
+         if (newperm & S_IROTH)
+            {
+            newperm |= S_IXOTH;
+            }
          }
-
-      if (newperm & S_IRGRP)
+      else
          {
-         newperm |= S_IXGRP;
+         CfOut(cf_verbose,"","NB: rxdirs is set to false - x for r bits not checked\n");
          }
-
-      if (newperm & S_IROTH)
-         {
-         newperm |= S_IXOTH;
-         }
-      }
-   else
-      {
-      CfOut(cf_verbose,"","NB: rxdirs is set to false - x for r bits not checked\n");
       }
    }
 
@@ -2123,11 +2212,11 @@ newflags &= ~(attr.perms.minus_flags);
 
 if ((newflags & CHFLAGS_MASK) == (dstat->st_flags & CHFLAGS_MASK))    /* file okay */
    {
-   Debug("BSD File okay, flags = %x, current = %x\n",(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
+   Debug("BSD File okay, flags = %lx, current = %lx\n",(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
    }
 else
    {
-   Debug("BSD Fixing %s, newflags = %x, flags = %x\n",file,(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
+   Debug("BSD Fixing %s, newflags = %lx, flags = %lx\n",file,(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
 
    switch (attr.transaction.action)
       {
@@ -2192,7 +2281,7 @@ save_gid = (attr.perms.groups)->gid;
 
 if (attr.copy.preserve)
    {
-   CfOut(cf_verbose,""," -> Attempting to preserve file permissions from the source");
+   CfOut(cf_verbose,""," -> Attempting to preserve file permissions from the source: %o",(sstat->st_mode & 07777));
       
    if ((attr.perms.owners)->uid == CF_SAME_OWNER)          /* Preserve uid and gid  */
       {
@@ -2206,16 +2295,14 @@ if (attr.copy.preserve)
 
 // Will this preserve if no mode set?
 
-   newplus = (sstat->st_mode & 07777) | attr.perms.plus;
-   newminus = ~(newplus & ~(attr.perms.minus)) & 07777;
+   newplus = (sstat->st_mode & 07777);
+   newminus = ~newplus & 07777;
    attr.perms.plus = newplus;
    attr.perms.minus = newminus;
    VerifyFileAttributes(file,dstat,attr,pp);
    }
 else
    {
-   CfOut(cf_verbose,""," -> Not attempting to preserve file permissions from the source");
-   
    if ((attr.perms.owners)->uid == CF_SAME_OWNER)          /* Preserve uid and gid  */
       {
       (attr.perms.owners)->uid = dstat->st_uid;
@@ -2242,7 +2329,7 @@ else
 
 /*******************************************************************/
 
-void AddSimpleUidItem(struct UidList **uidlist,int uid,char *uidname)
+void AddSimpleUidItem(struct UidList **uidlist,uid_t uid,char *uidname)
 
 { struct UidList *ulp, *u;
   char *copyuser;
@@ -2285,7 +2372,7 @@ else
 
 /*******************************************************************/
 
-void AddSimpleGidItem(struct GidList **gidlist,int gid,char *gidname)
+void AddSimpleGidItem(struct GidList **gidlist,gid_t gid,char *gidname)
 
 { struct GidList *glp,*g;
   char *copygroup;
