@@ -1,3 +1,4 @@
+
 /*
    Copyright (C) Cfengine AS
 
@@ -38,21 +39,48 @@ extern void CheckOpts(int argc,char **argv);
 
 /*****************************************************************************/
 
+static void SanitizeEnvironment()
+{
+ /* ps(1) and other utilities invoked by Cfengine may be affected */
+unsetenv("COLUMNS");
+ 
+ /* Make sure subprocesses output is not localized */
+unsetenv("LANG");
+unsetenv("LANGUAGE");
+unsetenv("LC_MESSAGES");
+}
+
+/*****************************************************************************/
+
 void GenericInitialize(int argc,char **argv,char *agents)
 
 { enum cfagenttype ag = Agent2Type(agents);
   char vbuff[CF_BUFSIZE];
-  int ok;
+  int ok = false;
 
+#ifdef HAVE_NOVA
+CF_DEFAULT_DIGEST = cf_sha256;
+CF_DEFAULT_DIGEST_LEN = CF_SHA256_LEN;
+#else
+CF_DEFAULT_DIGEST = cf_md5;
+CF_DEFAULT_DIGEST_LEN = CF_MD5_LEN;
+#endif
+ 
 InitializeGA(argc,argv);
 
 SetReferenceTime(true);
 SetStartTime(false);
 SetSignals();
+SanitizeEnvironment();
 
-// See cf3.defs.h for these settings
+strcpy(THIS_AGENT,CF_AGENTTYPES[ag]);
+NewClass(THIS_AGENT);
+THIS_AGENT_TYPE = ag;
 
-if (EnterpriseExpiry(LIC_DAY,LIC_MONTH,LIC_YEAR)) 
+// need scope sys to set vars in expiry function
+SetNewScope("sys");
+
+if (EnterpriseExpiry(LIC_DAY,LIC_MONTH,LIC_YEAR,LIC_COMPANY)) 
    {
    CfOut(cf_error,"","Cfengine - autonomous configuration engine. This enterprise license is invalid.\n");
    exit(1);
@@ -63,7 +91,6 @@ if (!NOHARDCLASSES)
    NewScope("const");
    NewScope("match");
    NewScope("mon");
-   SetNewScope("sys");
    GetNameInfo3();
    CfGetInterfaceInfo(ag);
       
@@ -77,41 +104,85 @@ if (!NOHARDCLASSES)
 LoadPersistentContext();
 LoadSystemConstants();
 
-strcpy(THIS_AGENT,CF_AGENTTYPES[ag]);
-NewClass(THIS_AGENT);
-THIS_AGENT_TYPE = ag;
-
 snprintf(vbuff,CF_BUFSIZE,"control_%s",THIS_AGENT);
-
 SetNewScope(vbuff);
 NewScope("this");
 NewScope("match");
 
 if (BOOTSTRAP)
    {
-   SetPolicyServer(POLICY_SERVER);
    CheckAutoBootstrap();
    }
-
-ok = BOOTSTRAP || CheckPromises(ag);
-
-if (ok)
-   {
-   ReadPromises(ag,agents);
-   }
 else
    {
-   CfOut(cf_error,"","cf-agent was not able to get confirmation of promises from cf-promises, so going to failsafe\n");
-   snprintf(VINPUTFILE,CF_BUFSIZE-1,"failsafe.cf");
-   ReadPromises(ag,agents);
+   if (strlen(POLICY_SERVER) > 0)
+      {
+      CfOut(cf_verbose,""," -> Found a policy server (hub) on %s",POLICY_SERVER);
+      }
+   else
+      {
+      CfOut(cf_verbose,""," -> No policy server (hub) watch yet registered");
+      }
    }
 
-if (SHOWREPORTS || ERRORCOUNT)
+SetPolicyServer(POLICY_SERVER);
+
+if (ag != cf_keygen)
    {
-   CompilationReport(VINPUTFILE);
-   }
+   if (!MissingInputFile())
+      {
+      bool check_promises = false;
 
-CheckLicenses();
+      if (SHOWREPORTS)
+         {
+         check_promises = true;
+         CfOut(cf_verbose, "", " -> Reports mode is enabled, force-validating policy");
+         }
+      if (IsFileOutsideDefaultRepository(VINPUTFILE))
+         {
+         check_promises = true;
+         CfOut(cf_verbose, "", " -> Input file is outside default repository, validating it");
+         }
+      if (NewPromiseProposals())
+         {
+         check_promises = true;
+         CfOut(cf_verbose, "", " -> Input file is changed since last valiadtion, validating it");
+         }
+
+      if (check_promises)
+         {
+         ok = CheckPromises(ag);
+         if (BOOTSTRAP && !ok)
+            {
+            CfOut(cf_verbose, "", " -> Policy is not valid, but proceeding with bootstrap");
+            ok = true;
+            }
+         }
+      else
+         {
+         CfOut(cf_verbose, "", " -> Policy is already validated");
+         ok = true;
+         }
+      }
+   
+   if (ok)
+      {
+      ReadPromises(ag,agents);
+      }
+   else
+      {
+      CfOut(cf_error,"","cf-agent was not able to get confirmation of promises from cf-promises, so going to failsafe\n");
+      snprintf(VINPUTFILE,CF_BUFSIZE-1,"failsafe.cf");
+      ReadPromises(ag,agents);
+      }
+
+   if (SHOWREPORTS)
+      {
+      CompilationReport(VINPUTFILE);
+      }
+
+   CheckLicenses();
+   }
 
 XML = 0;
 }
@@ -135,15 +206,17 @@ CloseAllDB();
 
 int CheckPromises(enum cfagenttype ag)
 
-{ char cmd[CF_BUFSIZE],path[CF_BUFSIZE];
+{ char cmd[CF_BUFSIZE];
+  char filename[CF_MAXVARSIZE];
   struct stat sb;
+  int fd;
 
 if ((ag != cf_agent) && (ag != cf_executor) && (ag != cf_server))
    {
    return true;
    }
 
-CfOut(cf_verbose,""," > Verifying the syntax of the inputs...\n");
+CfOut(cf_verbose,""," -> Verifying the syntax of the inputs...\n");
 
 snprintf(cmd,CF_BUFSIZE-1,"%s%cbin%ccf-promises%s",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,EXEC_SUFFIX);
 
@@ -155,7 +228,7 @@ if (cfstat(cmd,&sb) == -1)
 
 /* If we are cf-agent, check syntax before attempting to run */
 
-if ((*VINPUTFILE == '.') || IsAbsoluteFileName(VINPUTFILE))
+if (IsFileOutsideDefaultRepository(VINPUTFILE))
    {
    snprintf(cmd,CF_BUFSIZE-1,"\"%s%cbin%ccf-promises%s\" -f \"%s\"",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,EXEC_SUFFIX,VINPUTFILE);
    }
@@ -168,6 +241,29 @@ else
 
 if (ShellCommandReturnsZero(cmd,true))
    {
+   if (MINUSF)
+      {
+      snprintf(filename,CF_MAXVARSIZE,"%s/state/validated_%s",CFWORKDIR,CanonifyName(VINPUTFILE));
+      MapName(filename);   
+      }
+   else
+      {
+      snprintf(filename,CF_MAXVARSIZE,"%s/masterfiles/cf_promises_validated",CFWORKDIR);
+      MapName(filename);
+      }
+   
+   MakeParentDirectory(filename,true);
+   
+   if ((fd = creat(filename,0600)) != -1)
+      {
+      close(fd);
+      CfOut(cf_verbose,""," -> Caching the state of validation\n");
+      }
+   else
+      {
+      CfOut(cf_verbose,"creat"," -> Failed to cache the state of validation\n");
+      }
+   
    return true;
    }
 else
@@ -183,7 +279,6 @@ void ReadPromises(enum cfagenttype ag,char *agents)
 { char *v,rettype;
   void *retval;
   char vbuff[CF_BUFSIZE];
-  struct Constraint *cp;
 
 if (ag == cf_keygen)
    {
@@ -216,11 +311,6 @@ else
    v = "not specified";
    }
 
-if (strchr(retval,':'))
-   {
-   CfOut(cf_error,""," !! The version string may not contain the \":\" character");
-   }
-
 snprintf(vbuff,CF_BUFSIZE-1,"Expanded promises for %s",agents);
 CfHtmlHeader(FREPORT_HTML,vbuff,STYLESHEET,WEBDRIVER,BANNER);
 
@@ -297,12 +387,16 @@ switch (ag)
 
 void InitializeGA(int argc,char *argv[])
 
-{ char *sp;
-  int i,j,seed,force = false;
+{
+  int seed,force = false;
   struct stat statbuf,sb;
-  unsigned char s[16],vbuff[CF_BUFSIZE];
+  unsigned char s[16];
+  char vbuff[CF_BUFSIZE];
   char ebuff[CF_EXPANDSIZE];
 
+SHORT_CFENGINEPORT =  htons((unsigned short)5308);
+snprintf(STR_CFENGINEPORT,15,"5308");
+
 #ifdef NT
 if (cfstat("/cygdrive",&statbuf) == 0)
    {
@@ -321,7 +415,7 @@ strcpy(FILE_SEPARATOR_STR,"/");
 
 NewClass("any");
 
-#ifdef HAVE_LIBCFNOVA
+#ifdef HAVE_NOVA
 NewClass("nova_edition");
 #else
 NewClass("community_edition");
@@ -412,7 +506,7 @@ if (!LOOKUP) /* cf-know should not do this in lookup mode */
 
    if (cfstat(vbuff,&sb) == -1)
       {
-      FatalError(" !!! No access to workspace");
+      FatalError(" !!! No access to WORKSPACE/inputs dir");
       }
    else
       {
@@ -423,7 +517,7 @@ if (!LOOKUP) /* cf-know should not do this in lookup mode */
 
    if (cfstat(vbuff,&sb) == -1)
       {
-      FatalError(" !!! No access to workspace");
+      FatalError(" !!! No access to WORKSPACE/outputs dir");
       }
    else
       {
@@ -455,17 +549,17 @@ if (!LOOKUP) /* cf-know should not do this in lookup mode */
 
 OpenNetwork();
 
-// cleanup db file from v. 3.0.2 (can be removed later)
-char oldLockDb[CF_BUFSIZE];
-snprintf(oldLockDb, sizeof(oldLockDb), "%s/cfengine_lock_db", CFWORKDIR);
-unlink(oldLockDb);
-
 /* Init crypto stuff */
 
 OpenSSL_add_all_algorithms();
 OpenSSL_add_all_digests();
 ERR_load_crypto_strings();
-CheckWorkingDirectories();
+
+if(!LOOKUP)
+  {
+  CheckWorkingDirectories();
+  }
+
 RandomSeed();
 
 RAND_bytes(s,16);
@@ -498,7 +592,7 @@ if (BOOTSTRAP)
    if (!IsEnterprise() && cfstat(vbuff,&statbuf) == -1)
       {
       CfOut(cf_inform,"","Didn't find established file %s, so looking for one in current directory\n",vbuff);
-          snprintf(VINPUTFILE,CF_BUFSIZE-1,".%cfailsafe.cf",FILE_SEPARATOR);
+      snprintf(VINPUTFILE,CF_BUFSIZE-1,".%cfailsafe.cf",FILE_SEPARATOR);
       }
    else
       {
@@ -525,7 +619,7 @@ Cf3ParseFile(VINPUTFILE);
 
 // Expand any lists in this list now
 
-HashVariables();
+HashVariables(NULL);
 HashControls();
 
 if (VINPUTLIST != NULL)
@@ -553,37 +647,89 @@ if (VINPUTLIST != NULL)
                    }
                 break;
             }
+
+         DeleteRvalItem(returnval.item,returnval.rtype);
          }
 
-      HashVariables();
+      HashVariables(NULL);
       HashControls();
       }
    }
 
-HashVariables();
+HashVariables(NULL);
 
 PARSING = false;
 }
 
 /*******************************************************************/
 
+int MissingInputFile()
+
+{ struct stat sb;
+
+if (cfstat(InputLocation(VINPUTFILE),&sb) == -1)
+   {
+   CfOut(cf_error,"stat","There is no readable input file at %s",VINPUTFILE);
+   return true;
+   }
+
+return false;
+}
+
+/*******************************************************************/
+
 int NewPromiseProposals()
 
 { struct Rlist *rp,*sl;
   struct stat sb;
   int result = false;
+  char filename[CF_MAXVARSIZE];
+
+if (MINUSF)
+   {
+   snprintf(filename,CF_MAXVARSIZE,"%s/state/validated_%s",CFWORKDIR,CanonifyName(VINPUTFILE));
+   MapName(filename);   
+   }
+else
+   {
+   snprintf(filename,CF_MAXVARSIZE,"%s/masterfiles/cf_promises_validated",CFWORKDIR);
+   MapName(filename);
+   }
+  
+if (stat(filename,&sb) != -1)
+   {
+   PROMISETIME = sb.st_mtime;
+   }
+else
+   {
+   PROMISETIME = 0;
+   }
 
 if (cfstat(InputLocation(VINPUTFILE),&sb) == -1)
    {
-   CfOut(cf_error,"stat","There is no readable input file at %s",VINPUTFILE);
-   return false;
+   CfOut(cf_verbose,"stat","There is no readable input file at %s",VINPUTFILE);
+   return true;
    }
 
 if (sb.st_mtime > PROMISETIME)
    {
+   CfOut(cf_verbose,""," -> Promises seem to change");
    return true;
    }
 
+// Check the directories first for speed and because non-input/data files should trigger an update
+
+snprintf(filename,CF_MAXVARSIZE,"%s/inputs",CFWORKDIR);
+MapName(filename);
+
+if (IsNewerFileTree(filename,PROMISETIME))
+   {
+   CfOut(cf_verbose,""," -> Quick search detected file changes");
+   return true;
+   }
+
+// Check files in case there are any abs paths
+
 if (VINPUTLIST != NULL)
    {
    for (rp = VINPUTLIST; rp != NULL; rp=rp->next)
@@ -602,7 +748,8 @@ if (VINPUTLIST != NULL)
 
                 if (cfstat(InputLocation((char *)returnval.item),&sb) == -1)
                    {
-                   CfOut(cf_error,"stat","There are no readable promise proposals at %s",(char *)returnval.item);
+                   CfOut(cf_error,"stat","Unreadable promise proposals at %s",(char *)returnval.item);
+                   result = true;
                    break;
                    }
 
@@ -619,7 +766,8 @@ if (VINPUTLIST != NULL)
                    {
                    if (cfstat(InputLocation((char *)sl->item),&sb) == -1)
                       {
-                      CfOut(cf_error,"stat","There are no readable promise proposals at %s",(char *)sl->item);
+                      CfOut(cf_error,"stat","Unreadable promise proposals at %s",(char *)sl->item);
+                      result = true;
                       break;
                       }
 
@@ -643,7 +791,7 @@ if (VINPUTLIST != NULL)
       }
    }
 
-return result;
+return result | ALWAYS_VALIDATE;
 }
 
 /*******************************************************************/
@@ -677,6 +825,8 @@ if (SHOWREPORTS)
       CfOut(cf_error,"fopen","Cannot open output file %s",name);
       FKNOW = fopen(NULLFILE,"w");
       }
+
+   CfOut(cf_inform,""," -> Writing knowledge output to %s",CFWORKDIR);
    }
 else
    {
@@ -719,17 +869,24 @@ void CloseReports(char *agents)
 
 { char name[CF_BUFSIZE];
 
+#ifndef HAVE_NOVA 
 if (SHOWREPORTS)
    {
-   CfOut(cf_cmdout,"","Wrote compilation report %s%creports%cpromise_output_%s.txt",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,agents);
-   CfOut(cf_cmdout,"","Wrote compilation report %s%creports%cpromise_output_%s.html",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,agents);
-   CfOut(cf_cmdout,"","Wrote knowledge map %s%cpromise_knowledge.cf",CFWORKDIR,FILE_SEPARATOR,agents);
+   CfOut(cf_verbose,"","Wrote compilation report %s%creports%cpromise_output_%s.txt",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,agents);
+   CfOut(cf_verbose,"","Wrote compilation report %s%creports%cpromise_output_%s.html",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR,agents);
+   CfOut(cf_verbose,"","Wrote knowledge map %s%cpromise_knowledge.cf",CFWORKDIR,FILE_SEPARATOR,agents);
    }
+#endif
 
 fprintf(FKNOW,"}\n");
 fclose(FKNOW);
 fclose(FREPORT_HTML);
 fclose(FREPORT_TXT);
+
+// Make the knowledge readable in situ
+
+snprintf(name,CF_BUFSIZE,"%s%cpromise_knowledge.cf",CFWORKDIR,FILE_SEPARATOR);
+chmod(name,0644);
 }
 
 /*******************************************************************/
@@ -738,13 +895,10 @@ fclose(FREPORT_TXT);
 
 void Cf3ParseFile(char *filename)
 
-{ FILE *save_yyin = yyin;
+{
   struct stat statbuf;
-  struct Rlist *rp;
-  int access = false;
   char wfilename[CF_BUFSIZE];
 
-
 strncpy(wfilename,InputLocation(filename),CF_BUFSIZE);
 
 if (cfstat(wfilename,&statbuf) == -1)
@@ -782,15 +936,17 @@ P.line_no = 1;
 P.line_pos = 1;
 P.list_nesting = 0;
 P.arg_nesting = 0;
-P.filename = strdup(wfilename);
+strncpy(P.filename,wfilename,CF_MAXVARSIZE);
 
-P.currentid = NULL;
+P.currentid[0] = '\0';
 P.currentstring = NULL;
-P.currenttype = NULL;
+P.currenttype[0] = '\0';
 P.currentclasses = NULL;
 P.currentRlist = NULL;
 P.currentpromise = NULL;
 P.promiser = NULL;
+P.blockid[0] = '\0';
+P.blocktype[0] = '\0';
 
 while (!feof(yyin))
    {
@@ -811,7 +967,6 @@ fclose (yyin);
 struct Constraint *ControlBodyConstraints(enum cfagenttype agent)
 
 { struct Body *body;
-  char scope[CF_BUFSIZE];
 
 for (body = BODIES; body != NULL; body = body->next)
    {
@@ -941,7 +1096,7 @@ CfOut(cf_verbose,"","***********************************************************
 
 if (VERBOSE || DEBUG)
    {
-   printf("%s BUNDLE %s",VPREFIX,bp->name);
+   printf("%s> BUNDLE %s",VPREFIX,bp->name);
    }
 
 if (params && (VERBOSE||DEBUG))
@@ -957,6 +1112,7 @@ else
 
 CfOut(cf_verbose,"","*****************************************************************\n");
 CfOut(cf_verbose,"","\n");
+LastSawBundle(bp->name);
 }
 
 /**************************************************************/
@@ -969,7 +1125,7 @@ CfOut(cf_verbose,"","      * * * * * * * * * * * * * * * * * * * * * * * * * * *
 
 if (VERBOSE || DEBUG)
    {
-   printf("%s       BUNDLE %s",VPREFIX,bp->name);
+   printf("%s>       BUNDLE %s",VPREFIX,bp->name);
    }
 
 if (params && (VERBOSE||DEBUG))
@@ -984,6 +1140,7 @@ else
    }
 CfOut(cf_verbose,"","      * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
 CfOut(cf_verbose,"","\n");
+LastSawBundle(bp->name);
 }
 
 /**************************************************************/
@@ -1006,8 +1163,8 @@ CfOut(cf_verbose,"","    .......................................................
 
 if (VERBOSE||DEBUG)
    {
-   printf ("%s     Promise handle: %s\n",VPREFIX,handle);
-   printf ("%s     Promise made by: %s",VPREFIX,pp->promiser);
+   printf ("%s>     Promise handle: %s\n",VPREFIX,handle);
+   printf ("%s>     Promise made by: %s",VPREFIX,pp->promiser);
    }
 
 if (pp->promisee)
@@ -1042,8 +1199,7 @@ void CheckWorkingDirectories()
 /* NOTE: We do not care about permissions (ACLs) in windows */
 
 { struct stat statbuf;
-  int result;
-  char *sp,vbuff[CF_BUFSIZE];
+  char vbuff[CF_BUFSIZE];
   char output[CF_BUFSIZE];
 
 Debug("CheckWorkingDirectories()\n");
@@ -1079,11 +1235,14 @@ if (cfstat(CFWORKDIR,&statbuf) != -1)
 snprintf(vbuff,CF_BUFSIZE,"%s%cstate%c.",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
 MakeParentDirectory(vbuff,false);
 
-snprintf(CFPRIVKEYFILE,CF_BUFSIZE,"%s%cppkeys%clocalhost.priv",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
-snprintf(CFPUBKEYFILE,CF_BUFSIZE,"%s%cppkeys%clocalhost.pub",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
+if (strlen(CFPRIVKEYFILE) == 0)
+   {
+   snprintf(CFPRIVKEYFILE,CF_BUFSIZE,"%s%cppkeys%clocalhost.priv",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
+   snprintf(CFPUBKEYFILE,CF_BUFSIZE,"%s%cppkeys%clocalhost.pub",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
+   }
 
 CfOut(cf_verbose,"","Checking integrity of the state database\n");
-snprintf(vbuff,CF_BUFSIZE,"%s%cstate",CFWORKDIR,FILE_SEPARATOR,FILE_SEPARATOR);
+snprintf(vbuff,CF_BUFSIZE,"%s%cstate",CFWORKDIR,FILE_SEPARATOR);
 
 if (cfstat(vbuff,&statbuf) == -1)
    {
@@ -1164,14 +1323,14 @@ char *InputLocation(char *filename)
 
 { static char wfilename[CF_BUFSIZE], path[CF_BUFSIZE];
 
-if (MINUSF && (filename != VINPUTFILE) && (*VINPUTFILE == '.' || IsAbsoluteFileName(VINPUTFILE)) && !IsAbsoluteFileName(filename))
+if (MINUSF && (filename != VINPUTFILE) && IsFileOutsideDefaultRepository(VINPUTFILE) && !IsAbsoluteFileName(filename))
    {
    /* If -f assume included relative files are in same directory */
    strncpy(path,VINPUTFILE,CF_BUFSIZE-1);
    ChopLastNode(path);
    snprintf(wfilename,CF_BUFSIZE-1,"%s%c%s",path,FILE_SEPARATOR,filename);
    }
-else if ((*filename == '.') || IsAbsoluteFileName(filename))
+else if (IsFileOutsideDefaultRepository(filename))
    {
    strncpy(wfilename,filename,CF_BUFSIZE-1);
    }
@@ -1187,25 +1346,17 @@ return MapName(wfilename);
 
 void CompilationReport(char *fname)
 
-{ char filename[CF_BUFSIZE],output[CF_BUFSIZE];
-
-snprintf(filename,CF_BUFSIZE-1,"%s.txt",fname);
-printf("Summarizing promises as text to %s\n",filename);
-
-if ((FREPORT_TXT = fopen(filename,"w")) == NULL)
+{
+if (THIS_AGENT_TYPE != cf_common)
    {
-   snprintf(output,CF_BUFSIZE,"Could not write output log to %s",filename);
-   FatalError(output);
+   return;
    }
 
-snprintf(filename,CF_BUFSIZE-1,"%s.html",fname);
-printf("Summarizing promises as html to %s\n",filename);
-
-if ((FREPORT_HTML = fopen(filename,"w")) == NULL)
-   {
-   snprintf(output,CF_BUFSIZE,"Could not write output log to %s",filename);
-   FatalError(output);
-   }
+#if defined(HAVE_NOVA)
+Nova_OpenCompilationReportFiles(fname);
+#else
+OpenCompilationReportFiles(fname);
+#endif
 
 if ((FKNOW = fopen(NULLFILE,"w")) == NULL)
    {
@@ -1219,19 +1370,39 @@ fclose(FREPORT_TXT);
 fclose(FKNOW);
 }
 
+void OpenCompilationReportFiles(const char *fname)
+{
+char filename[CF_BUFSIZE];
+
+snprintf(filename,CF_BUFSIZE-1,"%s.txt",fname);
+CfOut(cf_inform,"","Summarizing promises as text to %s\n",filename);
+
+if ((FREPORT_TXT = fopen(filename,"w")) == NULL)
+   {
+   FatalError("Could not write output log to %s",filename);
+   }
+
+snprintf(filename,CF_BUFSIZE-1,"%s.html",fname);
+CfOut(cf_inform,"","Summarizing promises as html to %s\n",filename);
+
+if ((FREPORT_HTML = fopen(filename,"w")) == NULL)
+   {
+   FatalError("Could not write output log to %s",filename);
+   }
+}
+
+
 /*******************************************************************/
 
 void VerifyPromises(enum cfagenttype agent)
 
-{ struct Bundle *bp,*bundles;
+{ struct Bundle *bp;
   struct SubType *sp;
   struct Promise *pp;
   struct Body *bdp;
-  struct Scope *ptr;
-  struct Rlist *rp,*params;
+  struct Rlist *rp;
   struct FnCall *fp;
-  char buf[CF_BUFSIZE], *scope;
-  char rettype,*name;
+  char *scope;
 
 Debug("\n\nVerifyPromises()\n");
 
@@ -1279,7 +1450,7 @@ for (rp = SUBBUNDLES; rp != NULL; rp=rp->next)
       {
       case CF_SCALAR:
           
-          if (!IGNORE_MISSING_BUNDLES && !IsBundle(BUNDLES,(char *)rp->item))
+          if (!IGNORE_MISSING_BUNDLES && !IsCf3VarString(rp->item) && !IsBundle(BUNDLES,(char *)rp->item))
              {
              CfOut(cf_error,"","Undeclared promise bundle \"%s()\" was referenced in a promise\n",(char *)rp->item);
              ERRORCOUNT++;
@@ -1290,7 +1461,7 @@ for (rp = SUBBUNDLES; rp != NULL; rp=rp->next)
 
           fp = (struct FnCall *)rp->item;
 
-          if (!IGNORE_MISSING_BUNDLES && !IsBundle(BUNDLES,fp->name))
+          if (!IGNORE_MISSING_BUNDLES && !IsCf3VarString(fp->name) && !IsBundle(BUNDLES,fp->name))
              {
              CfOut(cf_error,"","Undeclared promise bundle \"%s()\" was referenced in a promise\n",fp->name);
              ERRORCOUNT++;
@@ -1321,7 +1492,7 @@ for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
       }
    }
 
-HashVariables();
+HashVariables(NULL);
 HashControls();
 
 /* Now look once through the sequences bundles themselves */
@@ -1336,7 +1507,7 @@ if (BadBundleSequence(agent))
 
 void PrependAuditFile(char *file)
 
-{ struct stat statbuf;;
+{ struct stat statbuf;
 
 if ((AUDITPTR = (struct Audit *)malloc(sizeof(struct Audit))) == NULL)
    {
@@ -1349,7 +1520,7 @@ if (cfstat(file,&statbuf) == -1)
    return;
    }
 
-HashFile(file,AUDITPTR->digest,cf_md5);
+HashFile(file,AUDITPTR->digest,CF_DEFAULT_DIGEST);
 
 AUDITPTR->next = VAUDIT;
 AUDITPTR->filename = strdup(file);
@@ -1393,36 +1564,12 @@ for (pp = classlist; pp != NULL; pp=pp->next)
 
 /*******************************************************************/
 
-void CheckBundleParameters(char *scope,struct Rlist *args)
-
-{ struct Rlist *rp;
-  struct Rval retval;
-  char *lval,rettype;;
-
-for (rp = args; rp != NULL; rp = rp->next)
-   {
-   lval = (char *)rp->item;
-
-   if (GetVariable(scope,lval,(void *)&retval,&rettype) != cf_notype)
-      {
-      CfOut(cf_error,"","Variable and bundle parameter %s collide",lval);
-      FatalError("Aborting");
-      }
-   }
-}
-
-/*******************************************************************/
-
 void CheckControlPromises(char *scope,char *agent,struct Constraint *controllist)
 
 { struct Constraint *cp;
-  struct SubTypeSyntax *sp;
   struct BodySyntax *bp = NULL;
-  char *lval;
-  void *rval = NULL,*retval;
-  int i = 0,override = true;
+  int i = 0;
   struct Rval returnval;
-  char rettype,rtype;
 
 Debug("CheckControlPromises(%s)\n",agent);
 
@@ -1458,10 +1605,10 @@ for (cp = controllist; cp != NULL; cp=cp->next)
       }
 
    DeleteVariable(scope,cp->lval);
-   
+
    if (!AddVariableHash(scope,cp->lval,returnval.item,returnval.rtype,GetControlDatatype(cp->lval,bp),cp->audit->filename,cp->lineno))
       {
-      CfOut(cf_error,"","Rule from %s at/before line %d\n",cp->audit->filename,cp->lineno);
+      CfOut(cf_error,""," !! Rule from %s at/before line %d\n",cp->audit->filename,cp->lineno);
       }
 
    if (strcmp(cp->lval,CFG_CONTROLBODY[cfg_output_prefix].lval) == 0)
@@ -1546,12 +1693,11 @@ for (i=0; options[i].name != NULL; i++)
       }
    }
 
-
-printf("\nBug reports: bug-cfengine@cfengine.org, ");
-printf("Community help: help-cfengine@cfengine.org\n");
+printf("\nBug reports: http://bug.cfengine.com, ");
+printf("Community help: http://forum.cfengine.com\n");
 printf("Community info: http://www.cfengine.org, ");
 printf("Support services: http://www.cfengine.com\n\n");
-printf("This software is Copyright (C) 2008-present Cfengine AS.\n");
+printf("This software is Copyright (C) 2008,2010-present Cfengine AS.\n");
 }
 
 /*******************************************************************/
@@ -1594,8 +1740,8 @@ printf(".SH AUTHOR\n"
        "Mark Burgess and Cfengine AS\n"
        ".SH INFORMATION\n");
 
-printf("\nBug reports: bug-cfengine@cfengine.org\n");
-printf(".pp\nCommunity help: help-cfengine@cfengine.org\n");
+printf("\nBug reports: http://bug.cfengine.com, ");
+printf(".pp\nCommunity help: http://forum.cfengine.com\n");
 printf(".pp\nCommunity info: http://www.cfengine.org\n");
 printf(".pp\nSupport services: http://www.cfengine.com\n");
 printf(".pp\nThis software is Copyright (C) 2008- Cfengine AS.\n");
@@ -1631,16 +1777,20 @@ fclose(fp);
 
 /*******************************************************************/
 
-void HashVariables()
+void HashVariables(char *name)
 
-{ struct Bundle *bp,*bundles;
+{ struct Bundle *bp;
   struct SubType *sp;
-  struct Scope *ptr;
 
 CfOut(cf_verbose,"","Initiate variable convergence...\n");
-
+    
 for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
    {
+   if (name && strcmp(name,bp->name) != 0)
+      {
+      continue;
+      }
+   
    SetNewScope(bp->name);
    THIS_BUNDLE = bp->name;
 
@@ -1657,9 +1807,12 @@ for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
          {
          CheckCommonClassPromises(sp->promiselist);
          }
-      }
 
-   CheckBundleParameters(bp->name,bp->args);
+      if (THIS_AGENT_TYPE == cf_common)
+         {
+         CheckBundleParameters(bp->name,bp->args);
+         }
+      }
    }
 }
 
@@ -1689,7 +1842,7 @@ for (bdp = BODIES; bdp != NULL; bdp = bdp->next) /* get schedule */
 
 void UnHashVariables()
 
-{ struct Bundle *bp,*bundles;
+{ struct Bundle *bp;
 
 for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
    {
@@ -1707,7 +1860,9 @@ int BadBundleSequence(enum cfagenttype agent)
   int ok = true;
   struct FnCall *fp;
 
-if (THIS_AGENT_TYPE != cf_agent && THIS_AGENT_TYPE != cf_know && THIS_AGENT_TYPE != cf_common)
+if ((THIS_AGENT_TYPE != cf_agent) && 
+    (THIS_AGENT_TYPE != cf_know) && 
+    (THIS_AGENT_TYPE != cf_common))
    {
    return false;
    }
@@ -1736,7 +1891,7 @@ if (rettype != CF_LIST)
    FatalError("Promised bundlesequence was not a list");
    }
 
-if (agent == cf_agent || agent == cf_common)
+if ((agent == cf_agent) || (agent == cf_common))
    {
    for (rp = (struct Rlist *)retval; rp != NULL; rp=rp->next)
       {
@@ -1763,6 +1918,11 @@ if (agent == cf_agent || agent == cf_common)
              break;
          }
 
+      if (strcmp(name,CF_NULL_VALUE) == 0)
+         {
+         continue;
+         }             
+
       if (!IGNORE_MISSING_BUNDLES && !GetBundle(name,NULL))
          {
          CfOut(cf_error,"","Bundle \"%s\" listed in the bundlesequence is not a defined bundle\n",name);
@@ -1782,3 +1942,24 @@ if (agent == cf_agent || agent == cf_common)
 
 return false;
 }
+
+/*******************************************************************/
+
+void CheckBundleParameters(char *scope,struct Rlist *args)
+
+{ struct Rlist *rp;
+  struct Rval retval;
+  char *lval,rettype;
+
+for (rp = args; rp != NULL; rp = rp->next)
+   {
+   lval = (char *)rp->item;
+   
+   if (GetVariable(scope,lval,(void *)&retval,&rettype) != cf_notype)
+      {
+      CfOut(cf_error,"","Variable and bundle parameter \"%s\" collide in scope \"%s\"",lval,scope);
+      FatalError("Aborting");
+      }
+   }
+}
+
