@@ -65,8 +65,6 @@ return false;
 struct Bundle *AppendBundle(struct Bundle **start,char *name, char *type, struct Rlist *args)
 
 { struct Bundle *bp,*lp;
-  char *sp;
-  struct Rlist *rp;
 
 if (INSTALL_SKIP)
    {
@@ -89,12 +87,6 @@ if ((bp = (struct Bundle *)malloc(sizeof(struct Bundle))) == NULL)
    FatalError("");
    }
 
-if ((sp = strdup(name)) == NULL)
-   {
-   CfOut(cf_error,"malloc","Unable to allocate Bundle");
-   FatalError("");
-   }
-
 if (*start == NULL)
    {
    *start = bp;
@@ -108,9 +100,9 @@ else
    lp->next = bp;
    }
 
-bp->name = sp;
+bp->name = strdup(name);
 bp->next = NULL;
-bp->type = type;
+bp->type = strdup(type);
 bp->args = args;
 bp->subtypes = NULL;
 return bp;
@@ -121,7 +113,6 @@ return bp;
 struct Body *AppendBody(struct Body **start,char *name, char *type, struct Rlist *args)
 
 { struct Body *bp,*lp;
-  char *sp;
   struct Rlist *rp;
 
 Debug("Appending new promise body %s %s(",type,name);
@@ -140,12 +131,6 @@ if ((bp = (struct Body *)malloc(sizeof(struct Body))) == NULL)
    FatalError("");
    }
 
-if ((sp = strdup(name)) == NULL)
-   {
-   CfOut(cf_error,"malloc","Unable to allocate Body");
-   FatalError("");
-   }
-
 if (*start == NULL)
    {
    *start = bp;
@@ -159,9 +144,9 @@ else
    lp->next = bp;
    }
 
-bp->name = sp;
+bp->name = strdup(name);
 bp->next = NULL;
-bp->type = type;
+bp->type = strdup(type);
 bp->args = args;
 bp->conlist = NULL;
 return bp;
@@ -232,6 +217,7 @@ struct Promise *AppendPromise(struct SubType *type,char *promiser, void *promise
 
 { struct Promise *pp,*lp;
   char *sp = NULL,*spe = NULL;
+  char output[CF_BUFSIZE];
 
 if (INSTALL_SKIP)
    {
@@ -277,11 +263,6 @@ else
       }
    }
 
-if ((strcmp(spe,"any") == 0) && (strcmp(type->name,"reports") == 0))
-   {
-   yyerror("reports promises may not be in class \'any\' - risk of a notification explosion");
-   }
-
 if (strcmp(type->name,"classes") == 0 || strcmp(type->name,"vars") == 0)
    {
    if (isdigit(*promiser) && Str2Int(promiser) != CF_NOINT)
@@ -289,7 +270,16 @@ if (strcmp(type->name,"classes") == 0 || strcmp(type->name,"vars") == 0)
       yyerror("Variable or class identifier is purely numerical, which is not allowed");
       }
    }
-   
+
+if (strcmp(type->name,"vars") == 0)
+   {
+   if (!CheckParseVariableName(promiser))
+      {
+      snprintf(output,CF_BUFSIZE,"Use of a reserved or illegal variable name \"%s\" ",promiser);
+      ReportError(output);      
+      }
+   }
+
 if (type->promiselist == NULL)
    {
    type->promiselist = pp;
@@ -319,9 +309,9 @@ pp->cache = NULL;
 pp->conn = NULL;
 pp->inode_cache = NULL;
 
-pp->bundletype = bundletype;   /* cache agent,common,server etc*/
-pp->agentsubtype = type->name; /* Cache the typename */
-pp->ref = NULL;                /* cache a reference if given*/
+pp->bundletype = strdup(bundletype); /* cache agent,common,server etc*/
+pp->agentsubtype = type->name;       /* Cache the typename */
+pp->ref = NULL;                      /* cache a reference if given*/
 pp->ref_alloc = 'n';
 pp->next = NULL;
 return pp;
@@ -349,8 +339,14 @@ if (bp->name != NULL)
    free(bp->name);
    }
 
+if (bp->type != NULL)
+   {
+   free(bp->type);
+   }
+
 DeleteRlist(bp->args);
 DeleteSubTypes(bp->subtypes);
+free(bp);
 }
 
 /*******************************************************************/
@@ -374,6 +370,8 @@ if (tp->name != NULL)
    {
    free(tp->name);
    }
+
+free(tp);
 }
 
 /*******************************************************************/
@@ -396,6 +394,12 @@ if (bp->name != NULL)
    free(bp->name);
    }
 
+if (bp->type != NULL)
+   {
+   free(bp->type);
+   }
+
 DeleteRlist(bp->args);
 DeleteConstraintList(bp->conlist);
+free(bp);
 }
