@@ -47,12 +47,12 @@ struct BodySyntax CF_TRANSACTION_BODY[] =
    {"log_priority",cf_opts,"emergency,alert,critical,error,warning,notice,info,debug","The priority level of the log message, as interpreted by a syslog server"},
    {"log_repaired",cf_str,CF_LOGRANGE,"This should be filename of a file to which log_string will be saved, if undefined it goes to the system logger"},
    {"log_failed",cf_str,CF_LOGRANGE,"This should be filename of a file to which log_string will be saved, if undefined it goes to the system logger"},
-   {"value_kept",cf_real,"","A real number value attributed to keeping this promise"},
-   {"value_repaired",cf_real,"","A real number value attributed to reparing this promise"},
-   {"value_notkept",cf_real,"","A real number value (possibly negative) attributed to not keeping this promise"},
+   {"value_kept",cf_real,CF_REALRANGE,"A real number value attributed to keeping this promise"},
+   {"value_repaired",cf_real,CF_REALRANGE,"A real number value attributed to reparing this promise"},
+   {"value_notkept",cf_real,CF_REALRANGE,"A real number value (possibly negative) attributed to not keeping this promise"},
    {"audit",cf_opts,CF_BOOL,"true/false switch for detailed audit records of this promise"},
    {"background",cf_opts,CF_BOOL,"true/false switch for parallelizing the promise repair"},
-   {"report_level",cf_opts,"inform,verbose,error,log","The reporting level for standard output"},
+   {"report_level",cf_opts,"inform,verbose,error,log","The reporting level for standard output for this promise"},
    {"measurement_class",cf_str,"","If set performance will be measured and recorded under this identifier"},
    {NULL,cf_notype,NULL,NULL}
    };
@@ -69,6 +69,9 @@ struct BodySyntax CF_DEFINECLASS_BODY[] =
    {"cancel_kept",cf_slist,CF_IDRANGE,"A list of classes to be cancelled if the promise is kept"},
    {"cancel_repaired",cf_slist,CF_IDRANGE,"A list of classes to be cancelled if the promise is repaired"},
    {"cancel_notkept",cf_slist,CF_IDRANGE,"A list of classes to be cancelled if the promise is not kept for any reason"},
+   {"kept_returncodes",cf_slist,CF_INTLISTRANGE,"A list of return codes indicating a kept command-related promise"},
+   {"repaired_returncodes",cf_slist,CF_INTLISTRANGE,"A list of return codes indicating a repaired command-related promise"},
+   {"failed_returncodes",cf_slist,CF_INTLISTRANGE,"A list of return codes indicating a failed command-related promise"},
    {"persist_time",cf_int,CF_VALRANGE,"A number of minutes the specified classes should remain active"},
    {"timer_policy",cf_opts,"absolute,reset","Whether a persistent class restarts its counter when rediscovered"},
    {NULL,cf_notype,NULL,NULL}
@@ -95,9 +98,10 @@ struct BodySyntax CF_CLASSBODY[] =
    {"or",cf_clist,CF_CLASSRANGE,"Combine class sources with inclusive OR"}, 
    {"and",cf_clist,CF_CLASSRANGE,"Combine class sources with AND"},
    {"xor",cf_clist,CF_CLASSRANGE,"Combine class sources with XOR"},
-   {"dist",cf_rlist,CF_REALRANGE,"Generate a probabilistic class distribution (strategy in cfengine 2)"},
+   {"dist",cf_rlist,CF_REALRANGE,"Generate a probabilistic class distribution (from strategies in cfengine 2)"},
    {"expression",cf_class,CF_CLASSRANGE,"Evaluate string expression of classes in normal form"},
    {"not",cf_class,CF_CLASSRANGE,"Evaluate the negation of string expression in normal form"},
+   {"select_class",cf_rlist,CF_CLASSRANGE,"Select one of the named list of classes to define based on host identity"},
    {NULL,cf_notype,NULL,NULL}
    };
 
@@ -119,6 +123,7 @@ struct BodySyntax CFG_CONTROLBODY[] =
    {"host_licenses_paid",cf_int,CF_VALRANGE,"The number of licenses that you promise to have paid for by setting this value (legally binding for commercial license)"},
    {"syslog_host",cf_str,CF_IPRANGE,"The name or address of a host to which syslog messages should be sent directly by UDP"},
    {"syslog_port",cf_int,CF_VALRANGE,"The port number of a UDP syslog service"},
+   {"fips_mode",cf_opts,CF_BOOL,"Activate full FIPS mode restrictions"},
    {NULL,cf_notype,NULL,NULL}
    };
 
@@ -129,6 +134,7 @@ struct BodySyntax CFA_CONTROLBODY[] =
    {"addclasses",cf_slist,".*","A list of classes to be defined always in the current context"},
    {"agentaccess",cf_slist,".*","A list of user names allowed to execute cf-agent"},
    {"agentfacility",cf_opts,CF_FACILITY,"The syslog facility for cf-agent"},
+   {"alwaysvalidate",cf_opts,CF_BOOL,"true/false flag to determine whether configurations will always be checked before executing, or only after updates"},
    {"auditing",cf_opts,CF_BOOL,"true/false flag to activate the cf-agent audit log"},
    {"binarypaddingchar",cf_str,"","Character used to pad unequal replacements in binary editing"},
    {"bindtointerface",cf_str,".*","Use this interface for outgoing connections"},
@@ -153,12 +159,13 @@ struct BodySyntax CFA_CONTROLBODY[] =
    {"mountfilesystems",cf_opts,CF_BOOL,"true/false mount any filesystems promised"},
    {"nonalphanumfiles",cf_opts,CF_BOOL,"true/false warn about filenames with no alphanumeric content"},
    {"repchar",cf_str,".","The character used to canonize pathnames in the file repository"},
+   {"refresh_processes",cf_slist,CF_IDRANGE,"Reload the process table before verifying the bundles named in this list (lazy evaluation)"},
    {"default_repository",cf_str,CF_PATHRANGE,"Path to the default file repository"},
    {"secureinput",cf_opts,CF_BOOL,"true/false check whether input files are writable by unauthorized users"},
    {"sensiblecount",cf_int,CF_VALRANGE,"Minimum number of files a mounted filesystem is expected to have"},
    {"sensiblesize",cf_int,CF_VALRANGE,"Minimum number of bytes a mounted filesystem is expected to have"},
    {"skipidentify",cf_opts,CF_BOOL,"Do not send IP/name during server connection because address resolution is broken"},
-   {"suspiciousnames",cf_slist,"List of names to warn about if found during any file search"},
+   {"suspiciousnames",cf_slist,"","List of names to warn about if found during any file search"},
    {"syslog",cf_opts,CF_BOOL,"true/false switches on output to syslog at the inform level"},
    {"track_value",cf_opts,CF_BOOL,"true/false switches on tracking of promise valuation"},
    {"timezone",cf_slist,"","List of allowed timezones this machine must comply with"},
@@ -169,23 +176,24 @@ struct BodySyntax CFA_CONTROLBODY[] =
 
 struct BodySyntax CFS_CONTROLBODY[] =
    {
-   {"cfruncommand",cf_str,CF_PATHRANGE,"Path to the cf-agent command or cf-execd wrapper for remote execution"},
-   {"maxconnections",cf_int,CF_VALRANGE,"Maximum number of connections that will be accepted by cf-serverd"},
-   {"denybadclocks",cf_opts,CF_BOOL,"true/false accept connections from hosts with clocks that are out of sync"},
-   {"allowconnects",cf_slist,"","List of IPs or hostnames that may connect to the server port"},
-   {"denyconnects",cf_slist,"","List of IPs or hostnames that may NOT connect to the server port"},
    {"allowallconnects",cf_slist,"","List of IPs or hostnames that may have more than one connection to the server port"},
-   {"trustkeysfrom",cf_slist,"","List of IPs from whom we accept public keys on trust"},
+   {"allowconnects",cf_slist,"","List of IPs or hostnames that may connect to the server port"},
    {"allowusers",cf_slist,"","List of usernames who may execute requests from this server"},
-   {"dynamicaddresses",cf_slist,"","List of IPs or hostnames for which the IP/name binding is expected to change"},
-   {"skipverify",cf_slist,"","List of IPs or hostnames for which we expect no DNS binding and cannot verify"},
-   {"logallconnections",cf_opts,CF_BOOL,"true/false causes the server to log all new connections to syslog"},
-   {"logencryptedtransfers",cf_opts,CF_BOOL,"true/false log all successful transfers required to be encrypted"},
-   {"hostnamekeys",cf_opts,CF_BOOL,"true/false store keys using hostname lookup instead of IP addresses"},
    {"auditing",cf_opts,CF_BOOL,"true/false activate auditing of server connections"},
    {"bindtointerface",cf_str,"","IP of the interface to which the server should bind on multi-homed hosts"},
-   {"serverfacility",cf_opts,CF_FACILITY,"Menu option for syslog facility level"},
+   {"cfruncommand",cf_str,CF_PATHRANGE,"Path to the cf-agent command or cf-execd wrapper for remote execution"},
+   {"denybadclocks",cf_opts,CF_BOOL,"true/false accept connections from hosts with clocks that are out of sync"},
+   {"denyconnects",cf_slist,"","List of IPs or hostnames that may NOT connect to the server port"},
+   {"dynamicaddresses",cf_slist,"","List of IPs or hostnames for which the IP/name binding is expected to change"},
+   {"hostnamekeys",cf_opts,CF_BOOL,"true/false store keys using hostname lookup instead of IP addresses"},
+   {"keycacheTTL",cf_int,CF_VALRANGE,"Maximum number of hours to hold public keys in the cache"},
+   {"logallconnections",cf_opts,CF_BOOL,"true/false causes the server to log all new connections to syslog"},
+   {"logencryptedtransfers",cf_opts,CF_BOOL,"true/false log all successful transfers required to be encrypted"},
+   {"maxconnections",cf_int,CF_VALRANGE,"Maximum number of connections that will be accepted by cf-serverd"},
    {"port",cf_int,"1024,99999","Default port for cfengine server"},
+   {"serverfacility",cf_opts,CF_FACILITY,"Menu option for syslog facility level"},
+   {"skipverify",cf_slist,"","List of IPs or hostnames for which we expect no DNS binding and cannot verify"},
+   {"trustkeysfrom",cf_slist,"","List of IPs from whom we accept public keys on trust"},
    {NULL,cf_notype,NULL,NULL}
    };
 
@@ -194,7 +202,7 @@ struct BodySyntax CFM_CONTROLBODY[] =
    {
    {"forgetrate",cf_real,"0,1","Decimal fraction [0,1] weighting of new values over old in 2d-average computation"},
    {"monitorfacility",cf_opts,CF_FACILITY,"Menu option for syslog facility"},
-   {"histograms",cf_opts,CF_BOOL,"true/false store signal histogram data"},
+   {"histograms",cf_opts,CF_BOOL,"Ignored, kept for backward compatibility"},
    {"tcpdump",cf_opts,CF_BOOL,"true/false use tcpdump if found"},
    {"tcpdumpcommand",cf_str,CF_PATHRANGE,"Path to the tcpdump command on this system"},
    {NULL,cf_notype,NULL,NULL}
@@ -210,6 +218,7 @@ struct BodySyntax CFR_CONTROLBODY[] =
    {"background_children",cf_opts,CF_BOOL,"true/false parallelize connections to servers"},
    {"max_children",cf_int,CF_VALRANGE,"Maximum number of simultaneous connections to attempt"},
    {"output_to_file",cf_opts,CF_BOOL,"true/false whether to send collected output to file(s)"},
+   {"timeout",cf_int,"1,9999","Connection timeout, sec"},
    {NULL,cf_notype,NULL,NULL}
    };
 
@@ -229,9 +238,12 @@ struct BodySyntax CFEX_CONTROLBODY[] = /* enum cfexcontrol */
 struct BodySyntax CFK_CONTROLBODY[] =
    {
    {"build_directory",cf_str,".*","The directory in which to generate output files"},
+   {"document_root",cf_str,".*","The directory in which the web root resides"},
    {"generate_manual",cf_opts,CF_BOOL,"true/false generate texinfo manual page skeleton for this version"},
    {"graph_directory",cf_str,CF_PATHRANGE,"Path to directory where rendered .png files will be created"},
    {"graph_output",cf_opts,CF_BOOL,"true/false generate png visualization of topic map if possible (requires lib)"},
+   {"goal_categories",cf_slist,"","A list of context names that represent parent categories for goals (goal patterns)"},
+   {"goal_patterns",cf_slist,"","A list of regular expressions that match promisees/topics considered to be organizational goals"},
    {"html_banner",cf_str,"","HTML code for a banner to be added to rendered in html after the header"},
    {"html_footer",cf_str,"","HTML code for a page footer to be added to rendered in html before the end body tag"},
    {"id_prefix",cf_str,".*","The LTM identifier prefix used to label topic maps (used for disambiguation in merging)"},
@@ -267,6 +279,15 @@ struct BodySyntax CFRE_CONTROLBODY[] = /* enum cfrecontrol */
    {NULL,cf_notype,NULL,NULL}
    };
 
+struct BodySyntax CFH_CONTROLBODY[] = /* enum cfh_control */
+   {
+   {"export_zenoss",cf_opts,CF_BOOL,"Make data available for Zenoss integration in docroot/reports/summary.z"},
+   {"federation",cf_slist,"","The list of cfengine servers supporting constellation integration with this hub"},
+   {"hub_schedule",cf_slist,"","The class schedule used by cf-hub for report collation"},
+   {"port",cf_int,"1024,99999","Default port for contacting hub nodes"},
+   {NULL,cf_notype,NULL,NULL}
+   };
+
 
 /*********************************************************/
 
@@ -282,6 +303,7 @@ struct SubTypeSyntax CF_ALL_BODIES[] =
    {CF_EXECC,"control",CFEX_CONTROLBODY},
    {CF_KNOWC,"control",CFK_CONTROLBODY},
    {CF_REPORTC,"control",CFRE_CONTROLBODY},
+   {CF_HUBC,"control",CFH_CONTROLBODY},
 
    //  get others from modules e.g. "agent","files",CF_FILES_BODIES,
 
@@ -321,7 +343,7 @@ struct SubTypeSyntax CF_COMMON_SUBTYPES[] =
      {"*","vars",CF_VARBODY},
      {"*","classes",CF_CLASSBODY},
      {"*","reports",CF_REPORT_BODIES},
-     {"agent","*",CF_COMMON_BODIES},
+     {"*","*",CF_COMMON_BODIES},
      {NULL,NULL,NULL}
      };
 
