@@ -67,6 +67,17 @@ struct FnCallArg AGO_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg LATERTHAN_ARGS[] =
+    {
+    {"0,1000",cf_int,"Years"},
+    {"0,1000",cf_int,"Months"},    
+    {"0,1000",cf_int,"Days"},      
+    {"0,1000",cf_int,"Hours"},     
+    {"0,1000",cf_int,"Minutes"},   
+    {"0,40000",cf_int,"Seconds"},   
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg CANONIFY_ARGS[] =
     {
     {CF_ANYSTRING,cf_str,"String containing non-identifer characters"},
@@ -153,6 +164,13 @@ struct FnCallArg GETINDICES_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg GETUSERS_ARGS[] =
+    {
+    {CF_ANYSTRING,cf_str,"Comma separated list of User names"},
+    {CF_ANYSTRING,cf_str,"Comma separated list of UserID numbers"},
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg GETENV_ARGS[] =
    {
    {CF_IDRANGE,cf_str,"Name of environment variable"},
@@ -206,6 +224,12 @@ struct FnCallArg HOST2IP_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg IP2HOST_ARGS[] =
+    {
+    {CF_ANYSTRING,cf_str,"IP address (IPv4 or IPv6)"},
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg HOSTINNETGROUP_ARGS[] =
     {
     {CF_ANYSTRING,cf_str,"Host name"},
@@ -227,8 +251,6 @@ struct FnCallArg HOSTSSEEN_ARGS[] =
     {"name,address",cf_opts,"Type of return value desired"},
     {NULL,cf_notype,NULL}
     };
-
-
     
 struct FnCallArg IPRANGE_ARGS[] =
     {
@@ -322,6 +344,18 @@ struct FnCallArg NOW_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg SUM_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"A list of arbitrary real values"},
+    {NULL,cf_notype,NULL}
+    };
+
+struct FnCallArg PRODUCT_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"A list of arbitrary real values"},
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg DATE_ARGS[] =
     {
     {"1970,3000",cf_int,"Year"},
@@ -383,6 +417,39 @@ struct FnCallArg READSTRINGARRAY_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg PARSESTRINGARRAY_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"Array identifer to populate"},
+    {CF_PATHRANGE,cf_str,"A string to parse for input data"},
+    {CF_ANYSTRING,cf_str,"Regex matching comments"},
+    {CF_ANYSTRING,cf_str,"Regex to split data"},
+    {CF_VALRANGE,cf_int,"Maximum number of entries to read"},
+    {CF_VALRANGE,cf_int,"Maximum bytes to read"},
+    {NULL,cf_notype,NULL}
+    };
+
+struct FnCallArg READSTRINGARRAYIDX_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"Array identifer to populate"},
+    {CF_PATHRANGE,cf_str,"A string to parse for input data"},
+    {CF_ANYSTRING,cf_str,"Regex matching comments"},
+    {CF_ANYSTRING,cf_str,"Regex to split data"},
+    {CF_VALRANGE,cf_int,"Maximum number of entries to read"},
+    {CF_VALRANGE,cf_int,"Maximum bytes to read"},
+    {NULL,cf_notype,NULL}
+    };
+
+struct FnCallArg PARSESTRINGARRAYIDX_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"Array identifer to populate"},
+    {CF_PATHRANGE,cf_str,"File name to read"},
+    {CF_ANYSTRING,cf_str,"Regex matching comments"},
+    {CF_ANYSTRING,cf_str,"Regex to split data"},
+    {CF_VALRANGE,cf_int,"Maximum number of entries to read"},
+    {CF_VALRANGE,cf_int,"Maximum bytes to read"},
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg READSTRINGLIST_ARGS[] =
     {
     {CF_PATHRANGE,cf_str,"File name to read"},
@@ -465,6 +532,12 @@ struct FnCallArg REMOTESCALAR_ARGS[] =
     {NULL,cf_notype,NULL}
     };
 
+struct FnCallArg HUB_KNOWLEDGE_ARGS[] =
+    {
+    {CF_IDRANGE,cf_str,"Variable identifier"},
+    {NULL,cf_notype,NULL}
+    };
+
 struct FnCallArg REMOTECLASSESMATCHING_ARGS[] =
     {
     {CF_ANYSTRING,cf_str,"Regular expression"},
@@ -563,22 +636,28 @@ struct FnCallType CF_FNCALL_TYPES[] =
    {"execresult",cf_str,2,EXECRESULT_ARGS,"Execute named command and assign output to variable"},
    {"fileexists",cf_class,1,FILESTAT_ARGS,"True if the named file can be accessed"},
    {"filesexist",cf_class,1,FILESEXIST_ARGS,"True if the named list of files can ALL be accessed"},
-   {"getfields",cf_int,4,GETFIELDS_ARGS,"Get an array of fields in the lines matching regex arg1 in file arg2, split on regex arg3 as array name arg4"},
-   {"getindices",cf_slist,1,GETINDICES_ARGS,"Get a list of keys to the array whose id is the argument and assign to variable"},
+   {"filesize",cf_int,1,FILESTAT_ARGS,"Returns the size in bytes of the file"},
    {"getenv",cf_str,2,GETENV_ARGS,"Return the environment variable named arg1, truncated at arg2 characters"},
+   {"getfields",cf_int,4,GETFIELDS_ARGS,"Get an array of fields in the lines matching regex arg1 in file arg2, split on regex arg3 as array name arg4"},
    {"getgid",cf_int,1,GETGID_ARGS,"Return the integer group id of the named group on this host"},
+   {"getindices",cf_slist,1,GETINDICES_ARGS,"Get a list of keys to the array whose id is the argument and assign to variable"},
    {"getuid",cf_int,1,GETUID_ARGS,"Return the integer user id of the named user on this host"},
-   {"grep",cf_str,2,GREP_ARGS,"Extract the sub-list if items matching the regular expression in arg1 of the list named in arg2"},
+   {"getusers",cf_slist,2,GETUSERS_ARGS,"Get a list of all system users defined, minus those names defined in args 1 and uids in args"},
+   {"getvalues",cf_slist,1,GETINDICES_ARGS,"Get a list of values corresponding to the right hand sides in an array whose id is the argument and assign to variable"},
+   {"grep",cf_slist,2,GREP_ARGS,"Extract the sub-list if items matching the regular expression in arg1 of the list named in arg2"},
    {"groupexists",cf_class,1,GROUPEXISTS_ARGS,"True if group or numerical id exists on this host"},
    {"hash",cf_str,2,HASH_ARGS,"Return the hash of arg1, type arg2 and assign to a variable"},
    {"hashmatch",cf_class,3,HASHMATCH_ARGS,"Compute the hash of arg1, of type arg2 and test if it matches the value in arg 3"},
    {"host2ip",cf_str,1,HOST2IP_ARGS,"Returns the primary name-service IP address for the named host"},
+   {"ip2host",cf_str,1,IP2HOST_ARGS,"Returns the primary name-service host name for the IP address"},
    {"hostinnetgroup",cf_class,1,HOSTINNETGROUP_ARGS,"True if the current host is in the named netgroup"},
    {"hostrange",cf_class,2,HOSTRANGE_ARGS,"True if the current host lies in the range of enumerated hostnames specified"},
-   {"hostsseen",cf_str,3,HOSTSSEEN_ARGS,"Extract the list of hosts last seen/not seen within the last arg1 hours"},
+   {"hostsseen",cf_slist,3,HOSTSSEEN_ARGS,"Extract the list of hosts last seen/not seen within the last arg1 hours"},
+   {"hubknowledge",cf_str,1,HUB_KNOWLEDGE_ARGS,"Read global knowledge from the hub host by id (commercial extension)"},
    {"iprange",cf_class,1,IPRANGE_ARGS,"True if the current host lies in the range of IP addresses specified"},
    {"irange",cf_irange,2,IRANGE_ARGS,"Define a range of integer values for cfengine internal use"},
    {"isdir",cf_class,1,FILESTAT_ARGS,"True if the named object is a directory"},
+   {"isexecutable",cf_class,1,FILESTAT_ARGS,"True if the named object has execution rights for the current user"},
    {"isgreaterthan",cf_class,2,ISGREATERTHAN_ARGS,"True if arg1 is numerically greater than arg2, else compare strings like strcmp"},
    {"islessthan",cf_class,2,ISLESSTHAN_ARGS,"True if arg1 is numerically less than arg2, else compare strings like NOT strcmp"},
    {"islink",cf_class,1,FILESTAT_ARGS,"True if the named object is a symbolic link"},
@@ -587,14 +666,20 @@ struct FnCallType CF_FNCALL_TYPES[] =
    {"isvariable",cf_class,1,ISVARIABLE_ARGS,"True if the named variable is defined"},
    {"join",cf_str,2,JOIN_ARGS,"Join the items of arg2 into a string, using the conjunction in arg1"},
    {"lastnode",cf_str,2,LASTNODE_ARGS,"Extract the last of a separated string, e.g. filename from a path"},
+   {"laterthan",cf_class,6,LATERTHAN_ARGS,"True if the current time is later than the given date"},
    {"ldaparray",cf_class,6,LDAPARRAY_ARGS,"Extract all values from an ldap record"},
    {"ldaplist",cf_slist,6,LDAPLIST_ARGS,"Extract all named values from multiple ldap records"},
    {"ldapvalue",cf_str,6,LDAPVALUE_ARGS,"Extract the first matching named value from ldap"},
    {"now",cf_int,0,NOW_ARGS,"Convert the current time into system representation"},
    {"on",cf_int,6,DATE_ARGS,"Convert an exact date/time to an integer system representation"},
+   {"parseintarray",cf_int,6,PARSESTRINGARRAY_ARGS,"Read an array of integers from a file and assign the dimension to a variable"},
+   {"parserealarray",cf_int,6,PARSESTRINGARRAY_ARGS,"Read an array of real numbers from a file and assign the dimension to a variable"},
+   {"parsestringarray",cf_int,6,PARSESTRINGARRAY_ARGS,"Read an array of strings from a file and assign the dimension to a variable"},
+   {"parsestringarrayidx",cf_int,6,PARSESTRINGARRAYIDX_ARGS,"Read an array of strings from a file and assign the dimension to a variable with integer indeces"},
    {"peers",cf_slist,3,PEERS_ARGS,"Get a list of peers (not including ourself) from the partition to which we belong"},
    {"peerleader",cf_str,3,PEERLEADER_ARGS,"Get the assigned peer-leader of the partition to which we belong"},
    {"peerleaders",cf_slist,3,PEERLEADERS_ARGS,"Get a list of peer leaders from the named partitioning"},
+   {"product",cf_real,1,PRODUCT_ARGS,"Return the product of a list of reals"},
    {"randomint",cf_int,2,RANDOMINT_ARGS,"Generate a random integer between the given limits"},
    {"readfile",cf_str,2,READFILE_ARGS,"Read max number of bytes from named file and assign to variable"},
    {"readintarray",cf_int,6,READSTRINGARRAY_ARGS,"Read an array of integers from a file and assign the dimension to a variable"},
@@ -602,6 +687,7 @@ struct FnCallType CF_FNCALL_TYPES[] =
    {"readrealarray",cf_int,6,READSTRINGARRAY_ARGS,"Read an array of real numbers from a file and assign the dimension to a variable"},
    {"readreallist",cf_rlist,5,READSTRINGLIST_ARGS,"Read and assign a list variable from a file of separated real numbers"},
    {"readstringarray",cf_int,6,READSTRINGARRAY_ARGS,"Read an array of strings from a file and assign the dimension to a variable"},
+   {"readstringarrayidx",cf_int,6,READSTRINGARRAYIDX_ARGS,"Read an array of strings from a file and assign the dimension to a variable with integer indeces"},
    {"readstringlist",cf_slist,5,READSTRINGLIST_ARGS,"Read and assign a list variable from a file of separated strings"},
    {"readtcp",cf_str,4,READTCP_ARGS,"Connect to tcp port, send string and assign result to variable"},
    {"regarray",cf_class,2,REGARRAY_ARGS,"True if arg1 matches any item in the associative array with id=arg2"},
@@ -619,6 +705,7 @@ struct FnCallType CF_FNCALL_TYPES[] =
    {"splayclass",cf_class,2,SPLAYCLASS_ARGS,"True if the first argument's time-slot has arrived, according to a policy in arg2"},
    {"splitstring",cf_slist,3,SPLITSTRING_ARGS,"Convert a string in arg1 into a list of max arg3 strings by splitting on a regular expression in arg2"},
    {"strcmp",cf_class,2,STRCMP_ARGS,"True if the two strings match exactly"},
+   {"sum",cf_real,1,SUM_ARGS,"Return the sum of a list of reals"},
    {"translatepath",cf_str,1,TRANSLATEPATH_ARGS,"Translate path separators from Unix style to the host's native"},
    {"usemodule",cf_class,2,USEMODULE_ARGS,"Execute cfengine module script and set class if successful"},
    {"userexists",cf_class,1,USEREXISTS_ARGS,"True if user name or numerical id exists on this host"},
