@@ -44,7 +44,7 @@ int LoadMountInfo(struct Rlist **list)
 { FILE *pp;
   char buf1[CF_BUFSIZE],buf2[CF_BUFSIZE],buf3[CF_BUFSIZE];
   char host[CF_MAXVARSIZE],source[CF_BUFSIZE],mounton[CF_BUFSIZE],vbuff[CF_BUFSIZE];
-  int i;
+  int i, nfs = false;
 
 for (i=0; VMOUNTCOMM[VSYSTEMHARDCLASS][i] != ' '; i++)
    {
@@ -78,10 +78,15 @@ do
       CfOut(cf_error,"ferror","Error getting mount info\n");
       break;
       }
+
+   if (strstr(vbuff,"nfs"))
+      {
+      nfs = true;
+      }
    
    sscanf(vbuff,"%s%s%s",buf1,buf2,buf3);
 
-   if (vbuff[0] == '\n')
+   if (vbuff[0] == '\0' || vbuff[0] == '\n')
       {
       break;
       }
@@ -201,7 +206,14 @@ do
 
    Debug("GOT: host=%s, source=%s, mounton=%s\n",host,source,mounton);
 
-   AugmentMountInfo(list,host,source,mounton,NULL);
+   if (nfs)
+      {
+      AugmentMountInfo(list,host,source,mounton,"nfs");
+      }
+   else
+      {
+      AugmentMountInfo(list,host,source,mounton,NULL);
+      }
    }
 while (!feof(pp));
 
@@ -393,7 +405,7 @@ int VerifyNotInFstab(char *name,struct Attributes a,struct Promise *pp)
 
 /* Ensure filesystem is NOT in fstab, and return no of changes */
     
-{ char fstab[CF_BUFSIZE],regex[CF_BUFSIZE],aixcomm[CF_BUFSIZE],line[CF_BUFSIZE];
+{ char regex[CF_BUFSIZE],aixcomm[CF_BUFSIZE],line[CF_BUFSIZE];
   char *host,*rmountpt,*mountpt,*fstype,*opts;
   FILE *pfp;
   struct Item *ip;
@@ -495,7 +507,7 @@ int VerifyMount(char *name,struct Attributes a,struct Promise *pp)
 
 { char comm[CF_BUFSIZE],line[CF_BUFSIZE];
   FILE *pfp;
-  char *host,*rmountpt,*mountpt,*fstype,*opts;
+  char *host,*rmountpt,*mountpt,*fstype;
  
 host = a.mount.mount_server;
 rmountpt = a.mount.mount_source;
@@ -535,7 +547,7 @@ int VerifyUnmount(char *name,struct Attributes a,struct Promise *pp)
 
 { char comm[CF_BUFSIZE],line[CF_BUFSIZE];
   FILE *pfp;
-  char *host,*rmountpt,*mountpt,*fstype,*opts;
+  char *host,*rmountpt,*mountpt,*fstype;
  
 host = a.mount.mount_server;
 rmountpt = a.mount.mount_source;
