@@ -32,65 +32,50 @@
 #include "cf3.defs.h"
 #include "cf3.extern.h"
 
+int GLOBAL_ID = 1; // Used as a primary key for convenience, 0 reserved
+extern struct Occurrences *OCCURRENCES;
+
 /*****************************************************************************/
 
-void AddTopic(struct Topic **list,char *name,char *type)
+struct Topic *IdempInsertTopic(char *classified_name)
 
-{ struct Topic *tp;
+{ char context[CF_MAXVARSIZE],topic[CF_MAXVARSIZE];
+   
+context[0] = '\0';
+topic[0] = '\0';
 
-if (tp = TopicExists(*list,name,type))
-   {
-   CfOut(cf_verbose,""," ! Topic %s already defined\n",name);
-   }
-else
-   {
-   if ((tp = (struct Topic *)malloc(sizeof(struct Topic))) == NULL)
-      {
-      CfOut(cf_error,"malloc"," !! Memory failure in AddTopic");
-      FatalError("");
-      }
-   
-   if ((tp->topic_name = strdup(name)) == NULL)
-      {
-      CfOut(cf_error,"malloc"," !! Memory failure in AddTopic");
-      FatalError("");
-      }
-   
-   if ((tp->topic_type = strdup(type)) == NULL)
-      {
-      CfOut(cf_error,"malloc"," !! Memory failure in AddTopic");
-      FatalError("");
-      }
-   
-   tp->topic_comment = NULL;
-   tp->associations = NULL;
-   tp->occurrences = NULL;
-   tp->next = *list;
-   *list = tp;
+DeClassifyTopic(classified_name,topic,context);
 
-   CF_NODES++;
-   }
+return InsertTopic(topic,context);
 }
 
+/*****************************************************************************/
+
+struct Topic *InsertTopic(char *name,char *context)
+
+{ int slot = GetHash(ToLowerStr(name));
+
+return AddTopic(&(TOPICHASH[slot]),name,context);
+}
 
 /*****************************************************************************/
 
-void AddCommentedTopic(struct Topic **list,char *name,char *comment,char *type)
+struct Topic *FindTopic(char *name)
+
+{ int slot = GetHash(ToLowerStr(name));
+
+return GetTopic(TOPICHASH[slot],name);
+}
+
+/*****************************************************************************/
+
+struct Topic *AddTopic(struct Topic **list,char *name,char *context)
 
 { struct Topic *tp;
 
-if (tp = TopicExists(*list,name,type))
+if ((tp = TopicExists(name,context)))
    {
    CfOut(cf_verbose,""," -> Topic %s already defined, ok\n",name);
-
-   if (comment && tp->topic_comment == NULL)
-      {
-      if ((tp->topic_comment = strdup(comment)) == NULL)
-         {
-         CfOut(cf_error,"malloc","Memory failure in AddTopic");
-         FatalError("");
-         }
-      }
    }
 else
    {
@@ -99,57 +84,46 @@ else
       CfOut(cf_error,"malloc"," !! Memory failure in AddTopic");
       FatalError("");
       }
+
+   tp->topic_name = strdup(NormalizeTopic(name));
    
-   if ((tp->topic_name = strdup(name)) == NULL)
+   if (context && strlen(context) > 0)
       {
-      CfOut(cf_error,"malloc"," !! Memory failure in AddTopic");
-      FatalError("");
-      }
-   
-   if (comment)
-      {
-      if ((tp->topic_comment = strdup(comment)) == NULL)
-         {
-         CfOut(cf_error,"malloc","Memory failure in AddTopic");
-         FatalError("");
-         }
+      tp->topic_context = strdup(NormalizeTopic(context));
       }
    else
       {
-      tp->topic_comment = NULL;
+      tp->topic_context = strdup("any");
       }
-   
-   if ((tp->topic_type = strdup(type)) == NULL)
-      {
-      CfOut(cf_error,"malloc","Memory failure in AddTopic");
-      FatalError("");
-      }
-   
-   tp->occurrences = NULL;
+
+   tp->id = GLOBAL_ID++;
    tp->associations = NULL;
    tp->next = *list;
    *list = tp;
    CF_NODES++;
    }
+
+return tp;
 }
 
 /*****************************************************************************/
 
-void AddTopicAssociation(struct TopicAssociation **list,char *fwd_name,char *bwd_name,struct Rlist *associates,int verify)
+void AddTopicAssociation(struct Topic *this_tp,struct TopicAssociation **list,char *fwd_name,char *bwd_name,struct Rlist *passociates,int ok_to_add_inverse)
 
 { struct TopicAssociation *ta = NULL,*texist;
-  char assoc_type[CF_MAXVARSIZE];
-  struct Rlist *rp;
+  char fwd_context[CF_MAXVARSIZE];
+  struct Rlist *rp,*rpc;
+  struct Topic *new_tp;
 
-strncpy(assoc_type,CanonifyName(fwd_name),CF_MAXVARSIZE-1);
+strncpy(fwd_context,CanonifyName(fwd_name),CF_MAXVARSIZE-1);
 
-if (associates == NULL || associates->item == NULL)
+if (passociates == NULL || passociates->item == NULL)
    {
    CfOut(cf_error," !! A topic must have at least one associate in association %s",fwd_name);
    return;
    }
 
-if ((texist = AssociationExists(*list,fwd_name,bwd_name,verify)) == NULL)
+if ((texist = AssociationExists(*list,fwd_name,bwd_name)) == NULL)
    {
    if ((ta = (struct TopicAssociation *)malloc(sizeof(struct TopicAssociation))) == NULL)
       {
@@ -171,14 +145,14 @@ if ((texist = AssociationExists(*list,fwd_name,bwd_name,verify)) == NULL)
       FatalError("");
       }
    
-   if (assoc_type && (ta->assoc_type = strdup(assoc_type)) == NULL)
+   if ((ta->fwd_context = strdup(fwd_context)) == NULL)
       {
       CfOut(cf_error,"malloc","Memory failure in AddTopicAssociation");
       FatalError("");
       }
 
    ta->associates = NULL;
-   ta->associate_topic_type = NULL;
+   ta->bwd_context = NULL;
    ta->next = *list;
    *list = ta;
    }
@@ -189,24 +163,51 @@ else
 
 /* Association now exists, so add new members */
 
-for (rp = associates; rp != NULL; rp=rp->next)
+// First make sure topics pointed to exist so that they can point to us also
+
+for (rp = passociates; rp != NULL; rp=rp->next)
    {
-   /* Defer checking until we have whole ontlogy - all types */
-   CfOut(cf_verbose,""," ---> Adding associate '%s'",rp->item);
-   IdempPrependRScalar(&(ta->associates),rp->item,rp->type);
+   char normalform[CF_BUFSIZE] = {0};
+
+   strncpy(normalform,NormalizeTopic(rp->item),CF_BUFSIZE-1);
+   new_tp = IdempInsertTopic(rp->item);
+
+   if (ok_to_add_inverse)
+      {
+      CfOut(cf_verbose,""," --> Adding '%s' with id %d as an associate of '%s'",normalform,new_tp->id,this_tp->topic_name);
+      }
+   else
+      {
+      CfOut(cf_verbose,""," ---> Reverse '%s' with id %d as an associate of '%s' (inverse)",normalform,new_tp->id,this_tp->topic_name);
+      }
+
+   if (!IsItemIn(ta->associates,normalform))
+      {
+      PrependFullItem(&(ta->associates),normalform,NULL,new_tp->id,0);
+      }
+   
+   if (ok_to_add_inverse)
+      {
+      char rev[CF_BUFSIZE];
+      struct Rlist *rlist = 0;
+      snprintf(rev,CF_BUFSIZE-1,"%s::%s",this_tp->topic_context,this_tp->topic_name);
+      PrependRScalar(&rlist,rev,CF_SCALAR);
+      AddTopicAssociation(new_tp,&(new_tp->associations),bwd_name,fwd_name,rlist,false);
+      DeleteRlist(rlist);
+      }
+       
    CF_EDGES++;
    }
 }
 
 /*****************************************************************************/
 
-void AddOccurrence(struct Occurrence **list,char *reference,struct Rlist *represents,enum representations rtype)
+void AddOccurrence(struct Occurrence **list,char *reference,struct Rlist *represents,enum representations rtype,char *context)
 
 { struct Occurrence *op = NULL;
-  struct TopRepresentation *tr;
   struct Rlist *rp;
 
-if ((op = OccurrenceExists(*list,reference,rtype)) == NULL)
+if ((op = OccurrenceExists(*list,reference,rtype,context)) == NULL)
    {
    if ((op = (struct Occurrence *)malloc(sizeof(struct Occurrence))) == NULL)
       {
@@ -215,12 +216,14 @@ if ((op = OccurrenceExists(*list,reference,rtype)) == NULL)
       }
 
    op->represents = NULL;
+   op->occurrence_context = strdup(context);
    op->locator = strdup(reference);
    op->rep_type = rtype;   
    op->next = *list;
    *list = op;
    CF_EDGES++;
    CF_NODES++;
+   CfOut(cf_verbose,""," -> Noted occurrence for %s::%s",context,reference);
    }
 
 /* Occurrence now exists, so add new subtype promises */
@@ -239,14 +242,34 @@ for (rp = represents; rp != NULL; rp=rp->next)
 
 /*********************************************************************/
 
-char *TypedTopic(char *topic,char *type)
+void AddInference(struct Inference **list,char *result,char *pre,char *qual)
+
+{ struct Inference *ip;
+ 
+if ((ip = (struct Inference *)malloc(sizeof(struct Occurrence))) == NULL)
+   {
+   CfOut(cf_error,"malloc","Memory failure in AddOccurrence");
+   FatalError("");
+   }
+
+ip->inference = strdup(result);
+ip->precedent = strdup(pre);
+ip->qualifier = strdup(qual);
+ip->next = *list;
+*list = ip;
+}
+
+/*********************************************************************/
+
+char *ClassifiedTopic(char *topic,char *context)
 
 { static char name[CF_MAXVARSIZE];
 
-Debug("TYPE(%s)/TOPIC(%s)",type,topic);
-if (type && strlen(type) > 0)
+Debug("CONTEXT(%s)/TOPIC(%s)",context,topic);
+
+if (context && strlen(context) > 0)
    {
-   snprintf(name,CF_MAXVARSIZE,"%s::%s",type,topic);
+   snprintf(name,CF_MAXVARSIZE,"%s::%s",context,topic);
    }
 else
    {
@@ -258,81 +281,96 @@ return name;
 
 /*********************************************************************/
 
-void DeTypeTopic(char *typed_topic,char *topic,char *type)
+void DeClassifyTopic(char *classified_topic,char *topic,char *context)
 
 {
-type[0] = '\0';
+context[0] = '\0';
 topic[0] = '\0';
 
-if (*typed_topic == ':')
+if (classified_topic == NULL)
    {
-   sscanf(typed_topic,"::%255[^\n]",topic);
+   return;
    }
-else if (strstr(typed_topic,"::"))
+
+if (*classified_topic == ':')
    {
-   sscanf(typed_topic,"%255[^:]::%255[^\n]",type,topic);
+   sscanf(classified_topic,"::%255[^\n]",topic);
+   }
+else if (strstr(classified_topic,"::"))
+   {
+   sscanf(classified_topic,"%255[^:]::%255[^\n]",context,topic);
    
    if (strlen(topic) == 0)
       {
-      sscanf(typed_topic,"::%255[^\n]",topic);
+      sscanf(classified_topic,"::%255[^\n]",topic);
       }
    }
 else
    {
-   strncpy(topic,typed_topic,CF_MAXVARSIZE-1);
+   strncpy(topic,classified_topic,CF_MAXVARSIZE-1);
+   }
+
+if (strlen(context) == 0)
+   {
+   strcpy(context,"any");
    }
 }
 
 /*********************************************************************/
 
-void DeTypeCanonicalTopic(char *typed_topic,char *topic,char *type)
+void DeClassifyCanonicalTopic(char *classified_topic,char *topic,char *context)
 
 {
-type[0] = '\0';
+context[0] = '\0';
 topic[0] = '\0';
 
-if (*typed_topic == '.')
+if (*classified_topic == '.')
    {
-   sscanf(typed_topic,".%255[^\n]",topic);
+   sscanf(classified_topic,".%255[^\n]",topic);
    }
-else if (strstr(typed_topic,"."))
+else if (strstr(classified_topic,"."))
    {
-   sscanf(typed_topic,"%255[^.].%255[^\n]",type,topic);
+   sscanf(classified_topic,"%255[^.].%255[^\n]",context,topic);
    
    if (strlen(topic) == 0)
       {
-      sscanf(typed_topic,".%255[^\n]",topic);
+      sscanf(classified_topic,".%255[^\n]",topic);
       }
    }
 else
    {
-   strncpy(topic,typed_topic,CF_MAXVARSIZE-1);
+   strncpy(topic,classified_topic,CF_MAXVARSIZE-1);
+   }
+
+if (strlen(context) == 0)
+   {
+   strcpy(context,"any");
    }
 }
 
 /*********************************************************************/
 
-int TypedTopicMatch(char *ttopic1,char *ttopic2)
+int ClassifiedTopicMatch(char *ttopic1,char *ttopic2)
 
-{ char type1[CF_MAXVARSIZE],topic1[CF_MAXVARSIZE];
-  char type2[CF_MAXVARSIZE],topic2[CF_MAXVARSIZE];
+{ char context1[CF_MAXVARSIZE],topic1[CF_MAXVARSIZE];
+  char context2[CF_MAXVARSIZE],topic2[CF_MAXVARSIZE];
 
 if (strcmp(ttopic1,ttopic2) == 0)
    {
    return true;
    }
    
-type1[0] = '\0';
+context1[0] = '\0';
 topic1[0] = '\0';
-type2[0] = '\0';
+context2[0] = '\0';
 topic2[0] = '\0';
 
-DeTypeTopic(ttopic1,topic1,type1);
-DeTypeTopic(ttopic2,topic2,type2);
+DeClassifyTopic(ttopic1,topic1,context1);
+DeClassifyTopic(ttopic2,topic2,context2);
 
-if (strlen(type1) > 0 && strlen(type2) > 0)
+if (strlen(context1) > 0 && strlen(context2) > 0)
    {
-   if (strcmp(topic1,topic2) == 0 && strcmp(type1,type2) == 0)
+   if (strcmp(topic1,topic2) == 0 && strcmp(context1,context2) == 0)
       {
       return true;
       }
@@ -350,6 +388,27 @@ return false;
 
 /*****************************************************************************/
 
+int GetTopicPid(char *classified_topic)
+
+{ struct Topic *tp;
+  int slot;
+  char context[CF_MAXVARSIZE],name[CF_MAXVARSIZE];
+
+name[0] = '\0';
+
+DeClassifyTopic(classified_topic,name,context);
+slot = GetHash(ToLowerStr(name));
+
+if ((tp = GetTopic(TOPICHASH[slot],classified_topic)))
+   {
+   return tp->id;
+   }
+
+return 0;
+}
+
+/*****************************************************************************/
+
 char *URLHint(char *url)
 
 { char *sp;
@@ -365,32 +424,29 @@ return sp;
 /* Level                                                                     */
 /*****************************************************************************/
 
-struct Topic *TopicExists(struct Topic *list,char *topic_name,char *topic_type)
+struct Topic *TopicExists(char *topic_name,char *topic_context)
 
 { struct Topic *tp;
-  char l[CF_BUFSIZE],r[CF_BUFSIZE];
-  
-for (tp = list; tp != NULL; tp=tp->next)
-   {
-   if (strcmp(tp->topic_name,topic_name) == 0)
-      {
-      if (topic_type && strcmp(tp->topic_type,topic_type) != 0)
-         {
-         CfOut(cf_inform,""," !! Topic \"%s\" already exists, but it promises type \"%s\" not \"%s\"\n",topic_name,tp->topic_type,topic_type);
-         return NULL;         
-         }
-      else
-         {
-         return tp;
-         }
-      }
+  int slot;
 
-   strncpy(l,ToLowerStr(tp->topic_name),CF_MAXVARSIZE);
-   strncpy(r,ToLowerStr(topic_name),CF_MAXVARSIZE);
-   
-   if (strcmp(l,r) == 0)
+slot = GetHash(ToLowerStr(topic_name));
+  
+for (tp = TOPICHASH[slot]; tp != NULL; tp=tp->next)
+   {
+   if (strcmp(tp->topic_name,NormalizeTopic(topic_name)) == 0)
       {
-      CfOut(cf_inform,""," ! Topic \"%s\" exists with different capitalization \"%s\" this could be a broken promise\n",topic_name,tp->topic_name);
+      if (topic_context)
+         {
+         if (strlen(topic_context) > 0 && strcmp(tp->topic_context,NormalizeTopic(topic_context)) == 0)
+            {
+            return tp;
+            }
+
+         if (strlen(topic_context) == 0 && strcmp(tp->topic_context,"any") == 0)
+            {
+            return tp;
+            }
+         }
       }
    }
 
@@ -399,21 +455,14 @@ return NULL;
 
 /*****************************************************************************/
 
-struct TopicAssociation *AssociationExists(struct TopicAssociation *list,char *fwd,char *bwd,int verify)
+struct TopicAssociation *AssociationExists(struct TopicAssociation *list,char *fwd,char *bwd)
 
 { struct TopicAssociation *ta;
   int yfwd = false,ybwd = false;
   enum cfreport level;
   char l[CF_BUFSIZE],r[CF_BUFSIZE];
 
-if (verify)
-   {
-   level = cf_error;
-   }
-else
-   {
-   level = cf_verbose;
-   }
+level = cf_verbose;
 
 if (fwd == NULL || (fwd && strlen(fwd) == 0))
    {
@@ -430,7 +479,7 @@ for (ta = list; ta != NULL; ta=ta->next)
    {
    if (fwd && (strcmp(fwd,ta->fwd_name) == 0))
       {
-      CfOut(cf_verbose,"","Association %s exists already\n",fwd);
+      CfOut(cf_verbose,"","Association '%s' exists already\n",fwd);
       yfwd = true;
       }
    else if (fwd && ta->fwd_name)
@@ -455,7 +504,7 @@ for (ta = list; ta != NULL; ta=ta->next)
 
    if (bwd && (strcmp(bwd,ta->bwd_name) == 0))
       {
-      CfOut(cf_verbose,""," ! Association %s exists already\n",bwd);
+      CfOut(cf_verbose,""," ! Association '%s' exists already\n",bwd);
       ybwd = true;
       }
    else if (bwd && ta->bwd_name)
@@ -496,13 +545,13 @@ return NULL;
 
 /*****************************************************************************/
 
-struct Occurrence *OccurrenceExists(struct Occurrence *list,char *locator,enum representations rep_type)
+struct Occurrence *OccurrenceExists(struct Occurrence *list,char *locator,enum representations rep_type,char *context)
 
 { struct Occurrence *op;
   
 for (op = list; op != NULL; op=op->next)
    {
-   if (strcmp(locator,op->locator) == 0)
+   if (strcmp(locator,op->locator) == 0 && strcmp(op->occurrence_context,context) == 0)
       {
       return op;
       }
@@ -516,16 +565,16 @@ return NULL;
 struct Topic *GetTopic(struct Topic *list,char *topic_name)
 
 { struct Topic *tp;
-  char type[CF_MAXVARSIZE],name[CF_MAXVARSIZE],*sp;
+  char context[CF_MAXVARSIZE],name[CF_MAXVARSIZE];
 
-strncpy(type,topic_name,CF_MAXVARSIZE-1);
+strncpy(context,topic_name,CF_MAXVARSIZE-1);
 name[0] = '\0';
 
-DeTypeTopic(topic_name,name,type);
+DeClassifyTopic(topic_name,name,context);
 
 for (tp = list; tp != NULL; tp=tp->next)
    {
-   if (strlen(type) == 0)
+   if (strlen(context) == 0)
       {
       if (strcmp(topic_name,tp->topic_name) == 0)
          {
@@ -534,7 +583,7 @@ for (tp = list; tp != NULL; tp=tp->next)
       }
    else
       {
-      if ((strcmp(name,tp->topic_name)) == 0 && (strcmp(type,tp->topic_type) == 0))
+      if ((strcmp(name,tp->topic_name)) == 0 && (strcmp(context,tp->topic_context) == 0))
          {
          return tp;          
          }
@@ -549,13 +598,13 @@ return NULL;
 struct Topic *GetCanonizedTopic(struct Topic *list,char *topic_name)
 
 { struct Topic *tp;
-  char type[CF_MAXVARSIZE],name[CF_MAXVARSIZE],*sp;
+  char context[CF_MAXVARSIZE],name[CF_MAXVARSIZE];
 
-DeTypeCanonicalTopic(topic_name,name,type);
+DeClassifyCanonicalTopic(topic_name,name,context);
 
 for (tp = list; tp != NULL; tp=tp->next)
    {
-   if (strlen(type) == 0)
+   if (strlen(context) == 0)
       {
       if (strcmp(name,CanonifyName(tp->topic_name)) == 0)
          {
@@ -564,7 +613,7 @@ for (tp = list; tp != NULL; tp=tp->next)
       }
    else
       {
-      if ((strcmp(name,CanonifyName(tp->topic_name))) == 0 && (strcmp(type,CanonifyName(tp->topic_type)) == 0))
+      if ((strcmp(name,CanonifyName(tp->topic_name))) == 0 && (strcmp(context,CanonifyName(tp->topic_context)) == 0))
          {
          return tp;          
          }
@@ -576,27 +625,53 @@ return NULL;
 
 /*****************************************************************************/
 
-char *GetTopicType(struct Topic *list,char *topic_name)
+char *GetTopicContext(char *topic_name)
 
 { struct Topic *tp;
-  static char type1[CF_MAXVARSIZE],topic1[CF_MAXVARSIZE];
+  static char context1[CF_MAXVARSIZE],topic1[CF_MAXVARSIZE];
+  int slot = GetHash(topic_name);
+ 
+context1[0] = '\0';
+DeClassifyTopic(topic_name,topic1,context1);
 
-type1[0] = '\0';
-DeTypeTopic(topic_name,topic1,type1);
-
-if (strlen(type1) > 0)
+if (strlen(context1) > 0)
    {
-   return type1;
+   return context1;
    }
 
-for (tp = list; tp != NULL; tp=tp->next)
+for (tp = TOPICHASH[slot]; tp != NULL; tp=tp->next)
    {
    if (strcmp(topic1,tp->topic_name) == 0)
       {
-      return tp->topic_type;
+      return tp->topic_context;
       }
    }
 
 return NULL;
 }
 
+/*****************************************************************************/
+
+char *NormalizeTopic(char *s)
+
+{ char *sp;
+  int special = false;
+
+for (sp = s; *sp != '\0'; sp++)
+   {
+   if (strchr("/\\&|=$@", *sp))
+      {
+      special = true;
+      break;
+      }
+   }
+
+if (special)
+   {
+   return s;
+   }
+else
+   {
+   return ToLowerStr(s);
+   }
+}
