@@ -55,12 +55,13 @@ else
 /* We assume that s is at least MAX_FILENAME large.
  * MapName() is thread-safe, but the argument is modified. */
 
-char *MapName(char *s)
-
-{ char buffer[CF_BUFSIZE];
-  char *spf,*spto;
-
 #ifdef NT
+char *MapName(char *s)
+{
+char buffer[CF_BUFSIZE];
+char *spto;
+char *spf;
+
 memset(buffer,0,CF_BUFSIZE);
 
 if (UseUnixStandard(s))
@@ -127,10 +128,15 @@ else
 memset(s,0,MAX_FILENAME);
 strncpy(s,buffer,MAX_FILENAME-1);
 
-#endif  /* NT */
-
 return s;
 }
+#else
+char *MapName(char *s)
+{
+return s;
+}
+
+#endif  /* NT */
 
 /*********************************************************/
 
@@ -654,11 +660,20 @@ return times;
 int cf_closesocket(int sd)
 
 {
+int res;
+
 #ifdef MINGW
-return closesocket(sd);
+res = closesocket(sd);
 #else
-return close(sd);
+res = close(sd);
 #endif
+
+if (res != 0)
+  {
+  CfOut(cf_error,"cf_closesocket","!! Could not close socket");
+  }
+
+return res;
 }
 
 /*******************************************************************/
@@ -702,8 +717,7 @@ return rename(oldpath,newpath);
 void *cf_malloc(size_t size, char *errLocation)
 /* Stops on memory allocation error */
 {
- char buf[CF_SMALLBUF];
- void *ptr = NULL;
+void *ptr = NULL;
  
  ptr = malloc(size);
 
@@ -762,6 +776,7 @@ return 0;
 }
 #endif  /* MINGW */
 
+/*******************************************************************/
 
 #ifdef MINGW
 const char *inet_ntop(int af, const void *src, char *dst, socklen_t cnt)
@@ -812,5 +827,47 @@ int inet_pton(int af, const char *src, void *dst)
   freeaddrinfo(ressave);
   return 0;
 }
+#endif  /* MINGW */
 
-#endif
+/*******************************************************************/
+
+int LinkOrCopy(const char *from, const char *to, int sym)
+/**
+ *  Creates symlink to file on platforms supporting it, copies on
+ *  others.
+ **/
+{
+
+#ifdef MINGW  // only copy on Windows for now
+
+
+if (!CopyFile(from,to,TRUE))
+  {
+  return false;
+  }
+
+
+#else  /* NOT MINGW */
+
+
+if(sym)
+  {
+    if (symlink(from,to) == -1)
+      {
+	return false;
+      }
+  }
+ else  // hardlink
+   {
+     if(link(from,to) == -1)
+       {
+	 return false;
+       }
+   }
+
+   
+#endif  /* NOT MINGW */
+
+
+ return true;
+}
