@@ -36,14 +36,15 @@
 
 int SelectProcess(char *procentry,char **names,int *start,int *end,struct Attributes a,struct Promise *pp)
 
-{ struct Item *proc_attr = NULL;
- int result = true,s,e;
-  char *criteria = NULL;
+{ struct AlphaList proc_attr;
+ int result = true,i;
   char *column[CF_PROCCOLS];
   struct Rlist *rp;
-    
+
 Debug("SelectProcess(%s)\n",procentry);
 
+InitAlphaList(&proc_attr);
+
 if (!a.haveselect)
    {
    return true;
@@ -54,81 +55,89 @@ if (!SplitProcLine(procentry,names,start,end,column))
    return false;
    }
 
+if (DEBUG)
+   {
+   for (i = 0; names[i] != NULL; i++)
+      {
+      printf("COL[%s] = \"%s\"\n",names[i],column[i]);
+      }
+   }
+
 for (rp = a.process_select.owner; rp != NULL; rp = rp->next)
    {
    if (SelectProcRegexMatch("USER","UID",(char *)rp->item,names,column))
       {
-      PrependItem(&proc_attr,"process_owner","");
+      PrependAlphaList(&proc_attr,"process_owner");
       break;
       }
    }
 
 if (SelectProcRangeMatch("PID","PID",a.process_select.min_pid,a.process_select.max_pid,names,column))
    {
-   PrependItem(&proc_attr,"pid","");
+   PrependAlphaList(&proc_attr,"pid");
    }
 
 if (SelectProcRangeMatch("PPID","PPID",a.process_select.min_ppid,a.process_select.max_ppid,names,column))
    {
-   PrependItem(&proc_attr,"ppid","");
+   PrependAlphaList(&proc_attr,"ppid");
    }
 
 if (SelectProcRangeMatch("PGID","PGID",a.process_select.min_pgid,a.process_select.max_pgid,names,column))
    {
-   PrependItem(&proc_attr,"pgid","");
+   PrependAlphaList(&proc_attr,"pgid");
    }
 
-if (SelectProcRangeMatch("SZ","VSZ",a.process_select.min_vsize,a.process_select.max_vsize,names,column))
+if (SelectProcRangeMatch("VSZ","SZ",a.process_select.min_vsize,a.process_select.max_vsize,names,column))
    {
-   PrependItem(&proc_attr,"vsize","");
+   PrependAlphaList(&proc_attr,"vsize");
    }
 
 if (SelectProcRangeMatch("RSS","RSS",a.process_select.min_rsize,a.process_select.max_rsize,names,column))
    {
-   PrependItem(&proc_attr,"rsize","");
+   PrependAlphaList(&proc_attr,"rsize");
    }
 
 if (SelectProcTimeCounterRangeMatch("TIME","TIME",a.process_select.min_ttime,a.process_select.max_ttime,names,column))
    {
-   PrependItem(&proc_attr,"ttime","");
+   PrependAlphaList(&proc_attr,"ttime");
    }
 
 if (SelectProcTimeAbsRangeMatch("STIME","START",a.process_select.min_stime,a.process_select.max_stime,names,column))
    {
-   PrependItem(&proc_attr,"stime","");
+   PrependAlphaList(&proc_attr,"stime");
    }
 
 if (SelectProcRangeMatch("NI","PRI",a.process_select.min_pri,a.process_select.max_pri,names,column))
    {
-   PrependItem(&proc_attr,"priority","");
+   PrependAlphaList(&proc_attr,"priority");
    }
 
 if (SelectProcRangeMatch("NLWP","NLWP",a.process_select.min_thread,a.process_select.max_thread,names,column))
    {
-   PrependItem(&proc_attr,"threads","");
+   PrependAlphaList(&proc_attr,"threads");
    }
 
 if (SelectProcRegexMatch("S","STAT",a.process_select.status,names,column))
    {
-   PrependItem(&proc_attr,"status","");
+   PrependAlphaList(&proc_attr,"status");
    }
 
 if (SelectProcRegexMatch("CMD","COMMAND",a.process_select.command,names,column))
    {
-   PrependItem(&proc_attr,"command","");
+   PrependAlphaList(&proc_attr,"command");
    }
 
 if (SelectProcRegexMatch("TTY","TTY",a.process_select.tty,names,column))
    {
-   PrependItem(&proc_attr,"tty","");
+   PrependAlphaList(&proc_attr,"tty");
    }
 
-if (result = EvaluateORString(a.process_select.process_result,proc_attr,0))
+if ((result = EvalProcessResult(a.process_select.process_result,&proc_attr)))
    {
    //ClassesFromString(fp->defines);
    }
 
-DeleteItemList(proc_attr);
+DeleteAlphaList(&proc_attr);
 
 if (result)
    {
@@ -142,6 +151,11 @@ if (result)
       }
    }
 
+for (i = 0; column[i] != NULL; i++)
+   {
+   free(column[i]);
+   }
+
 return result; 
 }
 
@@ -153,27 +167,29 @@ int SelectProcRangeMatch(char *name1,char *name2,int min,int max,char **names,ch
 
 { int i;
   long value;
- 
-for (i = 0; names[i] != NULL; i++)
-   {
-   if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
-      {
-      value = Str2Int(line[i]);
 
-      if (value == CF_NOINT)
-         {
-         CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",names[i],line[i]);
-         return false;
-         }
-      
-      if (min < value && value < max)
-         {
-         return true;
-         }
-      else
-         {   
-         return false;
-         }
+if (min == CF_NOINT || max == CF_NOINT)
+   {
+   return false;
+   }
+
+if ((i = GetProcColumnIndex(name1,name2,names)) != -1)
+   {
+   value = Str2Int(line[i]);
+   
+   if (value == CF_NOINT)
+      {
+      CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",names[i],line[i]);
+      return false;
+      }
+   
+   if (min <= value && value <= max)
+      {
+      return true;
+      }
+   else
+      {   
+      return false;
       }
    }
 
@@ -186,28 +202,31 @@ int SelectProcTimeCounterRangeMatch(char *name1,char *name2,time_t min,time_t ma
 
 { int i;
   time_t value;
- 
-for (i = 0; names[i] != NULL; i++)
+
+if (min == CF_NOINT || max == CF_NOINT)
    {
-   if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
+   return false;
+   }
+
+if ((i = GetProcColumnIndex(name1,name2,names)) != -1)
+   {
+   value = (time_t) TimeCounter2Int(line[i]);
+   
+   if (value == CF_NOINT)
       {
-      value = (time_t) TimeCounter2Int(line[i]);
-               
-      if (value == CF_NOINT)
-         {
-         CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",name1[i],line[i]);
-         return false;
-         }
-      
-      if (min < value && value < max)
-         {
-         CfOut(cf_verbose,"","Selection filter matched %s/%s = %s in [%ld,%ld]\n",name1,name2,line[i],min,max);
-         return true;
-         }
-      else
-         {   
-         return false;
-         }
+      CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",name1[i],line[i]);
+      return false;
+      }
+
+   if (min <= value && value <= max)
+      {
+      CfOut(cf_verbose,"","Selection filter matched counter range %s/%s = %s in [%ld,%ld] (= %ld secs)\n",name1,name2,line[i],min,max,value);
+      return true;
+      }
+   else
+      {   
+      Debug("Selection filter REJECTED counter range %s/%s = %s in [%ld,%ld] (= %ld secs)\n",name1,name2,line[i],min,max,value);
+      return false;
       }
    }
 
@@ -220,28 +239,30 @@ int SelectProcTimeAbsRangeMatch(char *name1,char *name2,time_t min,time_t max,ch
 
 { int i;
   time_t value;
- 
-for (i = 0; names[i] != NULL; i++)
+
+if (min == CF_NOINT || max == CF_NOINT)
    {
-   if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
+   return false;
+   }
+
+if ((i = GetProcColumnIndex(name1,name2,names)) != -1)
+   {
+   value = (time_t)TimeAbs2Int(line[i]);
+   
+   if (value == CF_NOINT)
       {
-      value = (time_t)TimeAbs2Int(line[i]);
-               
-      if (value == CF_NOINT)
-         {
-         CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",name1[i],line[i]);
-         return false;
-         }
-      
-      if (min < value && value < max)
-         {
-         CfOut(cf_verbose,"","Selection filter matched %s/%s = %s in [%ld,%ld]\n",name1,name2,line[i],min,max);
-         return true;
-         }
-      else
-         {
-         return false;
-         }
+      CfOut(cf_inform,"","Failed to extract a valid integer from %s => \"%s\" in process list\n",name1[i],line[i]);
+      return false;
+      }
+   
+   if (min <= value && value <= max)
+      {
+      CfOut(cf_verbose,"","Selection filter matched absolute %s/%s = %s in [%ld,%ld]\n",name1,name2,line[i],min,max);
+      return true;
+      }
+   else
+      {
+      return false;
       }
    }
 
@@ -258,19 +279,16 @@ if (regex == NULL)
    {
    return false;
    }
- 
-for (i = 0; names[i] != NULL; i++)
+
+if ((i = GetProcColumnIndex(name1,name2,names)) != -1)
    {
-   if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
+   if (FullTextMatch(regex,line[i]))
       {
-      if (FullTextMatch(regex,line[i]))
-         {
-         return true;
-         }
-      else
-         {   
-         return false;
-         }
+      return true;
+      }
+   else
+      {   
+      return false;
       }
    }
 
@@ -283,6 +301,10 @@ int SplitProcLine(char *proc,char **names,int *start,int *end,char **line)
 
 { int i,s,e;
 
+char *sp = NULL;
+char cols1[CF_PROCCOLS][CF_SMALLBUF] = { "" };
+char cols2[CF_PROCCOLS][CF_SMALLBUF] = { "" };
+
 Debug("SplitProcLine(%s)\n",proc); 
 
 if (proc == NULL || strlen(proc) == 0)
@@ -290,13 +312,46 @@ if (proc == NULL || strlen(proc) == 0)
    return false;
    }
 
-for (i = 0; i < CF_PROCCOLS; i++)
+memset(line, 0, sizeof(char *) * CF_PROCCOLS);
+
+// First try looking at all the separable items
+
+sp = proc;
+
+for (i = 0; i < CF_PROCCOLS && names[i] != NULL; i++)
    {
-   line[i] = NULL;
+   while(*sp == ' ')
+      {
+      sp++;
+      }
+
+   if (strcmp(names[i],"CMD") == 0 || strcmp(names[i],"COMMAND") == 0)
+      {
+      sscanf(sp,"%127[^\n]",cols1[i]);
+      sp += strlen(cols1[i]);
+      }
+   else
+      {
+      sscanf(sp,"%127s",cols1[i]);
+      sp += strlen(cols1[i]);
+      }
+   
+   // Some ps stimes may contain spaces, e.g. "Jan 25"
+   if (strcmp(names[i],"STIME") == 0 && strlen(cols1[i]) == 3)
+      {
+      char s[CF_SMALLBUF] = {0};
+      sscanf(sp,"%127s",s);
+      strcat(cols1[i]," ");
+      strcat(cols1[i],s);
+      sp += strlen(s)+1;
+      }
    }
- 
-for (i = 0; names[i] != NULL; i++)
+
+// Now try looking at columne alignment
+
+for (i = 0; i < CF_PROCCOLS && names[i] != NULL; i++)
    {
+   // Start from the header/column tab marker and count backwards until we find 0 or space
    for (s = start[i]; (s >= 0) && !isspace((int)*(proc+s)); s--)
       {
       }
@@ -305,7 +360,8 @@ for (i = 0; names[i] != NULL; i++)
       {
       s = 0;
       }
-   
+
+   // Make sure to strip off leading spaces
    while (isspace((int)proc[s]))
       {
       s++;
@@ -332,16 +388,40 @@ for (i = 0; names[i] != NULL; i++)
    
    if (s <= e)
       {
-      line[i] = (char *)malloc(e-s+2);
-      memset(line[i],0,(e-s+2));
-      strncpy(line[i],(char *)(proc+s),(e-s+1));
+      strncpy(cols2[i],(char *)(proc+s),MIN(CF_SMALLBUF-1,(e-s+1)));
       }
    else
       {
-      line[i] = (char *)malloc(1);
-      line[i][0] = '\0';
+      cols2[i][0] = '\0';
       }
+
+   Chop(cols2[i]);
+
+   if (strcmp(cols2[i],cols1[i]) != 0)
+      {
+      CfOut(cf_inform,""," !! Unacceptable model uncertainty examining processes");
+      }
+
+   line[i] = strdup(cols1[i]);
    }
 
 return true;
 }
+
+/*******************************************************************/
+
+int GetProcColumnIndex(char *name1,char *name2,char **names)
+
+{ int i;
+ 
+for (i = 0; names[i] != NULL; i++)
+   {
+   if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
+      {
+      return i;
+      }
+   }
+
+CfOut(cf_verbose,""," INFO - process column %s/%s was not supported on this system",name1,name2);
+return -1;
+}
