@@ -80,7 +80,6 @@ struct Promise *DeRefCopyPromise(char *scopeid,struct Promise *pp)
 
 { struct Promise *pcopy;
   struct Constraint *cp,*scp;
-  char scope[CF_BUFSIZE],naked[CF_MAXVARSIZE];
   struct Rval returnval;
 
 if (pp->promisee)
@@ -129,12 +128,13 @@ if (pcopy->promiser == NULL || (pp->promisee != NULL && pcopy->promisee == NULL)
    FatalError("memory");
    }
 
-pcopy->bundletype = pp->bundletype;
+pcopy->bundletype = strdup(pp->bundletype);
 pcopy->audit = pp->audit;
 pcopy->lineno = pp->lineno;
 pcopy->petype = pp->petype;      /* rtype of promisee - list or scalar recipient? */
 pcopy->bundle = strdup(pp->bundle);
 pcopy->ref = pp->ref;
+pcopy->ref_alloc = pp->ref_alloc;
 pcopy->agentsubtype = pp->agentsubtype;
 pcopy->done = pp->done;
 pcopy->conlist = NULL;
@@ -148,16 +148,13 @@ pcopy->edcontext = pp->edcontext;
 
 Debug("Copying promise constraints\n\n");
 
-
-
 /* No further type checking should be necessary here, already done by CheckConstraintTypeMatch */
 
 for (cp = pp->conlist; cp != NULL; cp=cp->next)
    {
    struct Body *bp = NULL;
    struct FnCall *fp = NULL;
-   struct Rlist *rp,*rnew;
-   enum cfdatatype dtype;
+   struct Rlist *rnew;
    char *bodyname = NULL;
 
    /* A body template reference could look like a scalar or fn to the parser w/w () */
@@ -190,7 +187,7 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
       {
       if (strcmp(bp->type,cp->lval) != 0)
          {
-         CfOut(cf_error,"","Body type mismatch for body reference \"%s\" in promise at line %d of %s\n",bodyname,pp->lineno,(pp->audit)->filename);
+         CfOut(cf_error,"","Body type mismatch for body reference \"%s\" in promise at line %d of %s (%s != %s)\n",bodyname,pp->lineno,(pp->audit)->filename,bp->type,cp->lval);
          ERRORCOUNT++;
          }
       
@@ -232,7 +229,7 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
 
          if (fp != NULL)
             {
-            CfOut(cf_error,"","body \"%s()\" was undeclared, but used in a promise near line %d of %s",bodyname,pp->lineno,(pp->audit)->filename);
+            CfOut(cf_error,"","An apparent body \"%s()\" was undeclared or could have incorrect args, but used in a promise near line %d of %s (possible unquoted literal value)",bodyname,pp->lineno,(pp->audit)->filename);
             }
          else
             {
@@ -247,6 +244,11 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
       }
    else
       {
+      if (cp->isbody && !IsBundle(BUNDLES,bodyname))
+         {
+         CfOut(cf_error,"","Apparent body \"%s()\" was undeclared, but used in a promise near line %d of %s (possible unquoted literal value)",bodyname,pp->lineno,(pp->audit)->filename);
+         }
+      
       rnew = CopyRvalItem(cp->rval,cp->type);
       scp = AppendConstraint(&(pcopy->conlist),cp->lval,rnew,cp->type,cp->classes,false);
       }
@@ -261,8 +263,7 @@ return pcopy;
 struct Promise *ExpandDeRefPromise(char *scopeid,struct Promise *pp)
 
 { struct Promise *pcopy;
-  struct Constraint *cp,*scp;
-  char scope[CF_BUFSIZE],naked[CF_MAXVARSIZE];
+  struct Constraint *cp;
   struct Rval returnval,final;
 
 Debug("ExpandDerefPromise()\n");
@@ -303,14 +304,14 @@ if (pcopy->promiser == NULL || pcopy->classes == NULL)
    FatalError("memory");
    }
 
-pcopy->bundletype = pp->bundletype;
+pcopy->bundletype = strdup(pp->bundletype);
 pcopy->done = pp->done;
 pcopy->donep = pp->donep;
 pcopy->audit = pp->audit;
 pcopy->lineno = pp->lineno;
 pcopy->bundle = strdup(pp->bundle);
 pcopy->ref = pp->ref;
-
+pcopy->ref_alloc = pp->ref_alloc;
 pcopy->agentsubtype = pp->agentsubtype;
 pcopy->conlist = NULL;
 pcopy->next = NULL;
@@ -320,14 +321,13 @@ pcopy->this_server = pp->this_server;
 pcopy->conn = pp->conn;
 pcopy->edcontext = pp->edcontext;
 
+
 /* No further type checking should be necessary here, already done by CheckConstraintTypeMatch */
 
+
 for (cp = pp->conlist; cp != NULL; cp=cp->next)
    {
-   struct FnCall *fp;
-   struct Rlist *rp;
    struct Rval returnval;
-   char type;
 
    if (ExpectedDataType(cp->lval) == cf_bundle)
       {
@@ -337,6 +337,7 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
       {
       returnval = EvaluateFinalRval(scopeid,cp->rval,cp->type,false,pp);   
       final = ExpandDanglers(scopeid,returnval,pp);
+      DeleteRvalItem(returnval.item,returnval.rtype);
       }
 
    AppendConstraint(&(pcopy->conlist),cp->lval,final.item,final.rtype,cp->classes,false);
@@ -355,11 +356,8 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
             {
             DereferenceComment(pcopy);
             }
-         
-
          }
       }
-
    }
 
 return pcopy;
@@ -370,8 +368,7 @@ return pcopy;
 struct Promise *CopyPromise(char *scopeid,struct Promise *pp)
 
 { struct Promise *pcopy;
-  struct Constraint *cp,*scp;
-  char scope[CF_BUFSIZE],naked[CF_MAXVARSIZE];
+  struct Constraint *cp;
   struct Rval returnval,final;
 
 Debug("CopyPromise()\n");
@@ -411,13 +408,14 @@ if (pcopy->promiser == NULL || pcopy->classes == NULL)
    FatalError("memory");
    }
 
-pcopy->bundletype = pp->bundletype;
+pcopy->bundletype = strdup(pp->bundletype);
 pcopy->done = pp->done;
 pcopy->donep = pp->donep;
 pcopy->audit = pp->audit;
 pcopy->lineno = pp->lineno;
 pcopy->bundle = strdup(pp->bundle);
 pcopy->ref = pp->ref;
+pcopy->ref_alloc = pp->ref_alloc;
 pcopy->agentsubtype = pp->agentsubtype;
 pcopy->conlist = NULL;
 pcopy->next = NULL;
@@ -431,10 +429,7 @@ pcopy->edcontext = pp->edcontext;
 
 for (cp = pp->conlist; cp != NULL; cp=cp->next)
    {
-   struct FnCall *fp;
-   struct Rlist *rp;
    struct Rval returnval;
-   char type;
 
    if (ExpectedDataType(cp->lval) == cf_bundle)
       {
@@ -473,10 +468,8 @@ void DebugPromise(struct Promise *pp)
   struct Body *bp;
   struct FnCall *fp;
   struct Rlist *rp;
-  char *v,rettype,vbuff[CF_BUFSIZE];
+  char *v,rettype;
   void *retval;
-  time_t lastseen,last;
-  double val,av,var;
 
 if (GetVariable("control_common","version",&retval,&rettype) != cf_notype)
    {
@@ -491,13 +484,15 @@ if (pp->promisee != NULL)
    {
    fprintf(stdout,"%s promise by \'%s\' -> ",pp->agentsubtype,pp->promiser);
    ShowRval(stdout,pp->promisee,pp->petype);
-   fprintf(stdout," if context is %s\n\n",pp->classes);
+   fprintf(stdout," if context is %s\n",pp->classes);
    }
 else
    {
-   fprintf(stdout,"%s promise by \'%s\' (implicit) if context is %s\n\n",pp->agentsubtype,pp->promiser,pp->classes);
+   fprintf(stdout,"%s promise by \'%s\' (implicit) if context is %s\n",pp->agentsubtype,pp->promiser,pp->classes);
    }
 
+fprintf(stdout,"in bundle %s of type %s\n",pp->bundle,pp->bundletype);
+
 for (cp = pp->conlist; cp != NULL; cp = cp->next)
    {
    fprintf(stdout,"%10s => ",cp->lval);
@@ -505,7 +500,7 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
    switch (cp->type)
       {
       case CF_SCALAR:
-          if (bp = IsBody(BODIES,(char *)cp->rval))
+          if ((bp = IsBody(BODIES,(char *)cp->rval)))
              {
              ShowBody(bp,15);
              }
@@ -524,7 +519,7 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
       case CF_FNCALL:
           fp = (struct FnCall *)cp->rval;
 
-          if (bp = IsBody(BODIES,fp->name))
+          if ((bp = IsBody(BODIES,fp->name)))
              {
              ShowBody(bp,15);
              }
@@ -598,7 +593,7 @@ if (pp->this_server != NULL)
    free(pp->this_server);
    ThreadUnlock(cft_policy);
    }
- 
+
 if (pp->next != NULL)
    {
    DeletePromises(pp->next);
@@ -630,7 +625,7 @@ if ((pp = (struct Promise *)malloc(sizeof(struct Promise))) == NULL)
 
 pp->audit = AUDITPTR;
 pp->lineno = 0;
-pp->bundle =  strdup("independent");
+pp->bundle =  strdup("internal_bundle");
 pp->promiser = strdup(promiser);
 
 ThreadUnlock(cft_policy);
@@ -638,7 +633,6 @@ ThreadUnlock(cft_policy);
 pp->promisee = NULL;
 pp->petype = CF_NOPROMISEE;
 pp->classes = NULL;
-pp->conlist = NULL;
 pp->done = false;
 pp->donep = &(pp->done);
 
@@ -651,7 +645,13 @@ pp->cache = NULL;
 pp->bundletype = NULL;
 pp->agentsubtype = typename;   /* cache this, not copy strdup(typename);*/
 pp->ref = NULL;                /* cache a reference if given*/
+pp->ref_alloc = 'n';
 pp->next = NULL;
+
+
+pp->conlist = NULL;  // this fn is used for internal promises only
+AppendConstraint(&(pp->conlist), "handle", strdup("internal_promise"),CF_SCALAR,NULL,false);
+
 return pp;
 }
 
@@ -660,13 +660,13 @@ return pp;
 void DeletePromise(struct Promise *pp)
 
 {
-Debug("DeletePromise(%s->[%c])\n",pp->promiser,pp->petype);
-
 if (pp == NULL)
    {
    return;
    }
 
+Debug("DeletePromise(%s->[%c])\n",pp->promiser,pp->petype);
+
 ThreadLock(cft_policy);
 
 if (pp->promiser != NULL)
@@ -680,6 +680,7 @@ if (pp->promisee != NULL)
    }
 
 free(pp->bundle);
+free(pp->bundletype);
 free(pp->classes);
 
 // ref and agentsubtype are only references, do not free
@@ -696,7 +697,7 @@ void DeleteDeRefPromise(char *scopeid,struct Promise *pp)
 
 { struct Constraint *cp;
 
-Debug("ExpandDerefPromise()\n");
+Debug("DeleteDeRefPromise()\n");
 
 free(pp->promiser);
 
@@ -762,7 +763,7 @@ if (pp->ref)
 void HashPromise(char *salt,struct Promise *pp,unsigned char digest[EVP_MAX_MD_SIZE+1],enum cfhashes type)
 
 { EVP_MD_CTX context;
-  int len, md_len;
+  int md_len;
   const EVP_MD *md = NULL;
   struct Constraint *cp;
   struct Rlist *rp;
@@ -776,13 +777,22 @@ md = EVP_get_digestbyname(FileHashName(type));
    
 EVP_DigestInit(&context,md);
 
-EVP_DigestUpdate(&context,pp->promiser,strlen(pp->promiser));
+// multiple packages (promisers) may share same package_list_update_ifelapsed lock
+if (!(salt && (strncmp(salt, PACK_UPIFELAPSED_SALT, sizeof(PACK_UPIFELAPSED_SALT) - 1) == 0)))
+   {
+   EVP_DigestUpdate(&context,pp->promiser,strlen(pp->promiser));
+   }
 
 if (pp->ref)
    {
    EVP_DigestUpdate(&context,pp->ref,strlen(pp->ref));
    }
 
+if (pp->this_server)
+   {
+   EVP_DigestUpdate(&context,pp->this_server,strlen(pp->this_server));
+   }
+
 if (salt)
    {
    EVP_DigestUpdate(&context,salt,strlen(salt));
@@ -797,14 +807,14 @@ for (cp = pp->conlist; cp != NULL; cp=cp->next)
 
    for (i = 0; noRvalHash[i] != NULL; i++ )
       {
-      if(strcmp(cp->lval, noRvalHash[i]) == 0)
+      if (strcmp(cp->lval, noRvalHash[i]) == 0)
 	 {
          doHash = false;
          break;
 	 }
       }
    
-   if(!doHash)
+   if (!doHash)
       {
       continue;
       }
@@ -852,7 +862,7 @@ void DereferenceComment(struct Promise *pp)
 
 strncpy(pre_buffer,pp->ref,CF_BUFSIZE);
 
-if (sp = strstr(pre_buffer,"$(this.promiser)"))
+if ((sp = strstr(pre_buffer,"$(this.promiser)")))
    {
    *sp = '\0';
    offset = sp - pre_buffer + strlen("$(this.promiser)");
