@@ -35,48 +35,60 @@
 
 char *CFX[][2] =
    {
-    "<head>","</head>",
-    "<bundle>","</bundle>",
-    "<block>","</block>",
-    "<blockheader>","</blockheader>",
-    "<blockid>","</blockid>",
-    "<blocktype>","</blocktype>",
-    "<args>","</args>",
-    "<promise>","</promise>",
-    "<class>","</class>",
-    "<subtype>","</subtype>",
-    "<object>","</object>",
-    "<lval>","</lval>",
-    "<rval>","</rval>",
-    "<qstring>","</qstring>",
-    "<rlist>","</rlist>",
-    "<function>","</function>",
-    "\n","\n",
-    NULL,NULL
+    {"<head>","</head>"},
+    {"<bundle>","</bundle>"},
+    {"<block>","</block>"},
+    {"<blockheader>","</blockheader>"},
+    {"<blockid>","</blockid>"},
+    {"<blocktype>","</blocktype>"},
+    {"<args>","</args>"},
+    {"<promise>","</promise>"},
+    {"<class>","</class>"},
+    {"<subtype>","</subtype>"},
+    {"<object>","</object>"},
+    {"<lval>","</lval>"},
+    {"<rval>","</rval>"},
+    {"<qstring>","</qstring>"},
+    {"<rlist>","</rlist>"},
+    {"<function>","</function>"},
+    {"\n","\n"},
+    {NULL,NULL}
    };
 
 char *CFH[][2] =
    {
-    "<html><head>\n<link rel=\"stylesheet\" type=\"text/css\" href=\"/cf_enterprise.css\" />\n</head>\n","</html>",
-    "<div id=\"bundle\"><table class=border><tr><td><h2>","</td></tr></h2></table></div>",
-    "<div id=\"block\"><table class=border cellpadding=5 width=800>","</table></div>",
-    "<tr><th>","</th></tr>",
-    "<span class=\"bodyname\">","</span>",
-    "<span class=\"bodytype\">","</span>",
-    "<span class=\"args\">","</span>",
-    "<tr><td><table class=\"border\"><tr><td>","</td></tr></table></td></tr>",
-    "<span class=\"class\">","</span>",
-    "<span class=\"subtype\">","</span>",
-    "<b>","</b>",
-    "<br><span class=\"lval\">........................","</span>",
-    "<span class=\"rval\">","</span>",
-    "<span class=\"qstring\">","</span>",
-    "<span class=\"rlist\">","</span>",
-    "","",
-    "<tr><td>","</td></tr>",
-    NULL,NULL
+    {"<html><head>\n<link rel=\"stylesheet\" type=\"text/css\" href=\"/cf_enterprise.css\" />\n</head>\n","</html>"},
+    {"<div id=\"bundle\"><table class=border><tr><td><h2>","</td></tr></h2></table></div>"},
+    {"<div id=\"block\"><table class=border cellpadding=5 width=800>","</table></div>"},
+    {"<tr><th>","</th></tr>"},
+    {"<span class=\"bodyname\">","</span>"},
+    {"<span class=\"bodytype\">","</span>"},
+    {"<span class=\"args\">","</span>"},
+    {"<tr><td><table class=\"border\"><tr><td>","</td></tr></table></td></tr>"},
+    {"<span class=\"class\">","</span>"},
+    {"<span class=\"subtype\">","</span>"},
+    {"<b>","</b>"},
+    {"<br><span class=\"lval\">........................","</span>"},
+    {"<span class=\"rval\">","</span>"},
+    {"<span class=\"qstring\">","</span>"},
+    {"<span class=\"rlist\">","</span>"},
+    {"",""},
+    {"<tr><td>","</td></tr>"},
+    {NULL,NULL}
    };
 
+/* Prototypes */
+
+static void ShowControlBodies(void);
+static void ReportBanner(char *s);
+static void Indent(int i);
+static void ShowDataTypes(void);
+static void ShowBundleTypes(void);
+static void ShowPromiseTypesFor(char *s);
+static void ShowBodyParts(struct BodySyntax *bs);
+static void ShowRange(char *s,enum cfdatatype type);
+static void ShowBuiltinFunctions(void);
+
 /*******************************************************************/
 /* Generic                                                         */
 /*******************************************************************/
@@ -85,31 +97,32 @@ void ShowContext(void)
 
 { struct Item *ptr;
   char vbuff[CF_BUFSIZE];
-
+  int i;
+  
  /* Text output */
 
 CfOut(cf_verbose,"","");
 
-ptr = SortItemListNames(VHEAP);
-VHEAP = ptr;
+for (i = 0; i < CF_ALPHABETSIZE; i++)
+   {
+   ptr = SortItemListNames(VHEAP.list[i]);
+   VHEAP.list[i] = ptr;
+   }
 
 if (VERBOSE||DEBUG)
    {
    snprintf(vbuff,CF_BUFSIZE,"Host %s's basic classified context",VFQNAME);
    ReportBanner(vbuff);
    
-   printf("%s  -> Defined classes = { ",VPREFIX);
-   
-   for (ptr = VHEAP; ptr != NULL; ptr=ptr->next)
-      {
-      printf("%s ",ptr->name);
-      }
+   printf("%s>  -> Defined classes = { ",VPREFIX);
+
+   ListAlphaList(stdout,VHEAP,' ');
    
    printf("}\n");
 
    CfOut(cf_verbose,"","");
    
-   printf("%s  -> Negated Classes = { ",VPREFIX);
+   printf("%s>  -> Negated Classes = { ",VPREFIX);
    
    for (ptr = VNEGHEAP; ptr != NULL; ptr=ptr->next)
       {
@@ -124,38 +137,46 @@ CfOut(cf_verbose,"","");
 
 /*******************************************************************/
 
-void ShowControlBodies()
+static void ShowControlBodies()
 
 { int i;
 
 printf("<h1>Control bodies for cfengine components</h1>\n");
 
 printf("<div id=\"bundles\">");
-printf("<ul>\n");
 
 for (i = 0; CF_ALL_BODIES[i].btype != NULL; i++)
    {
-   printf("<li>COMPONENT %s</li>\n", CF_ALL_BODIES[i].btype);
+   printf("<h4>COMPONENT %s</h4>\n", CF_ALL_BODIES[i].btype);
 
-   printf("<li><h4>PROMISE TYPE %s</h4>\n",CF_ALL_BODIES[i].subtype);
+   printf("<h4>PROMISE TYPE %s</h4>\n",CF_ALL_BODIES[i].subtype);
    ShowBodyParts(CF_ALL_BODIES[i].bs);
-   printf("</li>\n");
    }
-
-printf("</ul></div>\n\n");
 }
 
 /*******************************************************************/
 
 void ShowPromises(struct Bundle *bundles,struct Body *bodies)
 
-{ struct Bundle *bp;
-  struct Rlist *rp;
-  struct SubType *sp;
-  struct Promise *pp;
-  struct Body *bdp;
-  char *v,rettype,vbuff[CF_BUFSIZE];
-  void *retval;
+{
+#if defined(HAVE_NOVA)
+Nova_StoreUnExpandedPromises(bundles,bodies);
+#else
+ShowPromisesInReport(bundles, bodies);
+#endif
+}
+
+void ShowPromisesInReport(struct Bundle *bundles, struct Body *bodies)
+{
+char rettype;
+void *retval;
+char *v;
+char vbuff[CF_BUFSIZE];
+struct Bundle *bp;
+struct Rlist *rp;
+struct SubType *sp;
+struct Promise *pp;
+struct Body *bdp;
 
 if (GetVariable("control_common","version",&retval,&rettype) != cf_notype)
    {
@@ -176,8 +197,6 @@ fprintf(FREPORT_HTML,"<p>");
   
 for (bp = bundles; bp != NULL; bp=bp->next)
    {
-   BundleNode(FREPORT_HTML,bp->name);
-
    fprintf(FREPORT_HTML,"%s Bundle %s%s%s %s%s%s\n",
            CFH[cfx_bundle][cfb],
            CFH[cfx_blocktype][cfb],bp->type,CFH[cfx_blocktype][cfe],
@@ -203,7 +222,6 @@ for (bp = bundles; bp != NULL; bp=bp->next)
    for (sp = bp->subtypes; sp != NULL; sp = sp->next)
       {
       fprintf(FREPORT_HTML,"%s",CFH[cfx_line][cfb]);
-      TypeNode(FREPORT_HTML,sp->name);
       fprintf(FREPORT_HTML,"%s",CFH[cfx_line][cfe]);
       fprintf(FREPORT_TXT,"   TYPE: %s\n\n",sp->name);
       
@@ -230,7 +248,6 @@ for (bdp = bodies; bdp != NULL; bdp=bdp->next)
    fprintf(FREPORT_HTML,"%s%s\n",CFH[cfx_line][cfb],CFH[cfx_block][cfb]);
    fprintf(FREPORT_HTML,"%s\n",CFH[cfx_promise][cfb]);
 
-   BodyNode(FREPORT_HTML,bdp->name,0);
    ShowBody(bdp,3);
 
    fprintf(FREPORT_TXT,"\n");
@@ -246,14 +263,9 @@ CfHtmlFooter(FREPORT_HTML,FOOTER);
 
 void ShowPromise(struct Promise *pp, int indent)
 
-{ struct Constraint *cp;
-  struct Body *bp;
-  struct FnCall *fp;
-  struct Rlist *rp;
-  char *v,rettype,vbuff[CF_BUFSIZE];
-  void *retval;
-  time_t lastseen,last;
-  double val,av,var;
+{
+char *v,rettype;
+void *retval;
 
 if (GetVariable("control_common","version",&retval,&rettype) != cf_notype)
    {
@@ -264,10 +276,26 @@ else
    v = "not specified";
    }
 
+#if defined(HAVE_NOVA)
+Nova_ShowPromise(v, pp, indent);
+#else
+ShowPromiseInReport(v, pp, indent);
+#endif
+}
+
+void ShowPromiseInReport(const char *version, struct Promise* pp, int indent)
+{
+struct Constraint *cp;
+struct Body *bp;
+struct Rlist *rp;
+struct FnCall *fp;
+double av;
+double var;
+double val;
+time_t last;
+
 fprintf(FREPORT_HTML,"%s\n",CFH[cfx_line][cfb]);
 fprintf(FREPORT_HTML,"%s\n",CFH[cfx_promise][cfb]);
-MapPromiseToTopic(FKNOW,pp,v);
-PromiseNode(FREPORT_HTML,pp,0);
 fprintf(FREPORT_HTML,"Promise type is %s%s%s, ",CFH[cfx_subtype][cfb],pp->agentsubtype,CFH[cfx_subtype][cfe]);
 fprintf(FREPORT_HTML,"<a href=\"#class_context\">context</a> is %s%s%s <br><hr>\n\n",CFH[cfx_class][cfb],pp->classes,CFH[cfx_class][cfe]);
 
@@ -303,7 +331,7 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
    switch (cp->type)
       {
       case CF_SCALAR:
-          if (bp = IsBody(BODIES,(char *)cp->rval))
+         if ((bp = IsBody(BODIES,(char *)cp->rval)))
              {
              ShowBody(bp,15);
              }
@@ -329,7 +357,7 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
       case CF_FNCALL:
           fp = (struct FnCall *)cp->rval;
 
-          if (bp = IsBody(BODIES,fp->name))
+          if ((bp = IsBody(BODIES,fp->name)))
              {
              ShowBody(bp,15);
              }
@@ -346,8 +374,7 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
       Indent(indent);
       fprintf(FREPORT_HTML," , if body <a href=\"#class_context\">context</a> <span class=\"context\">%s</span>\n",cp->classes);
       fprintf(FREPORT_TXT," if body context %s\n",cp->classes);
-      }
-     
+      }     
    }
 
 av = 0;
@@ -355,38 +382,10 @@ var = 0;
 val = 0;
 last = 0;
 
-#ifdef HAVE_LIBCFNOVA
-lastseen = GetPromiseCompliance(pp,&val,&av,&var,&last);
-
-if (lastseen) /* This only gives something in Nova or higher */
-   {
-   strncpy(vbuff,cf_ctime(&lastseen),CF_MAXVARSIZE);
-   Chop(vbuff);
-   
-   fprintf(FREPORT_HTML,"<hr><p><div id=\"compliance\">Compliance last checked on %s. At that time the system was ",vbuff);
-   if (val = 1.0)
-      {
-      fprintf(FREPORT_HTML,"COMPLIANT.");
-      }
-   else if (val = 0.5)
-      {
-      fprintf(FREPORT_HTML,"REPAIRED.");
-      }
-   else if (val = 0.0)
-      {
-      fprintf(FREPORT_HTML,"NON-COMPLIANT.");
-      }
-
-   fprintf(FREPORT_HTML," Average compliance %.1lf pm %.1lf percent. </div>",av*100.0,sqrt(var)*100.0);
-   }
-#else
-fprintf(FREPORT_HTML,"<hr><p><div id=\"compliance\">Compliance level checking only in Cfengine Nova and above</div>",vbuff);
-#endif
-
 if (pp->audit)
    {
    Indent(indent);
-   fprintf(FREPORT_HTML,"<p><small>Promise (version %s) belongs to bundle <b>%s</b> (type %s) in \'<i>%s</i>\' near line %d</small></p>\n",v,pp->bundle,pp->bundletype,pp->audit->filename,pp->lineno);
+   fprintf(FREPORT_HTML,"<p><small>Promise (version %s) belongs to bundle <b>%s</b> (type %s) in \'<i>%s</i>\' near line %d</small></p>\n",version,pp->bundle,pp->bundletype,pp->audit->filename,pp->lineno);
    }
 
 fprintf(FREPORT_HTML,"%s\n",CFH[cfx_promise][cfe]);
@@ -395,13 +394,13 @@ fprintf(FREPORT_HTML,"%s\n",CFH[cfx_line][cfe]);
 if (pp->audit)
    {
    Indent(indent);
-   fprintf(FREPORT_TXT,"Promise (version %s) belongs to bundle \'%s\' (type %s) in file \'%s\' near line %d\n",v,pp->bundle,pp->bundletype,pp->audit->filename,pp->lineno);
+   fprintf(FREPORT_TXT,"Promise (version %s) belongs to bundle \'%s\' (type %s) in file \'%s\' near line %d\n",version,pp->bundle,pp->bundletype,pp->audit->filename,pp->lineno);
    fprintf(FREPORT_TXT,"\n\n");
    }
 else
    {
    Indent(indent);
-   fprintf(FREPORT_TXT,"Promise (version %s) belongs to bundle \'%s\' (type %s) near line %d\n\n",v,pp->bundle,pp->bundletype,pp->lineno);
+   fprintf(FREPORT_TXT,"Promise (version %s) belongs to bundle \'%s\' (type %s) near line %d\n\n",version,pp->bundle,pp->bundletype,pp->lineno);
    }
 }
 
@@ -409,6 +408,8 @@ else
 
 void ShowScopedVariables()
 
+/* WARNING: Not thread safe (access to VSCOPE) */
+
 { struct Scope *ptr;
 
 fprintf(FREPORT_HTML,"<div id=\"showvars\">");
@@ -420,7 +421,7 @@ for (ptr = VSCOPE; ptr != NULL; ptr=ptr->next)
       continue;
       }
 
-   fprintf(FREPORT_HTML,"<p>\nScope %s:\n<br><p>",ptr->scope);
+   fprintf(FREPORT_HTML,"<h4>\nScope %s:<h4>",ptr->scope);
    fprintf(FREPORT_TXT,"\nScope %s:\n",ptr->scope);
    
    if (ptr->hashtable)
@@ -445,7 +446,7 @@ CfOut(cf_verbose,"","***********************************************************
     
 /*******************************************************************/
 
-void ReportBanner(char *s)
+static void ReportBanner(char *s)
 
 {
 fprintf(FREPORT_TXT,"***********************************************************\n");
@@ -469,7 +470,8 @@ CfOut(cf_verbose,"","\n");
 
 void BannerSubSubType(char *bundlename,char *type)
 
-{
+{ int i;
+ 
 if (strcmp(type,"processes") == 0)
    {
    struct Item *ip;
@@ -477,11 +479,14 @@ if (strcmp(type,"processes") == 0)
 
    CfOut(cf_verbose,"","     ??? Local class context: \n");
 
-   for (ip = VADDCLASSES; ip != NULL; ip=ip->next)
+   for (i = 0; i < CF_ALPHABETSIZE; i++)
       {
-      printf("       %sǹ",ip->name);
+      for (ip = VADDCLASSES.list[i]; ip != NULL; ip=ip->next)
+         {
+         printf("       %sǹ",ip->name);
+         }
       }
-
+   
    CfOut(cf_verbose,"","\n");
    }
 
@@ -504,7 +509,7 @@ Debug("----------------------------------------------------------------\n");
 
 /*******************************************************************/
 
-void Indent(int i)
+static void Indent(int i)
 
 { int j;
 
@@ -524,8 +529,6 @@ void ShowBody(struct Body *body,int indent)
 fprintf(FREPORT_TXT,"%s body for type %s",body->name,body->type);
 fprintf(FREPORT_HTML," %s%s%s ",CFH[cfx_blocktype][cfb],body->type,CFH[cfx_blocktype][cfe]);
 
-BodyNode(FREPORT_HTML,body->name,1);
-
 fprintf(FREPORT_HTML,"%s%s%s",CFH[cfx_blockid][cfb],body->name,CFH[cfx_blockid][cfe]);
 
 if (body->args == NULL)
@@ -542,7 +545,7 @@ else
       {
       if (rp->type != CF_SCALAR)
          {
-         FatalError("ShowBody - non-scalar paramater container");
+         FatalError("ShowBody - non-scalar parameter container");
          }
 
       fprintf(FREPORT_HTML,"%s%s%s,\n",CFH[cfx_args][cfb],(char *)rp->item,CFH[cfx_args][cfe]);
@@ -554,8 +557,6 @@ else
    fprintf(FREPORT_TXT,"\n");
    }
 
-BodyNode(FREPORT_HTML,body->name,2);
-
 Indent(indent);
 fprintf(FREPORT_TXT,"{\n");
 
@@ -592,24 +593,21 @@ fprintf(FREPORT_TXT,"}\n");
 void SyntaxTree(void)
 
 {
-printf("%s",CFH[0][0]);
-
-printf("<table class=frame><tr><td>\n");
 printf("<h1>CFENGINE %s SYNTAX</h1><p>",VERSION);
 
+printf("<table class=\"frame\"><tr><td>\n");
 ShowDataTypes();
 ShowControlBodies();
 ShowBundleTypes();
 ShowBuiltinFunctions();
 printf("</td></tr></table>\n");
-printf("%s",CFH[0][1]);
 }
 
 /*******************************************************************/
 /* Level 2                                                         */
 /*******************************************************************/
 
-void ShowDataTypes()
+static void ShowDataTypes()
 
 { int i;
 
@@ -626,35 +624,47 @@ printf("</ol></td></tr></table>\n\n");
 
 /*******************************************************************/
 
-void ShowBundleTypes()
+static void ShowBundleTypes()
 
 { int i;
+  struct SubTypeSyntax *st;
 
 printf("<h1>Bundle types (software components)</h1>\n");
 
 printf("<div id=\"bundles\">");
-printf("<ul>\n");
 
 for (i = 0; CF_ALL_BODIES[i].btype != NULL; i++)
    {
-   printf("<li>COMPONENT %s</li>\n", CF_ALL_BODIES[i].btype);
+   printf("<h4>COMPONENT %s</h4>\n", CF_ALL_BODIES[i].btype);
    ShowPromiseTypesFor(CF_ALL_BODIES[i].btype);
    }
 
-printf("</ul></div>\n\n");
-}
+printf("<h4>EMBEDDED BUNDLE edit_line<h4>\n");
 
+ShowPromiseTypesFor("*");
+
+st = CF_FILES_SUBTYPES;
+
+for (i = 0; st[i].btype != NULL; i++)
+   {
+   if (strcmp("edit_line",st[i].btype) == 0)
+      {
+      ShowBodyParts(st[i].bs);
+      }
+   }
+
+printf("</div>\n\n");
+}
 
 /*******************************************************************/
 
-void ShowPromiseTypesFor(char *s)
+static void ShowPromiseTypesFor(char *s)
 
 { int i,j;
   struct SubTypeSyntax *st;
 
 printf("<div id=\"promisetype\">");
 printf("<h4>Promise types for %s bundles</h4>\n",s);
-printf("<ul>\n");
 printf("<table class=border><tr><td>\n");
 
 for (i = 0; i < CF3_MODULES; i++)
@@ -665,20 +675,19 @@ for (i = 0; i < CF3_MODULES; i++)
       {
       if (strcmp(s,st[j].btype) == 0 || strcmp("*",st[j].btype) == 0)
          {
-         printf("<li><h4>PROMISE TYPE %s</h4>\n",st[j].subtype);
+         printf("<h4>PROMISE TYPE %s</h4>\n",st[j].subtype);
          ShowBodyParts(st[j].bs);
-         printf("</li>\n");
          }
       }
    }
 
 printf("</td></tr></table>\n");
-printf("</ul></div>\n\n");
+printf("</div>\n\n");
 }
 
 /*******************************************************************/
 
-void ShowBodyParts(struct BodySyntax *bs)
+static void ShowBodyParts(struct BodySyntax *bs)
 
 { int i;
 
@@ -687,7 +696,7 @@ if (bs == NULL)
    return;
    }
  
-printf("<div id=bodies><table class=border>\n");
+printf("<div id=\"bodies\"><table class=\"border\">\n");
 
 for (i = 0; bs[i].lval != NULL; i++)
    {
@@ -716,7 +725,7 @@ printf("</table></div>\n");
 
 /*******************************************************************/
 
-void ShowRange(char *s,enum cfdatatype type)
+static void ShowRange(char *s,enum cfdatatype type)
 
 { char *sp;
  
@@ -756,7 +765,7 @@ switch (type)
 
 /*******************************************************************/
 
-void ShowBuiltinFunctions()
+static void ShowBuiltinFunctions()
 
 { int i;
 
@@ -790,6 +799,6 @@ if (PARSING)
 else
    {
    Chop(s);
-   fprintf(stderr,"Validation: %s\n",s);
+   FatalError("Validation: %s\n",s);
    }
 }
