@@ -35,7 +35,9 @@
 /*******************************************************************/
 
 void DebugVariables(char *label)
-
+/* 
+ * Not thread safe 
+ */
 { struct Scope *ptr;
 
  printf("----------------------%s ---------------------------\n",label);
@@ -56,11 +58,13 @@ printf("--------------------------------------------------\n");
 /*******************************************************************/
 
 struct Scope *GetScope(char *scope)
-
+/* 
+ * Not thread safe - returns pointer to global memory
+ */
 { struct Scope *cp = NULL;
 
 Debug("Searching for scope context %s\n",scope);
- 
+
 for (cp = VSCOPE; cp != NULL; cp=cp->next)
    {
    if (strcmp(cp->scope,scope) == 0)
@@ -93,15 +97,24 @@ strncpy(CONTEXTID,id,31);
 /*******************************************************************/
 
 void NewScope(char *name)
-
+/*
+ * Thread safe
+ */
 { struct Scope *ptr;
   
 Debug("Adding scope data %s\n", name);
 
+if (!ThreadLock(cft_vscope))
+   {
+   CfOut(cf_error, "", "!! Could not lock VSCOPE");
+   return;
+   }
+ 
 for (ptr = VSCOPE; ptr != NULL; ptr=ptr->next)
    {
    if (strcmp(ptr->scope,name) == 0)
       {
+      ThreadUnlock(cft_vscope);
       Debug("SCOPE Object %s already exists\n",name);
       return;
       }
@@ -116,7 +129,8 @@ InitHashes((struct CfAssoc**)ptr->hashtable);
 
 ptr->next = VSCOPE;
 ptr->scope = strdup(name);
-VSCOPE = ptr; 
+VSCOPE = ptr;
+ThreadUnlock(cft_vscope);
 }
 
 /*******************************************************************/
@@ -126,8 +140,7 @@ void AugmentScope(char *scope,struct Rlist *lvals,struct Rlist *rvals)
 { struct Scope *ptr;
   struct Rlist *rpl,*rpr;
   struct Rval retval;
-  void *result;
-  char *lval,rettype,naked[CF_MAXVARSIZE];
+  char *lval,naked[CF_BUFSIZE];
   int i;
 
 if (RlistLen(lvals) != RlistLen(rvals))
@@ -145,23 +158,29 @@ for (rpl = lvals, rpr=rvals; rpl != NULL; rpl = rpl->next,rpr = rpr->next)
    {
    lval = (char *)rpl->item;
 
-   CfOut(cf_verbose,"","    ? Augment scope %s with %s\n",scope,lval);
+   CfOut(cf_verbose,"","    ? Augment scope %s with %s (%c)\n",scope,lval,rpr->type);
 
    // CheckBundleParameters() already checked that there is no namespace collision
    // By this stage all functions should have been expanded, so we only have scalars left
 
    if (IsNakedVar(rpr->item,'@'))
       {
+      enum cfdatatype vtype;
       GetNaked(naked,rpr->item);
 
-      if (GetVariable(scope,naked,&(retval.item),&(retval.rtype)) != cf_notype)
+      vtype = GetVariable(scope,naked,&(retval.item),&(retval.rtype));
+
+      switch(vtype)
          {
-         NewList(scope,lval,CopyRvalItem(retval.item,CF_LIST),cf_slist);
-         }
-      else
-         {
-         CfOut(cf_error,"","List parameter \"%s\" not found while constructing scope \"%s\" - use @(scope.variable) in calling reference",naked,scope);
-         NewScalar(scope,lval,rpr->item,cf_str);         
+         case cf_slist:
+         case cf_ilist:
+         case cf_rlist:
+             NewList(scope,lval,CopyRvalItem(retval.item,CF_LIST),cf_slist);
+             break;
+         default:
+             CfOut(cf_error,""," !! List parameter \"%s\" not found while constructing scope \"%s\" - use @(scope.variable) in calling reference",naked,scope);
+             NewScalar(scope,lval,rpr->item,cf_str);             
+             break;
          }
       }
    else
@@ -197,6 +216,12 @@ void DeleteAllScope()
   
 Debug("Deleting all scoped variables\n");
 
+if (!ThreadLock(cft_vscope))
+   {
+   CfOut(cf_error, "", "!! Could not lock VSCOPE");
+   return;
+   }
+ 
 ptr = VSCOPE;
 
 while (ptr != NULL)
@@ -210,30 +235,47 @@ while (ptr != NULL)
    }
 
 VSCOPE = NULL;
+
+ThreadUnlock(cft_vscope);
 }
 
 /*******************************************************************/
 
 void DeleteScope(char *name)
+/*
+ * Thread safe
+ */
 
 { struct Scope *ptr, *prev = NULL;
-  
+  int found = false;
+ 
 Debug1("Deleting scope %s\n", name);
 
+if (!ThreadLock(cft_vscope))
+   {
+   CfOut(cf_error, "", "!! Could not lock VSCOPE");
+   return;
+   }
+
+
 for (ptr = VSCOPE; ptr != NULL; ptr=ptr->next)
    {
    if (strcmp(ptr->scope,name) == 0)
       {
       Debug("Object %s exists\n",name);
+      found = true;
       break;
       }
-   
-   prev = ptr;
+   else
+      {
+      prev = ptr;
+      }
    }
 
-if (ptr == NULL)
+if (!found)
    {
    Debug("No such scope to delete\n");
+   ThreadUnlock(cft_vscope);
    return;
    }
 
@@ -246,9 +288,15 @@ else
    prev->next = ptr->next;
    }
 
-DeleteHashes(ptr->hashtable);
+if (ptr->hashtable)
+   {
+   DeleteHashes(ptr->hashtable);
+   }
+
 free(ptr->scope);
 free((char *)ptr);
+
+ThreadUnlock(cft_vscope);
 }
 
 /*******************************************************************/
@@ -268,24 +316,36 @@ for (rp = args; rp != NULL; rp=rp->next)
 /*******************************************************************/
 
 void CopyScope(char *new, char *old)
-
+/*
+ * Thread safe
+ */
 { struct Scope *op, *np;
  
 Debug("\n*\nCopying scope data %s to %s\n*\n",old,new);
 
 NewScope(new);
 
-if (op = GetScope(old))
+if (!ThreadLock(cft_vscope))
+   {
+   CfOut(cf_error, "", "!! Could not lock VSCOPE");
+   return;
+   }
+
+if ((op = GetScope(old)))
    {
    np = GetScope(new);
    CopyHashes(np->hashtable,op->hashtable);
    }
+
+ThreadUnlock(cft_vscope);
 }
 
 /*******************************************************************/
 
 void ShowScope(char *name)
-
+/*
+ * Not thread safe - access to global variable
+ */
 { struct Scope *ptr;
 
 for (ptr = VSCOPE; ptr != NULL; ptr=ptr->next)
