@@ -62,6 +62,7 @@ extern struct Auth *ROLESTOP;
 
 void KeepFileAccessPromise(struct Promise *pp);
 void KeepLiteralAccessPromise(struct Promise *pp, char *type);
+void KeepQueryAccessPromise(struct Promise *pp,char *type);
 
 /*******************************************************************/
 /* Level                                                           */
@@ -84,39 +85,35 @@ void Summarize()
 
 CfOut(cf_verbose,"","Summarize control promises\n");
   
-if (DEBUG || D2)
-   {
-   printf("\nACCESS GRANTED ----------------------:\n");
+ CfOut(cf_verbose, "", "Granted access to paths :\n");
 
-   for (ptr = VADMIT; ptr != NULL; ptr=ptr->next)
-      {
-      printf("Path: %s (encrypt=%d)\n",ptr->path,ptr->encrypt);
+ for (ptr = VADMIT; ptr != NULL; ptr=ptr->next)
+    {
+    CfOut(cf_verbose, "", "Path: %s (encrypt=%d)\n",ptr->path,ptr->encrypt);
 
-      for (ip = ptr->accesslist; ip != NULL; ip=ip->next)
-         {
-         printf("   Admit: %s root=",ip->name);
-         for (ipr = ptr->maproot; ipr !=NULL; ipr=ipr->next)
-            {
-            printf("%s,",ipr->name);
-            }
-         printf("\n");
-         }
-      }
+    for (ip = ptr->accesslist; ip != NULL; ip=ip->next)
+       {
+       CfOut(cf_verbose, "", "   Admit: %s root=",ip->name);
+       for (ipr = ptr->maproot; ipr !=NULL; ipr=ipr->next)
+          {
+          CfOut(cf_verbose, "", "%s,",ipr->name);
+          }
+       }
+    }
    
-   printf("\nACCESS DENIAL ------------------------ :\n");
+
+ CfOut(cf_verbose, "", "Denied access to paths :\n");
    
-   for (ptr = VDENY; ptr != NULL; ptr=ptr->next)
-      {
-      printf("Path: %s\n",ptr->path);
+ for (ptr = VDENY; ptr != NULL; ptr=ptr->next)
+    {
+    CfOut(cf_verbose, "", "Path: %s\n",ptr->path);
       
-      for (ip = ptr->accesslist; ip != NULL; ip=ip->next)
-         {
-         printf("   Deny: %s\n",ip->name);
-         }      
-      }
+    for (ip = ptr->accesslist; ip != NULL; ip=ip->next)
+       {
+       CfOut(cf_verbose, "", "   Deny: %s\n",ip->name);
+       }      
+    }
 
-   printf("\n");
-   }
 
 CfOut(cf_verbose,""," -> Host IPs allowed connection access :\n");
 
@@ -177,11 +174,8 @@ for (ip = DHCPLIST; ip != NULL; ip=ip->next)
 void KeepControlPromises()
     
 { struct Constraint *cp;
-  struct Scope *ptr;
-  struct Rval rrval;
   char rettype;
   void *retval;
-  int i;
 
 CFD_MAXPROCESSES = 30;
 MAXTRIES = 5;
@@ -372,6 +366,14 @@ for (cp = ControlBodyConstraints(cf_server); cp != NULL; cp=cp->next)
       SHORT_CFENGINEPORT = (short)Str2Int(retval);
       strncpy(STR_CFENGINEPORT,retval,15);
       CfOut(cf_verbose,"","SET default portnumber = %u = %s = %s\n",(int)SHORT_CFENGINEPORT,STR_CFENGINEPORT,retval);
+      SHORT_CFENGINEPORT = htons((short)Str2Int(retval));
+      continue;
+      }
+
+   if (strcmp(cp->lval,CFS_CONTROLBODY[cfs_keyttl].lval) == 0)
+      {
+      KEYTTL = (short)Str2Int(retval);
+      CfOut(cf_verbose,"","SET key TTL = %d\n",KEYTTL);
       continue;
       }
 
@@ -392,6 +394,12 @@ if (GetVariable("control_common",CFG_CONTROLBODY[cfg_syslog_port].lval,&retval,&
    {
    strncpy(SYSLOGHOST,Hostname2IPString(retval),CF_MAXVARSIZE-1);
    }
+
+if (GetVariable("control_common",CFG_CONTROLBODY[cfg_fips_mode].lval,&retval,&rettype) != cf_notype)
+   {
+   FIPS_MODE = GetBoolean(retval);
+   CfOut(cf_verbose,"","SET FIPS_MODE = %d\n",FIPS_MODE);
+   }
 }
 
 /*********************************************************************/
@@ -401,8 +409,7 @@ void KeepContextBundles()
 { struct Bundle *bp;
   struct SubType *sp;
   struct Promise *pp;
-  char *scope, *rettype;
-  void *retval;
+  char *scope;
 
 /* Dial up the generic promise expansion with a callback */
 
@@ -420,7 +427,7 @@ for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
       
       for (sp = bp->subtypes; sp != NULL; sp = sp->next) /* get schedule */
          {
-         if (strcmp(sp->name,"access") == 0)
+         if (strcmp(sp->name,"vars") != 0 && strcmp(sp->name,"classes") != 0)
             {
             continue;
             }
@@ -445,8 +452,7 @@ void KeepPromiseBundles()
 { struct Bundle *bp;
   struct SubType *sp;
   struct Promise *pp;
-  char *scope, *rettype;
-  void *retval;
+  char *scope;
 
 /* Dial up the generic promise expansion with a callback */
 
@@ -464,7 +470,7 @@ for (bp = BUNDLES; bp != NULL; bp = bp->next) /* get schedule */
       
       for (sp = bp->subtypes; sp != NULL; sp = sp->next) /* get schedule */
          {
-         if (strcmp(sp->name,"access") != 0)
+         if (strcmp(sp->name,"access") != 0 && strcmp(sp->name,"roles") != 0)
             {
             continue;
             }
@@ -519,6 +525,12 @@ if (strcmp(pp->agentsubtype,"access") == 0 && sp && strcmp(sp,"literal") == 0)
    return;
    }
 
+if (strcmp(pp->agentsubtype,"access") == 0 && sp && strcmp(sp,"query") == 0)
+   {
+   KeepQueryAccessPromise(pp,"query");
+   return;
+   }
+
 if (strcmp(pp->agentsubtype,"access") == 0 && sp && strcmp(sp,"context") == 0)
    {
    KeepLiteralAccessPromise(pp,"context");
@@ -545,8 +557,6 @@ if (strcmp(pp->agentsubtype,"roles") == 0)
 void KeepFileAccessPromise(struct Promise *pp)
 
 { struct Constraint *cp;
-  struct Body *bp;
-  struct FnCall *fp;
   struct Rlist *rp;
   struct Auth *ap,*dp;
   char *val;
@@ -625,8 +635,6 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
 void KeepLiteralAccessPromise(struct Promise *pp,char *type)
 
 { struct Constraint *cp;
-  struct Body *bp;
-  struct FnCall *fp;
   struct Rlist *rp;
   struct Auth *ap,*dp;
   char *handle = GetConstraint("handle",pp,CF_SCALAR);
@@ -716,15 +724,92 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
 
 /*********************************************************************/
 
-void KeepServerRolePromise(struct Promise *pp)
+void KeepQueryAccessPromise(struct Promise *pp,char *type)
 
 { struct Constraint *cp;
-  struct Body *bp;
-  struct FnCall *fp;
   struct Rlist *rp;
   struct Auth *ap,*dp;
   char *val;
 
+if (!GetAuthPath(pp->promiser,VARADMIT))
+   {
+   InstallServerAuthPath(pp->promiser,&VARADMIT,&VARADMITTOP);
+   }
+
+RegisterLiteralServerData(pp->promiser,pp);
+
+if (!GetAuthPath(pp->promiser,VARDENY))
+   {
+   InstallServerAuthPath(pp->promiser,&VARDENY,&VARDENYTOP);
+   }
+
+ap = GetAuthPath(pp->promiser,VARADMIT);
+dp = GetAuthPath(pp->promiser,VARDENY);
+
+if (strcmp(type,"query") == 0)
+   {
+   ap->literal = true;
+   }
+
+for (cp = pp->conlist; cp != NULL; cp = cp->next)
+   {
+   if (!IsDefinedClass(cp->classes))
+      {
+      continue;
+      }
+
+   switch (cp->type)
+      {
+      case CF_SCALAR:
+
+          val = (char *)cp->rval;
+
+          if (strcmp(cp->lval,CF_REMACCESS_BODIES[cfs_encrypted].lval) == 0)
+             {
+             ap->encrypt = true;
+             }
+             
+          break;
+
+      case CF_LIST:
+          
+          for (rp = (struct Rlist *)cp->rval; rp != NULL; rp=rp->next)
+             {
+             if (strcmp(cp->lval,CF_REMACCESS_BODIES[cfs_admit].lval) == 0)
+                {
+                PrependItem(&(ap->accesslist),rp->item,NULL);
+                continue;
+                }
+             
+             if (strcmp(cp->lval,CF_REMACCESS_BODIES[cfs_deny].lval) == 0)
+                {
+                PrependItem(&(dp->accesslist),rp->item,NULL);
+                continue;
+                }
+
+             if (strcmp(cp->lval,CF_REMACCESS_BODIES[cfs_maproot].lval) == 0)
+                {
+                PrependItem(&(ap->maproot),rp->item,NULL);
+                continue;
+                }
+             }
+          break;
+
+      case CF_FNCALL:
+          /* Shouldn't happen */
+          break;
+      }
+   }
+}
+
+/*********************************************************************/
+
+void KeepServerRolePromise(struct Promise *pp)
+
+{ struct Constraint *cp;
+  struct Rlist *rp;
+  struct Auth *ap;
+
 if (!GetAuthPath(pp->promiser,ROLES))
    {
    InstallServerAuthPath(pp->promiser,&ROLES,&ROLESTOP);
@@ -758,7 +843,14 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
           break;
 
       default:
-          CfOut(cf_error,"","RHS of authorize promise for %s should be a list\n",pp->promiser);
+
+          if (strcmp(cp->lval,"comment") == 0 || strcmp(cp->lval,"handle") == 0)
+             {
+             }
+          else
+             {
+             CfOut(cf_error,"","RHS of authorize promise for %s should be a list\n",pp->promiser);
+             }
           break;
       }
    }
@@ -771,7 +863,6 @@ for (cp = pp->conlist; cp != NULL; cp = cp->next)
 void InstallServerAuthPath(char *path,struct Auth **list,struct Auth **listtop)
 
 { struct Auth *ptr;
-  char ebuff[CF_EXPANDSIZE]; 
 
 #ifdef MINGW
 int i;
@@ -815,7 +906,6 @@ ptr->next = NULL;
 struct Auth *GetAuthPath(char *path,struct Auth *list)
 
 { struct Auth *ap;
-  char ebuff[CF_EXPANDSIZE]; 
 
 #ifdef MINGW
 int i;
