@@ -40,8 +40,15 @@ void CheckBundle(char *name,char *type)
 
 { struct Bundle *bp;
   char output[CF_BUFSIZE];
+  char *reserved[] = { "sys", "const", "mon", "edit", "match", "mon", "this", NULL };
 
 Debug("Checking for bundle (%s,%s)\n",name,type);
+  
+if (IsStrIn(name,reserved,false))
+   {
+   snprintf(output,CF_BUFSIZE,"Use of a reserved context as a bundle name \"%s\" ",name);
+   ReportError(output);      
+   }
 
 for (bp = BUNDLES; bp != NULL; bp=bp->next)
    {
@@ -64,7 +71,7 @@ for (bp = BODIES; bp != NULL; bp=bp->next)
    {
    if ((strcmp(name,bp->name) == 0) && (strcmp(type,bp->type) == 0))
       {
-      snprintf(output,CF_BUFSIZE,"Redefinition of body %s for %s is a broken promise",name,type);
+      snprintf(output,CF_BUFSIZE,"Redefinition of body \"%s\" for \"%s\" is a broken promise",name,type);
       ReportError(output);
       }
    }
@@ -80,7 +87,7 @@ struct SubTypeSyntax CheckSubType(char *bundletype,char *subtype)
   
 if (subtype == NULL)
    {
-   snprintf(output,CF_BUFSIZE,"Missing promise type category for %s bundle",subtype,bundletype);
+   snprintf(output,CF_BUFSIZE,"Missing promise type category for %s bundle",bundletype);
    ReportError(output);
    return CF_NOSTYPE;
    }
@@ -153,7 +160,7 @@ return cf_notype;
 void CheckConstraint(char *type,char *name,char *lval,void *rval,char rvaltype,struct SubTypeSyntax ss)
 
 { int lmatch = false;
-  int i,j,k,l, allowed = false;
+  int i,l, allowed = false;
   struct BodySyntax *bs;
   char output[CF_BUFSIZE];
   
@@ -241,20 +248,67 @@ for (i = 0; CF_COMMON_EDITBODIES[i].lval != NULL; i++)
 
 if (!lmatch || !allowed)
    {
-   snprintf(output,CF_BUFSIZE,"Constraint lvalue %s is not allowed in bundle category \'%s\'",lval,type);
+   snprintf(output,CF_BUFSIZE,"Constraint lvalue \'%s\' is not allowed in bundle category \'%s\'",lval,type);
    ReportError(output);
    }
 }
 
 /******************************************************************************************/
 
+int LvalWantsBody(char *stype,char *lval)
+
+{
+  int i,j,l;
+  struct SubTypeSyntax *ss;
+  struct BodySyntax *bs;
+
+for  (i = 0; i < CF3_MODULES; i++)
+   {
+   if ((ss = CF_ALL_SUBTYPES[i]) == NULL)
+      {
+      continue;
+      }
+  
+   for (j = 0; ss[j].subtype != NULL; j++)
+      {
+      if ((bs = ss[j].bs) == NULL)
+         {
+         continue;
+         }
+      
+      if (strcmp(ss[j].subtype,stype) != 0)
+         {
+         continue;
+         }
+
+      for (l = 0; bs[l].range != NULL; l++)
+         {
+         if (strcmp(bs[l].lval,lval) == 0)
+            {
+            if (bs[l].dtype == cf_body)
+               {
+               return true;
+               }
+            else
+               {
+               return false;
+               }
+            }
+         }
+      }
+   }
+
+return false;
+}
+
+/******************************************************************************************/
+
 void CheckSelection(char *type,char *name,char *lval,void *rval,char rvaltype)
 
 { int lmatch = false;
-  int i,j,k,l, allowed = false;
+  int i,j,k,l;
   struct SubTypeSyntax *ss;
   struct BodySyntax *bs,*bs2;
-  enum cfdatatype ltype;
   char output[CF_BUFSIZE];
   
 Debug("CheckSelection(%s,%s,",type,lval);
@@ -362,7 +416,7 @@ for  (i = 0; i < CF3_MODULES; i++)
 
 if (!lmatch)
    {
-   snprintf(output,CF_BUFSIZE,"Constraint lvalue %s is not allowed in \'%s\' constraint body",lval,type);
+   snprintf(output,CF_BUFSIZE,"Constraint lvalue \"%s\" is not allowed in \'%s\' constraint body",lval,type);
    ReportError(output);
    }
 }
@@ -507,7 +561,7 @@ Debug("end CheckConstraintTypeMatch---------\n");
 
 enum cfdatatype StringDataType(char *scopeid,char *string)
 
-{ struct Rlist *rp;
+{
   enum cfdatatype dtype;
   char rtype;
   void *rval;
@@ -629,16 +683,14 @@ return true;
 
 int CheckParseClass(char *lval,char *s,char *range)
 
-{ regex_t rx;
-  regmatch_t pmatch;
-  char output[CF_BUFSIZE];
+{ char output[CF_BUFSIZE];
 
 if (s == NULL)
    {
    return false;
    }
   
-Debug("\nCheckParseString(%s => %s/%s)\n",lval,s,range);
+Debug("\nCheckParseClass(%s => %s/%s)\n",lval,s,range);
   
 if (strlen(range) == 0)
    {
@@ -659,7 +711,7 @@ return false;
 
 void CheckParseInt(char *lval,char *s,char *range)
     
-{ struct Item *split,*ip;
+{ struct Item *split;
   int n;
   long max = CF_LOWINIT, min = CF_HIGHINIT, val;
   char output[CF_BUFSIZE];
@@ -805,7 +857,7 @@ Debug("CheckParseIntRange - syntax verified\n\n");
 
 void CheckParseReal(char *lval,char *s,char *range)
     
-{ struct Item *split,*ip;
+{ struct Item *split;
   double max = (double)CF_LOWINIT, min = (double)CF_HIGHINIT, val;
   int n;
   char output[CF_BUFSIZE];
@@ -944,7 +996,7 @@ Debug("CheckParseRealRange - syntax verified\n\n");
 
 void CheckParseOpts(char *lval,char *s,char *range)
 
-{ struct Item *split,*ip;
+{ struct Item *split;
   int err = false;
   char output[CF_BUFSIZE];
  
@@ -982,6 +1034,68 @@ if (!err)
 
 /****************************************************************************/
 
+int CheckParseVariableName(char *name)
+
+{ char *reserved[] = { "promiser", "handle", "promise_filename", "promise_linenumber", NULL };
+  char *sp,scopeid[CF_MAXVARSIZE],vlval[CF_MAXVARSIZE];
+  int count = 0, level = 0;
+  
+if (IsStrIn(name,reserved,false))
+   {
+   return false;
+   }
+
+scopeid[0] = '\0';
+
+if (strchr(name,'.'))
+   {
+   for (sp = name; *sp != '\0'; sp++)
+      {
+      switch (*sp)
+         {
+         case '.':
+             if (++count > 1 && level != 1)
+                {
+                return false;
+                }
+             break;
+             
+         case '[':
+             level++;
+             break;
+             
+         case ']':
+             level--;
+             break;
+             
+         default:
+             break;
+         }
+
+      if (level > 1)
+         {
+         yyerror("Too many levels of [] reserved for array use");
+         return false;
+         }
+
+      }
+
+   if (count == 1)
+      {
+      sscanf(name,"%[^.].%s",scopeid,vlval);
+   
+      if (strlen(scopeid) == 0 || strlen(vlval) == 0)
+         {
+         return false;
+         }
+      }
+   }
+
+return true;
+}
+
+/****************************************************************************/
+
 void CheckFnCallType(char *lval,char *s,enum cfdatatype dtype,char *range)
 
 { int i;
