@@ -43,6 +43,10 @@ NewScalar("const","r","\r",cf_str);
 NewScalar("const","t","\t",cf_str);
 NewScalar("const","endl","\n",cf_str);
 /* NewScalar("const","0","\0",cf_str);  - this cannot work */
+
+#ifdef HAVE_NOVA
+Nova_EnterpriseDiscovery();
+#endif
 }
 
 /*******************************************************************/
@@ -72,8 +76,17 @@ Debug("Setting local variable \"match.%s\" context; $(%s) = %s\n",lval,lval,rval
 void NewScalar(char *scope,char *lval,char *rval,enum cfdatatype dt)
 
 { struct Rval rvald;
+  struct Scope *ptr;
 
-  Debug("NewScalar(%s,%s,%s)\n",scope,lval,rval);
+Debug("NewScalar(%s,%s,%s)\n",scope,lval,rval);
+
+ptr = GetScope(scope);
+
+if (ptr == NULL)
+   {
+   CfOut(cf_error, "", "!! Attempt to add variable \"%s\" to non-existant scope \"%s\" - ignored", lval, scope);
+   return;
+   }
 
 // Newscalar allocates memory through NewAssoc
 
@@ -89,7 +102,7 @@ AddVariableHash(scope,lval,rval,CF_SCALAR,dt,NULL,0);
 
 void IdempNewScalar(char *scope,char *lval,char *rval,enum cfdatatype dt)
 
-{ char *sp1,*sp2;
+{
   struct Rval rvald;
  
 Debug("IdempNewScalar(%s,%s,%s)\n",scope,lval,rval);
@@ -118,7 +131,7 @@ if (ptr == NULL)
    return;
    }
  
-if (ap = (struct CfAssoc *)(ptr->hashtable[slot]))
+if ((ap = (struct CfAssoc *)(ptr->hashtable[slot])))
    {
    DeleteAssoc(ap);
    ptr->hashtable[slot] = NULL;
@@ -147,9 +160,52 @@ AddVariableHash(scope,sp1,rval,CF_LIST,dt,NULL,0);
 
 /*******************************************************************/
 
+void ExtendList(char *scope,char *lval,void *rval,enum cfdatatype dt)
+
+{
+  struct Rval rvald;
+  struct Rlist *list,*rp;
+
+if (GetVariable(scope,lval,&rvald.item,&rvald.rtype) != cf_notype)
+   {
+   list = rvald.item;
+   
+   switch(rvald.rtype)
+      {
+      case CF_SCALAR:
+          IdempAppendRlist(&list,rval,CF_SCALAR);
+          break;
+          
+      case CF_LIST:
+          for (rp = rval; rp != NULL; rp = rp->next)
+             {
+             IdempAppendRlist(&list,rval,CF_SCALAR);
+             }
+          break;
+          
+      case CF_FNCALL:
+          rp = IdempAppendRScalar(&list,"dummy",CF_SCALAR);
+          free(rp->item);
+          rp->item = rval;
+          rp->type = CF_FNCALL;
+          break;
+          
+      default:
+          CfOut(cf_error,"","Attempt to extend a list with something unknown");
+          break;
+      }
+   }
+else
+   {
+   NewList(scope,lval,rval,dt);
+   }
+}
+
+/*******************************************************************/
+
 enum cfdatatype GetVariable(char *scope,char *lval,void **returnv, char *rtype)
 
-{ char *sp;
+{
   int slot,i,found = false;
   struct Scope *ptr = NULL;
   char scopeid[CF_MAXVARSIZE],vlval[CF_MAXVARSIZE],sval[CF_MAXVARSIZE];
@@ -215,7 +271,7 @@ if (ptr == NULL || ptr->hashtable == NULL)
    return cf_notype;
    }
 
-Debug("GetVariable(%s,%s): using scope '%s' for variable '%s'\n",scopeid,vlval,ptr->scope,vlval);
+Debug("GetVariable(%s,%s): using scope '%s' for variable '%s' (slot =%d)\n",scopeid,vlval,ptr->scope,vlval,slot);
 
 if (CompareVariable(vlval,ptr->hashtable[slot]) != 0)
    {
@@ -229,7 +285,7 @@ if (CompareVariable(vlval,ptr->hashtable[slot]) != 0)
          {
          i = 0;
          }
-
+      
       if (CompareVariable(vlval,ptr->hashtable[i]) == 0)
          {
          found = true;
@@ -238,7 +294,7 @@ if (CompareVariable(vlval,ptr->hashtable[slot]) != 0)
 
       /* Removed autolookup in Unix environment variables -
          implement as getenv() fn instead */
-
+      
       if (i == slot)
          {
          found = false;
@@ -248,7 +304,7 @@ if (CompareVariable(vlval,ptr->hashtable[slot]) != 0)
 
    if (!found)
       {
-      Debug("No such variable found %s.%s\n",scope,lval);
+      Debug("No such variable found %s.%s\n\n",scopeid,lval);
       *returnv = lval;
       *rtype   = CF_SCALAR;
       return cf_notype;
@@ -304,7 +360,7 @@ if (CompareVariable(id,ptr->hashtable[slot]) != 0)
       
       if (CompareVariable(id,ptr->hashtable[i]) == 0)
          {
-         DeleteAssoc(ptr->hashtable[i]);
+	 DeleteAssoc(ptr->hashtable[i]);
          ptr->hashtable[i] = NULL;
          }
       }
@@ -320,7 +376,7 @@ else
 
 int CompareVariable(char *lval,struct CfAssoc *ap)
 
-{ char buffer[CF_BUFSIZE];
+{
 
 if (ap == NULL || lval == NULL)
    {
@@ -334,7 +390,7 @@ return strcmp(ap->lval,lval);
 
 int CompareVariableValue(void *rval,char rtype,struct CfAssoc *ap)
 
-{ char buffer[CF_BUFSIZE];
+{
   struct Rlist *list, *rp;
 
 if (ap == NULL || rval == NULL)
@@ -371,7 +427,7 @@ return strcmp(ap->rval,rval);
 
 int UnresolvedVariables(struct CfAssoc *ap,char rtype)
 
-{ char buffer[CF_BUFSIZE];
+{
   struct Rlist *list, *rp;
 
 if (ap == NULL)
@@ -468,6 +524,7 @@ if (strstr(s,varstr) != NULL)
    }
 
 snprintf(varstr,CF_MAXVARSIZE-1,"@{%s}",v);
+
 if (strstr(s,varstr) != NULL)
    {
    return true;
@@ -556,7 +613,7 @@ for (sp = str; *sp != '\0' ; sp++)       /* check for varitems */
 if (dollar && (bracks != 0))
    {
    char output[CF_BUFSIZE];
-   snprintf(output,CF_BUFSIZE,"Broken variable syntax or bracket mismatch in (%s)",str);
+   snprintf(output,CF_BUFSIZE,"Broken variable syntax or bracket mismatch in string (%s)",str);
    yyerror(output);
    return false;
    }
@@ -638,7 +695,7 @@ for (sp = str; *sp != '\0' ; sp++)       /* check for varitems */
 if (dollar && (bracks != 0))
    {
    char output[CF_BUFSIZE];
-   snprintf(output,CF_BUFSIZE,"Broken variable syntax or bracket mismatch in (%s)",str);
+   snprintf(output,CF_BUFSIZE,"Broken scalar variable syntax or bracket mismatch in \"%s\"",str);
    yyerror(output);
    return false;
    }
@@ -722,13 +779,13 @@ for (sp = str+2; *sp != '\0' ; sp++)       /* check for varitems */
           break;
           
       default:
-          if (isalnum((int)*sp) || IsIn(*sp,"_[]$.:-"))
+          if (isalnum((int)*sp) || strchr("_[]$.:-", *sp))
              {
              }
           else
              {
              Debug("Illegal character found: '%c'\n", *sp);
-             CfOut(cf_error,"","Illegal character somewhere in variable \"%s\" or nested expansion",str);
+             Debug("Illegal character somewhere in variable \"%s\" or nested expansion",str);
              }
       }
    
@@ -743,8 +800,11 @@ for (sp = str+2; *sp != '\0' ; sp++)       /* check for varitems */
 if (bracks != 0)
    {
    char output[CF_BUFSIZE];
-   snprintf(output,CF_BUFSIZE,"Broken variable syntax or bracket mismatch - inner (%s/%s)",str,substr);
-   yyerror(output);
+   if (strlen(substr) > 0)
+      {
+      snprintf(output,CF_BUFSIZE,"Broken variable syntax or bracket mismatch - inner (%s/%s)",str,substr);
+      yyerror(output);
+      }
    return NULL;
    }
 
@@ -851,3 +911,23 @@ for (sp = var; *sp != '\0'; sp++)
 return false;
 }
 
+/*********************************************************************/
+
+int IsCfList(char *type)
+{
+  char *listTypes[] = { "sl", "il", "rl", "ml", NULL };
+  int i;
+  
+
+  for(i = 0; listTypes[i] != NULL; i++)
+    {
+      if(strcmp(type, listTypes[i]) == 0)
+	{
+	  return true;
+	}
+    }
+
+  return false;
+}
+
+
