@@ -36,7 +36,7 @@
 
 void VerifyExecPromise(struct Promise *pp)
 
-{ struct Attributes a;
+{ struct Attributes a = {{0}};
 
 a = GetExecAttributes(pp);
 ExecSanityChecks(a,pp);
@@ -94,14 +94,13 @@ return true;
 void VerifyExec(struct Attributes a, struct Promise *pp)
     
 { struct CfLock thislock;
-  char line[CF_BUFSIZE],eventname[CF_BUFSIZE];
-  char comm[20], *sp;
+  char unsafeLine[CF_BUFSIZE], line[sizeof(unsafeLine)*2],eventname[CF_BUFSIZE];
+  char comm[20];
   char execstr[CF_EXPANDSIZE];
   struct timespec start;
-  int print, outsourced,count = 0;
+  int outsourced,count = 0;
   mode_t maskval = 0;
   FILE *pfp;
-  int preview = false;
   char cmdOutBuf[CF_BUFSIZE];
   int cmdOutBufPos = 0;
   int lineOutLen;
@@ -110,7 +109,7 @@ if (!IsExecutable(GetArg0(pp->promiser)))
    {
    cfPS(cf_error,CF_FAIL,"",pp,a,"%s promises to be executable but isn't\n",pp->promiser);
 
-   if(IsIn(' ', pp->promiser))
+   if (strchr(pp->promiser, ' '))
      {
      CfOut(cf_verbose, "", "Paths with spaces must be inside escaped quoutes (e.g. \\\"%s\\\")", pp->promiser);
      }
@@ -133,7 +132,7 @@ else
    strncpy(execstr,pp->promiser,CF_BUFSIZE);
    }
 
-thislock = AcquireLock(execstr,VUQNAME,CFSTARTTIME,a,pp);
+thislock = AcquireLock(execstr,VUQNAME,CFSTARTTIME,a,pp,false);
 
 if (thislock.lock == NULL)
    {
@@ -150,9 +149,12 @@ if (DONTDO && !a.contain.preview)
    {
    CfOut(cf_error,"","-> Would execute script %s\n",execstr);
    }
+else if(a.transaction.action != cfa_fix)
+   {
+   cfPS(cf_error,CF_WARN,"",pp,a," !! Command \"%s\" needs to be executed, but only warning was promised", execstr);
+   }
 else
    {
-   
    CommPrefix(execstr,comm);
    
    if (a.transaction.background)
@@ -197,8 +199,8 @@ else
 
       if (pfp == NULL)
          {
-         cfPS(cf_error,CF_FAIL,"cf_popen",pp,a,"Couldn't open pipe to command %s\n",execstr);
-         YieldCurrentLock(thislock);
+         cfPS(cf_error,CF_FAIL,"cf_popen",pp,a,"!! Couldn't open pipe to command %s\n",execstr);
+         YieldCurrentLock(thislock);         
          return;
          }
 
@@ -206,12 +208,14 @@ else
          {
          if (ferror(pfp))  /* abortable */
             {
-            cfPS(cf_error,CF_TIMEX,"ferror",pp,a,"Command pipe %s\n",execstr);
+            cfPS(cf_error,CF_TIMEX,"ferror",pp,a,"!! Command pipe %s\n",execstr);
+            cf_pclose(pfp);
             YieldCurrentLock(thislock);
             return;
             }
 
-         CfReadLine(line,CF_BUFSIZE-1,pfp);
+         CfReadLine(unsafeLine,CF_BUFSIZE-1,pfp);
+	 ReplaceStr(unsafeLine,line,sizeof(line),"%","%%");  // escape format char
          
          if (strstr(line,"cfengine-die"))
             {
@@ -220,7 +224,8 @@ else
          
          if (ferror(pfp))  /* abortable */
             {
-            cfPS(cf_error,CF_TIMEX,"ferror",pp,a,"Command pipe %s\n",execstr);
+            cfPS(cf_error,CF_TIMEX,"ferror",pp,a,"!! Command pipe %s\n",execstr);
+            cf_pclose(pfp);
             YieldCurrentLock(thislock);   
             return;
             }
@@ -236,7 +241,6 @@ else
             }
          else if (!a.contain.nooutput && NonEmptyLine(line))
             {
-
 	    lineOutLen = strlen(comm) + strlen(line) + 12;
 
 	    // if buffer is to small for this line, output it directly
@@ -247,10 +251,10 @@ else
 	    else
 	      {
 	      if(cmdOutBufPos + lineOutLen > sizeof(cmdOutBuf))
-	        {
-		CfOut(cf_cmdout, "", "%s", cmdOutBuf);
-		cmdOutBufPos = 0;
-		}
+                 {
+                 CfOut(cf_cmdout, "", "%s", cmdOutBuf);
+                 cmdOutBufPos = 0;
+                 }
 	      sprintf(cmdOutBuf + cmdOutBufPos, "Q: \"...%s\": %s\n",comm, line);
        	      cmdOutBufPos += (lineOutLen - 1);
 	      }
@@ -258,7 +262,7 @@ else
             }
          }
 #ifdef MINGW
-      if(outsourced)  // only get return value if we waited for command execution
+      if (outsourced)  // only get return value if we waited for command execution
 	{
 	cf_pclose(pfp);
 	}
@@ -273,12 +277,12 @@ else
 
    if (count)
       {
-      if(cmdOutBufPos)
-	{
-	CfOut(cf_cmdout, "", "%s", cmdOutBuf);
-	}
-
-      cfPS(cf_cmdout,CF_CHG,"",pp,a,"I: Last %d QUOTed lines were generated by promiser \"%s\"\n",count,execstr);
+      if (cmdOutBufPos)
+         {
+         CfOut(cf_cmdout, "", "%s", cmdOutBuf);
+         }
+      
+      CfOut(cf_cmdout,"","I: Last %d quoted lines were generated by promiser \"%s\"\n",count,execstr);
       }
    
    if (a.contain.timeout != CF_NOINT)
@@ -359,3 +363,6 @@ if (line[0] == ':')
 
 CfOut(cf_verbose,"","%s (preview of %s)\n",message,comm);
 }
+
+
+
