@@ -38,13 +38,15 @@
 
 void VerifyReportPromise(struct Promise *pp)
 
-{ struct Attributes a;
+{ struct Attributes a = {{0}};
   struct CfLock thislock;
   struct Rlist *rp;
+  char unique_name[CF_EXPANDSIZE];
 
 a = GetReportsAttributes(pp);
 
-thislock = AcquireLock(pp->promiser,VUQNAME,CFSTARTTIME,a,pp);
+snprintf(unique_name,CF_EXPANDSIZE-1,"%s_%d",pp->promiser,pp->lineno);
+thislock = AcquireLock(unique_name,VUQNAME,CFSTARTTIME,a,pp,false);
 
 if (thislock.lock == NULL)
    {
@@ -53,7 +55,7 @@ if (thislock.lock == NULL)
 
 PromiseBanner(pp);
 
-cfPS(cf_verbose,CF_CHG,"",pp,a,"Reporting about this...");
+cfPS(cf_verbose,CF_CHG,"",pp,a,"Report: %s", pp->promiser);
 
 if (a.report.to_file)
    {
@@ -187,10 +189,10 @@ if (cfstat(buffer,&statbuf) == 0)
             /* If we get here this is a process thing */
             if (offset == NULL)
                {
-               if (offset = strstr(buffer,"CMD"))
+               if ((offset = strstr(buffer,"CMD")))
                   {
                   }
-               else if (offset = strstr(buffer,"COMMAND"))
+               else if ((offset = strstr(buffer,"COMMAND")))
                   {
                   }
                
@@ -349,14 +351,14 @@ void VerifyFriendConnections(int hours,struct Attributes a,struct Promise *pp)
   char *key;
   void *value;
   int ksize,vsize;
-  int ret, secs = CF_TICKS_PER_HOUR*hours, criterion, overdue, regex=false;
+  int secs = CF_TICKS_PER_HOUR*hours, criterion, overdue;
   time_t now = time(NULL),lsea = (time_t)CF_WEEK, tthen, then;
   char name[CF_BUFSIZE],hostname[CF_BUFSIZE],datebuf[CF_MAXVARSIZE];
   char addr[CF_BUFSIZE],type[CF_BUFSIZE],output[CF_BUFSIZE];
   struct QPoint entry;
   double average = 0.0, var = 0.0, ticksperminute = 60.0;
-  double ticksperhour = (double)CF_TICKS_PER_HOUR,ticksperday = (double)CF_TICKS_PER_DAY;
- 
+  double ticksperhour = (double)CF_TICKS_PER_HOUR;
+
 CfOut(cf_verbose,"","CheckFriendConnections(%d)\n",hours);
 snprintf(name,CF_BUFSIZE-1,"%s/lastseen/%s",CFWORKDIR,CF_LASTDB_FILE);
 MapName(name);
@@ -499,7 +501,7 @@ void VerifyFriendReliability(struct Attributes a,struct Promise *pp)
 
 { CF_DB *dbp;
   CF_DBC *dbcp;
-  int i,ret,ksize,vsize;
+  int i,ksize,vsize;
   char *key;
   void *value;
   double n[CF_RELIABLE_CLASSES],n_av[CF_RELIABLE_CLASSES],total;
@@ -507,7 +509,7 @@ void VerifyFriendReliability(struct Attributes a,struct Promise *pp)
   char name[CF_BUFSIZE],hostname[CF_BUFSIZE],timekey[CF_MAXVARSIZE];
   struct QPoint entry;
   struct Item *ip, *hostlist = NULL;
-  double entropy,average,var,sum,sum_av,expect,actual;
+  double average,var,sum,sum_av,expect,actual;
   time_t now = time(NULL), then, lastseen = CF_WEEK;
 
 CfOut(cf_verbose,"","CheckFriendReliability()\n");
